@@ -2,7 +2,7 @@
 // Class: dviWindow
 // Author: Stefan Kebekus
 //
-// (C) 2001, Stefan Kebekus.
+// (C) 2001-2003, Stefan Kebekus.
 //
 // Previewer for TeX DVI files.
 //
@@ -50,7 +50,7 @@ void dviWindow::exportText(void)
     return;
   if (dviFile->dvi_Data == 0 )
     return;
-  if (pixmap->paintingActive())
+  if (currentlyDrawnPage.pixmap->paintingActive())
     return;
 
   if (KMessageBox::warningContinueCancel( this,
@@ -72,8 +72,8 @@ void dviWindow::exportText(void)
     return;
   QFileInfo finfo(fileName);
   if (finfo.exists()) {
-    int r = KMessageBox::warningYesNo (this, i18n("The file %1\nexists. Shall I overwrite that file?").arg(fileName),
-				       i18n("Overwrite file"));
+    int r = KMessageBox::warningYesNo (this, i18n("The file %1\nexists. Do you want to overwrite that file?").arg(fileName),
+				       i18n("Overwrite File"));
     if (r == KMessageBox::No)
       return;
   }
@@ -106,8 +106,8 @@ void dviWindow::exportText(void)
     draw_page(); // We gracefully ingore any errors (bad dvi-file, etc.) which may occur during draw_page()
     foreGroundPaint.end();
 
-    for(unsigned int i=0; i<textLinkList.size(); i++)
-      stream << textLinkList[i].linkText << endl;
+    for(unsigned int i=0; i<currentlyDrawnPage.textLinkList.size(); i++)
+      stream << currentlyDrawnPage.textLinkList[i].linkText << endl;
   }
 
   // Switch off the progress dialog, etc.
@@ -176,8 +176,8 @@ void dviWindow::exportPDF(void)
     return;
   QFileInfo finfo(fileName);
   if (finfo.exists()) {
-    int r = KMessageBox::warningYesNo (this, i18n("The file %1\nexists. Shall I overwrite that file?").arg(fileName),
-				       i18n("Overwrite file"));
+    int r = KMessageBox::warningYesNo (this, i18n("The file %1\nexists. Do you want to overwrite that file?").arg(fileName),
+				       i18n("Overwrite File"));
     if (r == KMessageBox::No)
       return;
   }
@@ -265,7 +265,7 @@ void dviWindow::exportPS(QString fname, QString options, KPrinter *printer)
       return;
     QFileInfo finfo(fileName);
     if (finfo.exists()) {
-      int r = KMessageBox::warningYesNo (this, i18n("The file %1\nexists. Shall I overwrite that file?").arg(fileName),
+      int r = KMessageBox::warningYesNo (this, i18n("The file %1\nexists. Do you want to overwrite that file?").arg(fileName),
 					 i18n("Overwrite File"));
       if (r == KMessageBox::No)
 	return;
@@ -435,10 +435,8 @@ void dviWindow::editorCommand_terminated(KProcess *sproc)
 
 void dviWindow::abortExternalProgramm(void)
 {
-  if (proc != 0) {
     delete proc; // Deleting the KProcess kills the child.
     proc = 0;
-  }
 
   if (export_tmpFileName.isEmpty() != true) {
     unlink(QFile::encodeName(export_tmpFileName)); // That should delete the file.
