@@ -52,15 +52,9 @@
 #include <kdebug.h>
 #include <kmessagebox.h>
 #include <klocale.h>
-#include <stdlib.h>
-#include <unistd.h>
+
 #include "dviwin.h"
-
-#include "oconfig.h"
-#include "dvi.h"
-
-#include "fontpool.h"
-#include "glyph.h"
+#include "xdvi.h"
 
 
 
@@ -89,20 +83,6 @@ please report the problem."));
   exit(1);
 }
 
-/*
- *	Allocate bitmap for given font and character
- */
-
-void alloc_bitmap(bitmap *bitmap)
-{
-  register unsigned int	size;
-
-  /* width must be multiple of 16 bits for raster_op */
-  bitmap->bytes_wide = ROUNDUP((int) bitmap->w, BITS_PER_BMUNIT) * BYTES_PER_BMUNIT;
-  size = bitmap->bytes_wide * bitmap->h;
-  bitmap->bits = new char[size != 0 ? size : 1];
-}
-
 /*
  *
  *      Read size bytes from the FILE fp, constructing them into a
