@@ -30,9 +30,10 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "dviwin.h"
-#include "oconfig.h"
 #include "dvi.h"
+#include "dviwin.h"
+#include "fontpool.h"
+#include "xdvi.h"
 
 extern void oops(QString message);
 
@@ -62,10 +63,10 @@ extern void oops(QString message);
  *	The main routine
  */
 
-void font::read_VF_index(void)
+void TeXFontDefinition::read_VF_index(void)
 {
 #ifdef DEBUG_FONTS
-  kdDebug() << "read_VF_index" << endl;
+  kdDebug(4300) << "font::read_VF_index(void)" << endl;
 #endif
   FILE *VF_file = file;
   unsigned char	cmnd;
@@ -74,14 +75,14 @@ void font::read_VF_index(void)
   flags      |= FONT_VIRTUAL;
   set_char_p  = &dviWindow::set_vf_char;
 #ifdef DEBUG_FONTS
-  kdDebug() << "Reading VF pixel file " << filename << endl;
+  kdDebug(4300) << "TeXFontDefinition::read_VF_index: reading VF pixel file " << filename << endl;
 #endif
   // Read preamble.
   fseek(VF_file, (long) one(VF_file), 1);	/* skip comment */
   long file_checksum = four(VF_file);
 
   if (file_checksum && checksum && file_checksum != checksum)
-    kdError() << i18n("Checksum mismatch") << "(dvi = " << checksum << "u, vf = " << file_checksum << 
+    kdError(4300) << i18n("Checksum mismatch") << "(dvi = " << checksum << "u, vf = " << file_checksum << 
       "u)" << i18n(" in font file ") << filename << endl;
   (void) four(VF_file);		/* skip design size */
 
@@ -89,25 +90,27 @@ void font::read_VF_index(void)
   first_font = NULL;
   while ((cmnd = one(VF_file)) >= FNTDEF1 && cmnd <= FNTDEF4) {
     int   TeXnumber = num(VF_file, (int) cmnd - FNTDEF1 + 1);
-    long  checksum  = four(VF_file);
-    int   scale     = four(VF_file);
-    int   design    = four(VF_file);
-    int   len       = one(VF_file) + one(VF_file); /* sequence point in the middle */
-    char *fontname  = new char[(unsigned) len + 1];
+    Q_UINT32 checksum  = four(VF_file);
+    Q_UINT32 scale     = four(VF_file);
+    Q_UINT32 design    = four(VF_file);
+    Q_UINT16 len       = one(VF_file) + one(VF_file); /* sequence point in the middle */
+    char *fontname  = new char[len + 1];
     fread(fontname, sizeof(char), len, VF_file);
     fontname[len] = '\0';
-	
+    
 #ifdef DEBUG_FONTS
     kdDebug() << "Virtual font defines subfont \"" << fontname << "\" scale=" << scale << " design=" << design << endl;
 #endif
 
-    // The scaled size is given in units of vfparent->scale * 2 ** -20
-    // SPELL units, so we convert it into SPELL units by multiplying
-    // by vfparent->dimconv. The design size is given in units of 2
-    // -20 pt, so we convert into SPELL units by multiplying by
-    // (pixels_per_inch * 2**16) / (72.27 * 2**20).
-    struct font *newfontp = font_pool->appendx(fontname, checksum, scale, design, 
-					       (72.27 * (1<<4)) * dimconv * scale / design, dimconv);
+    // According to Knuth's documentation found in the web source code
+    // of the "vftovp" program (which seems to be the standard
+    // definition of virtual fonts), the "scale" is a fixed point
+    // number which describes extra enlargement that the virtual font
+    // imposes. One obtains the enlargement by dividing 2^20. 
+    double enlargement_factor = double(scale)/(1<<20) * enlargement;
+
+    //    TeXFontDefinition *newfontp = font_pool->appendx(fontname, checksum, (Q_UINT32)(scaled_size_in_DVI_units*enlargement_factor), enlargement_factor);
+    TeXFontDefinition *newfontp = font_pool->appendx(fontname, checksum, (Q_UINT32)((double(scale)/(1<<20))*scaled_size_in_DVI_units), enlargement_factor);
 
     // Insert font in dictionary and make sure the dictionary is big
     // enough.
@@ -152,13 +155,14 @@ void font::read_VF_index(void)
       width = num(VF_file, 3);
     }
     m = &macrotable[cc];
-    m->dvi_adv = (int)(width * dimconv + 0.5);
+
+    m->dvi_advance_in_units_of_design_size_by_2e20 = width;
     if (len > 0) {
       if (len <= availend - avail) {
 	m->pos = avail;
 	avail += len;
       } else {
-	m->free_me = True;
+	m->free_me = true;
 	if (len <= VF_PARM_1) {
 	  m->pos = avail = new unsigned char [VF_PARM_2];
 	  availend = avail + VF_PARM_2;
@@ -171,7 +175,7 @@ void font::read_VF_index(void)
     }
   }
   if (cmnd != POST)
-    oops(QString(i18n("Wrong command byte found in VF macro list: %1")).arg(cmnd));
+    oops(i18n("Wrong command byte found in VF macro list: %1").arg(cmnd));
   
   fclose (VF_file);
   file = NULL;
