@@ -1,6 +1,6 @@
    /*
 
-    $Id: kfax.cpp,v 1.79.2.2 2003/05/15 11:28:19 waba Exp $
+    $Id: kfax.cpp,v 1.90 2003/11/11 01:55:33 bmeyer Exp $
 
     Copyright (C) 1997 Bernd Johannes Wuebben
                        wuebben@math.cornell.edu
@@ -17,7 +17,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
     */
 
@@ -51,6 +51,7 @@
 #include <kstdaction.h>
 #include <kdebug.h>
 #include <kurldrag.h>
+#include <kkeydialog.h>
 
 #include "kfax.h"
 #include "kfax.moc"
@@ -223,24 +224,24 @@ void TopLevel::setupMenuBar()
 {
   // File menu
   KStdAction::open( this, SLOT( faxOpen() ), actionCollection() );
-  actRecent =  KStdAction::openRecent( this, SLOT( faxOpen( const KURL & ) ), 
+  actRecent =  KStdAction::openRecent( this, SLOT( faxOpen( const KURL & ) ),
           actionCollection() );
   actSave = KStdAction::save( this, SLOT( faxSave() ), actionCollection() );
-  actSaveAs = KStdAction::saveAs( this, SLOT( faxSaveAs() ), 
+  actSaveAs = KStdAction::saveAs( this, SLOT( faxSaveAs() ),
           actionCollection() );
   actPrint = KStdAction::print( this, SLOT( print() ), actionCollection() );
   KStdAction::quit( this, SLOT( close() ), actionCollection() );
-
-  actAdd = new KAction( i18n( "&Add..." ), "filenew", KShortcut(), this, 
+  KStdAction::keyBindings( this, SLOT( slotConfigureKeys() ), actionCollection() );
+  actAdd = new KAction( i18n( "&Add..." ), "filenew", KShortcut(), this,
       SLOT( faxAdd() ), actionCollection(), "file_add_fax" );
 
   actRecent->setMaxItems( 5 );
 
   // View Menu
-  actSize = KStdAction::actualSize( this, SLOT( actualSize() ), 
+  actSize = KStdAction::actualSize( this, SLOT( actualSize() ),
       actionCollection() );
   actZoomIn = KStdAction::zoomIn( this, SLOT( zoomin() ), actionCollection() );
-  actZoomOut = KStdAction::zoomOut( this, SLOT( zoomout() ), 
+  actZoomOut = KStdAction::zoomOut( this, SLOT( zoomout() ),
       actionCollection() );
 
   actRotate = new KAction( i18n( "&Rotate Page" ), "rotate", KShortcut(), this,
@@ -253,17 +254,25 @@ void TopLevel::setupMenuBar()
   // Go menu
   actNext = KStdAction::next( this, SLOT( nextPage() ), actionCollection() );
   actPrev = KStdAction::prior( this, SLOT( prevPage() ), actionCollection() );
-  actFirst = KStdAction::firstPage( this, SLOT( firstPage() ), 
+  actFirst = KStdAction::firstPage( this, SLOT( firstPage() ),
       actionCollection() );
-  actLast = KStdAction::lastPage( this, SLOT( lastPage() ), 
+  actLast = KStdAction::lastPage( this, SLOT( lastPage() ),
       actionCollection() );
 
   // Settings menu
-  actStatus = KStdAction::showStatusbar( this, SLOT( toggleStatusBar() ),
-      actionCollection() );
+  createStandardStatusBarAction();
+  setStandardToolBarMenuEnabled(true);
+
+
+  createStandardStatusBarAction();
   KStdAction::preferences( this, SLOT( faxoptions() ), actionCollection() );
 }
 
+void TopLevel::slotConfigureKeys()
+{
+  KKeyDialog::configure( actionCollection(), this );
+}
+
 void TopLevel::setupStatusBar()
 {
   statusbar = statusBar();
@@ -292,7 +301,7 @@ void TopLevel::readSettings()
 
   config->setGroup("General Options");
 
-  lastDir = config->readEntry("DefaultPath", QDir::currentDirPath());
+  lastDir = config->readPathEntry("DefaultPath", QDir::currentDirPath());
   editor_width = config->readNumEntry("Width", 550);
   editor_height = config->readNumEntry("Height", 400);
 
@@ -312,8 +321,8 @@ void TopLevel::readSettings()
 
   config->setGroup("Printing");
 
-  pi.cmd = config->readEntry("PrintCommand", "lpr");
-  pi.file = config->readEntry("PrintFile");
+  pi.cmd = config->readPathEntry("PrintCommand", "lpr");
+  pi.file = config->readPathEntry("PrintFile");
   pi.lpr = config->readNumEntry("print", 1);
   pi.scale = config->readNumEntry("scale", 0);
   pi.margins = config->readNumEntry("margins", 0);
@@ -363,10 +372,10 @@ bool TopLevel::queryClose()
 void TopLevel::writeSettings()
 {
   config = kapp->config();
-	
+
   config->setGroup("General Options");
 
-  config->writeEntry("DefaultPath", lastDir);
+  config->writePathEntry("DefaultPath", lastDir);
   config->writeEntry("Width", this->width());
   config->writeEntry("Height", this->height());
 
@@ -384,15 +393,15 @@ void TopLevel::writeSettings()
 
   config->setGroup("Printing");
 
-  config->writeEntry("PrintCommand",pi.cmd);
-  config->writeEntry("PrintFile",pi.file);
+  config->writePathEntry("PrintCommand",pi.cmd);
+  config->writePathEntry("PrintFile",pi.file);
   config->writeEntry("print",pi.lpr);
   config->writeEntry("scale",pi.scale);
   config->writeEntry("PageSize",pi.pagesize);
   config->writeEntry("margins",pi.margins);
   config->writeEntry("xmargin", pi.xmargin);
   config->writeEntry("ymargin", pi.ymargin);
-  config->sync();	
+  config->sync();
 }
 
 void TopLevel::faxOpen()
@@ -436,14 +445,14 @@ void TopLevel::faxAdd( const KURL & url )
     lastDir = url.directory();
   }
 
-  openNetFile(url);  
+  openNetFile(url);
 
   updateGoActions();
 }
 
 void TopLevel::faxSave()
 {
-  saveNetFile(fileURL);  
+  saveNetFile(fileURL);
 }
 
 void TopLevel::faxSaveAs()
@@ -480,7 +489,7 @@ void TopLevel::zoom( int factor )
       Images[i+1] = ZoomImage(Images[i]);
       Image = Images[i+1];
 
-      if(Images[i+1] == NULL){ // out of memory	
+      if(Images[i+1] == NULL){ // out of memory
 	Image = Images[i];
 	break;
       }
@@ -526,7 +535,7 @@ void TopLevel::openadd(QString filename)
 {
   auxpage = lastpage;
 
-  (void) notetiff(QFile::encodeName(filename));	
+  (void) notetiff(QFile::encodeName(filename));
 
   if( firstpage != lastpage )
   {
@@ -543,16 +552,6 @@ void TopLevel::openadd(QString filename)
   putImage();
 }
 
-void TopLevel::toggleStatusBar()
-{
-  if( actStatus->isChecked() )
-    statusbar->show();
-  else
-    statusbar->hide();
-
-  resizeView();
-}
-
 void TopLevel::resizeEvent(QResizeEvent *e)
 {
   KMainWindow::resizeEvent(e);
@@ -879,7 +878,7 @@ void TopLevel::printIt(){
 
 void TopLevel::saveNetFile( const KURL& u)
 {
-    if ( u.isMalformed() )
+    if ( !u.isValid() )
     {
 	KMessageBox::sorry(this, i18n("Malformed URL"));
 	return;
@@ -914,7 +913,7 @@ void TopLevel::saveNetFile( const KURL& u)
 
 void TopLevel::openNetFile( const KURL &u)
 {
-  if ( u.isMalformed() )
+  if ( !u.isValid() )
   {
 	KMessageBox::error(this, i18n("Malformed URL"));
 	return;
@@ -950,7 +949,7 @@ void TopLevel::dropEvent( QDropEvent * event)
 {
 
   KURL::List list;
-  
+
   if (KURLDrag::decode(event, list) && !list.isEmpty())
   {
     // Load the first file in this window
@@ -1029,12 +1028,12 @@ void TopLevel::handle_X_event(XEvent Event)
 
 	if(Event.xexpose.count != 0)
 	  break;
-	
+
 	if(!Image)
 	  break;
 
 	putimage = TRUE;
-      }	
+      }
     break;
 
     case KeyPress:
@@ -1074,7 +1073,7 @@ void TopLevel::handle_X_event(XEvent Event)
 	putimage = TRUE;
 	break;
       case XK_Home:
-      case XK_R7:	
+      case XK_R7:
 	if (Event.xkey.state & ShiftMask) {
 	  thispage = firstpage;
 	  newPage();
@@ -1460,7 +1459,7 @@ void TopLevel::newPage(){
 
     XDefineCursor(qtdisplay, Win, ReadyCursor);
     FreeFax();
-    /*    KMessageBox::sorry(i18n("Bad Fax File k=3"));*/
+    /*    KMessageBox::sorry(i18n("Bad fax file k=3"));*/
     return;
   }
 
@@ -1468,7 +1467,7 @@ void TopLevel::newPage(){
 
     XDefineCursor(qtdisplay, Win, ReadyCursor);
     FreeFax();
-    /*    KMessageBox::sorry(i18n("Bad Fax File k=0"));*/
+    /*    KMessageBox::sorry(i18n("Bad fax file k=0"));*/
     return;
  }
 
@@ -1546,7 +1545,7 @@ void TopLevel::FreeFax(){
       if(pn->prev){
 	free(pn->prev);
       }
-    }	
+    }
   }
 
   if(lastpage)
@@ -1590,7 +1589,7 @@ void TopLevel::uiUpdate(){
 
     if(thispage->type == FAX_TIFF){
       typestring = i18n("Type: Tiff ");
-    }	
+    }
     else if ( thispage->type == FAX_RAW){
       typestring = i18n("Type: Raw ");
     }
@@ -1775,7 +1774,7 @@ static KCmdLineOptions options[] =
    {"2" ,       I18N_NOOP( "Raw files are g3-2d" ), 0 },
    {"4",        I18N_NOOP( "Raw files are g4" ), 0 },
    {"+file(s)",     I18N_NOOP( "Fax file(s) to show" ), 0 },
-   { 0, 0, 0 },
+   KCmdLineLastOption
 };
 
 int main (int argc, char **argv)
