@@ -1,6 +1,6 @@
     /*
 
-    $Id: kfax.h,v 1.34.2.1 2003/05/15 11:28:19 waba Exp $
+    $Id: kfax.h,v 1.40 2003/11/11 01:55:33 bmeyer Exp $
 
     Requires the Qt widget libraries, available at no cost at
     http://www.troll.no
@@ -21,7 +21,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
     */
 
@@ -131,8 +131,6 @@ private:
     QFrame *faxqtwin;
 
     int indentID;
-    KMenuBar *menubar;
-    KToolBar *toolbar;
     QColor forecolor;
     QColor backcolor;
 
@@ -140,7 +138,6 @@ private:
     KStatusBar *statusbar;
 
     KRecentFilesAction *actRecent;
-    KToggleAction *actStatus;
     KAction *actAdd, *actSave, *actSaveAs, *actPrint;
     KAction *actSize, *actZoomIn, *actZoomOut, *actRotate, *actMirror;
     KAction *actFlip, *actNext, *actPrev, *actFirst, *actLast;
@@ -173,7 +170,6 @@ public slots:
     void resizeView();
     void faxSave();
     void faxSaveAs();
-    void toggleStatusBar();
 
     void faxoptions();
     void rotatePage();
@@ -191,6 +187,7 @@ public slots:
     void FreeFax();
     void scrollHorz(int);
     void scrollVert(int);
+    void slotConfigureKeys();
 };
 
 void kfaxerror(const QString&, const QString&);
