@@ -1,10 +1,10 @@
 /*
-    $Id: print.h,v 1.10 2002/04/23 11:59:52 rnolden Exp $
+    $Id: print.h,v 1.14 2003/11/10 00:12:24 bmeyer Exp $
 
-    Requires the Qt widget libraries, available at no cost at 
+    Requires the Qt widget libraries, available at no cost at
     http://www.troll.no
-       
-    Copyright (C) 1996 Bernd Johannes Wuebben   
+
+    Copyright (C) 1996 Bernd Johannes Wuebben
                        wuebben@math.cornell.edu
 
     This program is free software; you can redistribute it and/or modify
@@ -19,8 +19,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-  
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
 */
 
 
@@ -29,23 +29,26 @@
 
 #include <qapplication.h>
 #include <qfiledialog.h>
-#include <qframe.h> 
-#include <qbuttongroup.h>
-#include <qcheckbox.h>
-#include <qcombobox.h>
-#include <qgroupbox.h>
-#include <qlabel.h>
-#include <qlineedit.h>
+#include <qframe.h>
 #include <qlistbox.h>
-#include <qpushbutton.h>
-#include <qprinter.h>
-#include <qradiobutton.h>
 #include <qtooltip.h>
 
 #include <qstring.h>
 #include <qfont.h>
 #include <kapplication.h>
 
+
+class QLineEdit;
+class QLabel;
+
+class QRadioButton;
+class QCheckBox;
+class QComboBox;
+class QButtonGroup;
+class QGroupBox;
+class KPushButton;
+
+
 #define US_LETTER i18n("US Letter 8.5x11 in")
 #define US_LEGAL i18n("US Legal 8.5x14 in")
 #define US_LEDGER i18n("US Ledger 11x17 in")
@@ -101,10 +104,10 @@ private:
     QLineEdit *cmdedit;
     QLabel *cmdlabel;
     QLabel *pagesizelabel;
-    QLabel *xmarginlabel;    
+    QLabel *xmarginlabel;
     QLabel *ymarginlabel;
 
-    QLineEdit *xmarginedit;    
+    QLineEdit *xmarginedit;
     QLineEdit *ymarginedit;
 
     QRadioButton *filebutton;
@@ -117,12 +120,12 @@ private:
 
     QGroupBox	 *box1;
 
-    
-    QPushButton	 *ok_button;
-    QPushButton	 *cancel_button;
-    QPushButton	 *help_button;
+
+    KPushButton	 *ok_button;
+    KPushButton	 *cancel_button;
+    KPushButton	 *help_button;
     QPushButton	 *selectfile;
-    
+
   struct printinfo pi;
 
 };
