@@ -300,6 +300,12 @@ drawline(pixnum *run, int LineNum, struct pagenode *pn)
     int n;
 
     LineNum += pn->stripnum * pn->rowsperstrip;
+    if (LineNum >= pn->height) {
+       if (verbose && LineNum == pn->height)
+           fputs("Height exceeded\n", stderr);
+       return;
+    }
+    
     p = (t32bits *) (Pimage(pn)->data + LineNum*(2-pn->vres)*Pimage(pn)->bytes_per_line);
     p1 =(t32bits *)( pn->vres ? 0 : p + Pimage(pn)->bytes_per_line/sizeof(*p));
     r = run;
@@ -310,6 +316,9 @@ drawline(pixnum *run, int LineNum, struct pagenode *pn)
     while (tot < pn->width) {
 	n = *r++;
 	tot += n;
+        /* Watch out for buffer overruns, e.g. when n == 65535.  */
+        if (tot > pn->width)
+            break;
 	if (pix)
 	    acc |= (~(t32bits)0 >> nacc);
 	else if (nacc)
@@ -726,6 +735,7 @@ XImage *NewImage(int w, int h, char *data, int bit_order){
     if (data == 0)
 	data = xmalloc(((w + 31) & ~31) * h / 8);
     fake.byte_order = ByteOrder;
+    fake.bitmap_unit = 32;
     fake.bitmap_bit_order = bit_order;
 
     int returncode = -1;
