@@ -1,6 +1,6 @@
 /**
  * Copyright (C) 2000-2002 the KGhostView authors. See file AUTHORS.
- * 	
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -13,13 +13,15 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #include <qfile.h>
 #include <qfileinfo.h>
 #include <qframe.h>
+#include <qtable.h>
 #include <qlayout.h>
+#include <qregexp.h>
 
 #include <kaction.h>
 #include <kapplication.h>
@@ -30,65 +32,97 @@
 #include <kiconloader.h>
 #include <kinstance.h>
 #include <klocale.h>
+#include <kpopupmenu.h>
 #include <kstdaction.h>
 #include <ktempfile.h>
 #include <kio/scheduler.h>
+#include <kkeydialog.h>
+#include <kaboutdata.h>
+
 
 #include "kgv_view.h"
 #include "kgv_miniwidget.h"
+#include "kgvconfigdialog.h"
+#include "kgvdocument.h"
 #include "kgvpagedecorator.h"
 #include "kgvpageview.h"
 #include "kgvmainwidget.h"
 #include "kpswidget.h"
+#include "kgvfactory.h"
+#include "logwindow.h"
 #include "marklist.h"
 #include "scrollbox.h"
 #include "version.h"
 
-#define PAGELIST_WIDTH 75
+#include <cmath>
 
-K_EXPORT_COMPONENT_FACTORY( libkghostviewpart, KGVFactory )
+namespace KGV {
+    /*
+     * This is because Qt's iterators
+     * are not standard iterators bc of missing typedefs,
+     * so they are only *almost* STL compatible
+     */
+    template <typename T>
+    unsigned distance( T a, T b ) {
+	unsigned res = 0;
+	while ( a != b ) {
+	    ++res;
+	    ++a;
+	}
+	return res;
+    }
+}
 
-KGVPart::KGVPart( QWidget* parentWidget, const char*, 
+KGVPart::KGVPart( QWidget* parentWidget, const char*,
                   QObject* parent, const char* name,
-                  const QStringList &args ) :  
-    KParts::ReadOnlyPart( parent, name ), 
+                  const QStringList &args ) :
+    KParts::ReadOnlyPart( parent, name ),
     _job( 0 ),
     _mimetypeScanner( 0 ),
+    _dirtyHandler( new QTimer( this ) ),
     _isGuiInitialized( false ),
-    _isDocumentOpen( false ),
     _isFileDirty( false ),
-    _currentPage( 0 )
+    _stickyOptions( false )
 {
     setInstance( KGVFactory::instance() );
 
-    bool bBrowserView = args.contains( "Browser/View" );
-
     // Don't show the progress info dialog if we're embedded in Konqueror.
-    setProgressInfoEnabled( !bBrowserView );
+    setProgressInfoEnabled( !args.contains( "Browser/View") );
+
+    _document = new KGVDocument( this );
+    connect( _document, SIGNAL( fileChangeFailed() ),
+	     this, SLOT( slotCancelWatch() ) );
+    connect( _document, SIGNAL( completed() ),
+	     this, SLOT( slotOpenFileCompleted() ) );
+    connect( _document, SIGNAL( canceled( const QString& ) ),
+	     this, SIGNAL( canceled( const QString& ) ) );
 
     _fileWatcher = new KDirWatch( this );
     connect( _fileWatcher, SIGNAL( dirty( const QString& ) ),
 	     this, SLOT( slotFileDirty( const QString& ) ) );
+    connect( _dirtyHandler, SIGNAL( timeout() ),
+	     this, SLOT( slotDoFileDirty() ) );
 
     // Setup main widget
     _mainWidget = new KGVMainWidget( parentWidget );
     _mainWidget->setFocusPolicy( QWidget::StrongFocus );
     _mainWidget->installEventFilter( this );
+    _mainWidget->setAcceptDrops( true );
     connect( _mainWidget, SIGNAL( spacePressed() ),
              this, SLOT( slotReadDown() ) );
+    connect( _mainWidget, SIGNAL( urlDropped( const KURL& ) ),
+	    this, SLOT( openURL( const KURL& ) ) );
 
     QHBoxLayout* hlay = new QHBoxLayout( _mainWidget, 0, 0 );
     QVBoxLayout* vlay = new QVBoxLayout( hlay );
 
+    const int PAGELIST_WIDTH = 75;
+
     _scrollBox = new ScrollBox( _mainWidget , "scrollbox" );
-    _scrollBox->setMinimumWidth( PAGELIST_WIDTH );
+    _scrollBox->setFixedWidth( PAGELIST_WIDTH );
     _scrollBox->setMinimumHeight( PAGELIST_WIDTH );
     vlay->addWidget( _scrollBox );
 
-    _markList = new MarkList( _mainWidget, "marklist" );
-    _markList->setMinimumWidth( PAGELIST_WIDTH );
-    vlay->addWidget( _markList, 1 );
-
     _divider = new QFrame( _mainWidget, "divider" );
     _divider->setFrameStyle( QFrame::Panel | QFrame::Raised );
     _divider->setLineWidth( 1 );
@@ -104,49 +138,93 @@ KGVPart::KGVPart( QWidget* parentWidget, const char*,
     _pageDecorator = new KGVPageDecorator( _pageView->viewport() );
     _pageDecorator->hide();
 
+    _configDialog = new KGVConfigDialog( _mainWidget, "configdialog" );
+
     _psWidget = new KPSWidget( _pageDecorator );
+    _psWidget->readSettings( _configDialog );
     _pageView->setPage( _pageDecorator );
+    connect( _psWidget, SIGNAL( output( char*, int ) ),
+             this, SLOT( slotGhostscriptOutput( char*, int ) ) );
+
+    connect( _psWidget, SIGNAL( ghostscriptError( const QString& ) ),
+             this, SLOT( slotGhostscriptError( const QString& ) ) );
+
+
+    _logWindow = new LogWindow( i18n( "Ghostscript Messages" ), _mainWidget, "logwindow" );
+    _showLogWindow = false;
+
+    connect( _logWindow, SIGNAL( configureGS() ),
+	    _configDialog, SLOT( show() ) );
 
     _docManager = new KGVMiniWidget( this );
     _docManager->setPSWidget( _psWidget );
+    _docManager->setDocument( document() );
+
+    _markList = new MarkList( _mainWidget, "marklist", _docManager );
+    _markList->setFixedWidth( PAGELIST_WIDTH );
+    vlay->addWidget( _markList, 1 );
+    connect( _markList, SIGNAL( contextMenuRequested ( int, int, const QPoint& ) ),
+             this, SLOT( showPopup( int, int, const QPoint& ) ) );
+
 
-    // Connect up the necessary signals and slots.
     connect( _markList, SIGNAL( selected( int ) ),
 	     _docManager, SLOT( goToPage( int ) ) );
     connect( _docManager, SIGNAL( newPageShown( int ) ),
 	     _markList, SLOT( select( int ) ) );
-
+    connect( _docManager, SIGNAL( setStatusBarText( const QString& ) ),
+	     this, SIGNAL( setStatusBarText( const QString& ) ) );
     connect( _scrollBox, SIGNAL( valueChangedRelative( int, int ) ),
 	     _pageView, SLOT( scrollBy( int, int ) ) );
-    connect( _pageView, SIGNAL( pageSizeChanged( QSize ) ),
-	     _scrollBox, SLOT( setPageSize( QSize ) ) );
-    connect( _pageView, SIGNAL( viewSizeChanged( QSize ) ),
-	     _scrollBox, SLOT( setViewSize( QSize ) ) );
+    connect( _pageView, SIGNAL( pageSizeChanged( const QSize& ) ),
+	     _scrollBox, SLOT( setPageSize( const QSize& ) ) );
+    connect( _pageView, SIGNAL( viewSizeChanged( const QSize& ) ),
+	     _scrollBox, SLOT( setViewSize( const QSize& ) ) );
     connect( _pageView, SIGNAL( contentsMoving( int, int ) ),
 	     _scrollBox, SLOT( setViewPos( int, int ) ) );
 
-    connect( _docManager, SIGNAL( fileChangeFailed() ),
-	     this, SLOT( slotCancelWatch() ) );
-    connect( _docManager, SIGNAL( completed() ),
-	     this, SLOT( slotOpenFileCompleted() ) );
-    connect( _docManager, SIGNAL( canceled( const QString& ) ),
-	     this, SIGNAL( canceled( const QString& ) ) );
-    connect( _docManager, SIGNAL( setStatusBarText( const QString& ) ),
-	     this, SIGNAL( setStatusBarText( const QString& ) ) );
-
     //-- File Menu ----------------------------------------------------------
-    KStdAction::saveAs( miniWidget(), SLOT( saveAs() ), 
+    KStdAction::saveAs( document(), SLOT( saveAs() ),
                         actionCollection() );
     new KAction( i18n( "Document &Info" ), 0,
                  miniWidget(), SLOT( info() ),
                  actionCollection(), "info" );
 
+    //-- Edit Menu -----------------------------------------------------
+    _popup = new KPopupMenu( _markList, "marklist_menu" );
+
+    KAction *act = new KAction( i18n( "Mark Current Page" ), "flag", CTRL+SHIFT+Key_M,
+                 _markList, SLOT( markCurrent() ),
+                 actionCollection(), "mark_current" );
+    act->plug( _popup );
+    act = new KAction( i18n( "Mark &All Pages" ), 0,
+                 _markList, SLOT( markAll() ),
+                 actionCollection(), "mark_all" );
+    act->plug( _popup );
+    act = new KAction( i18n( "Mark &Even Pages" ), 0,
+                 _markList, SLOT( markEven() ),
+                 actionCollection(), "mark_even" );
+    act->plug( _popup );
+    act = new KAction( i18n( "Mark &Odd Pages" ), 0,
+                 _markList, SLOT( markOdd() ),
+                 actionCollection(), "mark_odd" );
+    act->plug( _popup );
+    act = new KAction( i18n( "&Toggle Page Marks" ), 0,
+                 _markList, SLOT( toggleMarks() ),
+                 actionCollection(), "toggle" );
+    act->plug( _popup );
+    act = new KAction( i18n("&Remove Page Marks"), 0,
+                 _markList, SLOT( removeMarks() ),
+                 actionCollection(), "remove" );
+    act->plug( _popup );
+
+    // TODO -- disable entry if there aren't any page names
+
     //-- View Menu ----------------------------------------------------------
     _selectOrientation = new KSelectAction( i18n( "&Orientation" ), 0, 0, 0,
                                     actionCollection(), "orientation_menu" );
     _selectMedia       = new KSelectAction( i18n( "Paper &Size" ), 0, 0, 0,
                                     actionCollection(), "media_menu" );
- 
+
     QStringList orientations;
     orientations.append( i18n( "Auto" ) );
     orientations.append( i18n( "Portrait" ) );
@@ -154,7 +232,7 @@ KGVPart::KGVPart( QWidget* parentWidget, const char*,
     orientations.append( i18n( "Upside Down" ) );
     orientations.append( i18n( "Seascape" ) );
     _selectOrientation->setItems( orientations );
-    
+
     connect( _selectOrientation, SIGNAL( activated( int ) ),
 	     this, SLOT( slotOrientation( int ) ) );
     connect( _selectMedia, SIGNAL( activated( int ) ),
@@ -163,7 +241,25 @@ KGVPart::KGVPart( QWidget* parentWidget, const char*,
     _zoomIn  = KStdAction::zoomIn( this,  SLOT( slotZoomIn() ),
                                    actionCollection(), "zoomIn" );
     _zoomOut = KStdAction::zoomOut( this,  SLOT( slotZoomOut() ),
-			            actionCollection(), "zoomOut" );
+                                    actionCollection(), "zoomOut" );
+    _zoomTo = new KSelectAction(  i18n( "Zoom" ), "zoomTo", 0, actionCollection(), "zoomTo" );
+    connect(  _zoomTo, SIGNAL(  activated(  const QString & ) ), this, SLOT(  slotZoom( const QString& ) ) );
+    _zoomTo->setEditable(  true );
+    _zoomTo->clear();
+    QValueList<double> mags = DisplayOptions::normalMagnificationValues();
+    QStringList translated;
+    int idx = 0;
+    int cur = 0;
+    for ( QValueList<double>::iterator first = mags.begin(), last = mags.end();
+	    first != last;
+	    ++first ) {
+	translated << QString( i18n( "zoom-factor (percentage)", "%1%" ) ).arg( *first * 100.0 );
+	if ( *first == 1.0 ) idx = cur;
+	++cur;
+    }
+    _zoomTo->setItems( translated );
+    _zoomTo->setCurrentItem( idx );
+
 
     /*
     _fitWidth = new KAction( i18n( "Fit Width" ),
@@ -173,13 +269,21 @@ KGVPart::KGVPart( QWidget* parentWidget, const char*,
 
     _prevPage  = KStdAction::prior( this, SLOT( slotPrevPage() ),
                                     actionCollection(), "prevPage" );
+    _prevPage->setWhatsThis( i18n( "Moves to the previous page of the document" ) );
+
     _nextPage  = KStdAction::next( this, SLOT( slotNextPage() ),
                                    actionCollection(), "nextPage" );
+    _nextPage->setWhatsThis( i18n( "Moves to the next page of the document" ) );
+
     _firstPage = KStdAction::firstPage( this, SLOT( slotGotoStart() ),
                                    actionCollection(), "goToStart" );
+    _firstPage->setWhatsThis( i18n( "Moves to the first page of the document" ) );
+
     _lastPage  = KStdAction::lastPage( this, SLOT( slotGotoEnd() ),
                                    actionCollection(), "goToEnd" );
-    _readUp    = new KAction( i18n( "Read up Document" ), "previous",
+    _lastPage->setWhatsThis( i18n( "Moves to the last page of the document" ) );
+
+    _readUp    = new KAction( i18n( "Read Up Document" ), "previous",
                               SHIFT+Key_Space, this, SLOT( slotReadUp() ),
                               actionCollection(), "readUp" );
     // don't specify Key_Space as Accel, it breaks the lineedit in konq! (Simon)
@@ -190,46 +294,27 @@ KGVPart::KGVPart( QWidget* parentWidget, const char*,
     _gotoPage = KStdAction::gotoPage( _docManager, SLOT( goToPage() ),
                                       actionCollection(), "goToPage" );
 
-    //-- PageMarks Menu -----------------------------------------------------
-    new KAction( i18n( "Mark Current Page" ), "flag", CTRL+SHIFT+Key_M,
-                 _markList, SLOT( markCurrent() ),
-                 actionCollection(), "mark_current" );
-    new KAction( i18n( "Mark &All Pages" ), 0,
-                 _markList, SLOT( markAll() ),
-                 actionCollection(), "mark_all" );
-    new KAction( i18n( "Mark &Even Pages" ), 0,
-                 _markList, SLOT( markEven() ),
-                 actionCollection(), "mark_even" );
-    new KAction( i18n( "Mark &Odd Pages" ), 0,
-                 _markList, SLOT( markOdd() ),
-                 actionCollection(), "mark_odd" );
-    new KAction( i18n( "&Toggle Page Marks" ), 0,
-                 _markList, SLOT( toggleMarks() ),
-                 actionCollection(), "toggle" );
-    new KAction( i18n("&Remove Page Marks"), 0,
-                 _markList, SLOT( removeMarks() ),
-                 actionCollection(), "remove" );
-
-    // TODO -- disable entry if there aren't any page names
-
     //-- Settings Menu ------------------------------------------------------
     _showScrollBars = new KToggleAction( i18n( "Show &Scrollbars" ), 0,
-                                this, SLOT( slotShowScrollBars() ),
                                 actionCollection(), "show_scrollbars" );
     _watchFile	    = new KToggleAction( i18n( "&Watch File" ), 0,
                                 this, SLOT( slotWatchFile() ),
                                 actionCollection(), "watch_file" );
     _showPageList   = new KToggleAction( i18n( "Show &Page List" ), 0,
-                                this, SLOT( slotShowMarkList() ),
                                 actionCollection(), "show_page_list" );
     _showPageLabels = new KToggleAction( i18n("Show Page &Labels"), 0,
-                                this, SLOT( slotShowPageLabels() ),
                                 actionCollection(), "show_page_labels" );
     new KAction( i18n( "&Configure KGhostView..." ), "configure", 0,
-                 miniWidget(), SLOT( configureGhostscript() ),
+                 this, SLOT( slotConfigure() ),
                  actionCollection(), "configure" );
+    KStdAction::keyBindings( this, SLOT( slotConfigureKeys() ), actionCollection() );
+    connect( _showScrollBars, SIGNAL( toggled( bool ) ),
+             SLOT( showScrollBars( bool ) ) );
+    connect( _showPageList, SIGNAL( toggled( bool ) ),
+             SLOT( showMarkList( bool ) ) );
+    connect( _showPageLabels, SIGNAL( toggled( bool ) ),
+             SLOT( showPageLabels( bool ) ) );
 
-    
     _extension = new KGVBrowserExtension( this );
 
     setXMLFile( "kgv_part.rc" );
@@ -238,7 +323,7 @@ KGVPart::KGVPart( QWidget* parentWidget, const char*,
 	     this, SLOT( slotNewPage( int ) ) );
     connect( _pageView, SIGNAL( contentsMoving( int, int ) ),
 	     this, SLOT( slotPageMoved( int, int ) ) );
-    
+
     connect( _pageView, SIGNAL( nextPage() ), SLOT( slotNextPage() ));
     connect( _pageView, SIGNAL( prevPage() ), SLOT( slotPrevPage() ));
     connect( _pageView, SIGNAL( zoomIn() ), SLOT( slotZoomIn() ));
@@ -248,33 +333,42 @@ KGVPart::KGVPart( QWidget* parentWidget, const char*,
 
     readSettings();
 
-    QStringList items = miniWidget()->mediaNames();
+    QStringList items = document()->mediaNames();
     items.prepend( i18n( "Auto ") );
     _selectMedia->setItems( items );
 }
 
 KGVPart::~KGVPart()
 {
-    if ( _job ) _job->kill();
+    if ( _job ) _job -> kill();
     delete _mimetypeScanner;
     writeSettings();
 }
 
+void KGVPart::slotConfigureKeys()
+{
+    KKeyDialog::configure( actionCollection(), this );
+}
+
 KAboutData* KGVPart::createAboutData()
 {
-    KAboutData* about = new KAboutData( "kghostview", I18N_NOOP( "KGhostView"), 
-            KGHOSTVIEW_VERSION, 
-            I18N_NOOP( "Viewer for PostScript (.ps, .eps) and Portable Document Format (.pdf) files."), 
+    KAboutData* about = new KAboutData( "kghostview", I18N_NOOP( "KGhostView"),
+            KGHOSTVIEW_VERSION,
+            I18N_NOOP( "Viewer for PostScript (.ps, .eps) and Portable Document Format (.pdf) files."),
             KAboutData::License_GPL,
             "(C) 1998 Mark Donohoe, (C) 1999-2000 David Sweet, "
-            "(C) 2000-2001 Wilco Greven",
+            "(C) 2000-2003 Wilco Greven",
             I18N_NOOP( "KGhostView displays, prints, and saves "
                        "PostScript and PDF files.\n"
                        "Based on original work by Tim Theisen." ) );
-    about->addAuthor( "Wilco Greven",
+    about->addAuthor( "Luís Pedro Coelho",
                       I18N_NOOP( "Current maintainer" ),
+                      "luis@luispedro.org",
+                      "http://luispedro.org" );
+    about->addAuthor( "Wilco Greven",
+                      I18N_NOOP( "Maintainer 2000-2003" ),
                       "greven@kde.org" );
-    about->addAuthor( "David Sweet", 
+    about->addAuthor( "David Sweet",
                       I18N_NOOP( "Maintainer 1999-2000" ),
                       "dsweet@kde.org",
                       "http://www.andamooka.org/~dsweet" );
@@ -292,7 +386,7 @@ KAboutData* KGVPart::createAboutData()
                       "espen@kde.org" );
     about->addCredit( "Russell Lang of Ghostgum Software Pty Ltd",
 		      I18N_NOOP( "for contributing GSView's DSC parser." ),
-                      0, 
+                      0,
                       "http://www.ghostgum.com.au/" );
     about->addCredit( "The Ghostscript authors",
                       0, 0,
@@ -302,12 +396,17 @@ KAboutData* KGVPart::createAboutData()
 
 bool KGVPart::closeURL()
 {
-    _psWidget->disableInterpreter(); 
-    _isDocumentOpen = false;
+    document()->close();
+    _psWidget->stopInterpreter();
+    _docManager->getThumbnailService()->reset();
+    _markList->clear();
+    _pageDecorator->hide();
+    _scrollBox->clear();
     _isFileDirty = false;
-    if ( _job ) {
-	    _job->kill();
-	    _job = 0;
+    if ( _job )
+    {
+	_job -> kill();
+	_job = 0;
     }
     if( _mimetypeScanner != 0 )
 	_mimetypeScanner->abort();
@@ -332,13 +431,15 @@ void KGVPart::readSettings()
 {
     KConfigGroup general( KGVFactory::instance()->config(), "General" );
     _showScrollBars->setChecked( general.readBoolEntry( "ShowScrollBars", true ) );
-    slotShowScrollBars();
+    showScrollBars( _showScrollBars->isChecked() );
     _watchFile->setChecked( general.readBoolEntry( "WatchFile", false ) );
     slotWatchFile();
     _showPageList->setChecked( general.readBoolEntry( "ShowPageList", true ) );
-    slotShowMarkList();
+    showMarkList( _showPageList->isChecked() );
     _showPageLabels->setChecked( general.readBoolEntry( "ShowPageNames", false ) );
-    slotShowPageLabels();
+    showPageLabels( _showPageLabels->isChecked() );
+
+    _showLogWindow = _configDialog->showMessages();
 }
 
 void KGVPart::slotScrollLeft()
@@ -363,7 +464,7 @@ void KGVPart::slotScrollDown()
 
 void KGVPart::slotReadUp()
 {
-    if( !_isDocumentOpen )
+    if( !( document() && document()->isOpen() ) )
 	return;
 
     if( !_pageView->readUp() ) {
@@ -374,23 +475,40 @@ void KGVPart::slotReadUp()
 
 void KGVPart::slotReadDown()
 {
-    if( !_isDocumentOpen )
+    if( !( document() && document()->isOpen() ) )
 	return;
-    
-    if( !_pageView->readDown() )
-	slotNextPage();
+
+    if( !_pageView->readDown() ) {
+	if( _docManager->nextPage() )
+	   _pageView->scrollTop();
+    }
 }
 
 void KGVPart::slotPrevPage()
 {
+    if( !( document() && document()->isOpen() ) )
+	return;
+
     _docManager->prevPage();
-    _pageView->scrollTop();
+    // I am commenting the line below in response to 58514
+    // I am afraid that someone will dislike the new behaviour, though :-)
+    //
+    // BTW, this seems to mirror kdvi, although I admit I don't use that app
+    // much, so I could be wrong.
+    //
+    // If no one does, then the comment should be reverted.
+    // LPC (27 Jun 2003)
+    //_pageView->scrollTop();
 }
 
 void KGVPart::slotNextPage()
 {
+
+    if( !( document() && document()->isOpen() ) )
+	return;
     if( _docManager->nextPage() ) {
-	_pageView->scrollTop();
+	// See above
+	//_pageView->scrollTop();
     }
     else
 	_pageView->scrollBottom();
@@ -412,8 +530,10 @@ void KGVPart::slotWatchFile()
 {
     if( _watchFile->isChecked() )
 	_fileWatcher->startScan();
-    else
+    else {
+	_dirtyHandler->stop();
 	_fileWatcher->stopScan();
+    }
 }
 
 void KGVPart::slotCancelWatch()
@@ -425,38 +545,64 @@ void KGVPart::slotCancelWatch()
 /*
 void KGVPart::slotFitWidth()
 {
-    _docManager->fitWidth( pageView()->viewport()->width() - 
+    _docManager->fitWidth( pageView()->viewport()->width() -
          2*( pageDecorator()->margin() + pageDecorator()->borderWidth() ) );
 }
 */
 
 void KGVPart::updateZoomActions()
 {
-    if( !_isDocumentOpen ) 
+    if( !( document() && document()->isOpen() ) )
 	return;
 
     _zoomIn->setEnabled(!_docManager->atMaxZoom());
     _zoomOut->setEnabled(!_docManager->atMinZoom());
+    _zoomTo->setEnabled( true );
+    QStringList items = _zoomTo->items();
+    double zoom = miniWidget()->displayOptions().magnification();
+    unsigned idx = 0;
+    for ( QStringList::iterator first = items.begin(), last = items.end();
+	    first != last;
+	    ++first ) {
+	QString cur = *first;
+	cur.remove(  cur.find(  '%' ), 1 );
+	cur = cur.simplifyWhiteSpace();
+	bool ok = false;
+	double z = cur.toDouble(&ok);
+	if ( ok && std::abs( z - zoom * 100.0 ) < 0.1 ) {
+	    _zoomTo->setCurrentItem( idx );
+	    return;
+	}
+	++idx;
+    }
+    /* TODO:
+     *
+     * One should reorder the _zoomTo->items() list here.
+     * This is because any values the user types in himself will get
+     * put at the end of the list, which is not optimal.
+     *
+     * LPC 19 Jun 2003
+     */
 }
 
 void KGVPart::updatePageDepActions()
 {
-    if( !_isDocumentOpen ) 
+    if( !( document() && document()->isOpen() ) )
 	return;
-    
+
     _prevPage->setEnabled( !_docManager->atFirstPage() );
     _firstPage->setEnabled( !_docManager->atFirstPage() );
     _nextPage->setEnabled( !_docManager->atLastPage() );
     _lastPage->setEnabled( !_docManager->atLastPage() );
-    
+
     updateReadUpDownActions();
 }
 
 void KGVPart::updateReadUpDownActions()
 {
-    if( !_isDocumentOpen ) 
+    if( !( document() && document()->isOpen() ) )
 	return;
-    
+
     if( _docManager->atFirstPage() && _pageView->atTop() )
 	_readUp->setEnabled( false );
     else
@@ -470,35 +616,35 @@ void KGVPart::updateReadUpDownActions()
 
 bool KGVPart::openURL( const KURL& url )
 {
-    if( url.isMalformed() )
+    if( !url.isValid() )
 	return false;
     if( !closeURL() )
 	return false;
 
     m_url = url;
-    _currentPage = 0;
+    if ( !_stickyOptions ) _options.reset();
 
     emit setWindowCaption( m_url.prettyURL() );
 
-    _mimetypeScanner = new KGVRun( m_url, m_url.isLocalFile(), false );
+    _mimetypeScanner = new KGVRun( m_url, 0, m_url.isLocalFile(), false );
     connect( _mimetypeScanner, SIGNAL( finished( const QString& ) ),
              SLOT( slotMimetypeFinished( const QString& ) ) );
     connect( _mimetypeScanner, SIGNAL( error() ),
              SLOT( slotMimetypeError() ) );
 
-    resetMenu();
     return true;
 }
 
 void KGVPart::openURLContinue()
 {
-    if( m_url.isLocalFile() ) 
+    kdDebug(4500) << "KGVPart::openURLContinue()" << endl;
+    if( m_url.isLocalFile() )
     {
 	emit started( 0 );
 	m_file = m_url.path();
-	_docManager->openFile( m_file, _mimetype );
+	document()->openFile( m_file, _mimetype );
     }
-    else 
+    else
     {
 	m_bTemp = true;
 	// Use same extension as remote file. This is important for
@@ -537,26 +683,51 @@ bool KGVPart::openFile()
 
 void KGVPart::slotOpenFileCompleted()
 {
+    _docManager->getThumbnailService()->setEnabled( true );
     if( _isFileDirty )
     {
-	_docManager->goToPage( _currentPage );
-	slotOrientation( _selectOrientation->currentItem() );
-	slotMedia( _selectMedia->currentItem() );
 	_docManager->redisplay();
 	_isFileDirty = false;
     }
-    else	
+    else
     {
-	_isDocumentOpen = true;
-	_docManager->goToPage( _currentPage );
+	if ( !_stickyOptions ) setDisplayOptions( DisplayOptions() );
+	_stickyOptions = false;
+
 	stateChanged( "documentState" );
-	if ( !_fileWatcher->contains( m_file ) ) 
+	if ( !_fileWatcher->contains( m_file ) )
 	    _fileWatcher->addFile( m_file );
 	slotWatchFile();
+	slotZoom( i18n( "zoom-factor (percentage)", "%1%" ). arg( 100 ) );
+	updateZoomActions();
 	emit completed();
     }
 }
 
+void KGVPart::slotGhostscriptOutput( char* data, int len )
+{
+    _logWindow->append( QString::fromLocal8Bit( data, len ) );
+    if( _showLogWindow )
+	_logWindow->show();
+}
+
+
+void KGVPart::slotGhostscriptError( const QString& error )
+{
+    // FIXME: find a significant error message
+    _logWindow->setLabel( i18n( "An error occurred in rendering.\n%1\n%2\n" )
+		    .arg( error )
+		    .arg( i18n( "You might see some errors in the display or it may work.\n"
+			    "The error messages below are from Ghostscript (\"gs\") and may "
+			    "help identify or correct the problem.\n" ) ),
+	   true );
+    // The true above makes it show a "configure gs" option, but maybe we
+    // should trigger an auto-redetection?
+    // LPC (13 Apr 2003)
+    _logWindow->show();
+}
+
+
 void KGVPart::guiActivateEvent( KParts::GUIActivateEvent* event )
 {
     if( event->activated() && !_isGuiInitialized )
@@ -578,9 +749,9 @@ void KGVPart::slotData( KIO::Job* job, const QByteArray& data )
 
 void KGVPart::slotMimetypeFinished( const QString& type )
 {
-    kdDebug(4500) << "KGVPart::slotMimetype: type=" << type << endl;
+    kdDebug(4500) << "KGVPart::slotMimetypeFinished( " << type << " )" << endl;
     _mimetype = type;
-    if( _mimetypeScanner->hasError() )
+    if( !_mimetypeScanner || _mimetypeScanner->hasError() )
 	emit canceled( QString::null );
     else
 	openURLContinue();
@@ -589,12 +760,11 @@ void KGVPart::slotMimetypeFinished( const QString& type )
 
 void KGVPart::slotMimetypeError()
 {
-    /*
+    kdDebug(4500) << "KGVPart::slotMimetypeError()" << endl;
     _mimetypeScanner = 0;
     emit started( 0 );
-    kapp->processEvents();
+    //kapp->processEvents();
     emit canceled( QString::null );
-    */
 }
 
 void KGVPart::slotJobFinished( KIO::Job* job )
@@ -609,28 +779,33 @@ void KGVPart::slotJobFinished( KIO::Job* job )
 
     if( job->error() )
 	emit canceled( job->errorString() );
-    else 
-    {
-	_docManager->openFile( m_file, _mimetype );
-	resetMenu();
-    }
+    else
+	document()->openFile( m_file, _mimetype );
 }
 
 void KGVPart::slotFileDirty( const QString& fileName )
 {
-    if( fileName == m_file )
+    // The beauty of this is that each start cancels the previous one.
+    // This means that timeout() is only fired when there have
+    // no changes to the file for the last 750 milisecs.
+    // This is supposed to ensure that we don't update on every other byte
+    // that gets written to the file.
+    if ( fileName == m_file )
     {
+	_dirtyHandler->start( 750, true );
+    }
+}
+
+void KGVPart::slotDoFileDirty()
+{
 	kdDebug(4500) << "KGVPart::File changed" << endl;
 	_isFileDirty = true;
-	_currentPage = _docManager->currentPage();
-	_docManager->openFile( m_file, _mimetype );
-    }
+	reloadFile();
 }
 
 void KGVPart::slotNewPage( int )
 {
     updatePageDepActions();
-    _currentPage = _docManager->currentPage();
     //media->setCurrentItem (miniWidget()->getSize()-1);
     //orientation->setCurrentItem (miniWidget()->getOrientation()-1);
     //TODO -- zoom
@@ -658,33 +833,24 @@ void KGVPart::slotMedia( int id )
     if( id == 0 )
 	miniWidget()->restoreOverridePageMedia();
     else
-	miniWidget()->setOverridePageMedia( miniWidget()->mediaNames()[id-1] );
+	miniWidget()->setOverridePageMedia( _document->mediaNames()[id-1] );
 }
 
-void KGVPart::slotShowScrollBars()
+void KGVPart::showScrollBars( bool show )
 {
-    _pageView->enableScrollBars( _showScrollBars->isChecked() );
+    _pageView->enableScrollBars( show );
 }
 
-void KGVPart::slotShowMarkList()
+void KGVPart::showMarkList( bool show )
 {
-    if( _showPageList->isChecked() ) 
-    {
-	_markList->show();
-	_scrollBox->show();
-	_divider->show();
-    }
-    else 
-    {
-	_markList->hide();
-	_scrollBox->hide();
-	_divider->hide();
-    }
+    _markList->setShown( show );
+    _scrollBox->setShown( show );
+    _divider->setShown( show );
 }
 
-void KGVPart::slotShowPageLabels()
+void KGVPart::showPageLabels( bool show )
 {
-    _docManager->enablePageLabels( _showPageLabels->isChecked() );
+    _docManager->enablePageLabels( show );
 }
 
 void KGVPart::slotZoomIn()
@@ -699,12 +865,57 @@ void KGVPart::slotZoomOut()
     updateZoomActions();
 }
 
-void KGVPart::resetMenu()
+void KGVPart::slotZoom( const QString& nz )
 {
-    _selectOrientation->setCurrentItem( 0 ); // Auto
-    _selectMedia->setCurrentItem( 0 ); // Auto
+    QString z = nz;
+    double zoom;
+    z.remove(  z.find(  '%' ), 1 );
+    zoom = KGlobal::locale()->readNumber(  z ) / 100;
+    kdDebug( 4500 ) << "ZOOM = "  << nz << ", setting zoom = " << zoom << endl;
+
+    DisplayOptions options = miniWidget()->displayOptions();
+    options.setMagnification( zoom );
+    miniWidget()->setDisplayOptions( options );
+    miniWidget()->redisplay();
+    _mainWidget->setFocus();
 }
 
+void KGVPart::reloadFile()
+{
+    _psWidget->stopInterpreter();
+    _docManager->getThumbnailService()->reset();
+    document()->openFile( m_file, _mimetype );
+}
+
+void KGVPart::slotConfigure()
+{
+    _configDialog->setup();
+    if( _configDialog->exec() )
+    {
+	_psWidget->readSettings( _configDialog );
+	miniWidget()->redisplay();
+    }
+}
+
+void KGVPart::setDisplayOptions( const DisplayOptions& options )
+{
+    kdDebug(4500) << "KGVPart::setDisplayOptions()" << endl;
+    _stickyOptions = true;
+    _markList->select( options.page() );
+    _docManager->setDisplayOptions( options );
+    _selectOrientation->setCurrentItem( options.overrideOrientation()  );
+    QStringList medias = document()->mediaNames();
+    QStringList::Iterator now = medias.find( options.overridePageMedia() );
+    if ( now != medias.end() ){
+	// The options are displayed in inverted order.
+	// Therefore, size() - index gets you the display index
+	_selectMedia->setCurrentItem(  medias.size() - KGV::distance( medias.begin(), now ) );
+    } else {
+	_selectMedia->setCurrentItem( 0 );
+    }
+}
+
+
 KGVBrowserExtension::KGVBrowserExtension( KGVPart *parent ) :
     KParts::BrowserExtension( parent, "KGVBrowserExtension" )
 {
@@ -714,11 +925,11 @@ KGVBrowserExtension::KGVBrowserExtension( KGVPart *parent ) :
 
 void KGVBrowserExtension::print()
 {
-    ((KGVPart *)parent())->miniWidget()->print();
+    ((KGVPart *)parent())->document()->print();
 }
 
 
-KGVRun::KGVRun( const KURL& url, mode_t mode, bool isLocalFile, 
+KGVRun::KGVRun( const KURL& url, mode_t mode, bool isLocalFile,
                 bool showProgressInfo ) :
     KRun( url, mode, isLocalFile, showProgressInfo )
 {
@@ -734,9 +945,13 @@ void KGVRun::foundMimeType( const QString& mimetype )
 
     if( m_job && m_job->inherits( "KIO::TransferJob" ) )
     {
+	/* The code below should be correct except for what seems to be a bug in kio.
+	 * See bug:69456 for details
 	KIO::TransferJob *job = static_cast< KIO::TransferJob* >( m_job );
 	job->putOnHold();
 	KIO::Scheduler::publishSlaveOnHold();
+	*/
+	m_job->kill(true);
 	m_job = 0;
     }
 
@@ -746,5 +961,19 @@ void KGVRun::foundMimeType( const QString& mimetype )
     m_timer.start( 0, true );
 }
 
+
+void KGVPart::updateFullScreen( bool fs )
+{
+    if ( fs ) showMarkList( false );
+    else showMarkList( _showPageList->isChecked() );
+}
+
+void KGVPart::showPopup( int, int, const QPoint& pos )
+{
+    _popup->exec( pos );
+}
+
 #include "kgv_view.moc"
 
+
+// vim:sw=4:sts=4:ts=8:sta:tw=78:noet
