@@ -1,6 +1,6 @@
-/** 
+/**
  * Copyright (C) 2000-2002 the KGhostView authors. See file AUTHORS.
- * 	
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #ifndef __KGV_VIEW_H
@@ -24,68 +24,80 @@
 
 #include <kio/job.h>
 #include <kparts/browserextension.h>
-#include <kparts/genericfactory.h>
 #include <krun.h>
 
+#include "displayoptions.h"
+
 class QFrame;
 class QWidget;
 
+class KAboutData;
 class KAction;
 class KActionCollection;
 class KDirWatch;
 class KInstance;
+class KPopupMenu;
+class KSelectAction;
 class KToggleAction;
 
 class KGVBrowserExtension;
+class KGVConfigDialog;
+class KGVDocument;
 class KGVMiniWidget;
 class KGVPageView;
 class KGVPageDecorator;
 class KGVRun;
 class KPSWidget;
+class LogWindow;
 class MarkList;
 class ScrollBox;
 
-class KSelectAction;
-class KAboutData;
-
 class KGVPart: public KParts::ReadOnlyPart
 {
   Q_OBJECT
 public:
-    KGVPart( QWidget* parentWidget, const char* widgetName, 
-             QObject* parent, const char* name, 
+    KGVPart( QWidget* parentWidget, const char* widgetName,
+             QObject* parent, const char* name,
              const QStringList& args = QStringList() );
-    
+
     virtual ~KGVPart();
 
     KGVMiniWidget*    miniWidget()    const { return _docManager;    }
     MarkList*         markList()      const { return _markList;      }
+    ScrollBox*        scrollBox()           { return _scrollBox;     }
     KGVPageView*      pageView()      const { return _pageView;      }
     KGVPageDecorator* pageDecorator() const { return _pageDecorator; }
+    KGVDocument*      document()      const { return _document;      }
+    const KGVConfigDialog* configDialog()   { return _configDialog;  }
+    
 
     /**
-     * Reimplemented from ReadOnlyPart in order to delete the file from 
+     * Reimplemented from ReadOnlyPart in order to delete the file from
      * KDirWatch's list.
      */
     virtual bool closeURL();
 
-    /**
-     * Sets the current page that will be selected in slotOpenFileCompleted().
-     * Useful for to restore the current page during session restoring.
-     */
-    void setCurrentPage( int page ) { _currentPage = page; }
-
     static KAboutData* createAboutData();
 
 public slots:
     /**
-     * Reimplemented from ReadOnlyPart so that incoming data can be sent 
+     * Reimplemented from ReadOnlyPart so that incoming data can be sent
      * through the DSC parser immediately on arrival.
      */
     virtual bool openURL( const KURL& );
     virtual void openURLContinue();
 
-    void slotScrollLeft();   
+    /**
+     * Reloads the current file.
+     * No action if no file is loaded
+     */
+    void reloadFile();
+
+    void updateFullScreen( bool );
+
+    void showPopup( int, int, const QPoint &pos );
+
+    void slotScrollLeft();
     void slotScrollRight();
     void slotScrollUp();
     void slotScrollDown();
@@ -96,23 +108,37 @@ public slots:
     void slotGotoStart();
     void slotGotoEnd();
     // void slotFitWidth();
-    
-    void slotShowScrollBars();
+
+    void showScrollBars( bool );
     void slotCancelWatch();
-    void slotShowMarkList();
-    void slotShowPageLabels();
+    void showMarkList( bool );
+    void showPageLabels( bool );
 
     void slotZoomIn();
     void slotZoomOut();
 
+    void slotZoom( const QString& );
+
+    void slotConfigure();
+
+    /**
+     * Sets the display options in a sticky way.
+     * This means that the file being opened or the next one to be open will
+     * get these options. This is useful for session management or commandline
+     * arguments
+     */
+    void setDisplayOptions( const DisplayOptions& opts );
+
 protected slots:
     void slotData( KIO::Job*, const QByteArray& );
     void slotJobFinished( KIO::Job* );
-    
+
     void slotMimetypeFinished( const QString& );
     void slotMimetypeError();
-    
+
     void slotFileDirty( const QString& );
+    void slotDoFileDirty();
+
     void slotOrientation (int);
     void slotMedia (int);
     void slotNewPage( int );
@@ -120,13 +146,14 @@ protected slots:
     void slotWatchFile();
 
     void slotOpenFileCompleted();
+    void slotConfigureKeys();
 
 protected:
     virtual void guiActivateEvent( KParts::GUIActivateEvent* );
-   
+
     // reimplemented from ReadOnlyPart
     virtual bool openFile();
-    
+
     void updatePageDepActions();
     void updateZoomActions();
     void updateReadUpDownActions();
@@ -134,14 +161,15 @@ protected:
     void readSettings();
     void writeSettings();
 
-    /**
-     * Resets the menu settings, i.e. sets View->Orientation to "Auto", etc.
-     */
-    void resetMenu();
+private slots:
+    void slotGhostscriptOutput( char* data, int len );
+    void slotGhostscriptError( const QString& );
 
 private:
     KGVBrowserExtension* _extension;
 
+    KGVDocument* _document;
+
     QWidget*	      _mainWidget;
     KGVPageView*      _pageView;
     KGVPageDecorator* _pageDecorator;
@@ -151,37 +179,45 @@ private:
     MarkList*	      _markList;
     KGVMiniWidget*    _docManager;
 
+    KGVConfigDialog* _configDialog;
+    LogWindow*       _logWindow;
+
     KSelectAction* _selectOrientation;
     KSelectAction* _selectMedia;
     KAction*       _zoomIn;
-    KAction*       _zoomOut; 
-    // KAction* _fitWidth; 
+    KAction*       _zoomOut;
+    KSelectAction* _zoomTo;
+    // KAction* _fitWidth;
     KAction*       _prevPage;
     KAction*       _nextPage;
-    KAction*       _firstPage; 
-    KAction*       _lastPage; 
-    KAction*       _readUp; 
-    KAction*       _readDown; 
+    KAction*       _firstPage;
+    KAction*       _lastPage;
+    KAction*       _readUp;
+    KAction*       _readDown;
     KAction*       _gotoPage;
     KToggleAction* _showScrollBars;
     KToggleAction* _watchFile;
     KToggleAction* _showPageList;
-    KToggleAction* _showPageLabels; 
-  
+    KToggleAction* _showPageLabels;
+    KPopupMenu*    _popup;
+
     QFile             _tmpFile;
     KIO::TransferJob* _job;
     KDirWatch*        _fileWatcher;
     KGVRun*           _mimetypeScanner;
+    QTimer*           _dirtyHandler;
 
     QString _mimetype;
 
     bool _isGuiInitialized : 1;
-    bool _isDocumentOpen   : 1;
     bool _isFileDirty      : 1;
+    bool _showLogWindow    : 1;
+    bool _stickyOptions    : 1;
 
-    int _currentPage;
+    DisplayOptions _options;
 };
 
+
 class KGVBrowserExtension : public KParts::BrowserExtension
 {
   Q_OBJECT
@@ -195,14 +231,12 @@ public slots:
     void print();
 };
 
-typedef KParts::GenericFactory<KGVPart> KGVFactory;
-
 class KGVRun : public KRun
 {
     Q_OBJECT
-	
+
 public:
-    KGVRun( const KURL& url, mode_t mode = 0, 
+    KGVRun( const KURL& url, mode_t mode = 0,
             bool isLocalFile = false, bool showProgressInfo = true );
 
     virtual ~KGVRun();
@@ -222,4 +256,4 @@ private:
 
 #endif
 
-
+// vim:sw=4:sts=4:ts=8:sta:tw=78:noet
