@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
 #ifndef _INTERPRETER_DIALOG_H_
@@ -22,24 +22,20 @@
 #include <qstring.h>
 #include <kdialogbase.h>
 
-class QButtonGroup;
-class QCheckBox;
-class QPushButton;
-class QRadioButton;
-class KLineEdit;
-class KURLRequester;
+class GeneralSettingsWidget;
+class GSSettingsWidget;
 
 class KGVConfigDialog : public KDialogBase 
 {
     Q_OBJECT
 
 public:
-    enum {COLOR_PALETTE, GRAY_PALETTE, MONO_PALETTE};
+    enum {COLOR_PALETTE=3, GRAY_PALETTE=2, MONO_PALETTE=1};
 
     KGVConfigDialog( QWidget* parent = 0, const char* name = 0, 
 		     bool modal = true );
     ~KGVConfigDialog();
-    
+
     void setup();
 
     void readSettings();
@@ -49,7 +45,7 @@ public:
     bool    showMessages()	const { return mShowMessages; }
     bool    platformFonts()	const { return mPlatformFonts; }
     int	    paletteType()	const { return mPaletteType; }
-    
+
     QString interpreterPath()	const { return mInterpreterPath; }
     QString nonAntialiasArgs()	const { return mNonAntiAliasArgs; }
     QString antialiasArgs()	const { return mAntiAliasArgs; }
@@ -57,34 +53,26 @@ public:
 protected slots:
     virtual void slotOk();
     virtual void slotDefault();
-    
+
     bool slotConfigureGhostscript();
-    void slotAaToggled( bool );
-    void slotMonoOrGrayToggled( bool );
 
 private:
-    QCheckBox* mAntiAliasCheck;
-    QCheckBox* mPlatformFontCheck;
-    QCheckBox* mMessagesCheck;
-    QRadioButton* mMonoRadio;
-    QRadioButton* mGrayscaleRadio;
-    QRadioButton* mColorRadio;
-    QButtonGroup* mPaletteGroup;
 
-    QPushButton* mConfigureButton;
-    KURLRequester* mInterpreterReq;
-    KLineEdit* mNonAntiAliasArgsEdit;
-    KLineEdit* mAntiAliasArgsEdit;
+    GeneralSettingsWidget *mGeneralWidget;
+    GSSettingsWidget      *mGSWidget;
 
     bool    mAntialias;
     bool    mShowMessages;
     bool    mPlatformFonts;
     int	    mPaletteType;
-    
+
     QString mInterpreterPath;
     QString mNonAntiAliasArgs;
     QString mAntiAliasArgs;
+    QString mDetectedVersion;
 };
 
 #endif 
 
+
+// vim:sw=4:sts=4:ts=8:noet
