@@ -21,16 +21,6 @@
 #ifndef __KICON_H__
 #define __KICON_H__
 
-#include <qdir.h>
-#include <qobject.h>
-#include <qimage.h>
-#include <qfile.h>
-#include <qfileinfo.h>
-#include <kapplication.h>
-#include <kfiledialog.h>
-#include "kcolorgrid.h"
-#include "utils.h"
-
 #include <stdio.h>
 #include <errno.h>
 #include <assert.h>
@@ -51,12 +41,13 @@
 #include <sys/stat.h>
 #include <sys/file.h>
 
+class QImage;
 
 class KIconEditIcon : public QObject
 {
   Q_OBJECT
 public:
-  KIconEditIcon(QObject*, const QImage*, const QString &filename = QString::null );
+  KIconEditIcon(QObject*, const QImage*, KURL url = "" );
   ~KIconEditIcon();
 
   bool isLocal() { return local; }
@@ -64,7 +55,7 @@ public:
   void setUrl( const QString & u ) { _url = u; };
 
 public slots:
-  bool open(const QImage*, const QString &xpm= QString::null);
+  bool open(const QImage*, KURL url = "");
   bool promptForFile(const QImage*);
   bool save(const QImage*, const QString &filename=QString::null);
   bool saveAs(const QImage*);
@@ -72,6 +63,7 @@ public slots:
 signals:
   void newmessage( const QString &);
   void newname(const QString &);
+  void addrecent(const QString &);
   void opennewwin(const QString &);
   void loaded(QImage *);
   void saved();
