@@ -24,18 +24,18 @@
 #include <qlayout.h>
 #include <qlineedit.h>
 #include <qvbox.h>
+#include <qbuttongroup.h>
+#include <qradiobutton.h>
 
 #include <kaccel.h>
 #include <kbuttonbox.h>
 #include <kapplication.h>
-#include <kkeydialog.h>
 #include <klocale.h>
 #include <kcolorbutton.h>
 #include <kfiledialog.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
 #include <kurlrequester.h>
-#include <kwin.h>
 
 #include "kiconconfig.h"
 #include "kiconedit.h"
@@ -52,23 +52,26 @@ static inline QPixmap loadIcon( const char * name )
     ->loadIcon( QString::fromLatin1(name), KIcon::NoGroup, KIcon::SizeMedium );
 }
 
-KTemplateEditDlg::KTemplateEditDlg(QWidget *parent) : KDialog(parent, 0, true)
+KTemplateEditDlg::KTemplateEditDlg(QWidget *parent) 
+    : KDialogBase(parent, "KTemplateEditDlg", true, i18n( "Icon Template" ),
+      Ok|Cancel )
 {
-  ok = cancel = 0L;
-
-  QVBoxLayout *ml = new QVBoxLayout(this, 10);
-  QGroupBox *grp = new QGroupBox(i18n("Template"), this);
+  QFrame *frame = makeMainWidget();
+  QVBoxLayout *ml = new QVBoxLayout(frame);
+  QGroupBox *grp = new QGroupBox(i18n("Template"), frame);
   grp->setColumnLayout(0, Qt::Vertical);
   grp->layout()->setSpacing(spacingHint());
   grp->layout()->setMargin(marginHint());
-  QGridLayout *l = new QGridLayout(grp->layout(),  25);
+  QGridLayout *l = new QGridLayout(grp->layout());
 
   ln_name = new QLineEdit( grp );
-  connect( ln_name, SIGNAL( textChanged( const QString & ) ), SLOT( slotTextChanged( const QString & ) ) );
+  connect( ln_name, SIGNAL( textChanged( const QString & ) ), 
+      SLOT( slotTextChanged() ) );
   QLabel* lb_name = new QLabel( ln_name, i18n( "Description:" ), grp );
 
   ln_path = new KURLRequester(grp);
-  connect( ln_path, SIGNAL( textChanged( const QString & ) ), SLOT( slotTextChanged( const QString & ) ) );
+  connect( ln_path, SIGNAL( textChanged( const QString & ) ), 
+      SLOT( slotTextChanged() ) );
   QLabel* lb_path = new QLabel( ln_path, i18n( "Path:" ), grp );
 
   l->addWidget(lb_name, 0, 0);
@@ -76,43 +79,17 @@ KTemplateEditDlg::KTemplateEditDlg(QWidget *parent) : KDialog(parent, 0, true)
   l->addWidget(lb_path, 1, 0);
   l->addWidget(ln_path, 1, 1);
 
-  KButtonBox *bbox = new KButtonBox( this );
-
-  ok = bbox->addButton( i18n( "&OK" ) );
-  connect( ok, SIGNAL( clicked() ), SLOT( accept() ) );
-  ok->setEnabled(false);
-
-  cancel = bbox->addButton( i18n( "&Cancel" ) );
-  connect( cancel, SIGNAL( clicked() ), SLOT( reject() ) );
-
-  bbox->addStretch( 1 );
-
-//  browse = bbox->addButton( i18n( "&Browse..." ) );
-//  connect( browse, SIGNAL( clicked() ), SLOT( slotBrowse() ) );
-
-  bbox->layout();
-
   ml->addWidget( grp, 1);
-  ml->addWidget( bbox );
 
-  QSize min(500, sizeHint().height());
+  slotTextChanged();
 
-  if (500 < sizeHint().width()) { min.setWidth(sizeHint().width()); }
-
-  resize(min);
+  setMinimumSize( 400, 150 );
 }
 
-void KTemplateEditDlg::slotTextChanged(const QString &)
+void KTemplateEditDlg::slotTextChanged()
 {
   QString name = ln_name->text(), path = ln_path->url();
-  if(name.length() && path.length())
-  {
-    if(ok) ok->setEnabled(true);
-  }
-  else
-  {
-    if(ok) ok->setEnabled(false);
-  }
+  enableButtonOK(name.length() && path.length());
 }
 
 void KTemplateEditDlg::setName(const QString & name)
@@ -120,7 +97,7 @@ void KTemplateEditDlg::setName(const QString & name)
   ln_name->setText(name);
 }
 
-QString KTemplateEditDlg::getName()
+QString KTemplateEditDlg::name()
 {
   return ln_name->text();
 }
@@ -130,43 +107,30 @@ void KTemplateEditDlg::setPath(const QString & path)
   ln_path->setURL(path);
 }
 
-QString KTemplateEditDlg::getPath()
+QString KTemplateEditDlg::path()
 {
   return ln_path->url();
 }
 
-void KTemplateEditDlg::slotBrowse()
-{
-  //  KURL url = KFileDialog::getOpenURL( getenv( "HOME" ), "*.xpm" );
-  KURL url = KFileDialog::getImageOpenURL( getenv( "HOME" ) );
-
-  if( url.isEmpty() )
-    return;
-
-  if( !url.isLocalFile() )
-  {
-    KMessageBox::sorry( 0L, i18n( "Only local files are supported yet." ) );
-    return;
-  }
-
-  setPath( url.path() );
-}
-
 KTemplateConfig::KTemplateConfig(QWidget *parent) : QWidget(parent)
 {
-  kdDebug() << "KTemplateConfig constructor" << endl;
+  kdDebug(4640) << "KTemplateConfig constructor" << endl;
 
   btadd = btremove = btedit = 0L;
 
   QGroupBox* grp = new QGroupBox( i18n( "Templates" ), this );
+  grp->setColumnLayout( 0, Qt::Horizontal );
+
   templates = new KIconListBox( grp );
-  connect( templates, SIGNAL( highlighted( int ) ), SLOT( checkSelection( int ) ) );
-  connect( templates, SIGNAL(doubleClicked( QListBoxItem * ) ), SLOT( edit() ) );
-  QVBoxLayout* ml = new QVBoxLayout( this, 5 );
+  connect( templates, SIGNAL( highlighted( int ) ), 
+      SLOT( checkSelection( int ) ) );
+  connect( templates, SIGNAL(doubleClicked( QListBoxItem * ) ), 
+      SLOT( edit() ) );
+
+  QVBoxLayout* ml = new QVBoxLayout( this );
   ml->addWidget( grp );
 
-  QVBoxLayout* l = new  QVBoxLayout( grp, 15 );
-  l->addSpacing( 10 );
+  QVBoxLayout* l = new  QVBoxLayout( grp->layout(), KDialog::spacingHint() );
   l->addWidget( templates );
 
   for( unsigned int i = 0; i <  KIconTemplateContainer::self()->count(); i++ )
@@ -184,9 +148,9 @@ KTemplateConfig::KTemplateConfig(QWidget *parent) : QWidget(parent)
   btremove = bbox->addButton( i18n( "&Remove" ) );
   connect( btremove, SIGNAL( clicked() ), SLOT( remove() ) );
   btremove->setEnabled( false );
-		
+	
   bbox->addStretch( 1 );
-		
+	
   bbox->layout();
   l->addWidget( bbox );
 }
@@ -197,7 +161,7 @@ KTemplateConfig::~KTemplateConfig()
 
 void KTemplateConfig::saveSettings()
 {
-  kdDebug() << "KTemplateConfig::saveSettings" << endl;
+  kdDebug(4640) << "KTemplateConfig::saveSettings" << endl;
   
   KIconTemplateContainer::self()->clear();
 
@@ -206,13 +170,13 @@ void KTemplateConfig::saveSettings()
       KIconTemplateContainer::self()->append(templates->iconTemplate(i));
   }
   KIconTemplateContainer::self()->save();
-  kdDebug() << "KTemplateConfig::saveSettings - done" << endl;
+  kdDebug(4640) << "KTemplateConfig::saveSettings - done" << endl;
 
 }
 
 void KTemplateConfig::checkSelection(int)
 {
-  kdDebug() << "KTemplateConfig::checkSelection" << endl;
+  kdDebug(4640) << "KTemplateConfig::checkSelection" << endl;
   if(templates->currentItem() != -1)
   {
     if(btremove) btremove->setEnabled(true);
@@ -223,7 +187,7 @@ void KTemplateConfig::checkSelection(int)
     if(btremove) btremove->setEnabled(false);
     if(btedit) btedit->setEnabled(false);
   }
-  kdDebug() << "KTemplateConfig::checkSelection - done" << endl;
+  kdDebug(4640) << "KTemplateConfig::checkSelection - done" << endl;
 }
 
 void KTemplateConfig::remove()
@@ -237,8 +201,8 @@ void KTemplateConfig::add()
   if(dlg.exec())
   {
     KIconTemplate it;
-    it.path = dlg.getPath();
-    it.title = dlg.getName();
+    it.path = dlg.path();
+    it.title = dlg.name();
     templates->insertItem(new KIconListBoxItem(it));
   }
 }
@@ -251,13 +215,10 @@ void KTemplateConfig::edit()
   templates->item(templates->currentItem());
   if(dlg.exec())
   {
-   
-
     //Edit the entry
     KIconTemplate &it=templates->iconTemplate(templates->currentItem());
-    int pos=templates->currentItem();
-    it.path = dlg.getPath();
-    it.title = dlg.getName();
+    it.path = dlg.path();
+    it.title = dlg.name();
     static_cast<KIconListBoxItem*>(templates->item(templates->currentItem()))->reloadIcon();
     templates->update();
   }
@@ -266,24 +227,24 @@ void KTemplateConfig::edit()
 KBackgroundConfig::KBackgroundConfig( QWidget* parent )
   : QWidget( parent )
 {
-  kdDebug() << "KBackgroundConfig - constructor" << endl;
+  kdDebug(4640) << "KBackgroundConfig - constructor" << endl;
 
   lb_ex = 0L;
 
-  Properties *pprops = KIconEditProperties::self()->properties();
+  KIconEditProperties *props = KIconEditProperties::self();
 
-  pixpath = pprops->backgroundpixmap;
+  pixpath = props->bgPixmap();
   pix.load(pixpath);
   if(pix.isNull())
   {
-    kdDebug() << "BGPIX: " << pixpath << " not valid!" << endl;
+    kdDebug(4640) << "BGPIX: " << pixpath << " not valid!" << endl;
     QPixmap pmlogo((const char**)logo);
     pix = pmlogo;
   }
 
-  QVBoxLayout *mainLayout = new QVBoxLayout( this, 5 );
+  QVBoxLayout *mainLayout = new QVBoxLayout( this );
 
-  QGroupBox *grp1 = new QGroupBox( i18n( "Color or Pixmap" ), this );
+  QGroupBox *grp1 = new QGroupBox( i18n( "Select Background" ), this );
   mainLayout->addWidget( grp1 );
 
   QGridLayout *grp1Layout = new QGridLayout( grp1, 3, 2, 15 );
@@ -296,11 +257,11 @@ KBackgroundConfig::KBackgroundConfig( QWidget* parent )
 
   QVBoxLayout *bgl = new QVBoxLayout( btngrp, 5 );
 
-  QRadioButton *rbc = new QRadioButton( i18n( "Colored background" ), btngrp );
+  QRadioButton *rbc = new QRadioButton( i18n( "Use co&lor" ), btngrp );
   btngrp->insert( rbc, 0 );
   bgl->addWidget( rbc );
 
-  QRadioButton *rbp = new QRadioButton( i18n( "Pixmap background" ), btngrp );
+  QRadioButton *rbp = new QRadioButton( i18n( "Use pix&map" ), btngrp );
   btngrp->insert( rbp, 1 );
   bgl->addWidget( rbp );
 
@@ -309,13 +270,14 @@ KBackgroundConfig::KBackgroundConfig( QWidget* parent )
   QVBox *bbox = new QVBox( grp1 );
   grp1Layout->addWidget( bbox, 0, 1 );
 
-  btcolor = new KColorButton(pprops->backgroundcolor, bbox) ;
-  connect(btcolor, SIGNAL(changed(const QColor &)), this, SLOT( selectColor(const QColor &)));
+  btcolor = new KColorButton(props->bgColor(), bbox) ;
+  connect(btcolor, SIGNAL(changed(const QColor &)), 
+      SLOT( selectColor(const QColor &)));
 
-  btpix = new QPushButton(i18n( "&Pixmap..." ), bbox);
+  btpix = new QPushButton(i18n( "Choose..." ), bbox);
   connect( btpix, SIGNAL( clicked() ), SLOT( selectPixmap() ) );
 
-  QGroupBox *grp2 = new QGroupBox( i18n( "Example" ), this );
+  QGroupBox *grp2 = new QGroupBox( i18n( "Preview" ), this );
   mainLayout->addWidget( grp2, 1 );
 
   QBoxLayout *l2 = new QVBoxLayout( grp2, 15 );
@@ -330,8 +292,8 @@ KBackgroundConfig::KBackgroundConfig( QWidget* parent )
   l1->addWidget( btngrp, 0, AlignLeft );
   l1->addLayout( l1r );
 */
+  bgmode = props->bgMode();
 
-  bgmode = pprops->backgroundmode;
   if( bgmode == QWidget::FixedPixmap )
   {
     btngrp->setButton( 1 );
@@ -372,12 +334,12 @@ void KBackgroundConfig::slotBackgroundMode(int id)
 
 void KBackgroundConfig::saveSettings()
 {
-  kdDebug() << "KBackgroundConfig::saveSettings" << endl;
-  Properties *pprops = KIconEditProperties::self()->properties();
-  pprops->backgroundmode = bgmode;
-  pprops->backgroundpixmap = pixpath;
-  pprops->backgroundcolor = btcolor->color();
-  kdDebug() << "KBackgroundConfig::saveSettings - done" << endl;
+  kdDebug(4640) << "KBackgroundConfig::saveSettings" << endl;
+  KIconEditProperties *props = KIconEditProperties::self();
+  props->setBgMode( bgmode );
+  props->setBgPixmap( pixpath );
+  props->setBgColor( btcolor->color() );
+  kdDebug(4640) << "KBackgroundConfig::saveSettings - done" << endl;
 }
 
 void KBackgroundConfig::selectColor(const QColor & newColor)
@@ -410,16 +372,17 @@ void KBackgroundConfig::selectPixmap()
 
 KMiscConfig::KMiscConfig(QWidget *parent) : QWidget(parent)
 {
-  kdDebug() << "KMiscConfig - constructor" << endl;
+  kdDebug(4640) << "KMiscConfig - constructor" << endl;
 
-  Properties* pprops = KIconEditProperties::self()->properties();
+  KIconEditProperties* props = KIconEditProperties::self();
 
-  QBoxLayout *ml = new QVBoxLayout( this, 5 );
+  QBoxLayout *ml = new QVBoxLayout( this );
 
   QGroupBox *grp1 = new QGroupBox( i18n( "Paste Mode" ), this );
+  grp1->setColumnLayout( 0, Qt::Horizontal );
   ml->addWidget(grp1);
 
-  QBoxLayout *l1 = new  QVBoxLayout(grp1, 20);
+  QBoxLayout *l1 = new  QVBoxLayout(grp1->layout(), KDialog::spacingHint());
 
   QCheckBox *cbp = new QCheckBox( i18n( "Paste transparent pixels" ), grp1 );
   connect( cbp, SIGNAL( toggled( bool ) ), SLOT( pasteMode( bool ) ) );
@@ -431,13 +394,8 @@ KMiscConfig::KMiscConfig(QWidget *parent) : QWidget(parent)
 
   l1->addStretch(1);
 
-  pastemode = pprops->pastetransparent;
-  if(pastemode)
-    cbp->setChecked(true);
-
-  showrulers = pprops->showrulers;
-  if(showrulers)
-    cbr->setChecked(true);
+  cbp->setChecked( props->pasteTransparent() );
+  cbr->setChecked( props->showRulers() );
 }
 
 KMiscConfig::~KMiscConfig()
@@ -447,10 +405,10 @@ KMiscConfig::~KMiscConfig()
 
 void KMiscConfig::saveSettings()
 {
-  kdDebug() << "KMiscConfig::saveSettings" << endl;
-  Properties* pprops =  KIconEditProperties::self()->properties();
-  pprops->pastetransparent = pastemode;
-  pprops->showrulers = showrulers;
+  kdDebug(4640) << "KMiscConfig::saveSettings" << endl;
+  KIconEditProperties* props =  KIconEditProperties::self();
+  props->setPasteTransparent( pastemode );
+  props->setShowRulers( showrulers );
 }
 
 void KMiscConfig::pasteMode(bool mode)
@@ -473,7 +431,7 @@ KIconConfig::KIconConfig(QWidget *parent)
                          parent, "configDialog", true, true)
 {
   setHelp(QString::null);
-  KWin::setIcons(winId(), kapp->icon(), kapp->miniIcon());
+  //KWin::setIcons(winId(), kapp->icon(), kapp->miniIcon());
   connect(this, SIGNAL(finished()), this, SLOT(finis()));
 
   QVBox* page = addVBoxPage(i18n("Icon Templates"), QString::null, loadIcon("icons"));
@@ -500,7 +458,7 @@ KIconConfig::~KIconConfig()
 
 void KIconConfig::slotApply()
 {
-  kdDebug() << "KIconEditConfig::saveSettings" << endl;
+  kdDebug(4640) << "KIconEditConfig::saveSettings" << endl;
 
   temps->saveSettings();
   backs->saveSettings();
