@@ -21,15 +21,10 @@
 #ifndef __KICONEDIT_H__
 #define __KICONEDIT_H__
 
-#include <qdir.h>
 #include <qwidget.h>
 #include <qpixmap.h>
-#include <qfileinfo.h> 
 #include <qptrlist.h>
-#include <qkeycode.h>
-#include <qtimer.h>
 
-#include <kapplication.h>
 #include <kmainwindow.h>
 #include <kiconloader.h>
 #include <kmenubar.h>
@@ -37,6 +32,7 @@
 #include <ktoolbar.h>
 #include <kaccel.h>
 #include <kurl.h>
+#include <kaction.h>
 
 #include "knew.h"
 #include "kicon.h"
@@ -46,32 +42,13 @@
 #include "properties.h"
 
 class KIconEdit;
+class KCommandHistory;
 typedef QPtrList<KIconEdit> WindowList;
 
-#define ID_VIEW_ZOOM_IN      520
-#define ID_VIEW_ZOOM_OUT     521
-#define ID_VIEW_ZOOM_1TO1    522
-#define ID_VIEW_ZOOM_1TO5    523
-#define ID_VIEW_ZOOM_1TO10   524
-
-#define ID_OPTIONS_TOGGLE_GRID  560
-#define ID_OPTIONS_TOGGLE_TOOL1 561
-#define ID_OPTIONS_TOGGLE_TOOL2 562
-#define ID_OPTIONS_TOGGLE_STATS 563
-#define ID_OPTIONS_SAVE         565
-
-#define ID_EDIT_PASTE_AS_NEW 513
-
-#define ID_CUSTOM_COLORS        600
-#define ID_SYSTEM_COLORS        601
-#define ID_PREVIEW              602
-
-
 class QWhatsThis;
 class QToolButton;
 class QLabel;
-class KSysColors;
-class KCustomColors;
+class PaletteToolBar;
 
 /**
 * KIconEdit
@@ -83,7 +60,7 @@ class KIconEdit : public KMainWindow
 {
     Q_OBJECT
 public:
-  KIconEdit( const QString &xpm = QString::null, const char *name = "kiconedit");
+  KIconEdit( KURL url = "", const char *name = "kiconedit");
   KIconEdit( const QImage image, const char *name = "kiconedit");
   ~KIconEdit();
 
@@ -91,6 +68,7 @@ public:
   static WindowList windowList;
 
 signals:
+  void newname(const QString &);
 
 public slots:
   virtual void saveProperties(KConfig*);
@@ -98,8 +76,6 @@ public slots:
   void updateProperties();
 
 protected slots:
-  void addColors(uint, uint*);
-  void addColor(uint);
   void slotNewWin(const QString & url = 0);
   void slotNew();
   void slotOpen();
@@ -107,16 +83,18 @@ protected slots:
   void slotSave();
   void slotSaveAs();
   void slotPrint();
-  void slotView(int);
   void slotZoomIn();
   void slotZoomOut();
+  void slotZoom1();
+  void slotZoom2();
+  void slotZoom5();
+  void slotZoom10();
   void slotCopy();
   void slotCut();
   void slotPaste();
   void slotClear();
   void slotSaved();
   void slotSelectAll();
-  //void slotOpenRecent(int id);
   void slotOpenRecent(const KURL&);
   void slotToolPointer();
   void slotToolFreehand();
@@ -134,17 +112,16 @@ protected slots:
   void slotToolSelectCircle();
   void slotConfigureSettings();
   void slotConfigureKeys();
-  void slotConfigure(int);
-
+  void slotShowGrid();
+  void slotUpdateZoom( int );
   void slotUpdateStatusColors(uint);
   void slotUpdateStatusColors(uint, uint*);
   void slotUpdateStatusPos(int, int);
   void slotUpdateStatusSize(int, int);
-  void slotClearStatusMessage();
   void slotUpdateStatusMessage(const QString &);
   void slotUpdateStatusName(const QString &);
   void slotUpdateStatusModified(bool);
-  void slotUpdateStatusScaling(int, bool);
+  void slotUpdateStatusScaling(int);
   void slotUpdatePaste(bool);
   void slotUpdateCopy(bool);
   void slotOpenBlank(const QSize);
@@ -155,37 +132,26 @@ protected slots:
 
 protected:
   void init();
-  bool setupMenuBar();
-  void setupTools();
-  bool setupToolBar();
-  bool setupDrawToolBar();
-  bool setupPaletteToolBar();
+  void setupActions();
   bool setupStatusBar();
-  bool setupWhatsThis();
-  virtual bool queryClose();
-  virtual void resizeEvent(QResizeEvent*);
   void writeConfig();
-
   void updateAccels();
 
-  QLabel *syslabel, *customlabel;
-  QLabel *preview;
-  KSysColors *syscolors;
-  KCustomColors *customcolors;
+  virtual bool queryClose();
+  virtual QWidget *createContainer( QWidget*, int, const QDomElement&, int& );
 
-  KMenuBar *menubar;
-  KToolBar *toolbar, *drawtoolbar, *palettetoolbar;
+  KCommandHistory* history;
+  PaletteToolBar *m_paletteToolBar;
   KStatusBar *statusbar;
-  QPopupMenu *edit, *file, *view, *help;
-  QPopupMenu *tools, *options, *zoom, /*recent,*/ *select;
   KIconEditGrid *grid;
-  QTimer *msgtimer;
   KGridView *gridview;
   KIconEditIcon *icon;
   QImage img;
   QString m_name;
 
-  KAction* actPrint, *actCopy, *actPaste, *actCut;
+  KAction *m_actCopy, *m_actPaste, *m_actCut, *m_actPasteNew;
+  KAction *m_actZoomIn, *m_actZoomOut;
+  KRecentFilesAction *m_actRecent;
 };
 
 #endif //__KICONEDIT_H__
