@@ -24,21 +24,47 @@
 #include <stdlib.h>
 
 #include <qpainter.h>
+#include <qwhatsthis.h>
+#include <qscrollview.h>
+#include <qbitmap.h>
+#include <qclipboard.h>
 
-#include <kpixmap.h>
+#include <kiconloader.h>
 #include <kruler.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
 
-#include "kiconedit.h"
+#include "kresize.h"
+#include "properties.h"
 #include "kicongrid.h"
+#include "kiconedit.h"
 #ifndef PICS_INCLUDED
 #include "pics/logo.xpm"
 #define PICS_INCLUDED
 #endif
 
-KGridView::KGridView(QImage *image, QWidget *parent, const char *name)
+void DrawCommand::execute()
+{
+	oldcolor = *((uint*)image->scanLine(y) + x);
+	*((uint*)image->scanLine(y) + x) = newcolor; 
+        int cell = y * grid->numCols() + x;
+        grid->setUndoColor( cell, newcolor, false );
+}
+
+void DrawCommand::unexecute()
+{
+	*((uint*)image->scanLine(y) + x) = oldcolor; 
+        int cell = y * grid->numCols() + x;
+        grid->setUndoColor( cell, oldcolor, false );
+}
+
+void RepaintCommand::execute()
+{
+        grid->update( area);
+}
+
+KGridView::KGridView(QImage *image, KCommandHistory* history, QWidget *parent, const char *name)
 : QFrame(parent, name)
 {
   _corner = 0L;
@@ -47,20 +73,26 @@ KGridView::KGridView(QImage *image, QWidget *parent, const char *name)
 
   acceptdrop = false;
 
-  Properties *pprops = KIconEditProperties::self()->properties();
+  KIconEditProperties *props = KIconEditProperties::self();
 
   viewport = new QScrollView(this);
   Q_CHECK_PTR(viewport);
 
-  _grid = new KIconEditGrid(image, viewport->viewport());
+  _grid = new KIconEditGrid(image, history, viewport->viewport());
   Q_CHECK_PTR(_grid);
   viewport->addChild(_grid);
-  _grid->setGrid(pprops->showgrid);
-  _grid->setCellSize(pprops->gridscaling);
+  _grid->setGrid(props->showGrid());
+  _grid->setCellSize(props->gridScale());
 
-  if(pprops->backgroundmode == FixedPixmap)
+  QString str = i18n( "Icon draw grid\n\nThe icon grid is the area where"
+      " you draw the icons.\nYou can zoom in and out using the magnifying"
+      " glasses on the toolbar.\n(Tip: Hold the magnify button down for a"
+      " few seconds to zoom to a predefined scale)" );
+  QWhatsThis::add( _grid, str );
+
+  if(props->bgMode() == FixedPixmap)
   {
-    QPixmap pix(pprops->backgroundpixmap);
+    QPixmap pix(props->bgPixmap());
     if(pix.isNull())
     {
       QPixmap pmlogo((const char **)logo);
@@ -71,7 +103,7 @@ KGridView::KGridView(QImage *image, QWidget *parent, const char *name)
   }
   else
   {
-    viewport->viewport()->setBackgroundColor(pprops->backgroundcolor);
+    viewport->viewport()->setBackgroundColor(props->bgColor());
   }
 
   _corner = new QFrame(this);
@@ -87,13 +119,18 @@ KGridView::KGridView(QImage *image, QWidget *parent, const char *name)
   _vruler->setOffset( -2 );
   _vruler->setRange(0, 1000);
 
-  connect(_grid, SIGNAL(scalingchanged(int, bool)), SLOT(scalingChange(int, bool)));
+  str = i18n( "Rulers\n\nThis is a visual representation of the current"
+      " cursor position" );
+  QWhatsThis::add( _hruler, str );
+  QWhatsThis::add( _vruler, str );
+
+  connect(_grid, SIGNAL(scalingchanged(int)), SLOT(scalingChange(int)));
   connect(_grid, SIGNAL(sizechanged(int, int)), SLOT(sizeChange(int, int)));
   connect(_grid, SIGNAL(needPainting()), SLOT(paintGrid()));
   connect( _grid, SIGNAL(xposchanged(int)), _hruler, SLOT(slotNewValue(int)) );
   connect( _grid, SIGNAL(yposchanged(int)), _vruler, SLOT(slotNewValue(int)) );
   connect(viewport, SIGNAL(contentsMoving(int, int)), SLOT(moving(int, int)));
-
+  
   setSizes();
   QResizeEvent e(size(), size());
   resizeEvent(&e);
@@ -106,7 +143,7 @@ void KGridView::paintGrid()
 
 void KGridView::setSizes()
 {
-  if(KIconEditProperties::self()->properties()->showrulers)
+  if(KIconEditProperties::self()->showRulers())
   {
     _hruler->setLittleMarkDistance(_grid->scaling());
     _vruler->setLittleMarkDistance(_grid->scaling());
@@ -129,11 +166,6 @@ void KGridView::setSizes()
     _vruler->setShowBigMarks(true);
     _vruler->setShowEndMarks(true);
 
-    _hruler->setValuePerLittleMark(_grid->scaling());
-    _vruler->setValuePerLittleMark(_grid->scaling());
-    _hruler->setValuePerMediumMark(_grid->scaling()*5);
-    _vruler->setValuePerMediumMark(_grid->scaling()*5);
-
     _hruler->setPixelPerMark(_grid->scaling());
     _vruler->setPixelPerMark(_grid->scaling());
 
@@ -162,19 +194,18 @@ void KGridView::sizeChange(int, int)
 
 void KGridView::moving(int x, int y)
 {
-    //kdDebug() << "Moving: " << x << " x " << y << endl;
     _hruler->setOffset(abs(x));
     _vruler->setOffset(abs(y));
 }
 
-void KGridView::scalingChange(int, bool)
+void KGridView::scalingChange(int)
 {
     setSizes();
 }
 
 void KGridView::setShowRulers(bool mode)
 {
-    KIconEditProperties::self()->properties()->showrulers = mode;
+    KIconEditProperties::self()->setShowRulers( mode );
     setSizes();
     QResizeEvent e(size(), size());
     resizeEvent(&e);
@@ -238,11 +269,11 @@ void KGridView::paintEvent(QPaintEvent *)
 
 void KGridView::resizeEvent(QResizeEvent*)
 {
-  kdDebug() << "KGridView::resizeEvent" << endl;
+  kdDebug(4640) << "KGridView::resizeEvent" << endl;
 
   setSizes();
 
-  if(KIconEditProperties::self()->properties()->showrulers)
+  if(KIconEditProperties::self()->showRulers())
   {
     _hruler->setGeometry(_vruler->width(), 0, width(), _hruler->height());
     _vruler->setGeometry(0, _hruler->height(), _vruler->width(), height());
@@ -256,11 +287,13 @@ void KGridView::resizeEvent(QResizeEvent*)
 }
 
 
-KIconEditGrid::KIconEditGrid(QImage *image, QWidget *parent, const char *name)
+KIconEditGrid::KIconEditGrid(QImage *image, KCommandHistory* h, QWidget *parent, const char *name)
  : KColorGrid(parent, name, 1)
 {
     img = image;
+    history = h;
     selected = 0;
+    m_command = 0;
 
     // the 42 normal kde colors - there can be an additional
     // 18 custom colors in the custom colors palette
@@ -268,7 +301,6 @@ KIconEditGrid::KIconEditGrid(QImage *image, QWidget *parent, const char *name)
         iconcolors.append(iconpalette[i]);
 
     setupImageHandlers();
-    //kdDebug() << "Formats: " << formats->count() << endl;
     btndown = isselecting = ispasting = modified = false;
 
     img->create(32, 32, 32);
@@ -284,17 +316,18 @@ KIconEditGrid::KIconEditGrid(QImage *image, QWidget *parent, const char *name)
     fill(TRANSPARENT);
 
     connect( kapp->clipboard(), SIGNAL(dataChanged()), SLOT(checkClipboard()));
+    connect( h, SIGNAL(commandExecuted()), this, SLOT(updatePreviewPixmap() ));    
     createCursors();
 }
 
 KIconEditGrid::~KIconEditGrid()
 {
-    kdDebug() << "KIconEditGrid - destructor: done" << endl;
+    kdDebug(4640) << "KIconEditGrid - destructor: done" << endl;
 }
 
 void KIconEditGrid::paintEvent(QPaintEvent *e)
 {
-    //kdDebug() << "KIconGrid::paintEvent" << endl;
+    //kdDebug(4640) << "KIconGrid::paintEvent" << endl;
     KColorGrid::paintEvent(e);
     //kdDebug() << "KIconGrid::paintEvent - done" << endl;
 }
@@ -317,6 +350,7 @@ void KIconEditGrid::paintForeground(QPainter* p, QPaintEvent* e)
         for(int i = y; i < cy; i += cellSize())
             p->drawLine(x, i, cx, i);
     }
+    //kdDebug(4640) << "KIconGrid::paintEvent - done" << endl;
 }
 
 
@@ -392,7 +426,7 @@ void KIconEditGrid::paintCell( QPainter *painter, int row, int col )
     default:
       break;
   }
-  //kdDebug() << "KIconEditGrid::paintCell() - done" << endl;
+  //kdDebug(4640) << "KIconEditGrid::paintCell() - done" << endl;
 }
 
 void KIconEditGrid::mousePressEvent( QMouseEvent *e )
@@ -446,7 +480,7 @@ void KIconEditGrid::mouseMoveEvent( QMouseEvent *e )
 
     if(img->valid(col, row))
     {
-        //kdDebug() << col << " X " << row << endl;
+        //kdDebug(4640) << col << " X " << row << endl;
         emit poschanged(col, row);
         // for the rulers
         emit xposchanged((col*scaling())+scaling()/2);
@@ -499,6 +533,9 @@ void KIconEditGrid::mouseMoveEvent( QMouseEvent *e )
 
         case Freehand:
         {
+            if( !m_command )
+                m_command = new KMacroCommand( i18n("Free Hand") );
+            
             if(erase)
                 setColor( cell, TRANSPARENT );
             else
@@ -509,9 +546,16 @@ void KIconEditGrid::mouseMoveEvent( QMouseEvent *e )
                 setModified( true );
                 int prevSel = selected;
                 selected = cell;
-                update((prevSel%numCols())*cellsize,(prevSel/numCols())*cellsize, cellsize, cellsize);
-                update(col*cellsize,row*cellsize, cellsize, cellsize);
-                *((uint*)img->scanLine(row) + col) = (colorAt(cell));
+                QRect area = QRect( col*cellsize,row*cellsize, cellsize, cellsize ).unite( 
+                    QRect ( (prevSel%numCols())*cellsize,(prevSel/numCols())*cellsize, cellsize, cellsize ) );
+                    
+                m_command->addCommand( new RepaintCommand( area, this ) );
+                DrawCommand* dc = new DrawCommand( col, row, colorAt(cell), img, this );
+                RepaintCommand* rp = new RepaintCommand( area, this );
+                dc->execute();
+                rp->execute();
+                m_command->addCommand( dc );
+                m_command->addCommand( rp );
             }
             break;
         }
@@ -597,6 +641,12 @@ void KIconEditGrid::mouseReleaseEvent( QMouseEvent *e )
         *((uint*)img->scanLine(row) + col) = colorAt(cell);
         p = *img;
       //}
+      
+        if( m_command ) {
+            history->addCommand( m_command, false );
+            m_command = 0;
+        }
+        
       break;
     }
     case Ellipse:
@@ -651,6 +701,7 @@ void KIconEditGrid::mouseReleaseEvent( QMouseEvent *e )
     default:
       break;
   }
+  
   emit changed(QPixmap(p));
   //emit colorschanged(numColors(), data());
 }
@@ -671,13 +722,14 @@ void KIconEditGrid::loadBlank( int w, int h )
   fill(TRANSPARENT);
   emit sizechanged(numCols(), numRows());
   emit colorschanged(numColors(), data());
+  history->clear();
 }
 
 
 
 void KIconEditGrid::load( QImage *image)
 {
-    kdDebug() << "KIconEditGrid::load" << endl;
+    kdDebug(4640) << "KIconEditGrid::load" << endl;
 
     setUpdatesEnabled(false);
 
@@ -700,7 +752,7 @@ void KIconEditGrid::load( QImage *image)
         {
             setColor((y*numCols())+x, *l, false);
         }
-        //kdDebug() << "Row: " << y << endl;
+        //kdDebug(4640) << "Row: " << y << endl;
         kapp->processEvents(200);
     }
 
@@ -711,6 +763,7 @@ void KIconEditGrid::load( QImage *image)
     setUpdatesEnabled(true);
     emit needPainting();
     //repaint(viewRect(), false);
+    history->clear();
 }
 
 const QPixmap &KIconEditGrid::pixmap()
@@ -723,20 +776,19 @@ const QPixmap &KIconEditGrid::pixmap()
 
 void KIconEditGrid::getImage(QImage *image)
 {
-    kdDebug() << "KIconEditGrid::getImage" << endl;
+    kdDebug(4640) << "KIconEditGrid::getImage" << endl;
     *image = *img;
 }
 
 bool KIconEditGrid::zoomTo(int scale)
 {
     QApplication::setOverrideCursor(waitCursor);
-    emit scalingchanged(cellSize(), false);
     setUpdatesEnabled(false);
     setCellSize( scale );
     setUpdatesEnabled(true);
     emit needPainting();
     QApplication::restoreOverrideCursor();
-    emit scalingchanged(cellSize(), true);
+    emit scalingchanged(cellSize());
 
     if(scale == 1)
         return false;
@@ -745,17 +797,16 @@ bool KIconEditGrid::zoomTo(int scale)
 
 bool KIconEditGrid::zoom(Direction d)
 {
-    int f = (d == In) ? (cellSize()+1) : (cellSize()-1);
+    int f = (d == DirIn) ? (cellSize()+1) : (cellSize()-1);
     QApplication::setOverrideCursor(waitCursor);
-    emit scalingchanged(cellSize(), false);
     setUpdatesEnabled(false);
     setCellSize( f );
     setUpdatesEnabled(true);
     //emit needPainting();
     QApplication::restoreOverrideCursor();
 
-    emit scalingchanged(cellSize(), true);
-    if(d == Out && cellSize() <= 1)
+    emit scalingchanged(cellSize());
+    if(d == DirOut && cellSize() <= 1)
         return false;
     return true;
 }
@@ -890,7 +941,7 @@ void KIconEditGrid::editPaste(bool paste)
     bool ok = false;
     QImage tmp = clipboardImage(ok);
 
-    Properties *pprops = KIconEditProperties::self()->properties();
+    KIconEditProperties *props = KIconEditProperties::self();
 
     if(ok)
     {
@@ -898,7 +949,8 @@ void KIconEditGrid::editPaste(bool paste)
         || (tmp.size().height() > img->size().height()) )
         {
             if(KMessageBox::warningYesNo(this,
-                i18n("The clipboard image is larger than the current image!\nPaste as new image?")) == 0)
+                i18n("The clipboard image is larger than the current"
+                " image!\nPaste as new image?")) == 0)
             {
                 editPasteAsNew();
             }
@@ -913,7 +965,7 @@ void KIconEditGrid::editPaste(bool paste)
         }
         else
         {
-            //kdDebug() << "KIconEditGrid: Pasting at: " << insrect.x() << " x " << insrect.y() << endl;
+            //kdDebug(4640) << "KIconEditGrid: Pasting at: " << insrect.x() << " x " << insrect.y() << endl;
             QApplication::setOverrideCursor(waitCursor);
 
             for(int y = insrect.y(), ny = 0; y < numRows(), ny < insrect.height(); y++, ny++)
@@ -922,7 +974,7 @@ void KIconEditGrid::editPaste(bool paste)
                 uint *cl = (uint*)tmp.scanLine(ny);
                 for(int x = insrect.x(), nx = 0; x < numCols(), nx < insrect.width(); x++, nx++, l++, cl++)
                 {
-                    if(pprops->pastetransparent)
+                    if(props->pasteTransparent())
                     {
                         *l = *cl;
                     }
@@ -931,14 +983,14 @@ void KIconEditGrid::editPaste(bool paste)
                         // Porter-Duff Over composition
                         double alphaS = qAlpha(*cl) / 255.0;
                         double alphaD = qAlpha(*l) / 255.0;
-                        
+  
                         double r = qRed(*cl) * alphaS + (1 - alphaS) * qRed(*l) * alphaD;
                         double g = qGreen(*cl) * alphaS + (1 - alphaS) * qGreen(*l) * alphaD;
                         double b = qBlue(*cl) * alphaS + (1 - alphaS) * qBlue(*l) * alphaD;
                         double a = alphaS + (1 - alphaS) * alphaD;
-                        
+  
                         // Remove multiplication by alpha
-                        
+
                         if(a > 0)
                         {
                             r /= a;
@@ -953,7 +1005,7 @@ void KIconEditGrid::editPaste(bool paste)
                         }
                         
                         int ir = (int)(r + 0.5);
-                        
+
                         if(ir < 0)
                         {
                             ir = 0;
@@ -965,7 +1017,7 @@ void KIconEditGrid::editPaste(bool paste)
                         }
                         
                         int ig = (int)(g + 0.5);
-                        
+
                         if(ig < 0)
                         {
                             ig = 0;
@@ -977,7 +1029,7 @@ void KIconEditGrid::editPaste(bool paste)
                         }
                         
                         int ib = (int)(b + 0.5);
-                        
+
                         if(ib < 0)
                         {
                             ib = 0;
@@ -989,7 +1041,7 @@ void KIconEditGrid::editPaste(bool paste)
                         }
                         
                         int ia = (int)((a * 255) + 0.5);
-                        
+
                         if(ia < 0)
                         {
                             ia = 0;
@@ -1002,7 +1054,7 @@ void KIconEditGrid::editPaste(bool paste)
                         
                         *l = qRgba(ir, ig, ib, ia);
                     }
-                    
+
                     setColor((y*numCols())+x, (uint)*l, false);
                 }
             }
@@ -1052,6 +1104,7 @@ void KIconEditGrid::editPasteAsNew()
             emit sizechanged(numCols(), numRows());
             emit colorschanged(numColors(), data());
             emit newmessage(i18n("Done pasting"));
+            history->clear();
         }
     }
     else
@@ -1064,7 +1117,7 @@ void KIconEditGrid::editPasteAsNew()
 
 void KIconEditGrid::editResize()
 {
-    kdDebug() << "KIconGrid::editResize" << endl;
+    kdDebug(4640) << "KIconGrid::editResize" << endl;
     KResizeDialog *rs = new KResizeDialog(this, 0, QSize(numCols(), numRows()));
     if(rs->exec())
     {
@@ -1080,7 +1133,7 @@ void KIconEditGrid::editResize()
 
 void KIconEditGrid::setSize(const QSize s)
 {
-    kdDebug() << "::setSize: " << s.width() << " x " << s.height() << endl;
+    kdDebug(4640) << "::setSize: " << s.width() << " x " << s.height() << endl;
 
     img->create(s.width(), s.height(), 32);
     img->setAlphaBuffer(true);
@@ -1100,7 +1153,7 @@ void KIconEditGrid::createCursors()
   if(pix.isNull())
   {
     cursor_colorpicker = cursor_normal;
-    kdDebug() << "KIconEditGrid: Error loading colorpicker-cursor.xpm" << endl;
+    kdDebug(4640) << "KIconEditGrid: Error loading colorpicker-cursor.xpm" << endl;
   }
   else
   {
@@ -1113,7 +1166,7 @@ void KIconEditGrid::createCursors()
   if(pix.isNull())
   {
     cursor_paint = cursor_normal;
-    kdDebug() << "KIconEditGrid: Error loading paintbrush.xpm" << endl;
+    kdDebug(4640) << "KIconEditGrid: Error loading paintbrush.xpm" << endl;
   }
   else
   {
@@ -1126,7 +1179,7 @@ void KIconEditGrid::createCursors()
   if(pix.isNull())
   {
     cursor_flood = cursor_normal;
-    kdDebug() << "KIconEditGrid: Error loading fill-cursor.xpm" << endl;
+    kdDebug(4640) << "KIconEditGrid: Error loading fill-cursor.xpm" << endl;
   }
   else
   {
@@ -1139,7 +1192,7 @@ void KIconEditGrid::createCursors()
   if(pix.isNull())
   {
     cursor_aim = cursor_normal;
-    kdDebug() << "KIconEditGrid: Error loading aim-cursor.xpm" << endl;
+    kdDebug(4640) << "KIconEditGrid: Error loading aim-cursor.xpm" << endl;
   }
   else
   {
@@ -1152,7 +1205,7 @@ void KIconEditGrid::createCursors()
   if(pix.isNull())
   {
     cursor_spray = cursor_normal;
-    kdDebug() << "KIconEditGrid: Error loading airbrush-cursor.xpm" << endl;
+    kdDebug(4640) << "KIconEditGrid: Error loading airbrush-cursor.xpm" << endl;
   }
   else
   {
@@ -1165,13 +1218,13 @@ void KIconEditGrid::createCursors()
   if(pix.isNull())
   {
     cursor_erase = cursor_normal;
-    kdDebug() << "KIconEditGrid: Error loading eraser-cursor.xpm" << endl;
+    kdDebug(4640) << "KIconEditGrid: Error loading eraser-cursor.xpm" << endl;
   }
   else
   {
     mask = pix.createHeuristicMask(true);
     pix.setMask(mask);
-    cursor_erase = QCursor(pix, 5, 16);
+    cursor_erase = QCursor(pix, 1, 16);
   }
 }
 
@@ -1241,6 +1294,7 @@ void KIconEditGrid::drawFlood(int x, int y, uint oldcolor)
     drawFlood(x, y+1, oldcolor);
     drawFlood(x-1, y, oldcolor);
     drawFlood(x+1, y, oldcolor);
+    //TODO: add undo 
 }
 
 
@@ -1249,7 +1303,7 @@ void KIconEditGrid::drawSpray(QPoint point)
     int x = (point.x()-5);
     int y = (point.y()-5);
 
-    //kdDebug() << "drawSpray() - " << x << " X " << y << endl;
+    //kdDebug(4640) << "drawSpray() - " << x << " X " << y << endl;
 
     pntarray.resize(0);
     int points = 0;
@@ -1371,7 +1425,7 @@ void KIconEditGrid::drawEllipse(bool drawit)
 
     int d = (cx > cy) ? cx : cy;
 
-    //kdDebug() << x << ", " << y << " - " << d << " " << d << endl;
+    //kdDebug(4640) << x << ", " << y << " - " << d << " " << d << endl;
     pntarray.resize(0);
     drawPointArray(a, Mark);
 
@@ -1436,7 +1490,7 @@ void KIconEditGrid::drawRect(bool drawit)
     else
         cy = end.y() - y;
 
-    //kdDebug() << x << ", " << y << " - " << cx << " " << cy << endl;
+    //kdDebug(4640) << x << ", " << y << " - " << cx << " " << cy << endl;
     pntarray.resize(0);
     drawPointArray(a, Mark); // remove previous marking
 
@@ -1552,28 +1606,42 @@ void KIconEditGrid::drawLine(bool drawit, bool drawStraight)
 
 void KIconEditGrid::drawPointArray(QPointArray a, DrawAction action)
 {
-    QRect rect = a.boundingRect();
+    QRect area( a.boundingRect().x()*cellSize()-1, a.boundingRect().y()*cellSize()-1,
+                a.boundingRect().width()*cellSize()+1, a.boundingRect().height()*cellSize()+1 );
+    
+    KMacroCommand* macro = 0;
     bool doupdate = false;
 
+    if( a.size() > 0 && action == Draw ) {
+	// might cause a memmory leak, if
+	// macro is never used and never 
+	// added to the history! TODO: Fix this
+        macro = new KMacroCommand( i18n("Drawn Array") );
+	RepaintCommand* rc = new RepaintCommand( area, this );
+	macro->addCommand( rc );	
+    }
+    
     int s = a.size(); //((rect.size().width()) * (rect.size().height()));
     for(int i = 0; i < s; i++)
     {
         int x = a[i].x();
         int y = a[i].y();
 
-        if(img->valid(x, y) && rect.contains(QPoint(x, y)))
+        if(img->valid(x, y) && a.boundingRect().contains(a[ i ]))
         {
-            //kdDebug() << "x: " << x << " - y: " << y << endl;
+            //kdDebug(4640) << "x: " << x << " - y: " << y << endl;
             switch( action )
             {
                 case Draw:
                 {
-                    *((uint*)img->scanLine(y) + x) = currentcolor; //colors[cell]|OPAQUE;
-                    int cell = y * numCols() + x;
-                    setColor( cell, currentcolor, false );
-                    setModified( true );
+		    DrawCommand* dc = new DrawCommand( x, y, currentcolor, img, this );
+		    dc->execute();
+                    //*((uint*)img->scanLine(y) + x) = currentcolor; //colors[cell]|OPAQUE;
+                    //int cell = y * numCols() + x;
+                    //setColor( cell, currentcolor, false );
                     doupdate = true;
                     //updateCell( y, x, FALSE );
+		    macro->addCommand( dc );
                     break;
                 }
 
@@ -1589,14 +1657,23 @@ void KIconEditGrid::drawPointArray(QPointArray a, DrawAction action)
         }
     }
 
+    
     if(doupdate)
     {
+        setModified( true );
         updateColors();
-        update(rect.x()*cellSize()-1, rect.y()*cellSize()-1,
-            rect.width()*cellSize()+1, rect.height()*cellSize()+1);
-        pntarray.resize(0);
-    }
+	RepaintCommand* rc = new RepaintCommand( area, this );
+        rc->execute();
+	macro->addCommand( rc );
+	pntarray.resize(0);
+	// add to undo/redo history	
+	history->addCommand( macro, false );    }
+}
 
+void KIconEditGrid::updatePreviewPixmap()
+{
+    p = *img;
+    emit changed(QPixmap(p));
 }
 
 
@@ -1633,7 +1710,7 @@ static bool kdither_32_to_8( const QImage *src, QImage *dst )
 	//printf("kconvert_32_to_8\n");
 
     if ( !dst->create(src->width(), src->height(), 8, 256) ) {
-		kdWarning() << "KPixmap: destination image not valid" << endl;
+		kdWarning() << "OImage: destination image not valid" << endl;
 		return FALSE;
 	}
 
