@@ -14,23 +14,19 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 // A lot of this code is lifted from KMail. Thanks, guys!
 
-#include <signal.h>
 #include <stdlib.h>
 
-
-#include <kapplication.h>
 #include <klocale.h>
 #include <kconfig.h>
 #include <kaboutdata.h>
 #include <kcmdlineargs.h>
 #include <kdebug.h>
 
-#include "version.h"
 #include "kiconedit.h"
 #include "config.h"
 #include "properties.h"
@@ -41,55 +37,8 @@ static const char *description =
 static KCmdLineOptions options[] =
 {
   { "+[file(s)]" , I18N_NOOP("Icon file(s) to open"), 0 },
-  { 0, 0, 0 }
+  KCmdLineLastOption
 };
-/*
-static void signalHandler(int sigId);
-static void cleanup(void);
-static void setSignalHandler(void (*handler)(int));
-
-//-----------------------------------------------------------------------------
-// Crash recovery signal handler
-static void signalHandler(int sigId)
-{
-  QWidget* win;
-
-  if(!windowList)
-    exit(1);
-
-  kdDebug() << "*** KIconEdit got signal " << sigId << endl;
-
-  // try to cleanup all windows
-  while (windowList.first() != NULL)
-  {
-    win = windowList.take();
-    // if (win->inherits("KIconEdit")) ((KIconEdit*)win)->saveGoingDownStatus();
-    delete win;
-  }
-
-  setSignalHandler(SIG_DFL);
-  cleanup();
-  exit(1);
-}
-
-
-//-----------------------------------------------------------------------------
-static void setSignalHandler(void (*handler)(int))
-{
-  signal(SIGSEGV, handler);
-  signal(SIGKILL, handler);
-  signal(SIGTERM, handler);
-  signal(SIGHUP,  handler);
-  signal(SIGFPE,  handler);
-  signal(SIGABRT, handler);
-}
-
-static void cleanup(void)
-{
-  kapp->config()->sync();
-}
-*/
-#include <iostream>
 
 int main(int argc, char **argv)
 {
@@ -101,6 +50,8 @@ int main(int argc, char **argv)
   aboutData.addAuthor("John Califf",0, "jcaliff@compuzone.net");
   aboutData.addAuthor("Laurent Montel",0, "lmontel@mandrakesoft.com");
   aboutData.addAuthor("Aaron J. Seigo",0, "aseigo@olympusproject.org");
+  aboutData.addCredit( "Nadeem Hasan", I18N_NOOP("Rewrote UI to use XMLGUI\n"
+      "Lots of fixes and cleanup"), "nhasan@nadmm.com" );
   aboutData.addCredit( "Adrian Page", I18N_NOOP("Bug fixes and GUI tidy up"),
       "Adrian.Page@tesco.net" );
   KCmdLineArgs::init( argc, argv, &aboutData );
@@ -119,7 +70,7 @@ int main(int argc, char **argv)
     KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
     for(int i = 0; i < args->count(); i++)
     {
-        KIconEdit *ki = new KIconEdit(QFile::decodeName(args->arg(i)), "kiconedit");
+        KIconEdit *ki = new KIconEdit(args->url(i), "kiconedit");
         Q_CHECK_PTR(ki);
     }
 
