@@ -22,38 +22,7 @@
 #ifndef __PROPS_H__
 #define __PROPS_H__
 
-#include <qptrlist.h>
 #include <qwidget.h>
-#include <qimage.h>
-#include <kaccel.h>
-#include <ktoolbar.h>
-#include <kmenubar.h>
-#include <kstatusbar.h>
-#include "knew.h"
-
-struct Properties
-{
-  QPtrList<KIconTemplate> *tlist;
-  QString backgroundpixmap;
-  QColor backgroundcolor;
-  QWidget::BackgroundMode backgroundmode;
-  bool showgrid;
-  bool pastetransparent;
-  bool showrulers;
-
-  // initial settings from config
-  KToolBar::BarPosition maintoolbarpos;
-  KToolBar::BarPosition drawtoolbarpos;
-  int maintoolbartext;
-  int drawtoolbartext;
-  int maintoolbarstat;
-  int drawtoolbarstat;
-  int statusbarstat;
-
-  int winwidth;
-  int winheight;
-  int gridscaling;
-};
 
 class KIconEditProperties : public QObject
 {
@@ -61,18 +30,35 @@ public:
   static KIconEditProperties* self();
   ~KIconEditProperties();
 
-  /**
-  * The QPtrList must not be deleted! Delete the object instead as it automatically
-  * deletes the list when there are no more instances.
-  */
-  struct Properties *properties();
-  void saveProperties();
+  QString bgPixmap() { return m_bgPixmap; }
+  QColor bgColor() { return m_bgColor; }
+  QWidget::BackgroundMode bgMode() { return m_bgMode; }
+  bool showGrid() { return m_showGrid; }
+  bool pasteTransparent() { return m_pasteTransparent; }
+  bool showRulers() { return m_showRulers; }
+  int gridScale() { return m_gridScale; }
+
+  void setBgPixmap( const QString &p ) { m_bgPixmap = p; }
+  void setBgColor( const QColor &c ) { m_bgColor = c; }
+  void setBgMode( QWidget::BackgroundMode m ) { m_bgMode = m; }
+  void setShowGrid( bool b ) { m_showGrid = b; }
+  void setPasteTransparent( bool b ) { m_pasteTransparent = b; }
+  void setShowRulers( bool b ) { m_showRulers = b; }
+  void setGridScale( int s ) { m_gridScale = s; }
+
+  void save();
 
 protected:
   KIconEditProperties();
-  struct Properties *pprops;
+  QString m_bgPixmap;
+  QColor m_bgColor;
+  QWidget::BackgroundMode m_bgMode;
+  bool m_showGrid;
+  bool m_pasteTransparent;
+  bool m_showRulers;
+  int m_gridScale;
+
   static KIconEditProperties* m_self;
 };
 
-
 #endif //__PROPS_H__
