@@ -26,6 +26,7 @@
 
 #include "indexer.h"
 
+#include <kdeversion.h>
 #if KDE_VERSION < 306
   #define QUOTE( x ) x
 #else
@@ -68,8 +69,6 @@ void Indexer::startIndexing( const QStringList& dirs )
 
 void Indexer::processFinished( KProcess *proc )
 {
-    qDebug("*** FINISHED: ok: %i (dirs left: %i) ***", proc->normalExit(), m_dirs.count());
-
     // error handling
     if ( !proc->normalExit() || proc != m_process )
     {
@@ -114,7 +113,7 @@ void Indexer::processNext()
          *m_process << *it;
 #endif
 
-    emit progress( 0, i18n("<qt>Next Directory: <br><b>%1</b>").arg( m_currentDir ));
+    emit progress( 0, i18n("<qt>Next Folder: <br><b>%1</b>").arg( m_currentDir ));
     m_process->start();
 }
 
@@ -162,7 +161,7 @@ void Indexer::slotCanRead( KProcIO *proc )
                     if ( ok )
                     {
                         uint dirsLeft = m_dirs.count();
-                        QString message = i18n( "<qt>Processing directory %1 of %2: <br><b>%3</b><br>File %4 of %5.</qt>").arg( m_dirCount - dirsLeft ).arg( m_dirCount).arg( m_currentDir ).arg( currentFile ).arg( numFiles );
+                        QString message = i18n( "<qt>Processing folder %1 of %2: <br><b>%3</b><br>File %4 of %5.</qt>").arg( m_dirCount - dirsLeft ).arg( m_dirCount).arg( m_currentDir ).arg( currentFile ).arg( numFiles );
                         emit progress( perc, message );
                     }
                 }
