@@ -24,7 +24,7 @@
 
 #include <kurl.h>
 
-#include <kmrml_config.h>
+#include "kmrml_config.h"
 
 namespace KMrml
 {
