@@ -18,9 +18,12 @@
 
 #include "config.h"
 
-#include <sys/types.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #include <kconfig.h>
 #include <kdebug.h>
@@ -89,7 +92,7 @@ void Mrml::get( const KURL& url )
 
     if ( !checkLocalServer( url ) )
     {
-        error( KIO::ERR_SLAVE_DEFINED, i18n("Can't start the Indexing Server. "
+        error( KIO::ERR_SLAVE_DEFINED, i18n("Unable to start the Indexing Server. "
                                             "Aborting the query.") );
         return;
     }
