@@ -33,7 +33,6 @@
 #include <kconfig.h>
 #include <kdatastream.h>
 #include <kdebug.h>
-#include <kdebugclasses.h>
 #include <kglobal.h>
 #include <klocale.h>
 #include <kiconloader.h>
@@ -104,7 +103,7 @@ KParts::Part * PartFactory::createPartObject( QWidget *parentWidget,
 
 // can't use this due to MrmlShared ref-counting
 // typedef KParts::GenericFactory<KMrml::MrmlPart> PartFactory;
-// K_EXPORT_COMPONENT_FACTORY( mrmlpart, PartFactory );
+// K_EXPORT_COMPONENT_FACTORY( mrmlpart, PartFactory )
 
 ///////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////
@@ -126,7 +125,7 @@ MrmlPart::MrmlPart( QWidget *parentWidget, const char * /* widgetName */,
     setInstance( PartFactory::instance(), true ); // do load plugins :)
     KConfig *config = PartFactory::instance()->config();
     config->setGroup("MRML Settings");
-	
+
     QVBox *box = new QVBox( parentWidget, "main mrml box" );
     m_view = new MrmlView( box, "MrmlView" );
     connect( m_view, SIGNAL( activated( const KURL&, ButtonState )),
@@ -157,7 +156,7 @@ MrmlPart::MrmlPart( QWidget *parentWidget, const char * /* widgetName */,
     m_algoButton->setFixedSize( m_algoButton->sizeHint() );
     connect( m_algoButton, SIGNAL( clicked() ),
              SLOT( slotConfigureAlgorithm() ));
-    QToolTip::add( m_algoButton, i18n("Configure Algorithm") );
+    QToolTip::add( m_algoButton, i18n("Configure algorithm") );
 
     QWidget *spacer = new QWidget( m_panel );
     spacer->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding,
@@ -206,7 +205,7 @@ void MrmlPart::initCollections( const QDomElement& elem )
     {
         KMessageBox::information( widget(),
                                i18n("There is no image collection available\n"
-                                    "at %1\n."), i18n("No Image Collection"));
+                                    "at %1.\n"), i18n("No Image Collection"));
         setStatus( NeedCollection );
     }
     else
@@ -222,8 +221,8 @@ void MrmlPart::initAlgorithms( const QDomElement& elem )
 bool MrmlPart::openURL( const KURL& url )
 {
     closeURL();
-    
-    if ( url.protocol() != "mrml" || url.isMalformed() ) {
+
+    if ( url.protocol() != "mrml" || !url.isValid() ) {
         qWarning("MrmlPart::openURL: cannot handle url: %s", url.prettyURL().latin1());
         return false; // what to do with that?
     }
@@ -246,9 +245,9 @@ bool MrmlPart::openURL( const KURL& url )
     {
         if ( !list.isEmpty() )
             KMessageBox::sorry( m_view,
-                                i18n("You can only search by example images"
+                                i18n("You can only search by example images "
                                      "on a local Indexing Server"),
-                                i18n("Only local Servers possible") );
+                                i18n("Only Local Servers Possible") );
     }
 
     else // localhost query
@@ -261,7 +260,7 @@ bool MrmlPart::openURL( const KURL& url )
             else
                 u = *it;
 
-            if ( !u.isMalformed() )
+            if ( u.isValid() )
             {
                 if ( u.isLocalFile() )
                     m_queryList.append( u );
@@ -276,7 +275,7 @@ bool MrmlPart::openURL( const KURL& url )
         if ( !QFile::exists( m_config.mrmldDataDir() + "/gift-config.mrml" ) )
         {
             if ( KMessageBox::questionYesNo(0L,
-                             i18n("There are no indexable directories "
+                             i18n("There are no indexable folders "
                                   "specified. Do you want to configure them "
                                   "now?"),
                                             i18n("Configuration Missing"),
@@ -305,7 +304,7 @@ bool MrmlPart::openURL( const KURL& url )
 void MrmlPart::contactServer( const KURL& url )
 {
     m_job = transferJob( url );
-    
+
     m_job->addMetaData( MrmlShared::kio_task(), MrmlShared::kio_initialize() );
 
     QString host = url.host().isEmpty() ?
@@ -329,7 +328,7 @@ void MrmlPart::downloadReferenceFiles( const KURL::List& downloadList )
         int index = (*it).fileName().findRev( '.' );
         if ( index != -1 )
             extension = (*it).fileName().mid( index );
-        
+
         KTempFile tmpFile( QString::null, extension );
         if ( tmpFile.status() != 0 )
         {
@@ -398,11 +397,11 @@ KIO::TransferJob * MrmlPart::transferJob( const KURL& url )
     job->setWindow( widget() );
     if ( !m_sessionId.isEmpty() )
         job->addMetaData( MrmlShared::sessionId(), m_sessionId );
-	
+
     emit started( job );
     emit setWindowCaption( url.prettyURL() );
     setStatus( InProgress );
-	
+
     return job;
 }
 
@@ -687,7 +686,7 @@ void MrmlPart::performQuery( QDomDocument& doc )
                                          "\"query-step\" element is missing."),
                             i18n("Query Error") );
     }
-	
+
     m_job = transferJob( url() );
     slotSetStatusBar( randomSearch ? i18n("Random search...") :
                                      i18n("Searching...") );
@@ -788,7 +787,7 @@ void MrmlPart::setStatus( Status status )
             m_startButton->setText( i18n("&Search") );
             break;
         case InProgress:
-            m_startButton->setText( i18n("&Sto&p") );
+            m_startButton->setText( i18n("Sto&p") );
             break;
     };
 
