@@ -124,7 +124,7 @@ QStringList Watcher::runningDaemons() const
     QDictIterator<DaemonData> it( m_daemons );
     for ( ; it.current(); ++it )
         result.append( it.current()->commandline );
-        
+
     return result;
 }
 
@@ -143,7 +143,7 @@ void Watcher::slotProcExited( KProcess *proc )
         if ( --daemon->restartOnFailure <= 0 )
         {
             if ( KMessageBox::questionYesNo( 0L,
-                             i18n("<qt>The Server with the commandline"
+                             i18n("<qt>The server with the command line"
                                   "<br>%1<br>"
                                   "is not available anymore. Do you want to "
                                   "restart it?" ).arg( daemon->commandline ),
@@ -172,8 +172,8 @@ bool Watcher::startDaemon( DaemonData *daemon )
     else
     {
         if ( KMessageBox::questionYesNo( 0L,
-                      i18n("Can't start the Server with the "
-                           "commandline"
+                      i18n("Unable to start the server with the "
+                           "command line"
                            "<br>%1<br>"
                           "Try again?").arg( daemon->commandline ),
                                          i18n("Service Failure") )
