@@ -8,10 +8,19 @@
 
 /***************************************************************************
  *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *  This file may be distributed and/or modified under the terms of the    *
+ *  GNU General Public License version 2 as published by the Free Software *
+ *  Foundation and appearing in the file COPYING included in the           *
+ *  packaging of this file.                                                *
+ *
+ *  As a special exception, permission is given to link this program       *
+ *  with any version of the KADMOS ocr/icr engine of reRecognition GmbH,   *
+ *  Kreuzlingen and distribute the resulting executable without            *
+ *  including the source code for KADMOS in the source distribution.       *
+ *
+ *  As a special exception, permission is given to link this program       *
+ *  with any edition of Qt, and distribute the resulting executable,       *
+ *  without including the source code for Qt in the source distribution.   *
  *                                                                         *
  ***************************************************************************/
 #include "kooka.h"
@@ -19,8 +28,8 @@
 #include "resource.h"
 
 #include "kookapref.h"
+#include "imgprintdialog.h"
 
-#include <qdragobject.h>
 #include <qlineedit.h>
 #include <qprinter.h>
 #include <qprintdialog.h>
@@ -45,58 +54,64 @@
 #include <kedittoolbar.h>
 #include <kmessagebox.h>
 #include <kdockwidget.h>
-
+#include <kparts/partmanager.h>
 #include <kstdaccel.h>
 #include <kaction.h>
 #include <kstdaction.h>
+#include <qiconset.h>
+#include <kurldrag.h>
 
 #define DOCK_SIZES "DockSizes"
 
 
 Kooka::Kooka( const QCString& deviceToUse)
-   : KDockMainWindow( 0, "Kooka" ),
+   : KParts::DockMainWindow( 0, "Kooka" ),
       m_printer(0),
       m_prefDialogIndex(0)
 {
-   // accept dnd
-   // m_view = new KookaView(this, deviceToUse);
+    /* Start to create the main view framework */
+    m_view = new KookaView( this, deviceToUse);
 
+    /* Call createGUI on the ocr-result view */
+    setXMLFile( "kookaui.rc", true );
 
-   m_view = new KookaView( this, deviceToUse);
-   // setView( m_view->mainDockWidget() ); // central widget in a KDE mainwindow
-   // setMainDockWidget( m_view->mainDockWidget() );
-   setAcceptDrops(true);
-   KConfig *konf = KGlobal::config ();
-   readDockConfig ( konf, DOCK_SIZES );
+    setAcceptDrops(false); // Waba: Not (yet?) supported
+    KConfig *konf = KGlobal::config ();
+    readDockConfig ( konf, DOCK_SIZES );
 
+    // then, setup our actions
+    setupActions();
 
-   // tell the KMainWindow that this is indeed the main widget
-   // setCentralWidget(m_view->mainDockWidget());
+    createGUI(0L); // m_view->ocrResultPart());
+    // and a status bar
+    statusBar()->insertItem( QString(), KookaView::StatusTemp );
+    statusBar()->show();
 
-   // then, setup our actions
-   setupActions();
+    // allow the view to change the statusbar and caption
+    connect(m_view, SIGNAL(signalChangeStatusbar(const QString&)),
+            this,   SLOT(changeStatusbar(const QString&)));
+    connect(m_view, SIGNAL(signalCleanStatusbar(void)),
+            this, SLOT(cleanStatusbar()));
+    connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
+            this,   SLOT(changeCaption(const QString&)));
 
-   // and a status bar
-   statusBar()->show();
+    changeCaption( i18n( "KDE Scanning" ));
 
-   // allow the view to change the statusbar and caption
-   connect(m_view, SIGNAL(signalChangeStatusbar(const QString&)),
-	   this,   SLOT(changeStatusbar(const QString&)));
-   connect(m_view, SIGNAL(signalCleanStatusbar(void)),
-	   this, SLOT(cleanStatusbar()));
-   connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
-	   this,   SLOT(changeCaption(const QString&)));
-
-   changeCaption( i18n( "KDE Scanning" ));
-
-   setAutoSaveSettings(  QString::fromLatin1("General Options"),
-			 true );
+    setAutoSaveSettings(  QString::fromLatin1("General Options"),
+                          true );
 }
 
+void Kooka::createMyGUI( KParts::Part *part )
+{
+    kdDebug(28000) << "Part changed, Creating gui" << endl;
+    createGUI(part);
+
+}
 
 Kooka::~Kooka()
 {
    KConfig *konf = KGlobal::config ();
+   m_view->slCloseScanDevice();
    writeDockConfig ( konf, DOCK_SIZES );
    delete m_printer;
 }
@@ -114,10 +129,9 @@ void Kooka::setupActions()
     KStdAction::print(this, SLOT(filePrint()), actionCollection());
     KStdAction::quit(this , SLOT(close()), actionCollection());
 
-    m_toolbarAction = KStdAction::showToolbar(this, SLOT(optionsShowToolbar()),
-					      actionCollection());
-    m_statusbarAction = KStdAction::showStatusbar(this, SLOT(optionsShowStatusbar()),
-						  actionCollection());
+    // createStandardStatusBarAction();
+    // setStandardToolBarMenuEnabled(true);
+
     m_scanParamsAction = new KToggleAction(i18n("Show Scan &Parameters"),
                                            KShortcut(), this,
 					   SLOT(optionsShowScanParams()),
@@ -153,8 +167,25 @@ void Kooka::setupActions()
     m_view->connectViewerAction( act );
 
     act = new KAction(i18n("Original &Size"), "scaleorig", CTRL+Key_S,
-		       m_view, SLOT( slIVScaleOriginal()),
-		       actionCollection(), "scaleOriginal" );
+                      m_view, SLOT( slIVScaleOriginal()),
+                      actionCollection(), "scaleOriginal" );
+    m_view->connectViewerAction( act );
+
+#ifdef QICONSET_HONOUR_ON_OFF
+    /* The Toggleaction does not seem to handle the on/off icon from QIconSet */
+    QIconSet lockSet;
+    lockSet.setPixmap(BarIcon("lock")  , QIconSet::Automatic, QIconSet::Normal, QIconSet::On );
+    lockSet.setPixmap(BarIcon("unlock"), QIconSet::Automatic, QIconSet::Normal, QIconSet::Off);
+    act = new KToggleAction ( i18n("Keep &Zoom Setting"), lockSet, CTRL+Key_Z,
+                              actionCollection(), "keepZoom" );
+#else
+    act = new KToggleAction( i18n("Keep &Zoom Setting"), BarIcon("lockzoom"), CTRL+Key_Z,
+                             actionCollection(), "keepZoom" );
+#endif
+
+    connect( act, SIGNAL( toggled( bool ) ), m_view->getImageViewer(),
+             SLOT(setKeepZoom(bool)));
+
     m_view->connectViewerAction( act );
 
     /* thumbview and gallery actions */
@@ -199,16 +230,21 @@ void Kooka::setupActions()
     m_view->connectViewerAction( act );
 
     /* Gallery actions */
-    act = new KAction(i18n("&Create Directory..."), "folder_new", 0,
+    act = new KAction(i18n("&Create Folder..."), "folder_new", 0,
 		      m_view->gallery(), SLOT( slotCreateFolder() ),
 		       actionCollection(), "foldernew" );
     m_view->connectGalleryAction( act );
 
-    act = new KAction(i18n("&Save Image"), "filesave", 0,
+    act = new KAction(i18n("&Save Image..."), "filesave", 0,
 		      m_view->gallery(), SLOT( slotExportFile() ),
 		       actionCollection(), "saveImage" );
     m_view->connectGalleryAction( act );
 
+    act = new KAction(i18n("&Import Image..."), "inline_image", 0,
+		      m_view->gallery(), SLOT( slotImportFile() ),
+		       actionCollection(), "importImage" );
+    m_view->connectGalleryAction( act );
+
     act = new KAction(i18n("&Delete Image"), "edittrash", 0,
 		      m_view->gallery(), SLOT( slotDeleteItems() ),
 		       actionCollection(), "deleteImage" );
@@ -221,7 +257,7 @@ void Kooka::setupActions()
 
 #if 0
     /* not yet supported actions - coming post 3.1 */
-    (void) new KAction(i18n("&Load scan parameters"), "bookmark_add", CTRL+Key_L,
+    (void) new KAction(i18n("&Load Scan Parameters"), "bookmark_add", CTRL+Key_L,
                        m_view, SLOT(slLoadScanParams()),
                        actionCollection(), "loadscanparam" );
 
@@ -229,8 +265,8 @@ void Kooka::setupActions()
 		       m_view, SLOT(slSaveScanParams()),
 		       actionCollection(), "savescanparam" );
 #endif
-    
-    (void) new KAction(i18n("Select Scan Device"), "scanner", CTRL+Key_Q,
+
+    (void) new KAction(i18n("Select Scan Device"), "scanner", 0,
 		       m_view, SLOT( slSelectDevice()),
 		       actionCollection(), "selectsource" );
 
@@ -238,8 +274,10 @@ void Kooka::setupActions()
 			this,  SLOT(slEnableWarnings()),
 			actionCollection(), "enable_msgs");
 
-    createGUI("kookaui.rc");
 
+    m_saveOCRTextAction = new KAction( i18n("Save OCR Res&ult Text"), "filesaveas", CTRL+Key_U,
+                                       m_view, SLOT(slSaveOCRResult()),
+                                       actionCollection(), "saveOCRResult");
 }
 
 
@@ -249,8 +287,8 @@ void Kooka::saveProperties(KConfig *config)
     // config file.  anything you write here will be available
     // later when this app is restored
 
-   //if (m_view->currentURL() != QString::null)
-   //     config->writeEntry("lastURL", m_view->currentURL());
+   //if (!m_view->currentURL().isNull())
+   //     config->writePathEntry("lastURL", m_view->currentURL());
    kdDebug(28000) << "In kooka's saveProperties !" << endl;
    config->setGroup( KOOKA_STATE_GROUP );
    config->writeEntry( PREFERENCE_DIA_TAB, m_prefDialogIndex );
@@ -266,33 +304,36 @@ void Kooka::readProperties(KConfig *config)
     // in 'saveProperties'
    config->setGroup( KOOKA_STATE_GROUP );
    m_prefDialogIndex = config->readNumEntry( PREFERENCE_DIA_TAB, 0 );
-   // QString url = config->readEntry("lastURL");
+   // QString url = config->readPathEntry("lastURL");
 
 }
 
 void Kooka::dragEnterEvent(QDragEnterEvent *event)
 {
     // accept uri drops only
-    event->accept(QUriDrag::canDecode(event));
+    event->accept(KURLDrag::canDecode(event));
 }
 
+#if 0
 void Kooka::dropEvent(QDropEvent *event)
 {
     // this is a very simplistic implementation of a drop event.  we
     // will only accept a dropped URL.  the Qt dnd code can do *much*
     // much more, so please read the docs there
-    QStrList uri;
+    KURL::List uri;
 
     // see if we can decode a URI.. if not, just ignore it
-    if (QUriDrag::decode(event, uri))
+    if (KURLDrag::decode(event, uri) && !uri.isEmpty())
     {
         // okay, we have a URI.. process it
-        QString url, target;
-        url = uri.first();
+        const KURL &url = uri.first();
+	kdDebug(29000) << "Importing URI " << url.url() << endl;
+
+        // TODO: Do something with url
+        // Waba: See also setAcceptDrops() above
     }
 }
 
-#if 0
 void Kooka::fileNew()
 {
     // this slot is called whenever the File->New menu is selected,
@@ -337,41 +378,8 @@ void Kooka::filePrint()
     // this slot is called whenever the File->Print menu is selected,
     // the Print shortcut is pressed (usually CTRL+P) or the Print toolbar
     // button is clicked
-    if (!m_printer) m_printer = new KPrinter;
-    if (m_printer->setup(this))
-    {
-       // setup the printer.  with Qt, you always "print" to a
-       // QPainter.. whether the output medium is a pixmap, a screen,
-       // or paper
-       QPainter p;
-       QPaintDeviceMetrics metrics( m_printer );
-       p.begin( m_printer);
-       // we let our view do the actual printing
-       m_view->print(&p, m_printer, metrics );
+    m_view->print();
 
-       // and send the result to the printer
-       p.end();
-    }
-}
-
-void Kooka::optionsShowToolbar()
-{
-    // this is all very cut and paste code for showing/hiding the
-    // toolbar
-    if (m_toolbarAction->isChecked())
-        toolBar("mainToolBar")->show();
-    else
-        toolBar("mainToolBar")->hide();
-}
-
-void Kooka::optionsShowStatusbar()
-{
-    // this is all very cut and paste code for showing/hiding the
-    // statusbar
-    if (m_statusbarAction->isChecked())
-        statusBar()->show();
-    else
-        statusBar()->hide();
 }
 
 void Kooka::optionsShowScanParams()
@@ -387,18 +395,22 @@ void Kooka::optionsShowPreviewer()
 
 void Kooka::optionsConfigureKeys()
 {
-    KKeyDialog::configureKeys(actionCollection(), "kookaui.rc");
+    KKeyDialog::configure(actionCollection());
 }
 
 void Kooka::optionsConfigureToolbars()
 {
     // use the standard toolbar editor
-    KEditToolbar dlg(actionCollection());
-    if (dlg.exec())
-    {
-        // recreate our GUI
-        createGUI( "kookaui.rc");
-    }
+    saveMainWindowSettings(KGlobal::config(), autoSaveGroup());
+    KEditToolbar dlg(factory());
+    connect(&dlg, SIGNAL(newToolbarConfig()), SLOT(newToolbarConfig()));
+    dlg.exec();
+}
+
+void Kooka::newToolbarConfig()
+{
+    // OK/Apply pressed in the toolbar editor
+    applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
 }
 
 void Kooka::optionsPreferences()
@@ -419,7 +431,7 @@ void Kooka::optionsPreferences()
 void Kooka::changeStatusbar(const QString& text)
 {
     // display the text on the statusbar
-    statusBar()->message(text);
+    statusBar()->changeItem( text, KookaView::StatusTemp );
 }
 
 void Kooka::changeCaption(const QString& text)
