@@ -1,29 +1,40 @@
 /**************************************************************************
 			kooka.h  -  Main program class
-                             -------------------                                         
-    begin                : Sun Jan 16 2000                                           
-    copyright            : (C) 2000 by Klaas Freitag                         
-    email                : freitag@suse.de          
+                             -------------------
+    begin                : Sun Jan 16 2000
+    copyright            : (C) 2000 by Klaas Freitag
+    email                : freitag@suse.de
  ***************************************************************************/
 
 /***************************************************************************
  *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   * 
+ *  This file may be distributed and/or modified under the terms of the    *
+ *  GNU General Public License version 2 as published by the Free Software *
+ *  Foundation and appearing in the file COPYING included in the           *
+ *  packaging of this file.                                                *
+ *
+ *  As a special exception, permission is given to link this program       *
+ *  with any version of the KADMOS ocr/icr engine of reRecognition GmbH,   *
+ *  Kreuzlingen and distribute the resulting executable without            *
+ *  including the source code for KADMOS in the source distribution.       *
+ *
+ *  As a special exception, permission is given to link this program       *
+ *  with any edition of Qt, and distribute the resulting executable,       *
+ *  without including the source code for Qt in the source distribution.   *
  *                                                                         *
  ***************************************************************************/
+
 #ifndef KOOKA_H
 #define KOOKA_H
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif 
+#endif
 
 #include <kapplication.h>
 #include <kmainwindow.h>
 #include <kdockwidget.h>
+#include <kparts/dockmainwindow.h>
 
 #define KOOKA_STATE_GROUP "State"
 #define PREFERENCE_DIA_TAB "PreferencesTab"
@@ -41,7 +52,7 @@ class KookaView;
  * @author Klaas Freitag <freitag@suse.de>
  * @version 0.1
  */
-class Kooka : public KDockMainWindow
+class Kooka : public KParts::DockMainWindow
 {
     Q_OBJECT
 public:
@@ -60,13 +71,13 @@ public:
     **/
    void startup( void );
 
-    
+
 protected:
     /**
      * Overridden virtuals for Qt drag 'n drop (XDND)
      */
     virtual void dragEnterEvent(QDragEnterEvent *event);
-    virtual void dropEvent(QDropEvent *event);
+    // virtual void dropEvent(QDropEvent *event);
 
     /**
      * This function is called when it is time for the app to save its
@@ -83,11 +94,12 @@ protected:
 
 
 private slots:
+
+    void createMyGUI( KParts::Part* );
+
    void filePrint();
    /* ImageViewer-Actions */
-   
-   void optionsShowToolbar();
-   void optionsShowStatusbar();
+
    void optionsShowScanParams();
    void optionsShowPreviewer();
    void optionsConfigureKeys();
@@ -97,6 +109,7 @@ private slots:
    void changeStatusbar(const QString& text);
    void cleanStatusbar(void) { changeStatusbar(""); }
    void changeCaption(const QString& text);
+   void newToolbarConfig();
 
    // void fileSaveAs();
 
@@ -109,7 +122,7 @@ private slots:
    void slRotate180( void );
 
    void slEnableWarnings();
-   
+
 private:
    void setupAccel();
    void setupActions();
@@ -118,11 +131,11 @@ private:
    KookaView *m_view;
 
    KPrinter   *m_printer;
-   KToggleAction *m_toolbarAction;
-   KToggleAction *m_statusbarAction;
    KToggleAction *m_scanParamsAction;
    KToggleAction *m_previewerAction;
    KActionMenu   *m_settingsShowDocks;
+
+    KAction      *m_saveOCRTextAction;
    int m_prefDialogIndex;
 };
 
