@@ -1,19 +1,28 @@
 /***************************************************************************
               kookaview.cpp  -  kookas visible stuff
-                             -------------------                                         
+                             -------------------
     begin                : ?
-    copyright            : (C) 1999 by Klaas Freitag                         
+    copyright            : (C) 1999 by Klaas Freitag
     email                : freitag@suse.de
 
-    $Id: kookaview.cpp,v 1.53 2002/10/11 12:22:28 freitag Exp $
+    $Id: kookaview.cpp,v 1.69 2003/10/21 23:10:41 maragato Exp $
  ***************************************************************************/
 
 /***************************************************************************
  *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   * 
+ *  This file may be distributed and/or modified under the terms of the    *
+ *  GNU General Public License version 2 as published by the Free Software *
+ *  Foundation and appearing in the file COPYING included in the           *
+ *  packaging of this file.                                                *
+ *
+ *  As a special exception, permission is given to link this program       *
+ *  with any version of the KADMOS ocr/icr engine of reRecognition GmbH,   *
+ *  Kreuzlingen and distribute the resulting executable without            *
+ *  including the source code for KADMOS in the source distribution.       *
+ *
+ *  As a special exception, permission is given to link this program       *
+ *  with any edition of Qt, and distribute the resulting executable,       *
+ *  without including the source code for Qt in the source distribution.   *
  *                                                                         *
  ***************************************************************************/
 
@@ -27,6 +36,11 @@
 #include "imgnamecombo.h"
 #include "thumbview.h"
 #include "dwmenuaction.h"
+#include "kookaimage.h"
+#include "kookaimagemeta.h"
+#include "ocrresedit.h"
+#include "kookaprint.h"
+#include "imgprintdialog.h"
 #if 0
 #include "paramsetdialogs.h"
 #endif
@@ -43,6 +57,7 @@
 #include <kurl.h>
 #include <krun.h>
 #include <kapplication.h>
+#include <kstatusbar.h>
 #include <kconfig.h>
 #include <kdebug.h>
 #include <ktrader.h>
@@ -58,45 +73,60 @@
 #include <kdockwidget.h>
 #include <qobject.h>
 
-#define PACKAGER_TAB 0
-#define PREVIEWER_TAB 1
+#include <kparts/componentfactory.h>
+#include <qimage.h>
+#include <kpopupmenu.h>
+
 
 #define STARTUP_IMG_SELECTION   "SelectedImageOnStartup"
 
 
-KookaView::KookaView( KDockMainWindow *parent, const QCString& deviceToUse)
+KookaView::KookaView( KParts::DockMainWindow *parent, const QCString& deviceToUse)
    : QObject(),
+     m_ocrResultImg(0),
      ocrFabric(0),
      m_mainDock(0),
      m_dockScanParam(0),
      m_dockThumbs(0),
      m_dockPackager(0),
      m_dockRecent(0),
-     m_dockPreview(0)
+     m_dockPreview(0),
+     m_dockOCRText(0),
+     m_mainWindow(parent),
+     m_ocrResEdit(0)
 {
    KIconLoader *loader = KGlobal::iconLoader();
+   scan_params = 0L;
+   preview_canvas = 0L;
+
    m_mainDock = parent->createDockWidget( "Kookas MainDock",
-			       loader->loadIcon( "folder_image", KIcon::Small ),
-			       0L, i18n("Image Viewer"));
+                                          loader->loadIcon( "folder_image", KIcon::Small ),
+                                          0L, i18n("Image Viewer"));
    m_mainDock->setEnableDocking(KDockWidget::DockNone );
-   m_mainDock->setDockSite(KDockWidget::DockCorner);
+   m_mainDock->setDockSite( KDockWidget::DockFullSite );
+
    parent->setView( m_mainDock);
    parent->setMainDockWidget( m_mainDock);
-   
+
    img_canvas  = new ImageCanvas( m_mainDock );
    img_canvas->setMinimumSize(100,200);
    img_canvas->enableContextMenu(true);
+   connect( img_canvas, SIGNAL( imageReadOnly(bool)),
+	    this, SLOT(slViewerReadOnly(bool)));
+   
+   KPopupMenu *ctxtmenu = static_cast<KPopupMenu*>(img_canvas->contextMenu());
+   if( ctxtmenu )
+       ctxtmenu->insertTitle(i18n("Image View"));
    m_mainDock->setWidget( img_canvas );
 
    m_dockThumbs = parent->createDockWidget( "Thumbs",
 					    loader->loadIcon( "thumbnail", KIcon::Small ),
 					    0L,  i18n("Thumbnails"));
    m_dockThumbs->setDockSite(KDockWidget::DockFullSite );
-   
+
    /* thumbnail viewer widget */
    m_thumbview = new ThumbView( m_mainDock );
    m_dockThumbs->setWidget( m_thumbview );
-   m_dockThumbs->manualDock( m_mainDock, KDockWidget::DockLeft, 30 );
 
    /* make the main dock widget */
    /* A new packager to contain the already scanned images */
@@ -110,21 +140,23 @@ KookaView::KookaView( KDockMainWindow *parent, const QCString& deviceToUse)
                          KDockWidget::DockLeft, // dock site
                          30 );                  // relation target/this (in percent)
 
+   m_dockThumbs->manualDock( m_mainDock, KDockWidget::DockBottom, 5 );
+
    connect( packager, SIGNAL(showThumbnails( KFileTreeViewItem* )),
 	    this, SLOT( slShowThumbnails( KFileTreeViewItem* )));
    connect( m_thumbview, SIGNAL( selectFromThumbnail( const KURL& )),
 	    packager, SLOT( slSelectImage(const KURL&)));
-   
+
    /*
     * Create a Kombobox that holds the last folders visible even on the preview page
     */
    m_dockRecent  = parent->createDockWidget( "Recent",
 				loader->loadIcon( "image", KIcon::Small ),
-				0L, i18n("Gallery Directories"));
-   
+				0L, i18n("Gallery Folders"));
+
    m_dockRecent->setDockSite(KDockWidget::DockFullSite);
 
-   
+
    QHBox *recentBox = new QHBox( m_dockRecent );
    recentBox->setMargin(KDialog::marginHint());
    QLabel *lab = new QLabel( i18n("Gallery:"), recentBox );
@@ -137,7 +169,7 @@ KookaView::KookaView( KDockMainWindow *parent, const QCString& deviceToUse)
                          5 );                  // relation target/this (in percent)
 
 
-   
+
    connect( packager,  SIGNAL( galleryPathSelected( KFileTreeBranch*, const QString&)),
 	    recentFolder, SLOT( slotGalleryPathChanged( KFileTreeBranch*, const QString& )));
 
@@ -146,7 +178,7 @@ KookaView::KookaView( KDockMainWindow *parent, const QCString& deviceToUse)
 
    connect( recentFolder, SIGNAL(activated( const QString& )),
 	    packager, SLOT(slotSelectDirectory( const QString& )));
-   
+
    /* the object from the kscan lib to handle low level scanning */
    m_dockScanParam = parent->createDockWidget( "Scan Parameter",
  					     loader->loadIcon( "folder", KIcon::Small ),
@@ -162,13 +194,10 @@ KookaView::KookaView( KDockMainWindow *parent, const QCString& deviceToUse)
    KDockWidget::DockBottom, // dock site
    20 );                  // relation target/this (in percent)
    m_dockScanParam->hide();
-   
-   
+
    /* select the scan device, either user or from config, this creates and assembles
     * the complete scanner options dialog
     * scan_params must be zero for that */
-   scan_params = 0L;
-   preview_canvas = 0L;
 
    m_dockPreview = parent->createDockWidget( "Preview ",
 					   loader->loadIcon( "viewmag", KIcon::Small ),
@@ -177,25 +206,43 @@ KookaView::KookaView( KDockMainWindow *parent, const QCString& deviceToUse)
    preview_canvas = new Previewer( m_dockPreview );
    {
       preview_canvas->setMinimumSize( 100,100);
-   	
+
       /* since the scan_params will be created in slSelectDevice, do the
        * connections later
        */
    }
    m_dockPreview->setWidget( preview_canvas );
-   m_dockPreview->manualDock( m_dockPackager,              // dock target
-			    KDockWidget::DockCenter, // dock site
-			    100 );                  // relation target/this (in percent)
-   
-   
-   
+   m_dockPreview->manualDock( m_mainDock,              // dock target
+			      KDockWidget::DockCenter, // dock site
+			      100 );                  // relation target/this (in percent)
+
+   /* Create a text editor part for ocr results */
+
+   m_dockOCRText = parent->createDockWidget( "OCRResults",
+                                             loader->loadIcon("edit", KIcon::Small ),
+                                             0L, i18n("OCR Result Text"));
+   // m_textEdit
+   m_ocrResEdit  = new ocrResEdit( m_dockOCRText );
+
+   if( m_ocrResEdit )
+   {
+       m_dockOCRText->setWidget( m_ocrResEdit ); // m_textEdit->widget() );
+       m_dockOCRText->manualDock( m_dockThumbs,              // dock target
+                                  KDockWidget::DockCenter, // dock site
+                                  100 );                  // relation target/this (in percent)
+
+       m_ocrResEdit->setTextFormat( Qt::PlainText );
+       m_ocrResEdit->setWordWrap( QTextEdit::NoWrap );
+       // m_dockOCRText->hide();
+   }
+
    if( slSelectDevice(deviceToUse))
    {
       /* Load from config which tab page was selected last time */
    }
 
    /* New image created after scanning */
-   connect(sane, SIGNAL(sigNewImage(QImage*)), packager, SLOT(slAddImage(QImage*)));
+   connect(sane, SIGNAL(sigNewImage(QImage*)), this, SLOT(slNewImageScanned(QImage*)));
    /* New preview image */
    connect(sane, SIGNAL(sigNewPreview(QImage*)), this, SLOT( slNewPreview(QImage*)));
 
@@ -203,35 +250,57 @@ KookaView::KookaView( KDockMainWindow *parent, const QCString& deviceToUse)
    connect( sane, SIGNAL( sigScanFinished(KScanStat)), this, SLOT(slScanFinished(KScanStat)));
    connect( sane, SIGNAL( sigAcquireStart()), this, SLOT( slAcquireStart()));
    /* Image canvas should show a new document */
-   connect( packager, SIGNAL( showImage( QImage* )),
-            img_canvas, SLOT( newImage( QImage*)));
+   connect( packager, SIGNAL( showImage( KookaImage* )),
+            this,       SLOT( slShowAImage( KookaImage*)));
 
    connect( packager, SIGNAL( aboutToShowImage(const KURL&)),
-	    this, SLOT( slStartLoading( const KURL& )));
-   
+	    this,       SLOT( slStartLoading( const KURL& )));
+
    /* Packager unloads the image */
-   connect( packager, SIGNAL( unloadImage( QImage* )),
-            img_canvas, SLOT( deleteView( QImage*)));
+   connect( packager, SIGNAL( unloadImage( KookaImage* )),
+            this,       SLOT( slUnloadAImage( KookaImage*)));
 
    /* a image changed mostly through a image manipulation method like rotate */
-   connect( packager, SIGNAL( fileChanged( KFileItem* )),
+   connect( packager,  SIGNAL( fileChanged( KFileItem* )),
 	    m_thumbview, SLOT( slImageChanged( KFileItem* )));
 
-   connect( packager, SIGNAL( fileDeleted( KFileItem* )),
+   connect( packager, SIGNAL( fileRenamed( KFileItem*, const KURL& )),
+            m_thumbview, SLOT( slImageRenamed( KFileItem*, const KURL& )));
+
+   connect( packager,  SIGNAL( fileDeleted( KFileItem* )),
 	    m_thumbview, SLOT( slImageDeleted( KFileItem* )));
-   
-   m_mainDock->setDockSite( KDockWidget::DockFullSite );
+
 
    packager->openRoots();
+
+   /* Status Bar */
+   KStatusBar *statBar = m_mainWindow->statusBar();
+
+   // statBar->insertItem(QString("1"), SBAR_ZOOM,  0, true );
+   statBar->insertItem( QString("-"), StatusImage,  0, true );
+
+   /* Set a large enough size */
+   int w = statBar->fontMetrics().
+           width(img_canvas->imageInfoString(2000, 2000, 48));
+   kdDebug(28000) << "Fixed size for status bar: " << w << " from string " << img_canvas->imageInfoString(2000, 2000, 48) << endl;
+   statBar->setItemFixed( StatusImage, w );
+
 }
 
 
 KookaView::~KookaView()
 {
    saveProperties( KGlobal::config () );
+   delete preview_canvas;
+
    kdDebug(28000)<< "Finished saving config data" << endl;
 }
 
+void KookaView::slViewerReadOnly( bool )
+{
+    /* retrieve actions that could change the image */
+}
+
 
 bool KookaView::slSelectDevice( const QCString& useDevice )
 {
@@ -251,7 +320,7 @@ bool KookaView::slSelectDevice( const QCString& useDevice )
 	 selDevice = userDeviceSelection();
       }
    }
-   
+
    if( !selDevice.isEmpty() )
    {
       kdDebug(28000) << "Opening device " << selDevice << endl;
@@ -260,18 +329,19 @@ bool KookaView::slSelectDevice( const QCString& useDevice )
 	 kdDebug( 28000) << "Device " << selDevice << " is already selected!" << endl;
 	 return( true );
       }
-      
+
       if( scan_params )
       {
-	 /* This deletes the existing scan_params-object */
+	 /* This deletes the existing scan_params^-object */
 	 slCloseScanDevice();
       }
+
       /* This connects to the selected scanner */
       scan_params = new ScanParams( m_dockScanParam );
       Q_CHECK_PTR(scan_params);
-      m_dockScanParam->setWidget( scan_params );
-      m_dockScanParam->show();
-      
+      // m_dockScanParam->setWidget( scan_params );
+      // m_dockScanParam->show();
+
       if( sane->openDevice( selDevice ) == KSCAN_OK )
       {
          connect( scan_params,    SIGNAL( scanResolutionChanged( int, int )),
@@ -298,6 +368,9 @@ bool KookaView::slSelectDevice( const QCString& useDevice )
 	    if( preview_canvas )
 	    {
 	       preview_canvas->loadPreviewImage( selDevice );
+
+               /* Call this after the devic is actually open */
+               preview_canvas->slConnectScanner( sane );
 	    }
 	 }
       }
@@ -307,7 +380,11 @@ bool KookaView::slSelectDevice( const QCString& useDevice )
 	 scan_params->connectDevice(0);
       }
 
-      /* try to load the size from config */
+      /* show the widget again */
+
+      m_dockScanParam->setWidget( scan_params );
+
+      m_dockScanParam->show();
    }
    else
    {
@@ -326,7 +403,7 @@ QCString KookaView::userDeviceSelection( ) const
    /* a list of backends the scan backend knows */
    QStrList backends = sane->getDevices();
    QStrListIterator  it( backends );
-   
+
    QCString selDevice;
    if( backends.count() > 0 )
    {
@@ -367,7 +444,7 @@ void KookaView::loadStartupImage( void )
 
       if( wantReadOnStart )
       {
-	 QString startup = konf->readEntry( STARTUP_IMG_SELECTION, "" );
+	 QString startup = konf->readPathEntry( STARTUP_IMG_SELECTION );
 
 	 if( !startup.isEmpty() )
 	 {
@@ -383,59 +460,21 @@ void KookaView::loadStartupImage( void )
 }
 
 
-void KookaView::print(QPainter *p, KPrinter* printer, QPaintDeviceMetrics& metrics )
+void KookaView::print()
 {
-   int pheight = metrics.height();
-   int pwidth  = metrics.width();
+    /* For now, print a single file. Later, print multiple images to one page */
+    KookaImage *img = packager->getCurrImage();
+    if ( !img )
+        return;
+    KPrinter printer; // ( true, pMode );
+    printer.setUsePrinterResolution(true);
+    printer.addDialogPage( new ImgPrintDialog( img ));
 
-   /* Check the device */
-   if( pheight == 0 || pwidth == 0 ) return;
-   
-   (void) printer;
-
-   const QImage *img = img_canvas->rootImage();
-
-   kdDebug(28000) << "Printing canvas size: "<< pwidth << "x" << pheight << endl;
-   if( img )
-   {
-      int w = img->width();
-      int h = img->height();
-      bool scaled = false;
-      double ratio = 1.0;
-      
-      kdDebug(28000) << "printing image size " << w << " x " << h << endl;
-      if( w > pwidth || h > pheight )
-      {
-	 /* scaling is neccessary */
-	 double wratio = double(w) / double(pwidth);
-	 double hratio = double(h) / double(pheight);
-
-	 /* take the larger one */
-	 ratio = wratio > hratio ? wratio : hratio;
-	 if( ratio > 0.0 )
-	 {
-	    w = (int) (double(w) / ratio);
-	    h = (int) (double(h) / ratio);
-	 }
-	 scaled = true;
-	 kdDebug(28000) << "image ratio: " << ratio << " leads to new height " << h << endl;
-      }
-
-      QPoint poin ( 1+int(( pwidth-w)/2), 1+int((pheight-h)/2) );
-      kdDebug(28000) << "Printing on point " << poin.x() << "/" << poin.y() << endl;
-      if( scaled )
-      {
-         // Phys. image is larger than print area, needs  to scale
-         p->drawImage( poin, img->smoothScale( w, h ));
-	 kdDebug(28000) <<  "Needed to scale new size is " << w << "x" << h << endl;
-      }
-      else
-      {
-         // Phys. image fits on the page
-         p->drawImage( poin, *img );
-	 kdDebug(28000) <<  "Needed to scale new size is " << w << "x" << h << endl;
-      }
-   }
+    if( printer.setup( m_mainWindow, i18n("Print %1").arg(img->localFileName().section('/', -1)) ))
+    {
+	KookaPrint kookaprint( &printer );
+	kookaprint.printImage(img);
+    }
 }
 
 void KookaView::slNewPreview( QImage *new_img )
@@ -459,7 +498,7 @@ void KookaView::slNewPreview( QImage *new_img )
 	    m_dockPreview->makeDockVisible();
 	 }
       }
-      preview_canvas->newImage( new_img ); 
+      preview_canvas->newImage( new_img );
    }
 }
 
@@ -502,7 +541,7 @@ void KookaView::doOCRonSelection( void )
 {
    emit( signalChangeStatusbar( i18n("Starting OCR on selection" )));
 
-   QImage img;
+   KookaImage img;
 
    if( img_canvas->selectedImage(&img) )
    {
@@ -515,22 +554,50 @@ void KookaView::doOCRonSelection( void )
 void KookaView::doOCR( void )
 {
    emit( signalChangeStatusbar( i18n("Starting OCR on the entire image" )));
-   const QImage *img = img_canvas->rootImage();
+    KookaImage *img = packager->getCurrImage();
    startOCR( img );
    emit( signalCleanStatusbar( ));
 }
 
-void KookaView::startOCR( const QImage *img )
+void KookaView::startOCR( KookaImage *img )
 {
    if( img && ! img->isNull() )
    {
       if( ocrFabric == 0L )
-	 ocrFabric = new KSANEOCR( m_mainDock );
+      {
+          ocrFabric = new KSANEOCR( m_mainDock, KGlobal::config() );
+          ocrFabric->setImageCanvas( img_canvas );
+
+          connect( ocrFabric, SIGNAL( newOCRResultText( const QString& )),
+                   m_ocrResEdit, SLOT(setText( const QString& )));
+
+          connect( ocrFabric, SIGNAL( repaintOCRResImage( )),
+                   img_canvas, SLOT(repaint()));
+
+	  connect( ocrFabric, SIGNAL( clearOCRResultText()),
+		   m_ocrResEdit, SLOT(clear()));
+
+          connect( ocrFabric,    SIGNAL( updateWord(int, const QString&, const QString& )),
+                   m_ocrResEdit, SLOT( slUpdateOCRResult( int, const QString&, const QString& )));
+
+          connect( ocrFabric,    SIGNAL( ignoreWord(int, const ocrWord&)),
+                   m_ocrResEdit, SLOT( slIgnoreWrongWord( int, const ocrWord& )));
+
+          connect( ocrFabric, SIGNAL( markWordWrong(int, const ocrWord& )),
+                   m_ocrResEdit, SLOT( slMarkWordWrong( int, const ocrWord& )));
+
+          connect( ocrFabric,    SIGNAL( readOnlyEditor( bool )),
+                   m_ocrResEdit, SLOT( setReadOnly( bool )));
+
+          connect( ocrFabric,    SIGNAL( selectWord( int, const ocrWord& )),
+                   m_ocrResEdit, SLOT( slSelectWord( int, const ocrWord& )));
+
+      }
 
       Q_CHECK_PTR( ocrFabric );
-      ocrFabric->setImage( img );
+      ocrFabric->slSetImage( img );
 
-      if( !ocrFabric->startExternOcrVisible() )
+      if( !ocrFabric->startOCRVisible(m_mainDock) )
       {
 	 KMessageBox::sorry(0, i18n("Could not start OCR-Process.\n"
 				    "Probably there is already one running." ));
@@ -539,6 +606,24 @@ void KookaView::startOCR( const QImage *img )
    }
 }
 
+
+void KookaView::slOCRResultImage( const QPixmap& pix )
+{
+    kdDebug(28000) << "Showing OCR Result Image" << endl;
+    if( ! img_canvas ) return;
+
+    if( m_ocrResultImg )
+    {
+        img_canvas->newImage(0L);
+        delete m_ocrResultImg;
+    }
+
+    m_ocrResultImg = new QImage();
+    *m_ocrResultImg = pix;
+    img_canvas->newImage( m_ocrResultImg );
+    img_canvas->setReadOnly(true); // ocr result images should be read only.
+}
+
 void KookaView::slScanStart( )
 {
    kdDebug(28000) << "Scan starts " << endl;
@@ -567,6 +652,18 @@ void KookaView::slAcquireStart( )
    }
 }
 
+void KookaView::slNewImageScanned( QImage* img )
+{
+    KookaImageMeta *meta = new KookaImageMeta;
+
+    KScanOption res ( SANE_NAME_SCAN_RESOLUTION );
+    int resX;
+    res.get(&resX);
+    meta->setScanResolution(resX);
+
+    packager->slAddImage(img, meta);
+}
+
 
 
 void KookaView::slScanFinished( KScanStat stat )
@@ -594,6 +691,7 @@ void KookaView::slCloseScanDevice( )
       m_dockScanParam->setWidget(0L);
       m_dockScanParam->hide();
    }
+
    sane->slCloseDevice();
 }
 
@@ -615,7 +713,8 @@ void KookaView::slCreateNewImgFromSelection()
 
 void KookaView::slRotateImage(int angle)
 {
-   QImage *img = (QImage*) img_canvas->rootImage();
+    // QImage *img = (QImage*) img_canvas->rootImage();
+   KookaImage *img = packager->getCurrImage();
    bool doUpdate = true;
 
    if( img )
@@ -700,13 +799,20 @@ void KookaView::slMirrorImage( MirrorType m )
 }
 
 
+void KookaView::slSaveOCRResult()
+{
+    if( ! m_ocrResEdit ) return;
+    m_ocrResEdit->slSaveText();
+
+}
+
+
 void KookaView::slLoadScanParams( )
 {
    if( ! sane ) return;
-
-   /* not yet cooked */
 #if 0
-   LoadSetDialog loadDialog( m_mainDock, sane->shortScannerName() );
+   /* not yet cooked */
+   LoadSetDialog loadDialog( m_mainDock, sane->shortScannerName(), sane );
    if( loadDialog.exec())
    {
       kdDebug(28000)<< "Executed successfully" << endl;
@@ -720,19 +826,51 @@ void KookaView::slSaveScanParams( )
 
    /* not yet cooked */
 #if 0
-
    KScanOptSet optSet( "SaveSet" );
-   
+
    sane->getCurrentOptions( &optSet );
-   SaveSetDialog dialog( this, &optSet );
+   SaveSetDialog dialog( m_mainDock /* this */ , &optSet );
    if( dialog.exec())
    {
       kdDebug(28000)<< "Executed successfully" << endl;
+      QString name = dialog.paramSetName();
+      QString desc = dialog.paramSetDescription();
+      sane->slSaveScanConfigSet( name, desc );
    }
-   sane->slSaveScanConfigSet( "sysmtem-default", "default configuration" );
 #endif
 }
 
+void KookaView::slShowAImage( KookaImage *img )
+{
+   kdDebug(28000) << "Show new Image" << endl;
+   if( img_canvas )
+   {
+      img_canvas->newImage( img );
+      img_canvas->setReadOnly(false);
+   }
+
+   /* tell ocr about */
+   if( ocrFabric )
+   {
+       ocrFabric->slSetImage( img );
+   }
+
+   /* Status Bar */
+   KStatusBar *statBar = m_mainWindow->statusBar();
+   if( img_canvas )
+       statBar->changeItem( img_canvas->imageInfoString(), StatusImage );
+}
+
+void KookaView::slUnloadAImage( KookaImage * )
+{
+   kdDebug(28000) << "Unloading Image" << endl;
+   if( img_canvas )
+   {
+      img_canvas->newImage( 0L );
+   }
+}
+
+
 void KookaView::slShowThumbnails(KFileTreeViewItem *dirKfi, bool forceRedraw )
 {
    /* If no item is specified, use the current one */
@@ -750,7 +888,7 @@ void KookaView::slShowThumbnails(KFileTreeViewItem *dirKfi, bool forceRedraw )
       }
    }
 
-   kdDebug(28000) << "Showing thumbs for " << dirKfi->url().prettyURL() << endl;	   
+   kdDebug(28000) << "Showing thumbs for " << dirKfi->url().prettyURL() << endl;
 
    /* Only do the new thumbview if the old is on another dir */
    if( m_thumbview && (forceRedraw || m_thumbview->currentDir() != dirKfi->url()) )
@@ -758,7 +896,7 @@ void KookaView::slShowThumbnails(KFileTreeViewItem *dirKfi, bool forceRedraw )
       m_thumbview->clear();
       /* Find a list of child KFileItems */
       if( forceRedraw ) m_thumbview->readSettings();
-      
+
       KFileItemList fileItemsList;
 
       QListViewItem * myChild = dirKfi->firstChild();
@@ -792,9 +930,17 @@ void KookaView::slStartLoading( const KURL& url )
 
 void KookaView::updateCurrImage( QImage& img )
 {
-   emit( signalChangeStatusbar( i18n("Storing image changes" )));
-   packager->slotCurrentImageChanged( &img );
-   emit( signalCleanStatusbar());
+    if( ! img_canvas->readOnly() )
+    {
+	emit( signalChangeStatusbar( i18n("Storing image changes" )));
+	packager->slotCurrentImageChanged( &img );
+	emit( signalCleanStatusbar());
+    }
+    else
+    {
+	emit( signalChangeStatusbar( i18n("Can not save image, it is write protected!")));
+	kdDebug(28000) << "Image is write protected, no saving!" << endl;
+    }
 }
 
 
@@ -803,13 +949,7 @@ void KookaView::saveProperties(KConfig *config)
    kdDebug(28000) << "Saving Properties for KookaView !" << endl;
    config->setGroup( GROUP_STARTUP );
    /* Get with path */
-   config->writeEntry( STARTUP_IMG_SELECTION, packager->getCurrImageFileName(true));
-
-   QString tabwPre = "packager";
-   int idx = PREVIEWER_TAB; // FIXME !!
-   kdDebug(28000) << "Idx ist" << idx << endl;
-   if( idx == PREVIEWER_TAB )
-      tabwPre = "previewer";
+   config->writePathEntry( STARTUP_IMG_SELECTION, packager->getCurrImageFileName(true));
 
 }
 
@@ -817,18 +957,18 @@ void KookaView::saveProperties(KConfig *config)
 void KookaView::slOpenCurrInGraphApp( void )
 {
    QString file;
-   
+
    if( packager )
    {
       KFileTreeViewItem *ftvi = packager->currentKFileTreeViewItem();
 
       if( ! ftvi ) return;
-      
+
       kdDebug(28000) << "Trying to open <" << ftvi->url().prettyURL()<< ">" << endl;
       KURL::List urllist;
-	 
+
       urllist.append( ftvi->url());
-	 
+
       KRun::displayOpenWithDialog( urllist );
    }
 }
@@ -837,26 +977,26 @@ void KookaView::slOpenCurrInGraphApp( void )
 QImage KookaView::rotateLeft( QImage *m_img )
 {
    QImage rot;
-   
+
    if( m_img )
    {
       if ( m_img->depth() == 8)
       {
-	 rot.create( m_img->height(), m_img->width(), 
+	 rot.create( m_img->height(), m_img->width(),
 		     m_img->depth(), m_img->numColors() );
 	 for (int i=0; i<m_img->numColors(); i++)
 	 {
 	    rot.setColor( i, m_img->color(i) );
 	 }
-    
+
 	 unsigned char **ssl = m_img->jumpTable();
-    
+
 	 for (int y=0; y<m_img->height(); y++)
 	 {
 	    unsigned char *p = *ssl++;
 	    unsigned char **dsl = rot.jumpTable();
 	    dsl += m_img->width()-1;
-            
+
 	    for (int x=0; x<m_img->width(); x++)
 	    {
 	       *((*dsl--) + y ) = *p++;
@@ -865,17 +1005,17 @@ QImage KookaView::rotateLeft( QImage *m_img )
       }
       else
       {
-	 rot.create( m_img->height(), m_img->width(), 
+	 rot.create( m_img->height(), m_img->width(),
 		     m_img->depth() );
-    
+
 	 QRgb **ssl = (QRgb **)m_img->jumpTable();
-    
+
 	 for (int y=0; y<m_img->height(); y++)
 	 {
 	    QRgb *p = *ssl++;
 	    QRgb **dsl = (QRgb **)rot.jumpTable();
 	    dsl += m_img->width()-1;
-      
+
 	    for (int x=0; x<m_img->width(); x++)
 	    {
 	       *((*dsl--) + y ) = *p++;
@@ -894,20 +1034,20 @@ QImage KookaView::rotateRight( QImage *m_img )
    {
       if (m_img->depth() == 8)
       {
-	 rot.create( m_img->height(), m_img->width(), 
+	 rot.create( m_img->height(), m_img->width(),
 		     m_img->depth(), m_img->numColors() );
 	 for (int i=0; i<m_img->numColors(); i++)
 	 {
 	    rot.setColor( i, m_img->color(i) );
 	 }
-    
+
 	 unsigned char **ssl = m_img->jumpTable();
-    
+
 	 for (int y=0; y<m_img->height(); y++)
 	 {
 	    unsigned char *p = *ssl++;
 	    unsigned char **dsl = rot.jumpTable();
-            
+
 	    for (int x=0; x<m_img->width(); x++)
 	    {
 	       *((*dsl++) + m_img->height() - y - 1 ) = *p++;
@@ -916,16 +1056,16 @@ QImage KookaView::rotateRight( QImage *m_img )
       }
       else
       {
-	 rot.create( m_img->height(), m_img->width(), 
+	 rot.create( m_img->height(), m_img->width(),
 		     m_img->depth() );
-    
+
 	 QRgb **ssl = (QRgb **)m_img->jumpTable();
-    
+
 	 for (int y=0; y<m_img->height(); y++)
 	 {
 	    QRgb *p = *ssl++;
 	    QRgb **dsl = (QRgb **)rot.jumpTable();
-      
+
 	    for (int x=0; x<m_img->width(); x++)
 	    {
 	       *((*dsl++) + m_img->height() - y - 1 ) = *p++;
@@ -939,7 +1079,7 @@ QImage KookaView::rotateRight( QImage *m_img )
 void KookaView::connectViewerAction( KAction *action )
 {
    QPopupMenu *popup = img_canvas->contextMenu();
-
+   kdDebug(29000) << "This is the popup: " << popup << endl;
    if( popup && action )
    {
       action->plug( popup );
@@ -996,6 +1136,10 @@ void KookaView::createDockMenu( KActionCollection *col, KDockMainWindow *mainWin
    actionMenu->insert( new dwMenuAction( i18n("Show Scan Parameters"),
 					 KShortcut(), m_dockScanParam, col,
 					 mainWin, "dock_scanparams" ));
+
+   actionMenu->insert( new dwMenuAction( i18n("Show OCR Results"),
+					 KShortcut(), m_dockOCRText, col,
+					 mainWin, "dock_ocrResults" ));
 }
 
 
