@@ -3,14 +3,13 @@
 #ifndef AREASELECT_H
 #define AREASELECT_H
 
-#include <qpixmap.h>
 #include "tool.h"
 
 class AreaSelect : public Tool
 {
 Q_OBJECT
 public:
-  AreaSelect(const QString & toolname);
+  AreaSelect();
   const char* pixmap() const;
 
   virtual void mousePressEvent(QMouseEvent *e);
