@@ -1,41 +1,30 @@
-// $Id: brush.cpp,v 1.7 2002/09/25 23:45:28 waba Exp $
+// $Id: brush.cpp,v 1.8 2003/01/12 22:42:19 nhasan Exp $
 
-#include <stdio.h>
-#include <qcursor.h>
 #include <qpainter.h>
-#include <qstring.h>
-#include <qwmatrix.h>
 #include <qclipboard.h>
 
 #include <klocale.h>
 #include <kapplication.h>
 #include <kdebug.h>
 #include <kmessagebox.h>
-#include <kiconloader.h>
 
 #include "brush.h"
 
 /* #################  Paste Tool ############################*/
 
-PasteTool::PasteTool(const QString & toolname) : Tool(toolname)
+PasteTool::PasteTool() : Tool( i18n( "Paste" ) )
 {
   setSpecial( true );
   drawing= FALSE;
   showedSF= false;
-  tooltip= i18n("Paste");
   props= 0;
 }
 
-/*
-PasteTool::~PasteTool()
-{ }
-*/
-
 void PasteTool::activating()
 {
   kdDebug(4400) << "PasteTool::activating() hook called\n" << endl;
-  
-  cPixmap = kapp->clipboard()->pixmap();        
+
+  cPixmap = kapp->clipboard()->pixmap();
   noContent = cPixmap.isNull() ? true : false;
   if ( !noContent ) {
     lastx = -1;
@@ -57,7 +46,7 @@ void PasteTool::deactivating()
 void PasteTool::mousePressEvent(QMouseEvent *e)
 {
   kdDebug(4400) << "PasteTool::mousePressEvent() handler called\n" << endl;
-    
+
   if (isActive() && (e->button() == LeftButton) && !noContent) {
     if ( drawing )
       kdWarning(4400) << "PasteTool: Warning Left Button press received when pressed\n" << endl;
@@ -73,11 +62,11 @@ void PasteTool::mousePressEvent(QMouseEvent *e)
       lastx = e->pos().x();
       lasty = e->pos().y();
       drawing= true;
-        
+
       if ( !cPixmap.isNull() ) {
 	paint.setRasterOp( Qt::CopyROP );
 	paint.drawPixmap( lastx, lasty, cPixmap );
-      }    
+      }
       paint.end();
       canvas->repaint(0);
     }
@@ -87,14 +76,12 @@ void PasteTool::mousePressEvent(QMouseEvent *e)
   }
 }
 
-
-
 void PasteTool::mouseMoveEvent(QMouseEvent *e)
 {
   if ( isActive() && !noContent ) {
     int x = e->pos().x();
     int y = e->pos().y();
-    
+
     if ((lastx != x) || (lasty != y)) {
       QPainter paint;
       paint.begin( canvas->zoomedPixmap() );
@@ -109,8 +96,8 @@ void PasteTool::mouseMoveEvent(QMouseEvent *e)
 	  paint.drawPixmap( lastx, lasty, cPixmap );
 	}
 	paint.setRasterOp( Qt::NotROP );
-	paint.drawPixmap( x, y, cPixmap );                     
-      }    
+	paint.drawPixmap( x, y, cPixmap );
+      }
 
       paint.end();
       lastx = x;
