@@ -3,16 +3,13 @@
 #ifndef BRUSH_H
 #define BRUSH_H
 
-#include <qpixmap.h>
 #include "tool.h"
 
-
 class PasteTool : public Tool
 {
   Q_OBJECT
 public:
-  PasteTool(const QString & toolname);
-  ~PasteTool() {}
+  PasteTool();
 
   void mousePressEvent(QMouseEvent *e);
   void mouseMoveEvent(QMouseEvent *e);
