@@ -1,6 +1,6 @@
 // -*- c++ -*-
 
-// $Id: canvas.h,v 1.15 2002/09/25 23:45:28 waba Exp $
+// $Id: canvas.h,v 1.19 2003/05/19 17:44:07 mlaurent Exp $
 
 #ifndef CANVAS_H
 #define CANVAS_H
@@ -24,14 +24,14 @@ public:
   Canvas(const QString & filename, QWidget *parent= 0, const char *name=0);
   ~Canvas();
 
-  QPixmap *pixmap();
+  QPixmap *pixmap() const;
   void setPixmap(QPixmap *);
   void setDepth(int);
-  int getDepth() const { return pix->depth(); }
+  int depth() const { return m_pix->depth(); }
 
   const QRect &selection() const;
   void setSelection(const QRect&);
-  bool hasSelection() const { return haveSelection_; }
+  bool hasSelection() const { return m_haveSelection; }
   void clearSelection();
 
   /** copies from selection into clipboard and then cuts selection */
@@ -48,8 +48,10 @@ public:
 
   //static inline bool hasClipboardData();
   //static inline const QPixmap *clipboardData();
+  void rotateImage(int degrees);
+  void mirrorImage();
 
-  QPixmap *zoomedPixmap();
+  QPixmap *zoomedPixmap() const;
   const QWMatrix &zoomMatrix() const;
   const QWMatrix &zoomLineMatrix() const;
   void setZoom(int);
@@ -91,6 +93,7 @@ signals:
   void selection(bool);
   void clipboard(bool);
   void startDrawing();
+  void imageChanged();
 
 protected:
   enum state {
@@ -98,33 +101,33 @@ protected:
     INACTIVE
   };
 
-  QWMatrix matrix;
-  QWMatrix rev_matrix;
-  QWMatrix rev_line_matrix;
-  Tool *currentTool;
+  QWMatrix m_matrix;
+  QWMatrix m_rev_matrix;
+  QWMatrix m_rev_line_matrix;
+  Tool *m_currentTool;
 
   /** Zoomed copy */
-  QPixmap *zoomed;
+  QPixmap *m_zoomed;
 
   /** Master copy */
-  QPixmap *pix;
+  QPixmap *m_pix;
 
   /** Clipboard Data from Cut/Copy */
-  QPixmap clipboardPix;
+  QPixmap m_clipboardPix;
 
   /** % of original size */
-  int zoomFactor;
+  int m_zoomFactor;
 
   /** Has the pixmap been modified? */
-  bool modified_;
-  state s;
-  QRect selection_;
-  QRect clipboardRect_; // jwc
-  bool haveSelection_;
+  bool m_modified;
+  state m_state;
+  QRect m_selection;
+  QRect m_clipboardRect; // jwc
+  bool m_haveSelection;
 
   static int inst; // how many instances have we ?
-  
-  KUndo undoStack;
+
+  KUndo m_undoStack;
 };
 
 #endif // CANVAS_H
