@@ -1,28 +1,16 @@
-// $Id: circle.cpp,v 1.28 2002/09/25 23:45:28 waba Exp $
+// $Id: circle.cpp,v 1.29 2003/01/12 22:42:19 nhasan Exp $
 
 #include <kdebug.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <qcursor.h>
-#include <qstring.h>
 #include <qpainter.h>
-#include <qwmatrix.h>
 #include <klocale.h>
-#include "math.h"
+
+#include <math.h>
+
 #include "circle.h"
-#include <kiconloader.h>
 
-Circle::Circle() : Tool()
+Circle::Circle() : Tool( i18n( "Circle" ) )
 {
   drawing= FALSE;
-  tooltip= i18n("Circle");
-  props= Tool::HasLineProperties | Tool::HasFillProperties;
-}
-
-Circle::Circle(const QString & toolname) : Tool(toolname)
-{
-  drawing= FALSE;
-  tooltip= i18n("Circle");
   props= Tool::HasLineProperties | Tool::HasFillProperties;
 }
 
