@@ -1,11 +1,9 @@
 // -*- c++ -*-
-// $Id: ellipse.h,v 1.9 2002/09/02 16:56:27 winkvist Exp $
+// $Id: ellipse.h,v 1.10 2003/01/12 22:42:19 nhasan Exp $
 
 #ifndef ELLIPSE_H
 #define ELLIPSE_H
 
-#include <qpixmap.h>
-#include <qpainter.h>
 #include "tool.h"
 
 class Ellipse : public Tool
@@ -13,7 +11,6 @@ class Ellipse : public Tool
   Q_OBJECT
 public:
   Ellipse();
-  Ellipse(const QString & toolname);
   void mousePressEvent(QMouseEvent *e);
   void mouseMoveEvent(QMouseEvent *e);
   void mouseReleaseEvent(QMouseEvent *e);
