@@ -1,11 +1,9 @@
-// $Id: infodialog.cpp,v 1.16 2002/07/03 11:42:54 binner Exp $
+// $Id: infodialog.cpp,v 1.17 2003/01/12 05:03:20 nhasan Exp $
 
 #include <qgrid.h>
 #include <qimage.h>
 #include <qlabel.h>
 #include <qlayout.h>
-//#include <qlineedit.h>
-#include <qpushbutton.h>
 
 #include <kapplication.h>
 #include <kdebug.h>
@@ -17,20 +15,21 @@
 #include "infodialog.h"
 
 imageInfoDialog::imageInfoDialog(Canvas *c, QWidget* parent, const char* name)
-  : KDialog(parent, name, TRUE)
+  : KDialogBase(parent, name, true, i18n( "Information" ), Close )
 {
-  setCaption(i18n("Information"));
-  QVBoxLayout *mainLayout = new QVBoxLayout( this, marginHint(), spacingHint() );
+  QWidget *w = makeMainWidget();
+
+  QVBoxLayout *mainLayout = new QVBoxLayout( w );
 
   //
   // Title block
   //
-  QLabel *titleLabel = new QLabel( i18n( "Information" ), 
-				   this, "titleLabel" );
+  QLabel *titleLabel = new QLabel( i18n( "Image Information" ), 
+    w, "titleLabel" );
   QFont titleFont = titleLabel->font();
   titleFont.setBold( true );
   titleLabel->setFont( titleFont );
-  KSeparator *titleSep = new KSeparator( this );
+  KSeparator *titleSep = new KSeparator( w );
   titleSep->setFrameStyle( QFrame::HLine|QFrame::Plain );
   mainLayout->addWidget( titleLabel );
   mainLayout->addWidget( titleSep );
@@ -38,7 +37,7 @@ imageInfoDialog::imageInfoDialog(Canvas *c, QWidget* parent, const char* name)
   //
   // Content
   //
-  QGrid *info = new QGrid( 2, this );
+  QGrid *info = new QGrid( 2, w );
   info->setSpacing( spacingHint() );
   mainLayout->addWidget( info );
 
@@ -66,11 +65,6 @@ imageInfoDialog::imageInfoDialog(Canvas *c, QWidget* parent, const char* name)
   coloursUsed->setText( colUseStr );
   coloursUsed->setAlignment( AlignRight );
 
-  //  tmpQLabel = new QLabel( this, "Label_8" );
-  //  tmpQLabel->setText( i18n("Transparent color:") );
-  //  QLineEdit* tmpQLineEdit;
-  //  tmpQLineEdit = new QLineEdit( this, "LineEdit_1" );
-
   //
   // Image size
   //
@@ -94,20 +88,15 @@ imageInfoDialog::imageInfoDialog(Canvas *c, QWidget* parent, const char* name)
   height->setText( heightStr );
   height->setAlignment( AlignRight );
 
+  mainLayout->addStretch();
+
   //
   // Buttons
   //
-  KSeparator *btnSep = new KSeparator( this );
+  KSeparator *btnSep = new KSeparator( w );
   btnSep->setFrameStyle( QFrame::HLine|QFrame::Plain );
   mainLayout->addWidget( btnSep );
 
-  QBoxLayout *btnLayout = new QBoxLayout( mainLayout, QBoxLayout::RightToLeft );
-  QPushButton *okButton= new QPushButton( i18n("Dismiss"), this );
-  okButton->setFixedHeight( okButton->sizeHint().height() );
-  connect( okButton, SIGNAL(clicked()), SLOT(accept()) );
-  btnLayout->addWidget( okButton );
-  btnLayout->addStretch( 100 );
-
   resize( 200, 160 );
 }
 
