@@ -1,17 +1,16 @@
 // -*- c++ -*-
 
-// $Id: infodialog.h,v 1.5 2001/07/25 15:35:16 rich Exp $
+// $Id: infodialog.h,v 1.6 2003/01/12 05:03:20 nhasan Exp $
 
 #ifndef IMAGEINFODIALOG_H
 #define IMAGEINFODIALOG_H
 
-#include <qcombobox.h>
-#include <qlabel.h>
-#include <kdialog.h>
+#include <kdialogbase.h>
 
 class Canvas;
+class QLabel;
 
-class imageInfoDialog : public KDialog
+class imageInfoDialog : public KDialogBase
 {
     Q_OBJECT
 
