@@ -1,13 +1,12 @@
-// $Id: kpaint.cpp,v 1.105 2002/09/25 23:45:28 waba Exp $
+// $Id: kpaint.cpp,v 1.118.2.1 2004/02/11 19:29:41 coolo Exp $
 
 #include <qclipboard.h>
 #include <qlayout.h>
-#include <qregexp.h>
-#include <qwidget.h>
 #include <qsignalmapper.h>
+#include <qscrollview.h>
+
 #include <config.h>
 
-#include <kaccel.h>
 #include <kaction.h>
 #include <kapplication.h>
 #include <kcolorbutton.h>
@@ -17,17 +16,19 @@
 #include <kdebug.h>
 #include <kedittoolbar.h>
 #include <kfiledialog.h>
-#include <kiconloader.h>
 #include <kimageio.h>
+#include <kinputdialog.h>
 #include <kio/netaccess.h>
 #include <klocale.h>
 #include <kkeydialog.h>
 #include <kmessagebox.h>
 #include <kmainwindow.h>
-#include <kstdaccel.h>
 #include <kstdaction.h>
 #include <ktoolbar.h>
 #include <kurl.h>
+#include <ktempfile.h>
+#include <kstatusbar.h>
+#include <kstdguiitem.h>
 
 #include "canvas.h"
 #include "version.h"
@@ -60,11 +61,12 @@ KPaint::KPaint(const KURL & _url) : KMainWindow(0)
     initStatus();
     readOptions();
 
+    enableEditPaste();
     connect( kapp->clipboard(), SIGNAL( dataChanged() ), SLOT( enableEditPaste() ) );
     resize(640,480);
     show();
     m_canvas->clearModified();
-    
+
     if ( !_url.isEmpty() )
         loadFile(_url);
     else
@@ -91,8 +93,6 @@ void KPaint::setPixmap(QPixmap *p)
 
 bool KPaint::queryClose()
 {
-    saveMainWindowSettings(kapp->config(), "MainWindowSettings");
-
   if (!m_canvas->isModified())
     return true;
 
@@ -126,14 +126,6 @@ bool KPaint::queryClose()
 
 }
 
-void KPaint::updateControls()
-{
-    if (statusbarAction->isChecked())
-	statusBar()->show();
-    else
-	statusBar()->hide();
-}
-
 void KPaint::updateCommands()
 {
     revertAction->setEnabled(m_canvas->isModified());
@@ -143,9 +135,7 @@ void KPaint::updateCommands()
 
 void KPaint::readOptions()
 {
-    applyMainWindowSettings(kapp->config(), "MainWindowSettings");
-
-    statusbarAction->setChecked(!statusBar()->isHidden());
+    setAutoSaveSettings("MainWindowSettings");
 }
 
 void KPaint::enableEditPaste()
@@ -166,8 +156,9 @@ void KPaint::enableEditPaste()
 
 void KPaint::canvasSizeChanged()
 {
-    QString size("%1 x %2");
-    statusbar->changeItem(size.arg( m_canvas->width() ).arg( m_canvas->height() ), ID_FILE_SIZE);
+    QString size(i18n( "%1 x %2" ));
+    statusbar->changeItem(size.arg( m_canvas->width() )
+        .arg( m_canvas->height() ), ID_FILE_SIZE);
     m_mainView->updateLayout();
 }
 
@@ -178,10 +169,11 @@ void KPaint::initStatus()
 
     /* write the image size */
     QString size( "%1 x %2" );
-    statusbar->insertItem( size.arg( m_canvas->width() ).arg( m_canvas->height() ), ID_FILE_SIZE );
+    statusbar->insertItem( size.arg( m_canvas->width() )
+        .arg( m_canvas->height() ), ID_FILE_SIZE );
 
     /* write the color depth */
-    size.sprintf(" %d bpp", m_canvas->getDepth());
+    size = i18n( " %1 bpp" ).arg(m_canvas->depth());
     statusbar->insertItem(size, ID_COLOR_DEPTH);
 
     /* write the zoomfactor */
@@ -189,14 +181,14 @@ void KPaint::initStatus()
 
     /* write the toolname */
     /* get the max. font length of the toolnames */
-    ToolList tl = man->getToolList();
+    ToolList tl = man->toolList();
     QFontMetrics fm = statusbar->fontMetrics();
     Tool *maxt = 0;
     uint maxtlen= 0, tmp;
 
     for ( Tool *t= tl.first(); t; t= tl.next() )
     {
-        tmp = fm.width(t->getName());
+        tmp = fm.width(t->name());
         /* check the tool with the max. name len */
         if ( maxtlen < tmp )
         {
@@ -206,7 +198,7 @@ void KPaint::initStatus()
     }
     /* write the maxlen toolname */
     if (maxt)
-      size = maxt->getName();
+      size = maxt->name();
     else
       size = "XXXX";
     statusbar->insertItem(size, ID_TOOL_NAME);
@@ -215,7 +207,7 @@ void KPaint::initStatus()
     statusbar->insertItem(url.path(), ID_FILE_NAME);
 
     /* update to the current toolname */
-    statusbar->changeItem(man->getCurrentTool().getName(), ID_TOOL_NAME);
+    statusbar->changeItem(man->currentTool().name(), ID_TOOL_NAME);
     man->setStatusBar(statusbar);
 }
 
@@ -311,7 +303,7 @@ void KPaint::initMenus()
     pasteImageAction = new KAction(i18n("Paste &Image"), KShortcut(), this,
                        SLOT(editPasteImage()), actionCollection(),
                        "edit_paste_image");
-    (void) new KAction(i18n("&Clear"), KShortcut(), this, SLOT(editClear()), actionCollection(), "edit_clear");
+    (void) new KAction(KStdGuiItem::clear().text(), KShortcut(), this, SLOT(editClear()), actionCollection(), "edit_image_clear");
 
     // view menu
     zoomInAction = KStdAction::zoomIn(this, SLOT(editZoomIn()), actionCollection());
@@ -349,6 +341,25 @@ void KPaint::initMenus()
     (void) new KAction(i18n("&Resize..."), KShortcut(), this,
                        SLOT(imageResize()), actionCollection(),
                        "image_resize");
+
+	(void) new KAction(i18n("&Mirror"), KShortcut(), this,
+                       SLOT(imageMirror()), actionCollection(),
+                       "image_mirror");
+
+    (void) new KAction(i18n("Rotate &90°"), 0, this,
+        SLOT(imageRotate90()), actionCollection(),
+        "image_rotate90");
+    (void) new KAction(i18n("Rotate &180°"), 0, this,
+        SLOT(imageRotate180()), actionCollection(),
+        "image_rotate180");
+    (void) new KAction(i18n("Rotate &270°"), 0, this,
+        SLOT(imageRotate270()), actionCollection(),
+        "image_rotate270");
+    (void) new KAction(i18n("Free Rotate"), 0, this,
+        SLOT(imageRotateFree()), actionCollection(),
+        "image_rotatefree");
+
+
     paletteAction = new KAction(i18n("&Edit Palette..."), KShortcut(), this,
                        SLOT(imageEditPalette()), actionCollection(),
                        "image_palette");
@@ -357,20 +368,11 @@ void KPaint::initMenus()
                        "image_color");
 
     // settings menu
-    statusbarAction = KStdAction::showStatusbar(this, SLOT(updateControls()), actionCollection());
+    createStandardStatusBarAction();
+    setStandardToolBarMenuEnabled(true);
 
     KStdAction::keyBindings(this, SLOT(editKeyBindings()), actionCollection());
 
-#if 0
-    // window menu
-    (void) new KAction(i18n("&New Window"), KShortcut(), this,
-                       SLOT(newWindow()), actionCollection(),
-                       "window_new");
-    (void) new KAction(i18n("&Close Window"), KShortcut(), this,
-                       SLOT(closeWindow()), actionCollection(),
-                       "window_close");
-#endif
-
     // tool menu
     (void) new KAction(i18n("&Tool Properties"), "configure", KShortcut(), this,
                    SLOT(toolProperties()), actionCollection(),
@@ -381,13 +383,13 @@ void KPaint::initMenus()
 	     this, SLOT( setTool(int) ) );
 
     int id = 0;
-    ToolList tl = man->getToolList();
+    ToolList tl = man->toolList();
     for ( Tool *t= tl.first(); t; t= tl.next() )
     {
       if ( !t->isSpecial() ) {
-	KToggleAction *action = new KToggleAction(t->getName(), t->pixmap(), KShortcut(), 0, 0,
-						  actionCollection(),
-						  QCString().sprintf("tool_%d", id));
+	KToggleAction *action = new KToggleAction(t->name(), t->pixmap(),
+      KShortcut(), 0, 0, actionCollection(),
+      QCString().sprintf("tool_%d", id));
 	action->setExclusiveGroup( "tools" );
 
 	mapper->setMapping( action, id++ );
@@ -400,12 +402,21 @@ void KPaint::initMenus()
     createGUI();
 }
 
-
 void KPaint::editToolbars()
 {
+    saveMainWindowSettings(KGlobal::config(), autoSaveGroup());
     KEditToolbar dlg(actionCollection());
-    if (dlg.exec())
-	createGUI();
+    connect(&dlg, SIGNAL(newToolbarConfig()), SLOT(newToolbarConfig()));
+    dlg.exec();
+}
+
+void KPaint::newToolbarConfig()
+{
+    // OK/Apply pressed in the toolbar editor
+    createGUI();
+    // Our sidebar isn't XMLGUI-compatible. Recreate it...
+    initSideBar();
+    applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
 }
 
 void KPaint::editKeyBindings()
@@ -435,7 +446,7 @@ bool KPaint::fileOpen()
     if (url.isEmpty())
         return false;
 
-    return loadFile(url);
+	return loadFile(url);
 }
 
 
@@ -456,7 +467,7 @@ bool KPaint::loadFile(const KURL& _url)
        newTmpFile->setAutoDelete(true);
        newTmpFile->close();
        QString tmpname = newTmpFile->name();
-       if (KIO::NetAccess::download(_url, tmpname))
+       if (KIO::NetAccess::download(_url, tmpname, 0))
        {
           result =  m_canvas->load(tmpname);
        }
@@ -481,6 +492,8 @@ bool KPaint::loadFile(const KURL& _url)
     url = _url;
     format = KImageIO::type(url.fileName());
     statusbar->changeItem(url.prettyURL(), ID_FILE_NAME);
+    statusbar->changeItem( i18n( "%1 bpp" ).arg( m_canvas->depth() ),
+        ID_COLOR_DEPTH );
     orig_width = m_canvas->width();
     orig_height = m_canvas->height();
     canvasSizeChanged();
@@ -521,7 +534,7 @@ bool KPaint::saveFile(QString filename_, QString format_, KURL &url_)
 
     if (!url_.isLocalFile())
     {
-        if (!KIO::NetAccess::upload(filename_, url_))
+        if (!KIO::NetAccess::upload(filename_, url_, 0))
 	    return false;
     }
     filename = filename_;
@@ -744,7 +757,7 @@ void KPaint::editZoomIn()
 void KPaint::viewZoom( const QString &_strZoom )
 {
     QString z( _strZoom );
-    z = z.replace( QRegExp( "%" ), "" );
+    z = z.replace( "%", "" );
     z = z.simplifyWhiteSpace();
     int new_zoom = z.toInt();
 
@@ -851,8 +864,6 @@ void KPaint::imageEditPalette()
 
 void KPaint::imageChangeDepth()
 {
-    QString depthstr;
-    KStatusBar *sb = statusBar();
     depthDialog d(m_canvas);
 
     kdDebug(4400) << "imageChangeDepth()" << endl;
@@ -864,56 +875,18 @@ void KPaint::imageChangeDepth()
 
     case ID_COLOR_1:
       kdDebug(4400) << "setDepth to 1" << endl;
-      depthstr.sprintf(" %d bpp", 1);
-      sb->changeItem(depthstr, ID_COLOR_DEPTH);
       allowEditPalette= false;
       m_canvas->setDepth(1);
       break;
 
-    case ID_COLOR_4:
-      kdDebug(4400) << "setDepth to 4" << endl;
-      depthstr.sprintf(" %d bpp", 4);
-      sb->changeItem(depthstr, ID_COLOR_DEPTH);
-      allowEditPalette= false;
-      m_canvas->setDepth(4);
-      break;
-
     case ID_COLOR_8:
       kdDebug(4400) << "setDepth to 8" << endl;
-      depthstr.sprintf(" %d bpp", 8);
-      sb->changeItem(depthstr, ID_COLOR_DEPTH);
       allowEditPalette= true;
       m_canvas->setDepth(8);
       break;
 
-    case ID_COLOR_15:
-      kdDebug(4400) << "setDepth to 15" << endl;
-      depthstr.sprintf(" %d bpp", 15);
-      sb->changeItem(depthstr, ID_COLOR_DEPTH);
-      allowEditPalette= false;
-      m_canvas->setDepth(15);
-      break;
-
-    case ID_COLOR_16:
-      kdDebug(4400) << "setDepth to 16" << endl;
-      depthstr.sprintf(" %d bpp", 16);
-      sb->changeItem(depthstr, ID_COLOR_DEPTH);
-      allowEditPalette= false;
-      m_canvas->setDepth(16);
-      break;
-
-    case ID_COLOR_24:
-      kdDebug(4400) << "setDepth to 24" << endl;
-      depthstr.sprintf(" %d bpp", 24);
-      sb->changeItem(depthstr, ID_COLOR_DEPTH);
-      allowEditPalette= false;
-      m_canvas->setDepth(24);
-      break;
-
     case ID_COLOR_32:
       kdDebug(4400) << "setDepth to 32" << endl;
-      depthstr.sprintf("%d bpp", 32);
-      sb->changeItem(depthstr, ID_COLOR_DEPTH);
       allowEditPalette= false;
       m_canvas->setDepth(32);
       break;
@@ -937,4 +910,58 @@ void KPaint::toolProperties()
     man->showPropertiesDialog();
 }
 
+
+void KPaint::imageRotate90()
+{
+  kdDebug(4400) << "imageRotate90()" << endl;
+
+  m_canvas->rotateImage(90);
+  // width will become height,
+  // height will become width,
+  // so size is changed
+  canvasSizeChanged();
+}
+
+void KPaint::imageRotate180()
+{
+  kdDebug(4400) << "imageRotate180()" << endl;
+
+  m_canvas->rotateImage(180);
+}
+
+void KPaint::imageRotate270()
+{
+  kdDebug(4400) << "imageRotate270()" << endl;
+
+  m_canvas->rotateImage(270);
+  // width will become height,
+  // height will become width,
+  // so size is changed
+  canvasSizeChanged();
+}
+
+void KPaint::imageRotateFree()
+{
+  kdDebug(4400) << "imageRotateFree()" << endl;
+
+  bool ok;
+  int degree = KInputDialog::getInteger(
+	  i18n("Free Rotation"), i18n("Specify rotation in degrees:"),
+	   180, 0, 360, 1, &ok, this);
+  if (ok)
+    m_canvas->rotateImage(degree);
+
+  // size is probably changed
+  canvasSizeChanged();
+}
+
+void KPaint::imageMirror()
+{
+  kdDebug(4400) << "imageMirror()" << endl;
+
+  m_canvas->mirrorImage();
+}
+
 #include "kpaint.moc"
+
+
