@@ -3,20 +3,13 @@
 /* modified by jha (j.habenicht@usa.net)
  */
 
-// $Id: kpaint.h,v 1.40 2002/09/02 16:56:27 winkvist Exp $
+// $Id: kpaint.h,v 1.44 2003/05/01 21:48:28 danimo Exp $
 
 #ifndef KPAINT_H
 #define KPAINT_H
 
-#include <qwidget.h>
-#include <qscrollview.h>
 #include <kmainwindow.h>
-#include <ktoolbar.h>
-#include <kmenubar.h>
-#include <kstatusbar.h>
-#include <ktempfile.h>
 
-#include <qrect.h>
 #include "canvas.h"
 #include "manager.h"
 #include "commands.h"
@@ -33,6 +26,8 @@ class KSelectAction;
 class NavWidget;
 class KColorButton;
 class KToolbar;
+class KTempFile;
+class QScrollView;
 
 class KPaint : public KMainWindow
 {
@@ -76,6 +71,11 @@ public slots:
     void imageEditPalette();
     void imageChangeDepth();
 
+    void imageRotate90();
+    void imageRotate180();
+    void imageRotate270();
+    void imageRotateFree();
+    void imageMirror();
     // Tool
     void toolProperties();
     void setTool(int);
@@ -83,17 +83,19 @@ public slots:
     /**
     * Update toolbars/status bars etc.
     */
-    void updateControls();
     void updateCommands();
 
+protected slots:
+    void newToolbarConfig();
+
+    void enableEditPaste();
+
+protected:
     /**
     * Read the options
     */
     void readOptions();
 
-    void enableEditPaste();
-
-protected:
     // Load/Save files
     bool loadFile(const KURL& url);
     bool saveFile(QString filename_, QString format_, KURL &url_);
@@ -121,7 +123,6 @@ private:
     KAction *zoomOutAction;
     KAction *zoomNormalAction;
 
-    KToggleAction *statusbarAction;
     KSelectAction *zoomAction;
 
     /**
@@ -129,7 +130,7 @@ private:
      */
     void canvasSizeChanged();
 
-    
+
     // Info about the currently open image
     QString filename;          // actual local filename
     KURL url;                  // URL it came from
@@ -146,7 +147,7 @@ private:
 
     // Navigation (preview) widget
     NavWidget *nav;
-    // the color selection buttons    
+    // the color selection buttons
     KColorButton *lmbColourBtn;
     KColorButton *rmbColourBtn;
 
@@ -157,7 +158,7 @@ private:
     KToolBar *toolsToolbar;
     KToolBar *commandsToolbar;
     KToolBar *sideToolbar;
-  
+
     KStatusBar *statusbar;
     KMenuBar *menu;
     MainView *m_mainView;
