@@ -1,22 +1,18 @@
-// $Id: line.cpp,v 1.28 2002/09/25 23:45:28 waba Exp $
+// $Id: line.cpp,v 1.29 2003/01/12 22:42:19 nhasan Exp $
 
 // Note: When used to draw more than one line segment should use draw poly line
 // functions to avoid glitches at the joints.
 
-#include <kdebug.h>
-#include <stdio.h>
-#include <qcursor.h>
 #include <qpainter.h>
-#include <qstring.h>
-#include <qwmatrix.h>
-#include <klocale.h>
-#include "line.h"
-#include <kiconloader.h>
 
-Line::Line(const QString & toolname) : Tool(toolname)
+#include <kdebug.h>
+#include <klocale.h>
+
+#include "line.h"
+
+Line::Line() : Tool( i18n( "Line" ) )
 {
   drawing= FALSE;
-  tooltip= i18n("Line");
   props= Tool::HasLineProperties;
 }
 
