@@ -3,19 +3,18 @@
 // Line class
 // This class is a subclass of Tool and provides a rubber line drawing tool.
 
-// $Id: line.h,v 1.9 2002/09/02 16:56:27 winkvist Exp $
+// $Id: line.h,v 1.10 2003/01/12 22:42:19 nhasan Exp $
 
 #ifndef LINE_H
 #define LINE_H
 
-#include <qpixmap.h>
 #include "tool.h"
 
 class Line : public Tool
 {
   Q_OBJECT
 public:
-  Line(const QString & toolname);
+  Line();
   void mousePressEvent(QMouseEvent *e);
   void mouseMoveEvent(QMouseEvent *e);
   void mouseReleaseEvent(QMouseEvent *e);
