@@ -41,12 +41,12 @@ void MainView::updateLayout()
     int w = c->width();
     int h = c->height();
     int x, y;
-    if ( width() > w )
-       x = width()/2 - w/2;
+    if ( v->visibleWidth() > w )
+       x = v->visibleWidth()/2 - w/2;
     else
        x = 0;
-    if ( height() > h )
-       y = height()/2 - h/2;
+    if ( v->visibleHeight() > h )
+       y = v->visibleHeight()/2 - h/2;
     else
        y = 0;
     v->moveChild( c, x, y );
