@@ -1,4 +1,4 @@
-// $Id: manager.cpp,v 1.30 2002/02/08 21:16:45 rich Exp $
+// $Id: manager.cpp,v 1.32 2003/01/12 22:42:19 nhasan Exp $
 
 #include <kdebug.h>
 #include <qobject.h>
@@ -34,7 +34,7 @@ Manager::Manager(Canvas *c)
 
     canvas= c;
     createTools();
-    currentToolID= 0;
+    toolID= 0;
     list.first()->activate(c);
 
     lmbCol= red;
@@ -54,63 +54,57 @@ Manager::Manager(Canvas *c)
     connect(this, SIGNAL(modified()), c, SLOT(markModified()));
 }
 
-Manager::~Manager()
-{
-}
-
 void Manager::createTools()
 {
   AreaSelect *as; // handled different because of slot
   PasteTool  *pt;
-  
+
   // Create the Tools
   list.setAutoDelete(TRUE);
-  list.append(new Ellipse( i18n("Ellipse")));
+  list.append(new Ellipse());
   list.last()->id = 0;
-  list.append(new Circle( i18n("Circle")));
+  list.append(new Circle());
   list.last()->id = 1;
-  list.append(new Pen( i18n("Pen")));
+  list.append(new Pen());
   list.last()->id = 2;
-  list.append(new Line( i18n("Line")));
+  list.append(new Line());
   list.last()->id = 3;
-  list.append(new Rectangle( i18n("Rectangle")));
+  list.append(new Rectangle());
   list.last()->id = 4;
-  list.append(new Roundangle( i18n("Round Angle")));
+  list.append(new Roundangle());
   list.last()->id = 5;
-  list.append(new SprayCan( i18n("Spray Can")));
+  list.append(new SprayCan());
   list.last()->id = 6;
-  list.append(new Text( i18n("Text")));
+  list.append(new Text());
   list.last()->id = 7;
-  
-  as = new AreaSelect( i18n("Area Select"));
-  connect(canvas, SIGNAL(selection(bool)),
-	  as, SLOT(selection(bool)));
+
+  as = new AreaSelect();
+  connect(canvas, SIGNAL(selection(bool)), as, SLOT(selection(bool)));
   list.append(as);
   as->id = 8;
 
-  pt = new PasteTool( i18n("Paste"));
+  pt = new PasteTool();
   list.append(pt);
   as->id = 9;
 
   // Create the properties dialog
-  props= new propertiesDialog(1, 0, "tool_properties");
-  connect(props, SIGNAL(applyButtonPressed()),
-	  this, SLOT(updateProperties()) );
+  props= new propertiesDialog(0, "tool_properties");
+  connect(props, SIGNAL(okClicked()), SLOT(updateProperties()));
 }
 
 void Manager::updateProperties()
 {
-  p= props->getPen();
-  b= props->getBrush();
+  p= props->pen();
+  b= props->brush();
 
   p.setColor(lmbCol);
   b.setColor(lmbCol);
-  list.at(currentToolID)->setLeftPen(p);
-  list.at(currentToolID)->setLeftBrush(b);
+  list.at(toolID)->setLeftPen(p);
+  list.at(toolID)->setLeftBrush(b);
   p.setColor(rmbCol);
   b.setColor(rmbCol);
-  list.at(currentToolID)->setRightPen(p);
-  list.at(currentToolID)->setRightBrush(b);
+  list.at(toolID)->setRightPen(p);
+  list.at(toolID)->setRightBrush(b);
 }
 
 void Manager::setLMColorButton(KColorButton *cb)
@@ -141,19 +135,13 @@ void Manager::setRMBcolour(const QColor &c)
 
 void Manager::setCurrentTool(int tool)
 {
-
-    /*
-    kdDebug(4400) << "got setCurrentTool " << tool << "\n" << endl;
-    kdDebug(4400) << " old Tool is " << currentToolID << "\n" << endl;
-    */
-    
     Tool *t;
 
-    if (currentToolID != tool) 
+    if (toolID != tool) 
     {
         canvas->deactivate();
-        list.at(currentToolID)->deactivate();
-        currentToolID= tool;
+        list.at(toolID)->deactivate();
+        toolID= tool;
         t= list.at(tool);
         t->activate(canvas);
 
@@ -166,13 +154,13 @@ void Manager::setCurrentTool(int tool)
         t->setRightPen(p);
         t->setRightBrush(b);
 
-	canvas->updateZoomed();
+        canvas->updateZoomed();
         canvas->activate(t);
-        props->setPages(t->getPages());
+        props->setPages(t->pages());
         props->repaint(0);
 
         if ( statusBar )
-            statusBar->changeItem(t->getName(), ID_TOOL_NAME);
+            statusBar->changeItem(t->name(), ID_TOOL_NAME);
 
         emit toolChanged(tool);
     }
