@@ -4,7 +4,8 @@
 #define NAVWIDGET_H
 
 #include <qwidget.h>
-#include <qpixmap.h>
+
+class QPixmap;
 
 /**
  * A class to provide navigation information about the current view of
