@@ -1,4 +1,5 @@
 #include "palettedialog.h"
+#include "palette.h"
 
 #include <kdebug.h>
 #include <qpushbutton.h>
@@ -8,23 +9,21 @@
 #include <qlayout.h>
 
 paletteDialog::paletteDialog(QPixmap *p, QWidget *parent, const char *name)
-  : QDialog(parent, name, TRUE)
+  : KDialogBase(parent, name, true, i18n( "Edit Palette" ), Ok|Cancel,
+    Ok, true)
 {
     resize(340, 280);
-    QVBoxLayout *box = new QVBoxLayout( this, 10 );
+    QWidget *w = makeMainWidget();
+    QVBoxLayout *box = new QVBoxLayout( w, 0 );
 
-    pal= new paletteWidget(p, this);
+    pal= new paletteWidget(p, w);
     pal->move(10, 10);
     box->addWidget( pal );
 
-    QPushButton *dismiss = new QPushButton( i18n("Dismiss"), this );
-    box->addWidget( dismiss );
-
-    connect( dismiss, SIGNAL(clicked()), SLOT(accept()) );
     connect( pal, SIGNAL(colourSelected(int)), pal, SLOT(editColour(int)) );
 }
 
-QPixmap *paletteDialog::pixmap()
+QPixmap *paletteDialog::pixmap() const
 {
   return pal->pixmap();
 }
