@@ -1,18 +1,20 @@
-// $Id: palettedialog.h,v 1.3 1998/05/07 13:34:50 rich Exp $
+// $Id: palettedialog.h,v 1.5 2003/05/19 17:44:07 mlaurent Exp $
 
 #ifndef PALETTEDIALOG_H
 #define PALETTEDIALOG_H
 
-#include <qdialog.h>
-#include "palette.h"
+#include <kdialogbase.h>
 
-class paletteDialog : public QDialog
+class paletteWidget;
+class QPixmap;
+
+class paletteDialog : public KDialogBase
 {
   Q_OBJECT
 
 public:
   paletteDialog(QPixmap *, QWidget *parent= 0, const char *name= 0);
-  QPixmap *pixmap();
+  QPixmap *pixmap() const;
 private:
   paletteWidget *pal;
 };
