@@ -1,18 +1,16 @@
 // -*- c++ -*-
-// $Id: pen.h,v 1.9 2002/09/02 16:56:27 winkvist Exp $
+// $Id: pen.h,v 1.10 2003/01/12 22:42:19 nhasan Exp $
 
 #ifndef PEN_H
 #define PEN_H
 
-#include <qpixmap.h>
-#include <qpainter.h>
 #include "tool.h"
 
 class Pen : public Tool
 {
   Q_OBJECT
 public:
-  Pen(const QString & toolname);
+  Pen();
   void mousePressEvent(QMouseEvent *e);
   void mouseMoveEvent(QMouseEvent *e);
   void mouseReleaseEvent(QMouseEvent *e);
