@@ -1,14 +1,15 @@
 #ifndef PROPERTIES_H
 #define PROPERTIES_H
 
-#include <qtabdialog.h>
-#include <qpushbutton.h>
 #include <qlabel.h>
 #include <qlineedit.h>
 #include <qcombobox.h>
 #include <qbrush.h>
 #include <qpen.h>
-#include <qpixmap.h>
+
+#include <kdialogbase.h>
+
+class QPixmap;
 
 class fillPropertiesWidget : public QWidget
 {
@@ -16,10 +17,10 @@ class fillPropertiesWidget : public QWidget
 
 public:
   fillPropertiesWidget(QWidget *parent);
-  QBrush &getBrush();
+  QBrush &brush();
 
 public slots:
-    void setFillColour();
+  void setFillColour();
 
 private:
   QComboBox *patternBox;
@@ -37,11 +38,14 @@ private:
 class linePropertiesWidget : public QWidget
 {
   Q_OBJECT
+
 public:
   linePropertiesWidget(QWidget *parent);
-  QPen &getPen();
+  QPen &pen();
+
 public slots:
-    void setLineColour();
+  void setLineColour();
+
 private:
   QComboBox *styleBox;
   QLabel *styleLabel;
@@ -56,14 +60,14 @@ private:
   QPen p;
 };
 
-class propertiesDialog  : public QTabDialog
+class propertiesDialog  : public KDialogBase
 {
   Q_OBJECT
 public:
-  propertiesDialog(int pages, QWidget *parent= 0, const char *name= 0);
+  propertiesDialog(QWidget *parent= 0, const char *name= 0);
 
-  QBrush &getBrush();
-  QPen &getPen();
+  QBrush &brush();
+  QPen &pen();
   void setPages(int);
 
 private:
