@@ -3,14 +3,13 @@
 #ifndef RECTANGLE_H
 #define RECTANGLE_H
 
-#include <qpixmap.h>
 #include "tool.h"
 
 class Rectangle : public Tool
 {
   Q_OBJECT
 public:
-  Rectangle(const QString & toolname);
+  Rectangle();
   void mousePressEvent(QMouseEvent *e);
   void mouseMoveEvent(QMouseEvent *e);
   void mouseReleaseEvent(QMouseEvent *e);
