@@ -1,22 +1,17 @@
-// $Id: roundangle.cpp,v 1.17 2002/09/25 23:45:28 waba Exp $
+// $Id: roundangle.cpp,v 1.18 2003/01/12 22:42:19 nhasan Exp $
 
 #include <kdebug.h>
-#include <stdio.h>
-#include <qcursor.h>
 #include <qpainter.h>
-#include <qstring.h>
-#include <qwmatrix.h>
+
 #include <klocale.h>
+
 #include "roundangle.h"
-#include <kiconloader.h>
 
 #define ROUNDNESS (40)
 
-Roundangle::Roundangle(const QString & toolname) : Tool(toolname)
+Roundangle::Roundangle() : Tool( i18n( "Round Angle" ) )
 {
   drawing= FALSE;
-
-  tooltip= i18n("Roundangle");
   props= Tool::HasLineProperties | Tool::HasFillProperties;
 }
 
