@@ -3,14 +3,13 @@
 #ifndef ROUNDANGLE_H
 #define ROUNDANGLE_H
 
-#include <qpixmap.h>
 #include "tool.h"
 
 class Roundangle : public Tool
 {
   Q_OBJECT
 public:
-  Roundangle(const QString & toolname);
+  Roundangle();
   void mousePressEvent(QMouseEvent *e);
   void mouseMoveEvent(QMouseEvent *e);
   void mouseReleaseEvent(QMouseEvent *e);
