@@ -1,22 +1,19 @@
-// $Id: spraycan.cpp,v 1.27 2002/09/25 23:45:28 waba Exp $
+// $Id: spraycan.cpp,v 1.28 2003/01/12 22:42:19 nhasan Exp $
 
 #include <kdebug.h>
-#include <stdio.h>
 #include <stdlib.h>
-#include <qcursor.h>
-#include <qstring.h>
 #include <qpainter.h>
-#include <qwmatrix.h>
-#include <klocale.h>
-#include "spraycan.h"
-#include <kiconloader.h>
+#include <qtimer.h>
 
-SprayCan::SprayCan(const QString & toolname) : Tool(toolname)
+#include <klocale.h>
+
+#include "spraycan.h"
+
+SprayCan::SprayCan() : Tool( i18n( "Spray Can" ) )
 {
     drawing= FALSE;
     brushsize= 10;
     density= 100;
-    tooltip= i18n("Spray Can");
     props= Tool::HasLineProperties | Tool::HasCustomProperties;
 
     timer= new QTimer(this);
