@@ -1,21 +1,20 @@
 // -*- c++ -*-
 
-// $Id: spraycan.h,v 1.10 2002/09/02 16:56:27 winkvist Exp $
+// $Id: spraycan.h,v 1.11 2003/01/12 22:42:19 nhasan Exp $
 
 #ifndef SPRAYCAN_H
 #define SPRAYCAN_H
 
-#include <qpixmap.h>
-#include <qpainter.h>
-#include <qtimer.h>
 #include "tool.h"
 
+class QTimer;
+
 class SprayCan : public Tool
 {
   Q_OBJECT
 
 public:
-    SprayCan(const QString & toolname);
+    SprayCan();
     ~SprayCan();
     void mousePressEvent(QMouseEvent *e);
     void mouseMoveEvent(QMouseEvent *e);
