@@ -6,7 +6,7 @@
 #include <qpainter.h>
 #include <qkeycode.h>
 
-#include <kiconloader.h>
+#include <klocale.h>
 #include <kdebug.h>
 
 static const int TIMER_INTERVALL = 500;
@@ -14,18 +14,7 @@ static const int CURSOR_WIDTH = 9; // has to be odd number
 static const int CURSOR_HEIGHT = 15;
 
 Text::Text()
-  : Tool(),
-    startx(0), // starty(0),
-    cursorH(CURSOR_HEIGHT),
-    lastx(0), lasty(0),
-    drawing(false), showingC(false),
-    charBuffer(QString::null)
-{
-  lineStack.setAutoDelete(true);  
-}
-
-Text::Text(const QString & toolname)
-  : Tool(toolname),
+  : Tool( i18n( "Text" ) ),
     startx(0), // starty(0),
     cursorH(CURSOR_HEIGHT),
     lastx(0), lasty(0),
