@@ -1,4 +1,4 @@
-// $Id: tool.cpp,v 1.15 2001/07/25 15:35:16 rich Exp $
+// $Id: tool.cpp,v 1.16 2003/01/12 05:03:20 nhasan Exp $
 
 #include <kdebug.h>
 #include <stdio.h>
@@ -13,9 +13,9 @@ Tool::Tool()
 {
 }
 
-Tool::Tool(const QString & toolname)
+Tool::Tool(const QString & name)
   : id( -1 ), active( false ),
-  pix( 0 ), name(toolname), canvas( 0 ),
+  pix( 0 ), toolName(name), canvas( 0 ),
   props( 0 ), special( false )
 {
 }
@@ -36,25 +36,25 @@ void Tool::deactivate()
   canvas= 0;
 }
 
-void Tool::setLeftPen(QPen &p)
+void Tool::setLeftPen(const QPen &p)
 {
   kdDebug(4400) << "Tool::setLeftPen()" << endl;
   leftpen= p;
 }
 
-void Tool::setLeftBrush(QBrush &b)
+void Tool::setLeftBrush(const QBrush &b)
 {
   kdDebug(4400) << "Tool::setLeftBrush()" << endl;
   leftbrush= b;
 }
 
-void Tool::setRightPen(QPen &p)
+void Tool::setRightPen(const QPen &p)
 {
   kdDebug(4400) << "Tool::setRightPen()" << endl;
   rightpen= p;
 }
 
-void Tool::setRightBrush(QBrush &b)
+void Tool::setRightBrush(const QBrush &b)
 {
   kdDebug(4400) << "Tool::setRightBrush()" << endl;
   rightbrush= b;
