@@ -16,23 +16,33 @@
 **************************************************************************/
 
 #include "pmblendmapmodifiers.h"
-#include "pmoutputdevice.h"
 #include "pmxmlhelper.h"
 #include "pmmemento.h"
 #include "pmblendmapmodifiersedit.h"
 #include "pmvector.h"
+#include "pmenumproperty.h"
 
-#include <kdebug.h>
-#include "pmglobals.h"
 #include <klocale.h>
 
+PMDefinePropertyClass( PMBlendMapModifiers, PMProperty );
+PMDefineEnumPropertyClass( PMBlendMapModifiers,
+                           PMBlendMapModifiers::PMWaveFormType,
+                           PMWaveFormProperty );
+
+PMMetaObject* PMBlendMapModifiers::s_pMetaObject = 0;
+PMObject* createBlendMapModifiers( PMPart* part )
+{
+   return new PMBlendMapModifiers( part );
+}
+
 const double frequencyDefault = 1.0;
 const double phaseDefault = 0.0;
 const PMBlendMapModifiers::PMWaveFormType waveFormTypeDefault = PMBlendMapModifiers::RampWave;
 const double waveFormExponentDefault = 1.0;
 
-PMBlendMapModifiers::PMBlendMapModifiers( )
-      : Base( )
+
+PMBlendMapModifiers::PMBlendMapModifiers( PMPart* part )
+      : Base( part )
 {
    m_enableFrequency = false;
    m_frequency = frequencyDefault;
@@ -47,11 +57,53 @@ PMBlendMapModifiers::~PMBlendMapModifiers( )
 {
 }
 
-bool PMBlendMapModifiers::isA( PMObjectType t ) const
+PMMetaObject* PMBlendMapModifiers::metaObject( ) const
 {
-   if( t == PMTBlendMapModifiers )
-      return true;
-   return Base::isA( t );
+   if( !s_pMetaObject )
+   {
+      s_pMetaObject = new PMMetaObject( "BlendMapModifiers", Base::metaObject( ),
+                                        createBlendMapModifiers );
+      s_pMetaObject->addProperty(
+         new PMProperty( "frequencyEnabled", &PMBlendMapModifiers::enableFrequency,
+                         &PMBlendMapModifiers::isFrequencyEnabled ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "frequency", &PMBlendMapModifiers::setFrequency,
+                         &PMBlendMapModifiers::frequency ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "phaseEnabled", &PMBlendMapModifiers::enablePhase,
+                         &PMBlendMapModifiers::isPhaseEnabled ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "phase", &PMBlendMapModifiers::setPhase,
+                         &PMBlendMapModifiers::phase ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "waveFormEnabled", &PMBlendMapModifiers::enableWaveForm,
+                         &PMBlendMapModifiers::isWaveFormEnabled ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "waveFormExponent", &PMBlendMapModifiers::setWaveFormExponent,
+                         &PMBlendMapModifiers::waveFormExponent ) );
+
+      PMWaveFormProperty* p = new PMWaveFormProperty(
+         "waveForm", &PMBlendMapModifiers::setWaveFormType,
+         &PMBlendMapModifiers::waveFormType );
+      p->addEnumValue( QString( "RampWave" ), RampWave );
+      p->addEnumValue( QString( "TriangleWave" ), TriangleWave );
+      p->addEnumValue( QString( "SineWave" ), SineWave );
+      p->addEnumValue( QString( "ScallopWave" ), ScallopWave );
+      p->addEnumValue( QString( "CubicWave" ), CubicWave );
+      p->addEnumValue( QString( "PolyWave" ), PolyWave );
+      s_pMetaObject->addProperty( p );
+   }
+   return s_pMetaObject;
+}
+
+void PMBlendMapModifiers::cleanUp( ) const
+{
+   if( s_pMetaObject )
+   {
+      delete s_pMetaObject;
+      s_pMetaObject = 0;
+   }
+   Base::cleanUp( );
 }
 
 QString PMBlendMapModifiers::description( ) const
@@ -59,49 +111,6 @@ QString PMBlendMapModifiers::description( ) const
    return i18n( "blend map modifiers" );
 }
 
-void PMBlendMapModifiers::serialize( PMOutputDevice& dev ) const
-{
-   QString str;
-
-   if( m_enableFrequency )
-   {
-      str.setNum( m_frequency );
-      dev.writeLine( "frequency " + str );
-   }
-
-   if( m_enablePhase )
-   {
-      str.setNum( m_phase );
-      dev.writeLine( "phase " + str );
-   }
-
-   if( m_enableWaveForm )
-   {
-      switch( m_waveFormType )
-      {
-         case PMBlendMapModifiers::RampWave:
-            dev.writeLine( "ramp_wave" );
-            break;
-         case PMBlendMapModifiers::TriangleWave:
-            dev.writeLine( "triangle_wave" );
-            break;
-         case PMBlendMapModifiers::SineWave:
-            dev.writeLine( "sine_wave" );
-            break;
-         case PMBlendMapModifiers::ScallopWave:
-            dev.writeLine( "scallop_wave" );
-            break;
-         case PMBlendMapModifiers::CubicWave:
-            dev.writeLine( "cubic_wave" );
-            break;
-         case PMBlendMapModifiers::PolyWave:
-            str.setNum( m_waveFormExponent );
-            dev.writeLine( "poly_wave " + str );
-            break;
-      }
-   }
-}
-
 void PMBlendMapModifiers::serialize( QDomElement& e, QDomDocument& ) const
 {
    e.setAttribute( "frequency", m_frequency );
@@ -162,7 +171,7 @@ void PMBlendMapModifiers::setFrequency( double c )
    if( c != m_frequency )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBlendMapModifiers, PMFrequencyID, m_frequency );
+         m_pMemento->addData( s_pMetaObject, PMFrequencyID, m_frequency );
       m_frequency = c;
    }
 }
@@ -172,7 +181,7 @@ void PMBlendMapModifiers::enableFrequency( bool c )
    if( c != m_enableFrequency )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBlendMapModifiers, PMEnableFrequencyID, m_enableFrequency );
+         m_pMemento->addData( s_pMetaObject, PMEnableFrequencyID, m_enableFrequency );
       m_enableFrequency = c;
    }
 }
@@ -182,7 +191,7 @@ void PMBlendMapModifiers::setPhase( double c )
    if( c != m_phase )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBlendMapModifiers, PMPhaseID, m_phase );
+         m_pMemento->addData( s_pMetaObject, PMPhaseID, m_phase );
       m_phase = c;
    }
 }
@@ -192,7 +201,7 @@ void PMBlendMapModifiers::enablePhase( bool c )
    if( c != m_enablePhase )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBlendMapModifiers, PMEnablePhaseID, m_enablePhase );
+         m_pMemento->addData( s_pMetaObject, PMEnablePhaseID, m_enablePhase );
       m_enablePhase = c;
    }
 }
@@ -202,7 +211,7 @@ void PMBlendMapModifiers::enableWaveForm( bool c )
    if( c != m_enableWaveForm )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBlendMapModifiers, PMEnableWaveFormID, m_enableWaveForm );
+         m_pMemento->addData( s_pMetaObject, PMEnableWaveFormID, m_enableWaveForm );
       m_enableWaveForm = c;
    }
 }
@@ -212,7 +221,7 @@ void PMBlendMapModifiers::setWaveFormExponent( double c )
    if( c != m_waveFormExponent )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBlendMapModifiers, PMWaveFormExponentID, m_waveFormExponent );
+         m_pMemento->addData( s_pMetaObject, PMWaveFormExponentID, m_waveFormExponent );
       m_waveFormExponent = c;
    }
 }
@@ -222,7 +231,7 @@ void PMBlendMapModifiers::setWaveFormType( PMWaveFormType c )
    if( c != m_waveFormType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBlendMapModifiers, PMWaveFormTypeID, m_waveFormType );
+         m_pMemento->addData( s_pMetaObject, PMWaveFormTypeID, m_waveFormType );
       m_waveFormType = c;
    }
 }
@@ -240,7 +249,7 @@ void PMBlendMapModifiers::restoreMemento( PMMemento* s )
    for( ; it.current( ); ++it )
    {
       data = it.current( );
-      if( data->objectType( ) == PMTBlendMapModifiers )
+      if( data->objectType( ) == s_pMetaObject )
       {
          switch( data->valueID( ) )
          {
