@@ -21,13 +21,10 @@
 #include "pmpalettevalue.h"
 #include "pmpalettevaluememento.h"
 
-#include "pmoutputdevice.h"
 #include "pmxmlhelper.h"
 #include "pmcompositeobject.h"
 #include "pmmemento.h"
-
-#include <kdebug.h>
-#include "pmglobals.h"
+#include "pmenumproperty.h"
 
 #include <klocale.h>
 
@@ -43,8 +40,22 @@ const PMBumpMap::PMInterpolateType interpolateTypeDefault = PMBumpMap::Interpola
 const bool useIndexDefault = false;
 const double bumpSizeDefault = 0.0;
 
-PMBumpMap::PMBumpMap( )
-      : Base( )
+PMDefinePropertyClass( PMBumpMap, PMProperty );
+PMDefineEnumPropertyClass( PMBumpMap, PMBumpMap::PMBitmapType,
+                           PMBitmapTypeProperty );
+PMDefineEnumPropertyClass( PMBumpMap, PMBumpMap::PMInterpolateType,
+                           PMInterpolateTypeProperty );
+PMDefineEnumPropertyClass( PMBumpMap, PMBumpMap::PMMapType,
+                           PMMapTypeProperty );
+
+PMMetaObject* PMBumpMap::s_pMetaObject = 0;
+PMObject* createNewBumpMap( PMPart* part )
+{
+   return new PMBumpMap( part );
+}
+
+PMBumpMap::PMBumpMap( PMPart* part )
+      : Base( part )
 {
    m_bitmapType = bitmapTypeDefault;
    m_bitmapFile = bitmapFileDefault;
@@ -55,92 +66,22 @@ PMBumpMap::PMBumpMap( )
    m_bumpSize = bumpSizeDefault;
 }
 
+PMBumpMap::PMBumpMap( const PMBumpMap& m )
+      : Base( m )
+{
+   m_bitmapType = m.m_bitmapType;
+   m_bitmapFile = m.m_bitmapFile;
+   m_once = m.m_once;
+   m_mapType = m.m_mapType;
+   m_interpolateType = m.m_interpolateType;
+   m_useIndex = m.m_useIndex;
+   m_bumpSize = m.m_bumpSize;
+}
 
 PMBumpMap::~PMBumpMap( )
 {
 }
 
-void PMBumpMap::serialize( PMOutputDevice& dev ) const
-{
-   QString str1;
-
-   dev.objectBegin( "bump_map" );
-
-   switch( m_bitmapType )
-   {
-      case PMBumpMap::BitmapGif:
-         dev.writeLine( "gif" );
-         break;
-      case PMBumpMap::BitmapTga:
-         dev.writeLine( "tga" );
-         break;
-      case PMBumpMap::BitmapIff:
-         dev.writeLine( "iff" );
-         break;
-      case PMBumpMap::BitmapPpm:
-         dev.writeLine( "ppm" );
-         break;
-      case PMBumpMap::BitmapPgm:
-         dev.writeLine( "pgm" );
-         break;
-      case PMBumpMap::BitmapPng:
-         dev.writeLine( "png" );
-         break;
-      case PMBumpMap::BitmapJpeg:
-         dev.writeLine( "jpeg" );
-         break;
-      case PMBumpMap::BitmapTiff:
-         dev.writeLine( "tiff" );
-         break;
-      case PMBumpMap::BitmapSys:
-         dev.writeLine( "sys" );
-         break;
-   }
-
-   dev.writeLine( "\"" + m_bitmapFile + "\"" );
-
-   if( m_once )
-      dev.writeLine( "once" );
-
-   switch( m_mapType )
-   {
-      case PMBumpMap::MapPlanar:
-         dev.writeLine( "map_type 0" );
-         break;
-      case PMBumpMap::MapSpherical:
-         dev.writeLine( "map_type 1" );
-         break;
-      case PMBumpMap::MapCylindrical:
-         dev.writeLine( "map_type 2" );
-         break;
-      case PMBumpMap::MapToroidal:
-         dev.writeLine( "map_type 5" );
-         break;
-   }
-   switch( m_interpolateType )
-   { 
-      case PMBumpMap::InterpolateBilinear:
-         dev.writeLine( "interpolate 2" );
-	 break;
-      case PMBumpMap::InterpolateNormalized:
-         dev.writeLine( "interpolate 4" );
-	 break;
-      default:
-         break;
-   }
-
-   if( m_useIndex )
-      dev.writeLine( "use_index" );
-
-   if( m_bumpSize )
-   {
-      str1.setNum( m_bumpSize );
-      dev.writeLine( "bump_size " + str1 );
-   }
-
-   dev.objectEnd( );
-}
-
 void PMBumpMap::serialize( QDomElement& e, QDomDocument& doc ) const
 {
    switch( m_bitmapType )
@@ -259,13 +200,67 @@ void PMBumpMap::readAttributes( const PMXMLHelper& h )
    Base::readAttributes( h );
 }
 
-bool PMBumpMap::isA( PMObjectType t ) const
+PMMetaObject* PMBumpMap::metaObject( ) const
 {
-   if( t == PMTBumpMap )
-      return true;
-   return Base::isA( t );
+   if( !s_pMetaObject )
+   {
+      s_pMetaObject = new PMMetaObject( "BumpMap", Base::metaObject( ),
+                                        createNewBumpMap );
+
+      PMBitmapTypeProperty* bp = new PMBitmapTypeProperty(
+         "bitmapType", &PMBumpMap::setBitmapType,
+         &PMBumpMap::bitmapType );
+      bp->addEnumValue( "Gif", BitmapGif );
+      bp->addEnumValue( "Tga", BitmapTga );
+      bp->addEnumValue( "Iff", BitmapIff );
+      bp->addEnumValue( "Ppm", BitmapPpm );
+      bp->addEnumValue( "Pgm", BitmapPgm );
+      bp->addEnumValue( "Png", BitmapPng );
+      bp->addEnumValue( "Jpeg", BitmapJpeg );
+      bp->addEnumValue( "Tiff", BitmapTiff );
+      bp->addEnumValue( "Sys", BitmapSys );
+      s_pMetaObject->addProperty( bp );
+
+      PMInterpolateTypeProperty* ip = new PMInterpolateTypeProperty(
+         "interpolateType", &PMBumpMap::setInterpolateType,
+         &PMBumpMap::interpolateType );
+      ip->addEnumValue( "None", InterpolateNone );
+      ip->addEnumValue( "Bilinear", InterpolateBilinear );
+      ip->addEnumValue( "Normalized", InterpolateNormalized );
+      s_pMetaObject->addProperty( ip );
+
+      PMMapTypeProperty* mp = new PMMapTypeProperty(
+         "mapType", &PMBumpMap::setMapType, &PMBumpMap::mapType );
+      mp->addEnumValue( "Planar", MapPlanar );
+      mp->addEnumValue( "Spherical", MapSpherical );
+      mp->addEnumValue( "Cylindrical", MapCylindrical );
+      mp->addEnumValue( "Toroidal", MapToroidal );
+      s_pMetaObject->addProperty( mp );
+
+      s_pMetaObject->addProperty(
+         new PMProperty( "bitmapFile", &PMBumpMap::setBitmapFileName,
+                         &PMBumpMap::bitmapFile ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "useIndex", &PMBumpMap::enableUseIndex,
+                         &PMBumpMap::isUseIndexEnabled ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "once", &PMBumpMap::enableOnce,
+                         &PMBumpMap::isOnceEnabled ) );
+   }
+   return s_pMetaObject;
 }
 
+void PMBumpMap::cleanUp( ) const
+{
+   if( s_pMetaObject )
+   {
+      delete s_pMetaObject;
+      s_pMetaObject = 0;
+   }
+   Base::cleanUp( );
+}
+
+
 QString PMBumpMap::description( ) const
 {
    return i18n( "bump map" );
@@ -276,9 +271,9 @@ void PMBumpMap::setBitmapType( PMBitmapType c )
    if( c != m_bitmapType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBumpMap, PMBitmapTypeID, m_bitmapType );
+         m_pMemento->addData( s_pMetaObject, PMBitmapTypeID, m_bitmapType );
       m_bitmapType = c;
-   } 
+   }
 }
 
 void PMBumpMap::setBitmapFileName( const QString& c )
@@ -286,9 +281,9 @@ void PMBumpMap::setBitmapFileName( const QString& c )
    if( c != m_bitmapFile )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBumpMap, PMBitmapFileID, m_bitmapFile );
+         m_pMemento->addData( s_pMetaObject, PMBitmapFileID, m_bitmapFile );
       m_bitmapFile = c;
-   } 
+   }
 }
 
 void PMBumpMap::enableUseIndex( bool c )
@@ -296,9 +291,9 @@ void PMBumpMap::enableUseIndex( bool c )
    if( c != m_useIndex )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBumpMap, PMUseIndexID, m_useIndex );
+         m_pMemento->addData( s_pMetaObject, PMUseIndexID, m_useIndex );
       m_useIndex = c;
-   } 
+   }
 }
 
 void PMBumpMap::setMapType( PMMapType c )
@@ -306,7 +301,7 @@ void PMBumpMap::setMapType( PMMapType c )
    if( c != m_mapType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBumpMap, PMMapTypeID, m_mapType );
+         m_pMemento->addData( s_pMetaObject, PMMapTypeID, m_mapType );
       m_mapType = c;
    }
 }
@@ -316,7 +311,7 @@ void PMBumpMap::setInterpolateType( PMInterpolateType c )
    if( c != m_interpolateType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBumpMap, PMInterpolateID, m_interpolateType );
+         m_pMemento->addData( s_pMetaObject, PMInterpolateID, m_interpolateType );
       m_interpolateType = c;
    }
 }
@@ -326,9 +321,9 @@ void PMBumpMap::enableOnce( bool c )
    if( c != m_once )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBumpMap, PMOnceID, m_once );
+         m_pMemento->addData( s_pMetaObject, PMOnceID, m_once );
       m_once = c;
-   } 
+   }
 }
 
 void PMBumpMap::setBumpSize( double c )
@@ -336,9 +331,9 @@ void PMBumpMap::setBumpSize( double c )
    if( c != m_bumpSize )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTBumpMap, PMBumpSizeID, m_bumpSize );
+         m_pMemento->addData( s_pMetaObject, PMBumpSizeID, m_bumpSize );
       m_bumpSize = c;
-   } 
+   }
 }
 
 PMDialogEditBase* PMBumpMap::editWidget( QWidget* parent ) const
@@ -354,7 +349,7 @@ void PMBumpMap::restoreMemento( PMMemento* s )
    for( ; it.current( ); ++it )
    {
       data = it.current( );
-      if( data->objectType( ) == PMTBumpMap )
+      if( data->objectType( ) == s_pMetaObject )
       {
          switch( data->valueID( ) )
          {
