@@ -18,20 +18,26 @@
 
 #include "pmcamera.h"
 
-#include "pmoutputdevice.h"
 #include "pmxmlhelper.h"
 #include "pmcameraedit.h"
 #include "pmmemento.h"
 #include "pmviewstructure.h"
 #include "pm3dcontrolpoint.h"
 #include "pmvectorcontrolpoint.h"
-
-#include <kdebug.h>
-#include "pmglobals.h"
+#include "pmenumproperty.h"
 
 #include <klocale.h>
 
+PMDefinePropertyClass( PMCamera, PMProperty );
+PMDefineEnumPropertyClass( PMCamera, PMCamera::CameraType,
+                           PMCameraTypeProperty );
+
 PMViewStructure* PMCamera::s_pDefaultViewStructure = 0;
+PMMetaObject* PMCamera::s_pMetaObject = 0;
+PMObject* createNewCamera( PMPart* part )
+{
+   return new PMCamera( part );
+}
 
 const PMVector locationDefault = PMVector( 0.0, 0.0, 0.0 );
 const PMVector lookAtDefault = PMVector( 0.0, 0.0, 1.0 );
@@ -50,19 +56,21 @@ const int blurSamplesDefault = 10;
 const PMVector focalPointDefault = PMVector( 0.0, 0.0, 0.0 );
 const double confidenceDefault = 0.9;
 const double varianceDefault = 0.008;
+const bool c_defaultExport = true;
 
-PMCamera::PMCamera( )
-      : Base( )
+PMCamera::PMCamera( PMPart* part )
+      : Base( part )
 {
    m_location = locationDefault;
    m_lookAt = lookAtDefault;
    m_direction = directionDefault;
-   
+
    m_up = upDefault;
    m_right = rightDefault;
-   
+
    m_sky = skyDefault;
    m_angle = angleDefault;
+   m_angleEnabled = angleEnabledDefault;
 
    m_cameraType = cameraTypeDefault;
    m_cylinderType = cylinderTypeDefault;
@@ -73,6 +81,33 @@ PMCamera::PMCamera( )
    m_focalPoint = focalPointDefault;
    m_confidence = confidenceDefault;
    m_variance = varianceDefault;
+   m_export = c_defaultExport;
+}
+
+PMCamera::PMCamera( const PMCamera& c )
+      : Base( c )
+{
+   m_location = c.m_location;
+   m_lookAt = c.m_lookAt;
+   m_direction = c.m_direction;
+
+   m_up = c.m_up;
+   m_right = c.m_right;
+
+   m_sky = c.m_sky;
+   m_angle = c.m_angle;
+   m_angleEnabled = c.m_angleEnabled;
+
+   m_cameraType = c.m_cameraType;
+   m_cylinderType = c.m_cylinderType;
+
+   m_focalBlurEnabled = c.m_focalBlurEnabled;
+   m_aperture = c.m_aperture;
+   m_blurSamples = c.m_blurSamples;
+   m_focalPoint = c.m_focalPoint;
+   m_confidence = c.m_confidence;
+   m_variance = c.m_variance;
+   m_export = c.m_export;
 }
 
 PMCamera::~PMCamera( )
@@ -84,51 +119,6 @@ QString PMCamera::description( ) const
    return i18n( "camera" );
 }
 
-void PMCamera::serialize( PMOutputDevice& dev ) const
-{
-   dev.objectBegin( "camera" );
-   QString str;
-
-   serializeName( dev );
-
-   if( m_cameraType == Cylinder )
-   {
-      str.setNum( m_cylinderType );
-      dev.writeLine( "cylinder " + str );
-   }
-   else
-      dev.writeLine( cameraTypeToString( m_cameraType ) );
-   
-   dev.writeLine( "location " + m_location.serialize( ) );
-   dev.writeLine( "sky " + m_sky.serialize( ) );
-   dev.writeLine( "direction " + m_direction.serialize( ) );
-   dev.writeLine( "right " + m_right.serialize( ) );
-   dev.writeLine( "up " + m_up.serialize( ) );
-   dev.writeLine( "look_at " + m_lookAt.serialize( ) );
-      
-   if( ( m_cameraType != Orthographic ) && ( m_cameraType != Omnimax )
-       && ( m_cameraType != Panoramic ) && m_angleEnabled )
-   {
-      str.setNum( m_angle );
-      dev.writeLine( "angle " + str );
-   }
-   if( m_focalBlurEnabled && ( m_cameraType == Perspective ) )
-   {
-      str.setNum( m_aperture );
-      dev.writeLine( "aperture " + str );
-      str.setNum( m_blurSamples );
-      dev.writeLine( "blur_samples " + str );
-      dev.writeLine( "focal_point " + m_focalPoint.serialize( ) );
-      str.setNum( m_confidence );
-      dev.writeLine( "confidence " + str );
-      str.setNum( m_variance );
-      dev.writeLine( "variance " + str );
-   }
-   
-   Base::serialize( dev );
-   dev.objectEnd( );
-}
-
 void PMCamera::serialize( QDomElement& e, QDomDocument& doc ) const
 {
    e.setAttribute( "camera_type", cameraTypeToString( m_cameraType ) );
@@ -147,7 +137,8 @@ void PMCamera::serialize( QDomElement& e, QDomDocument& doc ) const
    e.setAttribute( "focal_point", m_focalPoint.serializeXML( ) );
    e.setAttribute( "confidence", m_confidence );
    e.setAttribute( "variance", m_variance );
-   
+   e.setAttribute( "export", m_export );
+
    Base::serialize( e, doc );
 }
 
@@ -170,44 +161,90 @@ void PMCamera::readAttributes( const PMXMLHelper& h )
    m_focalPoint = h.vectorAttribute( "focal_point", focalPointDefault );
    m_confidence = h.doubleAttribute( "confidence", confidenceDefault );
    m_variance = h.doubleAttribute( "variance", varianceDefault );
+   m_export = h.boolAttribute( "export", c_defaultExport );
    Base::readAttributes( h );
 }
 
-bool PMCamera::isA( PMObjectType t ) const
+PMMetaObject* PMCamera::metaObject( ) const
 {
-   if( t == PMTCamera )
-      return true;
-   return Base::isA( t );
-}
-
-
-bool PMCamera::canInsert( PMObjectType t, const PMObject*,
-                          const PMObjectList* ) const
-{
-   switch( t )
+   if( !s_pMetaObject )
    {
-      case PMTComment:
-      case PMTRaw:
-      case PMTTranslate:
-      case PMTRotate:
-      case PMTScale:
-      case PMTMatrix:
-      case PMTNormal:
-         return true;
-         break;
-      default:
-         break;
-   }
-   return false;
-}
+      s_pMetaObject = new PMMetaObject( "Camera", Base::metaObject( ),
+                                        createNewCamera );
+      s_pMetaObject->addProperty(
+         new PMProperty( "location", &PMCamera::setLocation,
+                         &PMCamera::location ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "lookAt", &PMCamera::setLookAt,
+                         &PMCamera::lookAt ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "up", &PMCamera::setUp,
+                         &PMCamera::up ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "right", &PMCamera::setRight,
+                         &PMCamera::right ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "direction", &PMCamera::setDirection,
+                         &PMCamera::direction ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "sky", &PMCamera::setSky,
+                         &PMCamera::sky ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "aspect", 0, &PMCamera::aspect ) );
 
+      s_pMetaObject->addProperty(
+         new PMProperty( "angleEnabled", &PMCamera::enableAngle,
+                         &PMCamera::isAngleEnabled ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "angle", &PMCamera::setAngle,
+                         &PMCamera::angle ) );
+
+      PMCameraTypeProperty* p = new PMCameraTypeProperty(
+         "cameraType", &PMCamera::setCameraType,
+         &PMCamera::cameraType );
+      p->addEnumValue( "Perspective", Perspective );
+      p->addEnumValue( "Orthographic", Orthographic );
+      p->addEnumValue( "FishEye", FishEye );
+      p->addEnumValue( "UltraWideAngle", UltraWideAngle );
+      p->addEnumValue( "Omnimax", Omnimax );
+      p->addEnumValue( "Panoramic", Panoramic );
+      p->addEnumValue( "Cylinder", Cylinder );
+      s_pMetaObject->addProperty( p );
+
+      s_pMetaObject->addProperty(
+         new PMProperty( "cylinderType", &PMCamera::setCylinderType,
+                         &PMCamera::cylinderType ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "focalBlurEnabled", &PMCamera::enableFocalBlur,
+                         &PMCamera::isFocalBlurEnabled ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "aperture", &PMCamera::setAperture,
+                         &PMCamera::aperture ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "blurSamples", &PMCamera::setBlurSamples,
+                         &PMCamera::blurSamples ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "focalPoint", &PMCamera::setFocalPoint,
+                         &PMCamera::focalPoint ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "confidence", &PMCamera::setConfidence,
+                         &PMCamera::confidence ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "variance", &PMCamera::setVariance,
+                         &PMCamera::variance ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "export", &PMCamera::setExportPovray,
+                         &PMCamera::exportPovray ) );
+   }
+   return s_pMetaObject;
+}
 
 void PMCamera::setLocation( const PMVector& p )
 {
    if( p != m_location )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMLocationID, m_location );
+         m_pMemento->addData( s_pMetaObject, PMLocationID, m_location );
       m_location = p;
       m_location.resize( 3 );
       setViewStructureChanged( );
@@ -219,7 +256,7 @@ void PMCamera::setLookAt( const PMVector& p )
    if( p != m_lookAt )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMLookAtID, m_lookAt );
+         m_pMemento->addData( s_pMetaObject, PMLookAtID, m_lookAt );
       m_lookAt = p;
       m_lookAt.resize( 3 );
       setViewStructureChanged( );
@@ -231,7 +268,7 @@ void PMCamera::setUp( const PMVector& v )
    if( v != m_up )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMUpID, m_up );
+         m_pMemento->addData( s_pMetaObject, PMUpID, m_up );
       m_up = v;
       m_up.resize( 3 );
       setViewStructureChanged( );
@@ -243,7 +280,7 @@ void PMCamera::setRight( const PMVector& v )
    if( v != m_right )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMRightID, m_right );
+         m_pMemento->addData( s_pMetaObject, PMRightID, m_right );
       m_right = v;
       m_right.resize( 3 );
       setViewStructureChanged( );
@@ -255,7 +292,7 @@ void PMCamera::setDirection( const PMVector& v )
    if( v != m_direction )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMDirectionID, m_direction );
+         m_pMemento->addData( s_pMetaObject, PMDirectionID, m_direction );
       m_direction = v;
       m_direction.resize( 3 );
       setViewStructureChanged( );
@@ -267,7 +304,7 @@ void PMCamera::setSky( const PMVector& v )
    if( v != m_sky )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMSkyID, m_sky );
+         m_pMemento->addData( s_pMetaObject, PMSkyID, m_sky );
       m_sky = v;
       m_sky.resize( 3 );
       setViewStructureChanged( );
@@ -287,7 +324,7 @@ void PMCamera::enableAngle( bool yes )
    if( yes != m_angleEnabled )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMAngleEnabledID, m_angleEnabled );
+         m_pMemento->addData( s_pMetaObject, PMAngleEnabledID, m_angleEnabled );
       m_angleEnabled = yes;
       setViewStructureChanged( );
    }
@@ -298,10 +335,10 @@ void PMCamera::setAngle( double a )
    if( a != m_angle )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMAngleID, m_angle );
+         m_pMemento->addData( s_pMetaObject, PMAngleID, m_angle );
       m_angle = a;
       setViewStructureChanged( );
-   }   
+   }
 }
 
 void PMCamera::setCameraType( PMCamera::CameraType t )
@@ -309,7 +346,7 @@ void PMCamera::setCameraType( PMCamera::CameraType t )
    if( t != m_cameraType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMCameraTypeID, m_cameraType );
+         m_pMemento->addData( s_pMetaObject, PMCameraTypeID, m_cameraType );
       m_cameraType = t;
       setViewStructureChanged( );
    }
@@ -322,7 +359,7 @@ void PMCamera::setCylinderType( int t )
    else if( t != m_cylinderType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMCylinderTypeID, m_cylinderType );
+         m_pMemento->addData( s_pMetaObject, PMCylinderTypeID, m_cylinderType );
       m_cylinderType = t;
       setViewStructureChanged( );
    }
@@ -333,7 +370,7 @@ void PMCamera::enableFocalBlur( bool yes )
    if( yes != m_focalBlurEnabled )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMFocalBlurID, m_focalBlurEnabled );
+         m_pMemento->addData( s_pMetaObject, PMFocalBlurID, m_focalBlurEnabled );
       m_focalBlurEnabled = yes;
    }
 }
@@ -345,7 +382,7 @@ void PMCamera::setAperture( double a )
    else if( a != m_aperture )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMApertureID, m_aperture );
+         m_pMemento->addData( s_pMetaObject, PMApertureID, m_aperture );
       m_aperture = a;
    }
 }
@@ -357,7 +394,7 @@ void PMCamera::setBlurSamples( int s )
    else if( s != m_blurSamples )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMBlurSamplesID, m_blurSamples );
+         m_pMemento->addData( s_pMetaObject, PMBlurSamplesID, m_blurSamples );
       m_blurSamples = s;
    }
 }
@@ -367,7 +404,7 @@ void PMCamera::setFocalPoint( const PMVector& v )
    if( v != m_focalPoint )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMFocalPointID, m_focalPoint );
+         m_pMemento->addData( s_pMetaObject, PMFocalPointID, m_focalPoint );
       m_focalPoint = v;
    }
 }
@@ -379,7 +416,7 @@ void PMCamera::setConfidence( double c )
    else if( c != m_confidence )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMConfidenceID, m_confidence );
+         m_pMemento->addData( s_pMetaObject, PMConfidenceID, m_confidence );
       m_confidence = c;
    }
 }
@@ -391,11 +428,21 @@ void PMCamera::setVariance( double v )
    else if( v != m_variance )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTCamera, PMVarianceID, m_variance );
+         m_pMemento->addData( s_pMetaObject, PMVarianceID, m_variance );
       m_variance = v;
    }
 }
 
+void PMCamera::setExportPovray( bool ex )
+{
+   if( m_export != ex )
+   {
+      if( m_pMemento )
+         m_pMemento->addData( s_pMetaObject, PMExportID, m_export );
+      m_export = ex;
+   }
+}
+
 PMDialogEditBase* PMCamera::editWidget( QWidget* parent ) const
 {
    return new PMCameraEdit( parent );
@@ -409,7 +456,7 @@ void PMCamera::restoreMemento( PMMemento* s )
    for( ; it.current( ); ++it )
    {
       data = it.current( );
-      if( data->objectType( ) == PMTCamera )
+      if( data->objectType( ) == s_pMetaObject )
       {
          switch( data->valueID( ) )
          {
@@ -461,6 +508,9 @@ void PMCamera::restoreMemento( PMMemento* s )
             case PMApertureID:
                setAperture( data->doubleData( ) );
                break;
+            case PMExportID:
+               setExportPovray( data->boolData( ) );
+               break;
             default:
                kdError( PMArea ) << "Wrong ID in PMCamera::restoreMemento\n";
                break;
@@ -474,7 +524,7 @@ void PMCamera::createViewStructure( )
 {
    PMVector newUp, newRight, newDirection, tmp;
    double upLen, rightLen;
-   
+
    if( !m_pViewStructure )
    {
       m_pViewStructure = new PMViewStructure( defaultViewStructure( ) );
@@ -487,7 +537,7 @@ void PMCamera::createViewStructure( )
    points[0] = m_lookAt;
    points[1] = m_location;
 
-   upLen = newUp.abs( ); 
+   upLen = newUp.abs( );
    rightLen = newRight.abs( );
    if( rightLen > upLen )
    {
@@ -501,7 +551,7 @@ void PMCamera::createViewStructure( )
       newRight /= upLen;
       newDirection /= upLen;
    }
-   
+
    newRight /= 2.0;
    newUp /= 2.0;
    tmp = m_location + newDirection;
@@ -569,7 +619,7 @@ void PMCamera::calculateLookAtAngle( PMVector& right, PMVector& up, PMVector& di
 {
    PMVector tmpVector;
    double directionLen, upLen, rightLen, handedness, angle;
-   
+
    angle = m_angle;
    if( m_cameraType == Perspective )
    {
@@ -580,7 +630,7 @@ void PMCamera::calculateLookAtAngle( PMVector& right, PMVector& up, PMVector& di
    }
    else
       angle = 90.0;
-   
+
    directionLen = m_direction.abs( );
    upLen = m_up.abs( );
    rightLen = m_right.abs( );
@@ -593,7 +643,7 @@ void PMCamera::calculateLookAtAngle( PMVector& right, PMVector& up, PMVector& di
    }
    else
       direction = m_direction;
-   
+
    if( approxZero( upLen ) )
    {
       up = upDefault;
@@ -617,7 +667,7 @@ void PMCamera::calculateLookAtAngle( PMVector& right, PMVector& up, PMVector& di
       directionLen = 0.5 * rightLen / tan( angle * M_PI / 360.0 );
       direction *= directionLen;
    }
-   
+
    // calculate handedness
    tmpVector = PMVector::cross( up, direction );
    handedness = PMVector::dot( tmpVector, right );
@@ -631,7 +681,7 @@ void PMCamera::calculateLookAtAngle( PMVector& right, PMVector& up, PMVector& di
 
    // normalize new direction
    direction /= direction.abs( );
-       
+
    tmpVector = right;
    right = PMVector::cross( m_sky, direction );
 
@@ -681,7 +731,7 @@ QString PMCamera::cameraTypeToString( PMCamera::CameraType t )
 PMCamera::CameraType PMCamera::stringToCameraType( const QString& str )
 {
    CameraType t = Perspective;
-   
+
    if( str == "perspective" )
       t = Perspective;
    else if( str == "orthographic" )
@@ -705,7 +755,15 @@ PMCamera::CameraType PMCamera::stringToCameraType( const QString& str )
 void PMCamera::cleanUp( ) const
 {
    if( s_pDefaultViewStructure )
+   {
       delete s_pDefaultViewStructure;
-   s_pDefaultViewStructure = 0;
+      s_pDefaultViewStructure = 0;
+   }
+   if( s_pMetaObject )
+   {
+      delete s_pMetaObject;
+      s_pMetaObject = 0;
+   }
+
    Base::cleanUp( );
 }
