@@ -2,7 +2,7 @@
 **************************************************************************
                                  description
                              --------------------
-    copyright            : (C) 2000-2001 by Andreas Zehender
+    copyright            : (C) 2000-2002 by Andreas Zehender
     email                : zehender@kde.org
 **************************************************************************
 
@@ -23,7 +23,7 @@
 #include <qpushbutton.h>
 #include <qlabel.h>
 #include <qvbox.h>
-#include <qtextstream.h>
+#include <qbuffer.h>
 #include <qptrdict.h>
 #include <kdialog.h>
 #include <kconfig.h>
@@ -34,13 +34,13 @@
 #include "pmdefaults.h"
 #include "pmpovrayrenderwidget.h"
 #include "pmpovrayoutputwidget.h"
-#include "pmoutputdevice.h"
+#include "pmpovray31format.h"
+#include "pmserializer.h"
 #include "pmcolor.h"
 #include "pmrecursiveobjectiterator.h"
 #include "pmdeclare.h"
 
-#include "kdebug.h"
-#include "pmglobals.h"
+#include "pmdebug.h"
 
 int PMDialogEditBase::s_previewSize = c_defaultTPSize;
 bool PMDialogEditBase::s_showSphere = c_defaultTPShowSphere;
@@ -97,7 +97,7 @@ const QString c_globalSettingsCode = QString(
 
 PMDialogEditBase::PMDialogEditBase( QWidget* parent, const char* name )
       : QWidget( parent, name )
-{  
+{
    m_pDisplayedObject = 0;
    m_pPart = 0;
 
@@ -119,11 +119,11 @@ PMDialogEditBase::~PMDialogEditBase( )
 void PMDialogEditBase::createWidgets( )
 {
    m_pTopLayout = new QBoxLayout( this, QBoxLayout::TopToBottom,
-                                  KDialog::spacingHint( ) );   
+                                  KDialog::spacingHint( ) );
 
    createTopWidgets( );
    createBottomWidgets( );
-   
+
    m_pTopLayout->addStretch( );
    m_pTopLayout->activate( );
 }
@@ -145,6 +145,10 @@ bool PMDialogEditBase::saveData( )
    return false;
 }
 
+void PMDialogEditBase::saveContents( )
+{
+}
+
 void PMDialogEditBase::displayObject( PMObject* o )
 {
    bool newObject = ( m_pDisplayedObject != o );
@@ -172,7 +176,7 @@ void PMDialogEditBase::displayObject( PMObject* o )
          m_pRenderFrame->setLineWidth( 2 );
          m_pRenderFrame->hide( );
          vl->addWidget( m_pRenderFrame );
-         
+
          m_pRenderWidget = new PMPovrayRenderWidget( m_pRenderFrame );
          connect( m_pRenderWidget, SIGNAL( finished( int ) ),
                   SLOT( slotPreviewFinished( int ) ) );
@@ -181,7 +185,7 @@ void PMDialogEditBase::displayObject( PMObject* o )
          connect( m_pPreviewLocalBox, SIGNAL( toggled( bool ) ),
                   SLOT( slotPreviewLocal( bool ) ) );
          vl->addWidget( m_pPreviewLocalBox );
-         
+
          QHBoxLayout* hl = new QHBoxLayout( vl );
          m_pPreviewButton = new QPushButton( i18n( "&Preview" ), m_pTexturePreviewWidget );
          hl->addWidget( m_pPreviewButton );
@@ -192,7 +196,7 @@ void PMDialogEditBase::displayObject( PMObject* o )
          connect( m_pOutputButton, SIGNAL( clicked( ) ),
                   SLOT( slotPovrayOutput( ) ) );
          hl->addStretch( 1 );
-         
+
          m_pOutputWidget = new PMPovrayOutputWidget( );
          connect( m_pRenderWidget, SIGNAL( povrayMessage( const QString& ) ),
                   m_pOutputWidget, SLOT( slotText( const QString& ) ) );
@@ -316,43 +320,38 @@ void PMDialogEditBase::findTextures( PMObject*& global, PMObject*& local ) const
 
    for( o = m_pDisplayedObject; o; o = o->parent( ) )
    {
-      switch( o->type( ) )
+      if( o->type( ) == "Material" || o->type( ) == "Interior" ||
+          o->type( ) == "Texture" || o->type( ) == "Pigment" ||
+          o->type( ) == "InteriorTexture" )
       {
-         case PMTMaterial:
-         case PMTInterior:
-         case PMTTexture:
-         case PMTPigment:
-            if( !local )
+         if( !local )
+            local = o;
+         global = o;
+      }
+      else if( o->type( ) == "Declare" )
+      {
+         PMDeclare* d = ( PMDeclare* ) o;
+         if( d->declareType( ) == "Interior" ||
+             d->declareType( ) == "Pigment" ||
+             d->declareType( ) == "Material" )
+         {
+            if( !local || ( local == global ) )
                local = o;
             global = o;
-            break;
-         case PMTDeclare:
-            switch( ( ( PMDeclare* ) o )->declareType( ) )
+         }
+         else if( d->declareType( ) == "Texture" ||
+                  d->declareType( ) == "InteriorTexture" )
+         {
+            if( o->countChildren( ) > 1 )
             {
-               case PMDeclare::InteriorDeclare:
-               case PMDeclare::PigmentDeclare:
-               case PMDeclare::MaterialDeclare:
-                  if( !local || ( local == global ) )
-                     local = o;
-                  global = o;
-                  break;
-               case PMDeclare::TextureDeclare:
-                  if( o->countChildren( ) > 1 )
-                  {
-                     if( !local )
-                        local = o;
-                  }
-                  else
-                     if( !local || ( local == global ) )
-                        local = o;
-                  global = o;
-                  break;
-               default:
-                  break;
+               if( !local )
+                  local = o;
             }
-            break;
-         default:
-            break;
+            else
+               if( !local || ( local == global ) )
+                  local = o;
+            global = o;
+         }
       }
    }
 }
@@ -369,6 +368,8 @@ void PMDialogEditBase::slotTexturePreview( )
    findTextures( global, local );
    if( local )
    {
+      emit aboutToRender( );
+
       if( global == local )
          o = global;
       else
@@ -385,8 +386,11 @@ void PMDialogEditBase::slotTexturePreview( )
 
       int numObjects = 0;
       QByteArray scene;
-      QTextStream str( scene, IO_WriteOnly );
-      PMOutputDevice dev( str );
+      QBuffer buffer( scene );
+      buffer.open( IO_WriteOnly );
+      QTextStream str( &buffer );
+      PMPovray31Format format;
+      PMSerializer* dev = format.newSerializer( &buffer );
       PMRenderMode mode;
       PMObjectList neededDeclares, objectsToCheck;
       QPtrDict<PMObject> objectsToSerialize( 101 );
@@ -410,9 +414,9 @@ void PMDialogEditBase::slotTexturePreview( )
             }
             objectsToSerialize.insert( it.current( ), it.current( ) );
          }
-         
+
          objectsToCheck = neededDeclares;
-         neededDeclares.clear( );         
+         neededDeclares.clear( );
       }
       while( objectsToCheck.count( ) > 0 );
 
@@ -421,28 +425,28 @@ void PMDialogEditBase::slotTexturePreview( )
       if( numDeclares > 0 )
       {
          PMObject* otr = o;
-      
+
          // find the scene
          while( otr->parent( ) ) otr = otr->parent( );
-         
+
          for( otr = otr->firstChild( ); otr && ( numDeclares > 0 );
               otr = otr->nextSibling( ) )
          {
-            if( otr->type( ) == PMTDeclare )
+            if( otr->type( ) == "Declare" )
             {
                if( objectsToSerialize.find( otr ) )
                {
-                  otr->serialize( dev );
+                  dev->serialize( otr );
                   numDeclares--;
                }
             }
          }
-         str << "\n\n";  
+         str << "\n\n";
       }
       // if the previewed texture is a declare, serialize it
-      if( o->type( ) == PMTDeclare )
-         o->serialize( dev );
-      
+      if( o->type( ) == "Declare" )
+          dev->serialize( o );
+
       // build the preview scene
       str << "union {\n";
       if( s_showBox )
@@ -462,30 +466,28 @@ void PMDialogEditBase::slotTexturePreview( )
       }
 
       // serialize the texture
-      if( o->type( ) == PMTDeclare )
+      if( o->type( ) == "Declare" )
       {
          PMDeclare* dec = ( PMDeclare* ) o;
-         switch( dec->declareType( ) )
-         {
-            case PMDeclare::InteriorDeclare:
-               str << "interior { ";
-               break;
-            case PMDeclare::TextureDeclare:
-               str << "texture { ";
-               break;
-            case PMDeclare::PigmentDeclare:
-               str << "pigment { ";
-               break;
-            default:
-               kdError( PMArea ) << "Unhandled declare type in "
-                  "PMDialogEditBase::slotTexturePreview\n";
-               break;
-         }
+         if( dec->declareType( ) == "Interior" )
+            str << "interior { ";
+         else if( dec->declareType( ) == "Texture" )
+            str << "texture { ";
+         else if( dec->declareType( ) == "Pigment" )
+            str << "pigment { ";
+         else if( dec->declareType( ) == "InteriorTexture" )
+            str << "interior_texture { ";
+         else if( dec->declareType( ) == "Material" )
+            str << "material { ";
+         else
+            kdError( PMArea ) << "Unhandled declare type in "
+               "PMDialogEditBase::slotTexturePreview\n";
+
          str << dec->id( );
          str << " }\n";
       }
       else
-         o->serialize( dev );
+         dev->serialize( o );
       str << "}\n";
 
       PMColor c1, c2;
@@ -521,6 +523,8 @@ void PMDialogEditBase::slotTexturePreview( )
       emit sizeChanged( );
       m_pOutputWidget->slotClear( );
       m_pRenderWidget->render( scene, mode, m_pPart->url( ) );
+
+      delete dev;
    }
 }
 
@@ -538,6 +542,11 @@ void PMDialogEditBase::slotPreviewFinished( int exitStatus )
                                       "See the povray output for details." )
                           .arg( exitStatus ) );
    }
+   else if( m_pRenderWidget->povrayOutput( ).contains( "ERROR" ) )
+   {
+      KMessageBox::error( this, i18n( "There were errors while rendering.\n"
+                                      "See the povray output for details." ) );
+   }
 }
 
 void PMDialogEditBase::slotPovrayOutput( )
