@@ -2,7 +2,7 @@
 **************************************************************************
                                  description
                              --------------------
-    copyright            : (C) 2000-2002 by Andreas Zehender
+    copyright            : (C) 2000-2003 by Andreas Zehender
     email                : zehender@kde.org
 **************************************************************************
 
@@ -19,9 +19,10 @@
 #include "pmpart.h"
 #include "pmrendermanager.h"
 #include "pmcamera.h"
-#include "pmglviewconnector.h"
 #include "pmscene.h"
 #include "pmdatachangecommand.h"
+#include "pmdebug.h"
+#include "pmdefaults.h"
 
 #include <math.h>
 #include <qpopupmenu.h>
@@ -32,16 +33,16 @@
 #include <qglobal.h>
 #include <qlayout.h>
 #include <qlabel.h>
+#include <qcombobox.h>
+#include <qdom.h>
 
 #include <kxmlguifactory.h>
 #include <kaction.h>
 #include <kconfig.h>
 #include <kstaticdeleter.h>
 #include <klocale.h>
-
-#include <kdebug.h>
-#include "pmglobals.h"
-#include "pmdefaults.h"
+#include <kiconloader.h>
+#include <kdialog.h>
 
 #include <GL/glx.h>
 
@@ -91,7 +92,7 @@ public:
       if( m_visualInfo )
          XFree( m_visualInfo );
    }
-   
+
    Colormap m_colormap;
    GLXContext m_context;
    bool m_colormapAllocated;
@@ -101,14 +102,14 @@ public:
 
 static PMGLViewStatic* s_pSharedData = 0;
 static KStaticDeleter<PMGLViewStatic> s_staticDeleter;
+bool PMGLView::s_bDirect = true;
 
 
-PMGLView::PMGLView( PMPart* part, PMGLViewConnector* conn, PMViewType t,
+PMGLView::PMGLView( PMPart* part, PMViewType t,
                     QWidget* parent, const char* name, WFlags f )
-      : QWidget( parent, name, f | Qt::WWinOwnDC | Qt::WRepaintNoErase )
+      : PMViewBase( parent, name, f | Qt::WWinOwnDC | Qt::WRepaintNoErase )
 {
    m_pPart = part;
-   m_pConnector = conn;
    m_type = t;
    m_bScaleMode = false;
    m_scaleIntX = 0.0;
@@ -132,6 +133,7 @@ PMGLView::PMGLView( PMPart* part, PMGLViewConnector* conn, PMViewType t,
    m_bAboutToUpdate = false;
    m_projectionUpToDate = false;
    m_contextClickPosition = PMVector( 0.0, 0.0 );
+   m_objectActions.setAutoDelete( true );
 
    m_controlPointsPosition.setAutoDelete( true );
    m_pUnderMouse = 0;
@@ -150,30 +152,15 @@ PMGLView::PMGLView( PMPart* part, PMGLViewConnector* conn, PMViewType t,
 
    connect( part, SIGNAL( refresh( ) ), SLOT( slotRefresh( ) ) );
    connect( part, SIGNAL( clear( ) ), SLOT( slotClear( ) ) );
+
    connect( this, SIGNAL( objectChanged( PMObject*, const int, QObject* ) ),
             part, SLOT( slotObjectChanged( PMObject*, const int, QObject* ) ) );
+   connect( part, SIGNAL( objectChanged( PMObject*, const int, QObject* ) ),
+            SLOT( slotObjectChanged( PMObject*, const int, QObject* ) ) );
+
    connect( part, SIGNAL( activeRenderModeChanged( ) ),
             SLOT( slotActiveRenderModeChanged( ) ) );
 
-
-   // Connect this slot to the view and not to the part.
-   // The class PMGLViewConnector updates the control points in
-   // slotObjectChanged, and emits the signals newControlPoints and
-   // glViewObjectChanged.
-   connect( conn, SIGNAL( glViewObjectChanged( PMObject*, const int, QObject* ) ),
-            SLOT( slotObjectChanged( PMObject*, const int, QObject* ) ) );
-   connect( conn, SIGNAL( newControlPoints( const PMControlPointList&, const PMMatrix& ) ),
-            SLOT( slotNewControlPoints( const PMControlPointList&, const PMMatrix& ) ) );
-   connect( conn, SIGNAL( stopRendering( ) ), SLOT( slotStopRendering( ) ) );
-   connect( this, SIGNAL( viewActivated( PMGLView* ) ),
-            conn, SLOT( slotViewActivated( PMGLView* ) ) );
-   connect( this, SIGNAL( controlPointSelectionChanged( QObject* ) ),
-            conn, SLOT( slotControlPointSelectionChanged( QObject* ) ) );
-   connect( conn, SIGNAL( controlPointSelectionChanged( QObject* ) ),
-            this, SLOT( slotControlPointSelectionChanged( QObject* ) ) );
-   connect( this, SIGNAL( contextMenuAboutToShow( ) ),
-            conn, SLOT( slotContextMenuAboutToShow( ) ) );
-
    connect( &m_startTimer, SIGNAL( timeout( ) ),
             SLOT( slotMouseChangeTimer( ) ) );
    connect( &m_autoScrollTimer, SIGNAL( timeout( ) ),
@@ -187,18 +174,11 @@ PMGLView::PMGLView( PMPart* part, PMGLViewConnector* conn, PMViewType t,
             SLOT( slotRenderingFinished( PMGLView* ) ) );
    connect( rm, SIGNAL( renderingSettingsChanged( ) ),
             SLOT( slotRefresh( ) ) );
-   
+
    connect( this, SIGNAL( controlPointMessage( const QString& ) ),
             m_pPart, SIGNAL( controlPointMessage( const QString& ) ) );
 
-   m_pPart->addGLView( this );
-   connect( this, SIGNAL( destroyed( PMGLView* ) ),
-            part, SLOT( removeGLView( PMGLView* ) ) );
-   m_pConnector->addGLView( this );
-   connect( this, SIGNAL( destroyed( PMGLView* ) ),
-            conn, SLOT( removeGLView( PMGLView* ) ) );
-
-   slotNewControlPoints( conn->controlPoints( ), conn->transformation( ) );
+   updateControlPoints( );
 }
 
 void PMGLView::initializeGL( )
@@ -221,7 +201,7 @@ void PMGLView::initializeGL( )
          if( vi )
          {
             kdDebug( PMArea ) << "PMGLView: XVisual found\n";
-         
+
             // create a color map (from qgl_x11.cpp)
             // check if we can use the global colormap
             // should be the default ?
@@ -246,7 +226,7 @@ void PMGLView::initializeGL( )
                   int n;
                   Atom hp_cmaps = XInternAtom( display, "_HP_RGB_SMOOTH_MAP_LIST",
                                                TRUE );
-               
+
                   if( hp_cmaps && ( vi->visual->c_class == TrueColor )
                       && ( vi->depth == 8 ) )
                   {
@@ -274,7 +254,7 @@ void PMGLView::initializeGL( )
             {
                XStandardColormap* c;
                int n;
-            
+
                if( XmuLookupStandardColormap( display, vi->screen, vi->visualid,
                                               vi->depth, XA_RGB_DEFAULT_MAP,
                                               FALSE, TRUE ) )
@@ -297,7 +277,7 @@ void PMGLView::initializeGL( )
                }
             }
 #endif
-         
+
             if( !s_pSharedData->m_colormap )
             {
                // create a new colormap otherwise
@@ -308,14 +288,15 @@ void PMGLView::initializeGL( )
                                    vi->visual, AllocNone );
                s_pSharedData->m_colormapAllocated = true;
             }
-         }
 
-         // create the context
-         // this context is shared between all gl views!
-         s_pSharedData->m_context = glXCreateContext( display, vi, 0, GL_TRUE );
-      
-         if( s_pSharedData->m_context != NULL )
-            kdDebug( PMArea ) << "PMGLView: glx context created\n";
+            // create the context
+            // this context is shared between all gl views!
+            s_pSharedData->m_context = glXCreateContext( display, vi, 0, s_bDirect );
+
+            if( s_pSharedData->m_context != NULL )
+               kdDebug( PMArea ) << "PMGLView: glx context created\n";
+
+         }
       }
    }
 
@@ -328,19 +309,19 @@ void PMGLView::initializeGL( )
       swa.colormap = s_pSharedData->m_colormap;
       swa.border_pixel = 0;
       swa.background_pixel = 0;
-         
+
       Window p;
       p = RootWindow( display, vi->screen );
       QWidget* pw = parentWidget( );
       if( pw )
          p = pw->winId( );
-      
+
       Window w = XCreateWindow( display, p, x( ), y( ), width( ),
                                 height( ), 0, vi->depth, InputOutput,
                                 vi->visual,
                                 CWColormap | CWBorderPixel | CWBackPixel,
                                 &swa );
-      
+
       // tell the windowmanager to use the colormap
       Window* colorMapWindows = 0;
       Window* newWindows = 0;
@@ -351,7 +332,7 @@ void PMGLView::initializeGL( )
          // create a new list and append the new window
          bool replaced = false;
          newWindows = new Window[num+1];
-         
+
          for( i = 0; i < num; i++ )
          {
             newWindows[i] = colorMapWindows[i];
@@ -375,11 +356,11 @@ void PMGLView::initializeGL( )
       }
       // tell Qt to use this window
       create( w );
-      
+
       XSetWMColormapWindows( display, topLevelWidget( )->winId( ),
                              newWindows, num );
       delete[] newWindows;
-      
+
       XFlush( x11Display( ) );
    }
    else
@@ -389,7 +370,7 @@ void PMGLView::initializeGL( )
       label->setAlignment( Qt::AlignCenter );
       topLayout->addWidget( label );
    }
-   
+
    //setProjection( );
 }
 
@@ -457,7 +438,7 @@ void PMGLView::invalidateProjection( bool graphicalChange /*= true*/ )
 {
    m_viewTransformation = PMMatrix::identity( );
 
-   
+
    if( m_type != PMViewCamera )
    {
       m_viewTransformation = m_viewTransformation * PMMatrix::scale( m_dScale, m_dScale, m_dScale );
@@ -524,8 +505,6 @@ void PMGLView::enableScaleMode( bool yes )
 
 void PMGLView::mousePressEvent( QMouseEvent* e )
 {
-   emit viewActivated( this );
-
    if( m_bScaleMode || m_bTranslateMode )
    {
       if( ( e->button( ) & LeftButton ) && ( e->state( ) == 0 ) )
@@ -562,7 +541,7 @@ void PMGLView::mousePressEvent( QMouseEvent* e )
                      selectControlPoint( m_pUnderMouse, true );
                }
             }
-            else 
+            else
                selectControlPoint( m_pUnderMouse, true );
 
             // start the graphical change
@@ -612,17 +591,11 @@ void PMGLView::mousePressEvent( QMouseEvent* e )
                if( !m_pUnderMouse->selected( ) )
                   selectControlPoint( m_pUnderMouse, true );
             }
-            else 
+            else
                selectControlPoint( m_pUnderMouse, true );
          }
 
-         QPopupMenu* m =
-            ( QPopupMenu* ) m_pPart->factory( )->container( "viewPopup", m_pPart );
-         if( m )
-         {
-            emit contextMenuAboutToShow( );
-            m->exec( QCursor::pos( ) );
-         }
+         contextMenu( );
       }
    }
 
@@ -833,7 +806,7 @@ void PMGLView::wheelEvent( QWheelEvent* e )
 void PMGLView::keyPressEvent( QKeyEvent* e )
 {
    bool accept = true;
-   
+
    if( e->state( ) == 0 )
    {
       if( m_type != PMViewCamera )
@@ -883,7 +856,7 @@ void PMGLView::keyPressEvent( QKeyEvent* e )
    }
    else
       accept = false;
-   
+
    if( accept )
       invalidateProjection( );
    else
@@ -978,9 +951,17 @@ void PMGLView::graphicalChange( const QPoint& mousePos )
    else
       m_pUnderMouse->change( p );
 
-   m_pActiveObject->controlPointsChanged( m_controlPoints );
+   PMObjectList changedObjects;
+   m_pActiveObject->controlPointsChangedList( m_controlPoints, changedObjects );
 
-   emit objectChanged( m_pActiveObject, PMCGraphicalChange, this );
+   if( changedObjects.isEmpty( ) )
+      emit objectChanged( m_pActiveObject, PMCGraphicalChange, this );
+   else
+   {
+      PMObjectListIterator it( changedObjects );
+      for( ; it.current( ); ++it )
+         emit objectChanged( it.current( ), PMCGraphicalChange, this );
+   }
 }
 
 void PMGLView::checkUnderMouse( int x, int y )
@@ -1038,17 +1019,14 @@ void PMGLView::checkUnderMouse( int x, int y )
    }
 }
 
-void PMGLView::slotNewControlPoints( const PMControlPointList& list, const PMMatrix& t )
+void PMGLView::updateControlPoints( )
 {
-   PMControlPointListIterator it( list );
    m_controlPoints.clear( );
+   m_controlPoints = m_pPart->activeControlPoints( );
 
-   for( ; it.current( ); ++it )
-      m_controlPoints.append( it.current( ) );
-
-   if( m_controlPoints.count( ) > 0 )
+   if( ( m_controlPoints.count( ) > 0 ) && m_pActiveObject )
    {
-      m_objectsTransformation = t;
+      m_objectsTransformation = m_pActiveObject->transformedWith( );
       recalculateTransformations( );
    }
 
@@ -1196,17 +1174,27 @@ void PMGLView::slotObjectChanged( PMObject* obj, const int mode,
    {
       if( m_type == PMGLView::PMViewCamera )
       {
-         if( obj->type( ) == PMTCamera )
+         if( obj->type( ) == "Camera" )
             if( m_pCamera == ( PMCamera* ) obj )
                invalidateProjection( );
-         
+
          if( obj->parent( ) )
-            if( obj->parent( )->type( ) == PMTCamera )
+            if( obj->parent( )->type( ) == "Camera" )
                if( m_pCamera == ( PMCamera* ) obj->parent( ) )
                   if( obj->hasTransformationMatrix( ) )
                      invalidateProjection( );
       }
-      
+
+      redraw = true;
+   }
+   if( mode & PMCNewControlPoints )
+   {
+      updateControlPoints( );
+      m_pActiveObject = m_pPart->activeObject( );
+      redraw = true;
+   }
+   if( mode & PMCControlPointSelection )
+   {
       redraw = true;
    }
    if( mode & PMCDescription )
@@ -1218,11 +1206,11 @@ void PMGLView::slotObjectChanged( PMObject* obj, const int mode,
    {
       if( m_type == PMGLView::PMViewCamera )
       {
-         if( obj->type( ) == PMTCamera )
+         if( obj->type( ) == "Camera" )
             if( !m_pCamera )
                setCamera( ( PMCamera* ) obj );
          if( obj->parent( ) )
-            if( obj->parent( )->type( ) == PMTCamera )
+            if( obj->parent( )->type( ) == "Camera" )
                if( m_pCamera == ( PMCamera* ) obj->parent( ) )
                   if( obj->hasTransformationMatrix( ) )
                      invalidateProjection( );
@@ -1232,13 +1220,13 @@ void PMGLView::slotObjectChanged( PMObject* obj, const int mode,
 
    if( mode & PMCRemove )
    {
-      if( obj->type( ) == PMTCamera )
+      if( obj->type( ) == "Camera" )
          if( m_pCamera == ( PMCamera* ) obj )
             setCamera( 0 );
-      
+
       if( m_type == PMGLView::PMViewCamera )
          if( obj->parent( ) )
-            if( obj->parent( )->type( ) == PMTCamera )
+            if( obj->parent( )->type( ) == "Camera" )
                if( m_pCamera == ( PMCamera* ) obj->parent( ) )
                   if( obj->hasTransformationMatrix( ) )
                      invalidateProjection( );
@@ -1253,17 +1241,12 @@ void PMGLView::slotObjectChanged( PMObject* obj, const int mode,
       repaint( sender == this );
 }
 
-void PMGLView::slotControlPointSelectionChanged( QObject* sender )
-{
-   repaint( sender == this );
-}
-
 void PMGLView::repaint( bool graphicalChange )
 {
    if( isValid( ) )
    {
       PMObject* obj = m_pActiveObject;
-      
+
       if( obj )
          obj = topLevelRenderingObject( obj );
       else
@@ -1273,7 +1256,7 @@ void PMGLView::repaint( bool graphicalChange )
          if( it.current( ) )
             obj = topLevelRenderingObject( it.current( ) );
 
-         if( obj && ( obj->type( ) != PMTScene ) )
+         if( obj && ( obj->type( ) != "Scene" ) )
             for( ++it; it.current( ) && obj; ++it )
                if( topLevelRenderingObject( it.current( ) ) != obj )
                   obj = 0;
@@ -1290,7 +1273,7 @@ void PMGLView::repaint( bool graphicalChange )
             if( mode->height( ) != 0 )
                aspectRatio = ( double ) mode->width( )
                              / ( double ) mode->height( );
-      
+
          PMRenderManager* rm = PMRenderManager::theManager( );
          rm->addView( this, m_pActiveObject, obj,
                       &m_controlPoints, aspectRatio,
@@ -1303,14 +1286,14 @@ PMObject* PMGLView::topLevelRenderingObject( PMObject* o ) const
 {
    PMObject* obj = o;
    bool stop = false;
-   
+
    if( obj )
    {
       do
       {
          if( !obj )
             stop = true;
-         else if( obj->isA( PMTScene ) || obj->isA( PMTDeclare ) )
+         else if( obj->isA( "Scene" ) || obj->isA( "Declare" ) )
             stop = true;
          else
             obj = obj->parent( );
@@ -1319,7 +1302,7 @@ PMObject* PMGLView::topLevelRenderingObject( PMObject* o ) const
    }
    else
       obj = m_pPart->scene( );
-   
+
    return obj;
 }
 
@@ -1370,7 +1353,7 @@ void PMGLView::selectControlPoint( PMControlPoint* cp, bool select, bool deselec
    }
 
    if( selectionChanged )
-      emit controlPointSelectionChanged( this );
+      emit objectChanged( m_pActiveObject, PMCControlPointSelection, this );
 }
 
 void PMGLView::startSelection( )
@@ -1515,7 +1498,7 @@ void PMGLView::slotRenderingFinished( PMGLView* view )
 QString PMGLView::viewTypeAsString( PMViewType t )
 {
    QString str;
-   
+
    switch( t )
    {
       case PMViewPosX:
@@ -1551,4 +1534,320 @@ void PMGLView::restoreConfig( KConfig* /*cfg*/ )
 {
 }
 
+void PMGLView::contextMenu( )
+{
+   QPopupMenu* m = new QPopupMenu( );
+   m->insertItem( i18n( "Left View" ), this, SLOT( slotSetTypePosX( ) ) );
+   m->insertItem( i18n( "Right View" ), this, SLOT( slotSetTypeNegX( ) ) );
+   m->insertItem( i18n( "Top View" ), this, SLOT( slotSetTypeNegY( ) ) );
+   m->insertItem( i18n( "Bottom View" ), this, SLOT( slotSetTypePosY( ) ) );
+   m->insertItem( i18n( "Front View" ), this, SLOT( slotSetTypePosZ( ) ) );
+   m->insertItem( i18n( "Back View" ), this, SLOT( slotSetTypeNegZ( ) ) );
+
+   QPopupMenu* cm = new QPopupMenu( m );
+   QPtrListIterator<PMCamera> it = m_pPart->cameras( );
+   QString name;
+   if( !it.current( ) )
+      cm->insertItem( i18n( "No Cameras" ) );
+   else
+   {
+      int cnr = 0;
+      for( ; it.current( ); ++it, ++cnr )
+      {
+         name = it.current( )->name( );
+         if( name.isEmpty( ) )
+            name = i18n( "(unnamed)" );
+         cm->insertItem( name, cnr );
+      }
+   }
+   connect( cm, SIGNAL( activated( int ) ), SLOT( slotCameraView( int ) ) );
+
+   m->insertItem( SmallIconSet( "pmcamera" ), i18n( "Camera" ), cm );
+
+   m->insertSeparator( );
+
+   m->insertItem( i18n( "Snap to Grid" ), this, SLOT( slotSnapToGrid( ) ) );
+   m_objectActions.clear( );
+   if( m_pActiveObject )
+   {
+      m_pActiveObject->addObjectActions( m_controlPoints, m_objectActions );
+      if( !m_objectActions.isEmpty( ) )
+      {
+         PMObjectAction* oa = 0;
+         QPtrListIterator<PMObjectAction> ait( m_objectActions );
+
+         for( ; ait.current( ); ++ait )
+         {
+            oa = ait.current( );
+            oa->setMenuID( m->insertItem( oa->description( ) ) );
+         }
+      }
+   }
+   connect( m, SIGNAL( activated( int ) ), SLOT( slotObjectAction( int ) ) );
+
+   m->insertSeparator( );
+
+   QPopupMenu* menu = new QPopupMenu( m );
+   PMControlPointListIterator pit( m_controlPoints );
+
+   if( !pit.current( ) )
+      menu->insertItem( i18n( "No Control Points" ) );
+   else
+   {
+      int cnr = 0;
+      for( ; pit.current( ); ++pit, ++cnr )
+         menu->insertItem( pit.current( )->description( ), cnr );
+   }
+   connect( menu, SIGNAL( activated( int ) ), SLOT( slotControlPoint( int ) ) );
+
+   m->insertItem( i18n( "Control Points" ), menu );
+
+   m->exec( QCursor::pos( ) );
+   delete m;
+}
+
+void PMGLView::slotCameraView( int id )
+{
+   int i;
+   QPtrListIterator<PMCamera> it = m_pPart->cameras( );
+
+   for( i = 0; i < id; i++ )
+      ++it;
+   if( it.current( ) )
+   {
+      setCamera( it.current( ) );
+      setType( PMGLView::PMViewCamera );
+   }
+}
+
+void PMGLView::slotObjectAction( int id )
+{
+   QPtrListIterator<PMObjectAction> it( m_objectActions );
+   PMObjectAction* oa = 0;
+
+   for( ; it.current( ) && !oa; ++it )
+      if( it.current( )->menuID( ) == id )
+         oa = it.current( );
+
+   if( oa && m_pActiveObject )
+   {
+      // otherwise no object action was selected in the context menu
+
+      m_pActiveObject->createMemento( );
+      m_pActiveObject->objectActionCalled( oa, m_controlPoints,
+                                           m_controlPointsPosition,
+                                           m_contextClickPosition );
+      PMDataChangeCommand* cmd;
+      cmd = new PMDataChangeCommand( m_pActiveObject->takeMemento( ) );
+      cmd->setText( oa->description( ) );
+      m_pPart->executeCommand( cmd );
+   }
+}
+
+void PMGLView::slotControlPoint( int id )
+{
+   PMControlPoint* p = m_controlPoints.at( id );
+   if( p )
+   {
+      PMControlPointListIterator cit( m_controlPoints );
+      for( ; cit.current( ); ++cit )
+         cit.current( )->setSelected( p == cit.current( ) );
+      emit objectChanged( m_pActiveObject, PMCControlPointSelection, this );
+   }
+}
+
+void PMGLView::slotSnapToGrid( )
+{
+   if( m_pActiveObject )
+   {
+      if( !m_pActiveObject->mementoCreated( ) )
+         m_pActiveObject->createMemento( );
+
+      PMControlPointListIterator it( m_controlPoints );
+      for( ; it.current( ); ++it )
+         if( it.current( )->selected( ) )
+            it.current( )->snapToGrid( );
+
+      m_pActiveObject->controlPointsChanged( m_controlPoints );
+
+      PMDataChangeCommand* cmd;
+      cmd = new PMDataChangeCommand( m_pActiveObject->takeMemento( ) );
+      cmd->setText( i18n( "Snap to Grid" ) );
+      m_pPart->executeCommand( cmd );
+   }
+}
+
+QString PMGLView::description( ) const
+{
+   return viewTypeAsString( m_type );
+}
+
+void PMGLView::restoreViewConfig( PMViewOptions* vo )
+{
+   if( vo && vo->viewType( ) == "glview" )
+   {
+      PMGLViewOptions* o = ( PMGLViewOptions* ) vo;
+      m_type = o->glViewType( );
+   }
+}
+
+void PMGLView::saveViewConfig( PMViewOptions* vo ) const
+{
+   if( vo && vo->viewType( ) == "glview" )
+   {
+      PMGLViewOptions* o = ( PMGLViewOptions* ) vo;
+      o->setGLViewType( m_type );
+   }
+}
+
+void PMGLViewOptions::loadData( QDomElement& e )
+{
+   QString s = e.attribute( "type", "Camera" );
+   if( s == "Camera" ) m_glViewType = PMGLView::PMViewCamera;
+   else if( s == "X" ) m_glViewType = PMGLView::PMViewPosX;
+   else if( s == "Y" ) m_glViewType = PMGLView::PMViewPosY;
+   else if( s == "Z" ) m_glViewType = PMGLView::PMViewPosZ;
+   else if( s == "NegX" ) m_glViewType = PMGLView::PMViewNegX;
+   else if( s == "NegY" ) m_glViewType = PMGLView::PMViewNegY;
+   else if( s == "NegZ" ) m_glViewType = PMGLView::PMViewNegZ;
+}
+
+void PMGLViewOptions::saveData( QDomElement& e )
+{
+   switch( m_glViewType )
+   {
+      case PMGLView::PMViewCamera:
+         e.setAttribute( "type", "Camera" );
+         break;
+      case PMGLView::PMViewPosX:
+         e.setAttribute( "type", "X" );
+         break;
+      case PMGLView::PMViewPosY:
+         e.setAttribute( "type", "Y" );
+         break;
+      case PMGLView::PMViewPosZ:
+         e.setAttribute( "type", "Z" );
+         break;
+      case PMGLView::PMViewNegX:
+         e.setAttribute( "type", "NegX" );
+         break;
+      case PMGLView::PMViewNegY:
+         e.setAttribute( "type", "NegY" );
+         break;
+      case PMGLView::PMViewNegZ:
+         e.setAttribute( "type", "NegZ" );
+         break;
+      default:
+         kdError( PMArea ) << i18n( "Unknown GL view type." )
+                           << endl;
+         break;
+   }
+}
+
+QString PMGLViewFactory::description( ) const
+{
+   return i18n( "3D View" );
+}
+
+QString PMGLViewFactory::description( PMViewOptions* vo ) const
+{
+   if( vo && vo->viewType( ) == "glview" )
+   {
+      PMGLViewOptions* o = ( PMGLViewOptions* ) vo;
+      return i18n( "3D View (%1)" ).arg(
+         PMGLView::viewTypeAsString( o->glViewType( ) ) );
+   }
+   return description( );
+}
+
+PMViewOptionsWidget* PMGLViewFactory::newOptionsWidget( QWidget* parent,
+                                                        PMViewOptions* o )
+{
+   return new PMGLViewOptionsWidget( parent, o );
+}
+
+PMViewOptions* PMGLViewFactory::newOptionsInstance( ) const
+{
+   PMGLViewOptions* o = new PMGLViewOptions( );
+   return o;
+}
+
+PMGLViewOptionsWidget::PMGLViewOptionsWidget( QWidget* parent,
+                                              PMViewOptions* o )
+      : PMViewOptionsWidget( parent )
+{
+   m_pOptions = ( PMGLViewOptions* ) o;
+
+   QHBoxLayout* hl = new QHBoxLayout( this, 0, KDialog::spacingHint( ) );
+   QLabel* l = new QLabel( i18n( "3D view type:" ), this );
+   hl->addWidget( l );
+
+   m_pGLViewType = new QComboBox( false, this );
+   m_pGLViewType->insertItem( i18n( "Top" ) );
+   m_pGLViewType->insertItem( i18n( "Bottom" ) );
+   m_pGLViewType->insertItem( i18n( "Left" ) );
+   m_pGLViewType->insertItem( i18n( "Right" ) );
+   m_pGLViewType->insertItem( i18n( "Front" ) );
+   m_pGLViewType->insertItem( i18n( "Back" ) );
+   m_pGLViewType->insertItem( i18n( "Camera" ) );
+
+   switch( m_pOptions->glViewType( ) )
+   {
+      case PMGLView::PMViewNegY:
+         m_pGLViewType->setCurrentItem( 0 );
+         break;
+      case PMGLView::PMViewPosY:
+         m_pGLViewType->setCurrentItem( 1 );
+         break;
+      case PMGLView::PMViewPosX:
+         m_pGLViewType->setCurrentItem( 2 );
+         break;
+      case PMGLView::PMViewNegX:
+         m_pGLViewType->setCurrentItem( 3 );
+         break;
+      case PMGLView::PMViewPosZ:
+         m_pGLViewType->setCurrentItem( 4 );
+         break;
+      case PMGLView::PMViewNegZ:
+         m_pGLViewType->setCurrentItem( 5 );
+         break;
+      case PMGLView::PMViewCamera:
+         m_pGLViewType->setCurrentItem( 6 );
+         break;
+   }
+
+   connect( m_pGLViewType, SIGNAL( activated( int ) ),
+            SLOT( slotGLViewTypeChanged( int ) ) );
+   hl->addWidget( m_pGLViewType );
+}
+
+void PMGLViewOptionsWidget::slotGLViewTypeChanged( int index )
+{
+   switch( index )
+   {
+      case 0:
+         m_pOptions->setGLViewType( PMGLView::PMViewNegY );
+         break;
+      case 1:
+         m_pOptions->setGLViewType( PMGLView::PMViewPosY );
+         break;
+      case 2:
+         m_pOptions->setGLViewType( PMGLView::PMViewPosX );
+         break;
+      case 3:
+         m_pOptions->setGLViewType( PMGLView::PMViewNegX );
+         break;
+      case 4:
+         m_pOptions->setGLViewType( PMGLView::PMViewPosZ );
+         break;
+      case 5:
+         m_pOptions->setGLViewType( PMGLView::PMViewNegZ );
+         break;
+      case 6:
+         m_pOptions->setGLViewType( PMGLView::PMViewCamera );
+         break;
+   }
+   emit viewTypeDescriptionChanged( );
+}
+
 #include "pmglview.moc"
