@@ -21,13 +21,10 @@
 #include "pmpalettevalue.h"
 #include "pmpalettevaluememento.h"
 
-#include "pmoutputdevice.h"
 #include "pmxmlhelper.h"
 #include "pmcompositeobject.h"
 #include "pmmemento.h"
-
-#include <kdebug.h>
-#include "pmglobals.h"
+#include "pmenumproperty.h"
 
 #include <klocale.h>
 
@@ -41,8 +38,22 @@ const bool onceDefault = false;
 const PMImageMap::PMMapType mapTypeDefault = PMImageMap::MapPlanar;
 const PMImageMap::PMInterpolateType interpolateTypeDefault = PMImageMap::InterpolateNone;
 
-PMImageMap::PMImageMap( )
-      : Base( )
+PMDefinePropertyClass( PMImageMap, PMProperty );
+PMDefineEnumPropertyClass( PMImageMap, PMImageMap::PMBitmapType,
+                           PMBitmapTypeProperty );
+PMDefineEnumPropertyClass( PMImageMap, PMImageMap::PMInterpolateType,
+                           PMInterpolateTypeProperty );
+PMDefineEnumPropertyClass( PMImageMap, PMImageMap::PMMapType,
+                           PMMapTypeProperty );
+
+PMMetaObject* PMImageMap::s_pMetaObject = 0;
+PMObject* createNewImageMap( PMPart* part )
+{
+   return new PMImageMap( part );
+}
+
+PMImageMap::PMImageMap( PMPart* part )
+      : Base( part )
 {
    m_bitmapType = bitmapTypeDefault;
    m_bitmapFile = bitmapFileDefault;
@@ -55,111 +66,26 @@ PMImageMap::PMImageMap( )
    m_interpolateType = interpolateTypeDefault;
 }
 
+PMImageMap::PMImageMap( const PMImageMap& m )
+      : Base( m )
+{
+   m_bitmapType = m.m_bitmapType;
+   m_bitmapFile = m.m_bitmapFile;
+   m_enableFilterAll = m.m_enableFilterAll;
+   m_filterAll = m.m_filterAll;
+   m_enableTransmitAll = m.m_enableTransmitAll;
+   m_transmitAll = m.m_transmitAll;
+   m_once = m.m_once;
+   m_mapType = m.m_mapType;
+   m_interpolateType = m.m_interpolateType;
+   m_filters = m.m_filters;
+   m_transmits = m.m_transmits;
+}
 
 PMImageMap::~PMImageMap( )
 {
 }
 
-void PMImageMap::serialize( PMOutputDevice& dev ) const
-{
-   typedef QValueList<PMPaletteValue> PMPaletteValueList;
-   PMPaletteValueList::ConstIterator tmpPalette;
-   QString str1, str2;
-
-   dev.objectBegin( "image_map" );
-
-   switch( m_bitmapType )
-   {
-      case PMImageMap::BitmapGif:
-         dev.writeLine( "gif" );
-         break;
-      case PMImageMap::BitmapTga:
-         dev.writeLine( "tga" );
-         break;
-      case PMImageMap::BitmapIff:
-         dev.writeLine( "iff" );
-         break;
-      case PMImageMap::BitmapPpm:
-         dev.writeLine( "ppm" );
-         break;
-      case PMImageMap::BitmapPgm:
-         dev.writeLine( "pgm" );
-         break;
-      case PMImageMap::BitmapPng:
-         dev.writeLine( "png" );
-         break;
-      case PMImageMap::BitmapJpeg:
-         dev.writeLine( "jpeg" );
-         break;
-      case PMImageMap::BitmapTiff:
-         dev.writeLine( "tiff" );
-         break;
-      case PMImageMap::BitmapSys:
-         dev.writeLine( "sys" );
-         break;
-   }
-
-   dev.writeLine( "\"" + m_bitmapFile + "\"" );
-
-   for( tmpPalette = m_filters.begin( ); tmpPalette != m_filters.end( ); ++tmpPalette )
-   {
-      str1.setNum( ( *tmpPalette ).index( ) );
-      str2.setNum( ( *tmpPalette ).value( ) );
-      dev.writeLine( "filter " + str1 + ", " + str2 );
-   }
-
-   for( tmpPalette = m_transmits.begin( ); tmpPalette != m_transmits.end( ); ++tmpPalette )
-   {
-      str1.setNum( ( *tmpPalette ).index( ) );
-      str2.setNum( ( *tmpPalette ).value( ) );
-      dev.writeLine( "transmit " + str1 + ", " + str2 );
-   }
-
-   if( m_enableFilterAll )
-   {
-      str1.setNum( m_filterAll );
-      dev.writeLine( "filter all " + str1 );
-   }
-      
-   if( m_enableTransmitAll )
-   {
-      str1.setNum( m_transmitAll );
-      dev.writeLine( "transmit all " + str1 );
-   }
-      
-   if( m_once )
-      dev.writeLine( "once" );
-
-   switch( m_mapType )
-   {
-      case PMImageMap::MapPlanar:
-         dev.writeLine( "map_type 0" );
-         break;
-      case PMImageMap::MapSpherical:
-         dev.writeLine( "map_type 1" );
-         break;
-      case PMImageMap::MapCylindrical:
-         dev.writeLine( "map_type 2" );
-         break;
-      case PMImageMap::MapToroidal:
-         dev.writeLine( "map_type 5" );
-         break;
-   }
-   switch( m_interpolateType )
-   { 
-      case PMImageMap::InterpolateBilinear:
-         dev.writeLine( "interpolate 2" );
-	 break;
-      case PMImageMap::InterpolateNormalized:
-         dev.writeLine( "interpolate 4" );
-	 break;
-      default:
-         break;
-   }
-
-   dev.objectEnd( );
-}
-
 void PMImageMap::serialize( QDomElement& e, QDomDocument& doc ) const
 {
    switch( m_bitmapType )
@@ -212,7 +138,7 @@ void PMImageMap::serialize( QDomElement& e, QDomDocument& doc ) const
       data.appendChild( p );
    }
    extra_data.appendChild( data );
-   
+
    data = doc.createElement( "indexed_transmits" );
    for( it = m_transmits.begin( ); it != m_transmits.end( ); ++it )
    {
@@ -328,7 +254,7 @@ void PMImageMap::readAttributes( const PMXMLHelper& h )
          c = c.nextSibling( );
       }
    }
-   
+
    str = h.stringAttribute( "map_type", "planar" );
    if( str == "planar" )
       m_mapType = MapPlanar;
@@ -349,11 +275,73 @@ void PMImageMap::readAttributes( const PMXMLHelper& h )
    Base::readAttributes( h );
 }
 
-bool PMImageMap::isA( PMObjectType t ) const
+PMMetaObject* PMImageMap::metaObject( ) const
 {
-   if( t == PMTImageMap )
-      return true;
-   return Base::isA( t );
+   if( !s_pMetaObject )
+   {
+      s_pMetaObject = new PMMetaObject( "ImageMap", Base::metaObject( ),
+                                        createNewImageMap );
+
+      PMBitmapTypeProperty* bp = new PMBitmapTypeProperty(
+         "bitmapType", &PMImageMap::setBitmapType, &PMImageMap::bitmapType );
+      bp->addEnumValue( "Gif", BitmapGif );
+      bp->addEnumValue( "Tga", BitmapTga );
+      bp->addEnumValue( "Iff", BitmapIff );
+      bp->addEnumValue( "Ppm", BitmapPpm );
+      bp->addEnumValue( "Pgm", BitmapPgm );
+      bp->addEnumValue( "Png", BitmapPng );
+      bp->addEnumValue( "Jpeg", BitmapJpeg );
+      bp->addEnumValue( "Tiff", BitmapTiff );
+      bp->addEnumValue( "Sys", BitmapSys );
+      s_pMetaObject->addProperty( bp );
+
+      PMInterpolateTypeProperty* ip = new PMInterpolateTypeProperty(
+         "interpolateType", &PMImageMap::setInterpolateType,
+         &PMImageMap::interpolateType );
+      ip->addEnumValue( "None", InterpolateNone );
+      ip->addEnumValue( "Bilinear", InterpolateBilinear );
+      ip->addEnumValue( "Normalized", InterpolateNormalized );
+      s_pMetaObject->addProperty( ip );
+
+      PMMapTypeProperty* mp = new PMMapTypeProperty(
+         "mapType", &PMImageMap::setMapType, &PMImageMap::mapType );
+      mp->addEnumValue( "Planar", MapPlanar );
+      mp->addEnumValue( "Spherical", MapSpherical );
+      mp->addEnumValue( "Cylindrical", MapCylindrical );
+      mp->addEnumValue( "Toroidal", MapToroidal );
+      s_pMetaObject->addProperty( mp );
+
+      s_pMetaObject->addProperty(
+         new PMProperty( "bitmapFile", &PMImageMap::setBitmapFileName,
+                         &PMImageMap::bitmapFile ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "filterAllEnabled", &PMImageMap::enableFilterAll,
+                         &PMImageMap::isFilterAllEnabled ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "filterAll", &PMImageMap::setFilterAll,
+                         &PMImageMap::filterAll ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "transmitAllEnabled", &PMImageMap::enableTransmitAll,
+                         &PMImageMap::isTransmitAllEnabled ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "transmitAll", &PMImageMap::setTransmitAll,
+                         &PMImageMap::transmitAll ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "once", &PMImageMap::enableOnce, &PMImageMap::isOnceEnabled ) );
+
+      // TODO: filters and transmits properties
+   }
+   return s_pMetaObject;
+}
+
+void PMImageMap::cleanUp( ) const
+{
+   if( s_pMetaObject )
+   {
+      delete s_pMetaObject;
+      s_pMetaObject = 0;
+   }
+   Base::cleanUp( );
 }
 
 QString PMImageMap::description( ) const
@@ -366,9 +354,9 @@ void PMImageMap::setBitmapType( PMBitmapType c )
    if( c != m_bitmapType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTImageMap, PMBitmapTypeID, m_bitmapType );
+         m_pMemento->addData( s_pMetaObject, PMBitmapTypeID, m_bitmapType );
       m_bitmapType = c;
-   } 
+   }
 }
 
 void PMImageMap::setBitmapFileName( const QString& c )
@@ -376,9 +364,9 @@ void PMImageMap::setBitmapFileName( const QString& c )
    if( c != m_bitmapFile )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTImageMap, PMBitmapFileID, m_bitmapFile );
+         m_pMemento->addData( s_pMetaObject, PMBitmapFileID, m_bitmapFile );
       m_bitmapFile = c;
-   } 
+   }
 }
 
 void PMImageMap::enableFilterAll( bool c )
@@ -386,9 +374,9 @@ void PMImageMap::enableFilterAll( bool c )
    if( c != m_enableFilterAll )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTImageMap, PMEnableFilterAllID, m_enableFilterAll );
+         m_pMemento->addData( s_pMetaObject, PMEnableFilterAllID, m_enableFilterAll );
       m_enableFilterAll = c;
-   } 
+   }
 }
 
 void PMImageMap::enableTransmitAll( bool c )
@@ -396,9 +384,9 @@ void PMImageMap::enableTransmitAll( bool c )
    if( c != m_enableTransmitAll )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTImageMap, PMEnableTransmitAllID, m_enableTransmitAll );
+         m_pMemento->addData( s_pMetaObject, PMEnableTransmitAllID, m_enableTransmitAll );
       m_enableTransmitAll = c;
-   } 
+   }
 }
 
 void PMImageMap::setFilterAll( double c )
@@ -406,9 +394,9 @@ void PMImageMap::setFilterAll( double c )
    if( c != m_filterAll )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTImageMap, PMFilterAllID, m_filterAll );
+         m_pMemento->addData( s_pMetaObject, PMFilterAllID, m_filterAll );
       m_filterAll = c;
-   } 
+   }
 }
 
 void PMImageMap::setTransmitAll( double c )
@@ -416,9 +404,9 @@ void PMImageMap::setTransmitAll( double c )
    if( c != m_transmitAll )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTImageMap, PMFilterAllID, m_transmitAll );
+         m_pMemento->addData( s_pMetaObject, PMFilterAllID, m_transmitAll );
       m_transmitAll = c;
-   } 
+   }
 }
 
 void PMImageMap::enableOnce( bool c )
@@ -426,9 +414,9 @@ void PMImageMap::enableOnce( bool c )
    if( c != m_once )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTImageMap, PMOnceID, m_once );
+         m_pMemento->addData( s_pMetaObject, PMOnceID, m_once );
       m_once = c;
-   } 
+   }
 }
 
 void PMImageMap::setMapType( PMMapType c )
@@ -436,9 +424,9 @@ void PMImageMap::setMapType( PMMapType c )
    if( c != m_mapType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTImageMap, PMMapTypeID, m_mapType );
+         m_pMemento->addData( s_pMetaObject, PMMapTypeID, m_mapType );
       m_mapType = c;
-   } 
+   }
 }
 
 void PMImageMap::setInterpolateType( PMInterpolateType c )
@@ -446,9 +434,9 @@ void PMImageMap::setInterpolateType( PMInterpolateType c )
    if( c != m_interpolateType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTImageMap, PMInterpolateID, m_interpolateType );
+         m_pMemento->addData( s_pMetaObject, PMInterpolateID, m_interpolateType );
       m_interpolateType = c;
-   } 
+   }
 }
 
 void PMImageMap::setFilters( const QValueList<PMPaletteValue>& c )
@@ -485,7 +473,7 @@ void PMImageMap::restoreMemento( PMMemento* s )
    for( ; it.current( ); ++it )
    {
       data = it.current( );
-      if( data->objectType( ) == PMTImageMap )
+      if( data->objectType( ) == s_pMetaObject )
       {
          switch( data->valueID( ) )
          {
