@@ -18,7 +18,6 @@
 
 #include "pmlathe.h"
 
-#include "pmoutputdevice.h"
 #include "pmxmlhelper.h"
 #include "pmlatheedit.h"
 #include "pmmemento.h"
@@ -27,9 +26,8 @@
 #include "pmsplinememento.h"
 #include "pmsplinesegment.h"
 #include "pmdefaults.h"
-
-#include <kdebug.h>
-#include "pmglobals.h"
+#include "pmenumproperty.h"
+#include "pmobjectaction.h"
 
 #include <klocale.h>
 
@@ -45,12 +43,81 @@ const PMVector defaultPoint[defaultNumberOfPoints] =
 const bool defaultSturm = false;
 const PMLathe::SplineType defaultSplineType = PMLathe::LinearSpline;
 
+PMDefinePropertyClass( PMLathe, PMProperty );
+PMDefineEnumPropertyClass( PMLathe, PMLathe::SplineType, PMSplineTypeProperty );
+
+PMMetaObject* PMLathe::s_pMetaObject = 0;
+PMObject* createNewLathe( PMPart* part )
+{
+   return new PMLathe( part );
+}
+
+class PMPointProperty : public PMPropertyBase
+{
+public:
+   PMPointProperty( )
+         : PMPropertyBase( "splinePoints", PMVariant::Vector )
+   {
+      m_index = 0;
+   }
+   virtual int dimensions( ) const { return 1; }
+   virtual void setIndex( int /*dimension*/, int index )
+   {
+      m_index = index;
+   }
+   virtual int size( PMObject* object, int /*dimension*/ ) const
+   {
+      return ( ( PMLathe* ) object )->numberOfPoints( );
+   }
+protected:
+   virtual bool setProtected( PMObject* obj, const PMVariant& var )
+   {
+      PMLathe* p = ( PMLathe* ) obj;
+      QValueList<PMVector> list = p->points( );
+      QValueList<PMVector>::Iterator it = list.begin( );
+      int i;
+      PMVector v = var.vectorData( );
+      v.resize( 2 );
+
+      for( i = 0; i < m_index && it != list.end( ); i++ )
+         ++it;
+      // expand the list if necessary
+      for( ; i < m_index; i++ )
+         list.insert( it, v );
+      if( it == list.end( ) )
+         it = list.insert( it, v );
+      else
+         *it = v;
+
+      p->setPoints( list );
+      return true;
+   }
+   virtual PMVariant getProtected( const PMObject* obj )
+   {
+      PMLathe* p = ( PMLathe* ) obj;
+      QValueList<PMVector> list = p->points( );
+      QValueList<PMVector>::ConstIterator it = list.at( m_index );
+
+      if( it == list.end( ) )
+      {
+         kdError( PMArea ) << "Range error in PMLathe::PointProperty::get" << endl;
+         return PMVariant( );
+      }
+
+      return PMVariant( *it );
+   }
+
+private:
+   int m_index;
+};
+
+
 int PMLathe::s_rSteps = c_defaultLatheRSteps;
 int PMLathe::s_sSteps = c_defaultLatheSSteps;
 int PMLathe::s_parameterKey = 0;
 
-PMLathe::PMLathe( )
-      : Base( )
+PMLathe::PMLathe( PMPart* part )
+      : Base( part )
 {
    int i;
 
@@ -60,6 +127,14 @@ PMLathe::PMLathe( )
    m_sturm = defaultSturm;
 }
 
+PMLathe::PMLathe( const PMLathe& l )
+      : Base( l )
+{
+   m_points = l.m_points;
+   m_splineType = l.m_splineType;
+   m_sturm = l.m_sturm;
+}
+
 PMLathe::~PMLathe( )
 {
 }
@@ -69,49 +144,6 @@ QString PMLathe::description( ) const
    return i18n( "lathe" );
 }
 
-void PMLathe::serialize( PMOutputDevice& dev ) const
-{
-   dev.objectBegin( "lathe" );
-
-   serializeName( dev );
-
-   switch( m_splineType )
-   {
-      case LinearSpline:
-         dev.writeLine( "linear_spline" );
-         break;
-      case QuadraticSpline:
-         dev.writeLine( "quadratic_spline" );
-         break;
-      case CubicSpline:
-         dev.writeLine( "cubic_spline" );
-         break;
-      case BezierSpline:
-         dev.writeLine( "bezier_spline" );
-         break;
-   }
-   
-   int num = m_points.count( );
-   dev.writeLine( QString( "%1," ).arg( num ) );
-   
-   bool first = true;
-   QValueList<PMVector>::ConstIterator it = m_points.begin( );
-   for( ; it != m_points.end( ); ++it )
-   {
-      if( !first )
-         dev.write( ", " );
-      dev.write( ( *it ).serialize( ) );
-      first = false;
-   }
-   dev.writeLine( "" );
-
-   if( m_sturm )
-      dev.writeLine( "sturm" );
-   
-   Base::serialize( dev );
-   dev.objectEnd( );
-}
-
 void PMLathe::serialize( QDomElement& e, QDomDocument& doc ) const
 {
    QDomElement data = doc.createElement( "extra_data" );
@@ -166,11 +198,34 @@ void PMLathe::readAttributes( const PMXMLHelper& h )
    Base::readAttributes( h );
 }
 
-bool PMLathe::isA( PMObjectType t ) const
+PMMetaObject* PMLathe::metaObject( ) const
 {
-   if( t == PMTLathe )
-      return true;
-   return Base::isA( t );
+   if( !s_pMetaObject )
+   {
+      s_pMetaObject = new PMMetaObject( "Lathe", Base::metaObject( ),
+                                        createNewLathe );
+      s_pMetaObject->addProperty(
+         new PMProperty( "sturm", &PMLathe::setSturm, &PMLathe::sturm ) );
+      PMSplineTypeProperty* p = new PMSplineTypeProperty(
+         "splineType", &PMLathe::setSplineType, &PMLathe::splineType );
+      p->addEnumValue( "LinearSpline", LinearSpline );
+      p->addEnumValue( "QuadraticSpline", QuadraticSpline );
+      p->addEnumValue( "CubicSpline", CubicSpline );
+      p->addEnumValue( "BezierSpline", BezierSpline );
+      s_pMetaObject->addProperty( p );
+      s_pMetaObject->addProperty( new PMPointProperty( ) );
+   }
+   return s_pMetaObject;
+}
+
+void PMLathe::cleanUp( ) const
+{
+   if( s_pMetaObject )
+   {
+      delete s_pMetaObject;
+      s_pMetaObject = 0;
+   }
+   Base::cleanUp( );
 }
 
 void PMLathe::setSplineType( PMLathe::SplineType t )
@@ -178,7 +233,7 @@ void PMLathe::setSplineType( PMLathe::SplineType t )
    if( m_splineType != t )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLathe, PMSplineTypeID, ( int ) m_splineType );
+         m_pMemento->addData( s_pMetaObject, PMSplineTypeID, ( int ) m_splineType );
       setViewStructureChanged( );
       m_splineType = t;
    }
@@ -189,7 +244,7 @@ void PMLathe::setSturm( bool s )
    if( m_sturm != s )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLathe, PMSturmID, m_sturm );
+         m_pMemento->addData( s_pMetaObject, PMSturmID, m_sturm );
       m_sturm = s;
    }
 }
@@ -200,7 +255,7 @@ void PMLathe::setPoints( const QValueList<PMVector>& points )
    {
       if( m_pMemento )
          ( ( PMSplineMemento* ) m_pMemento )->setSplinePoints( m_points );
-      
+
       setViewStructureChanged( );
       m_points = points;
    }
@@ -227,7 +282,7 @@ void PMLathe::restoreMemento( PMMemento* s )
    for( ; it.current( ); ++it )
    {
       data = it.current( );
-      if( data->objectType( ) == PMTLathe )
+      if( data->objectType( ) == s_pMetaObject )
       {
          switch( data->valueID( ) )
          {
@@ -245,7 +300,7 @@ void PMLathe::restoreMemento( PMMemento* s )
    }
    if( m->splinePointsSaved( ) )
       setPoints( m->splinePoints( ) );
-   
+
    Base::restoreMemento( s );
 }
 
@@ -290,7 +345,7 @@ void PMLathe::createViewStructure( )
       vsl = ( 2 * vsp - 1 ) * s_rSteps;
    else
       vsl = ns * ( ( 2 * s_sSteps + 1 ) * s_rSteps );
-   
+
    vsp *= s_rSteps;
 
    if( m_pViewStructure )
@@ -298,7 +353,7 @@ void PMLathe::createViewStructure( )
       if( m_pViewStructure->points( ).size( ) != ( unsigned ) vsp )
          m_pViewStructure->points( ).resize( vsp );
       if( m_pViewStructure->lines( ).size( ) != ( unsigned ) vsl )
-         m_pViewStructure->lines( ).resize( vsl );      
+         m_pViewStructure->lines( ).resize( vsl );
    }
    else
       m_pViewStructure = new PMViewStructure( vsp, vsl );
@@ -308,7 +363,7 @@ void PMLathe::createViewStructure( )
    QValueList<PMSplineSegment> segments;
    QValueList<PMVector>::Iterator it1, it2, it3, it4;
    it1 = m_points.begin( );
-   it2 = it1; ++it2; 
+   it2 = it1; ++it2;
    it3 = it2; ++it3;
    it4 = it3; ++it4;
    PMSplineSegment s;
@@ -407,7 +462,7 @@ void PMLathe::createViewStructure( )
    PMVector point2, point3;
    QValueList<PMSplineSegment>::Iterator sit = segments.begin( );
    int pi = 0;
-   
+
    double poffset = 1.0 / s_sSteps;
    PMMatrix rot = PMMatrix::rotation( 0.0, M_PI * 2.0 / s_rSteps, 0.0 );
    PMPointArray& points = m_pViewStructure->points( );
@@ -422,7 +477,7 @@ void PMLathe::createViewStructure( )
             point3[0] = point2[0];
             point3[1] = point2[1];
             point3[2] = 0.0;
-         
+
             for( r = 0; r < s_rSteps; r++ )
             {
                points[pi] = PMPoint( point3 );
@@ -437,7 +492,7 @@ void PMLathe::createViewStructure( )
             point3[0] = point2[0];
             point3[1] = point2[1];
             point3[2] = 0.0;
-         
+
             for( r = 0; r < s_rSteps; r++ )
             {
                points[pi] = PMPoint( point3 );
@@ -458,7 +513,7 @@ void PMLathe::createViewStructure( )
             point3[0] = point2[0];
             point3[1] = point2[1];
             point3[2] = 0.0;
-         
+
             for( r = 0; r < s_rSteps; r++ )
             {
                points[pi] = PMPoint( point3 );
@@ -476,9 +531,9 @@ void PMLathe::controlPoints( PMControlPointList& list )
    QValueList<PMVector>::Iterator it;
    int i, d;
 
-   PM2DControlPoint::CPType type = PM2DControlPoint::PM2DXY;
    PM2DControlPoint* cp = 0;
-   
+   QPtrList<PM2DControlPoint> tmp[2];
+
    for( d = 0; d < 2; d++ )
    {
       if( m_splineType != BezierSpline )
@@ -489,14 +544,19 @@ void PMLathe::controlPoints( PMControlPointList& list )
          for( it = m_points.begin( ), i = 0; it != m_points.end( ); ++it, i++ )
          {
             lastPoint = cp;
-            cp = new PM2DControlPoint( *it, type, i,
-                                    i18n( "Point %1 (xy)" ).arg( i + 1 ) );
+            if( d == 0 )
+               cp = new PM2DControlPoint( *it, PM2DControlPoint::PM2DXY, i,
+                                          i18n( "Point %1 (xy)" ).arg( i + 1 ) );
+            else
+               cp = new PM2DControlPoint( *it, PM2DControlPoint::PM2DZY, i,
+                                          i18n( "Point %1 (xy)" ).arg( i + 1 ) );
+
             if( i == 0 )
                firstPoint = cp;
             if( ( i == 1 ) && ( m_splineType != LinearSpline ) )
                firstPoint->setBasePoint( cp );
-            
-            list.append( cp );
+
+            tmp[d].append( cp );
          }
          if( m_splineType == CubicSpline )
             cp->setBasePoint( lastPoint );
@@ -508,8 +568,12 @@ void PMLathe::controlPoints( PMControlPointList& list )
          for( it = m_points.begin( ), i = 0; it != m_points.end( ); ++it, i++ )
          {
             int imod4 = i % 4;
-            cp = new PM2DControlPoint( *it, type, i,
-                                    i18n( "Point %1 (xy)" ).arg( i + 1 ) );
+            if( d == 0 )
+               cp = new PM2DControlPoint( *it, PM2DControlPoint::PM2DXY, i,
+                                          i18n( "Point %1 (xy)" ).arg( i + 1 ) );
+            else
+               cp = new PM2DControlPoint( *it, PM2DControlPoint::PM2DZY, i,
+                                          i18n( "Point %1 (xy)" ).arg( i + 1 ) );
             switch( imod4 )
             {
                case 0:
@@ -527,13 +591,23 @@ void PMLathe::controlPoints( PMControlPointList& list )
                default:
                   break;
             }
-                  
-            list.append( cp );
+
+            tmp[d].append( cp );
          }
       }
-      
-      type = PM2DControlPoint::PM2DZY;
    }
+
+   QPtrListIterator<PM2DControlPoint> cit1( tmp[0] ), cit2( tmp[1] );
+
+   for( ; cit1.current( ) && cit2.current( ); ++cit1, ++cit2 )
+   {
+      ( *cit1 )->setLatheLink( *cit2 );
+      ( *cit2 )->setLatheLink( *cit1 );
+   }
+   for( cit1.toFirst( ); cit1.current( ); ++cit1 )
+      list.append( *cit1 );
+   for( cit2.toFirst( ); cit2.current( ); ++cit2 )
+      list.append( *cit2 );
 }
 
 void PMLathe::controlPointsChanged( PMControlPointList& list )
@@ -543,12 +617,12 @@ void PMLathe::controlPointsChanged( PMControlPointList& list )
    PM2DControlPoint* p1;
    PM2DControlPoint* p2;
    bool firstChange = true;
-   
+
    for( it2 += list.count( ) / 2; it2.current( ); ++it1, ++it2, ++pit )
    {
       p1 = ( PM2DControlPoint* ) it1.current( );
       p2 = ( PM2DControlPoint* ) it2.current( );
-      
+
       if( p1->changed( ) )
       {
          if( firstChange )
@@ -589,12 +663,12 @@ void PMLathe::addObjectActions( const PMControlPointList& /*cp*/,
 {
    PMObjectAction* a;
 
-   a = new PMObjectAction( PMTLathe, PMSplitSegmentID,
-                           i18n( "Split Segment" ) );
+   a = new PMObjectAction( s_pMetaObject, PMSplitSegmentID,
+                           i18n( "Add Point" ) );
    actions.append( a );
-   
-   a = new PMObjectAction( PMTLathe, PMJoinSegmentsID,
-                           i18n( "Join Segments" ) );
+
+   a = new PMObjectAction( s_pMetaObject, PMJoinSegmentsID,
+                           i18n( "Remove Point" ) );
    int np = m_points.count( );
    int minp =  3;
    switch( m_splineType )
@@ -612,7 +686,7 @@ void PMLathe::addObjectActions( const PMControlPointList& /*cp*/,
          minp = 8;
          break;
    }
-   
+
    if( np < minp )
       a->setEnabled( false );
    actions.append( a );
@@ -623,7 +697,7 @@ void PMLathe::objectActionCalled( const PMObjectAction* action,
                                   const QPtrList<PMVector>& cpViewPosition,
                                   const PMVector& clickPosition )
 {
-   if( action->objectType( ) == PMTLathe )
+   if( action->objectType( ) == s_pMetaObject )
    {
       switch( action->actionID( ) )
       {
@@ -684,7 +758,7 @@ void PMLathe::splitSegment( const PMControlPointList& /*cp*/,
             dist[1] = mid[1];
             dist -= clickPosition;
             abs = dist.abs( );
-            
+
             if( ( minabs > abs ) || ( ns < 0 ) )
             {
                minabs = abs;
@@ -738,7 +812,7 @@ void PMLathe::splitSegment( const PMControlPointList& /*cp*/,
          break;
    }
 
-   mid = segment.point( 0.5 );   
+   mid = segment.point( 0.5 );
    if( m_splineType != BezierSpline )
    {
       ++it;
@@ -750,7 +824,7 @@ void PMLathe::splitSegment( const PMControlPointList& /*cp*/,
       ++it;
       *it = end + ( *it - end ) / 2.0;
       ++it;
-      
+
       PMVector grad = segment.gradient( 0.5 ) / 4.0;
 
       newPoints.insert( it, mid - grad );
@@ -795,7 +869,7 @@ void PMLathe::joinSegments( const PMControlPointList& /*cp*/,
       kdError( PMArea ) << "Not enough points in PMLathe::joinSegments\n";
       return;
    }
-   
+
    double abs = 0.0, minabs = 1e10;
    int ns = -1;
    int i, j;
@@ -813,7 +887,7 @@ void PMLathe::joinSegments( const PMControlPointList& /*cp*/,
          dist[1] = (*p)[1];
          dist -= clickPosition;
          abs = dist.abs( );
-         
+
          if( ( minabs > abs ) || ( ns < 0 ) )
          {
             minabs = abs;
@@ -844,7 +918,7 @@ void PMLathe::joinSegments( const PMControlPointList& /*cp*/,
          ns = 0;
       if( ns >= ( nump / 4 - 1 ) )
          ns = nump / 4 - 2;
-      
+
       it = newPoints.at( ns * 4 + 2 );
       for( i = 0; i < 4; i++ )
          it = newPoints.remove( it );
