@@ -2,7 +2,7 @@
 **************************************************************************
                                  description
                              --------------------
-    copyright            : (C) 2000-2001 by Andreas Zehender
+    copyright            : (C) 2000-2002 by Andreas Zehender
     email                : zehender@kde.org
 **************************************************************************
 
@@ -18,7 +18,6 @@
 
 #include "pmlight.h"
 
-#include "pmoutputdevice.h"
 #include "pmxmlhelper.h"
 #include "pmlightedit.h"
 #include "pmmemento.h"
@@ -26,9 +25,7 @@
 #include "pm3dcontrolpoint.h"
 #include "pmmath.h"
 #include "pmmatrix.h"
-
-#include <kdebug.h>
-#include "pmglobals.h"
+#include "pmenumproperty.h"
 
 #include <klocale.h>
 
@@ -38,11 +35,13 @@ const double radiusDefault = 70.0;
 const double falloffDefault = 70.0;
 const double tightnessDefault = 10;
 const PMVector pointAtDefault = PMVector( 0, 0, 1 );
+const bool parallelDefault = false;
 const PMVector areaAxis1Default = PMVector( 1, 0, 0 );
 const PMVector areaAxis2Default = PMVector( 0, 1, 0 );
 const int areaSize1Default = 3;
 const int areaSize2Default = 3;
 const int adaptiveDefault = 0;
+const bool orientDefault = false;
 const bool jitterDefault = false;
 const int fadePowerDefault = 1;
 const double fadeDistanceDefault = 10.0;
@@ -53,10 +52,20 @@ PMViewStructure* PMLight::s_pDefaultCylindricalStructure = 0;
 double PMLight::s_pointLightSize = 0.25;
 int PMLight::s_nCylinderLines = 8;
 int PMLight::s_nSpotLines = 8;
-double PMLight::s_length = 0.5;
+double PMLight::s_length = 1.0;
 
-PMLight::PMLight( )
-      : Base( )
+PMDefinePropertyClass( PMLight, PMProperty );
+PMDefineEnumPropertyClass( PMLight, PMLight::PMLightType, PMTypeProperty );
+PMDefineEnumPropertyClass( PMLight, PMLight::PMAreaType, PMAreaProperty );
+
+PMMetaObject* PMLight::s_pMetaObject = 0;
+PMObject* createNewLight( PMPart* part )
+{
+   return new PMLight( part );
+}
+
+PMLight::PMLight( PMPart* part )
+      : Base( part )
 {
    m_location = locationDefault;
    m_color = colorDefault;
@@ -65,12 +74,15 @@ PMLight::PMLight( )
    m_falloff = falloffDefault;
    m_tightness = tightnessDefault;
    m_pointAt = pointAtDefault;
+   m_parallel = parallelDefault;
    m_bAreaLight = false;
+   m_areaType = Rectangular;
    m_areaAxis1 = areaAxis1Default;
    m_areaAxis2 = areaAxis2Default;
    m_areaSize1 = areaSize1Default;
    m_areaSize2 = areaSize2Default;
    m_adaptive = adaptiveDefault;
+   m_orient = orientDefault;
    m_jitter = jitterDefault;
    m_bFading = false;
    m_fadeDistance = fadeDistanceDefault;
@@ -79,6 +91,33 @@ PMLight::PMLight( )
    m_bMediaAttenuation = true;
 }
 
+PMLight::PMLight( const PMLight& l )
+      : Base( l )
+{
+   m_location = l.m_location;
+   m_color = l.m_color;
+   m_type = l.m_type;
+   m_radius = l.m_radius;
+   m_falloff = l.m_falloff;
+   m_tightness = l.m_tightness;
+   m_pointAt = l.m_pointAt;
+   m_parallel = l.m_parallel;
+   m_bAreaLight = l.m_bAreaLight;
+   m_areaType = l.m_areaType;
+   m_areaAxis1 = l.m_areaAxis1;
+   m_areaAxis2 = l.m_areaAxis2;
+   m_areaSize1 = l.m_areaSize1;
+   m_areaSize2 = l.m_areaSize2;
+   m_adaptive = l.m_adaptive;
+   m_orient = l.m_orient;
+   m_jitter = l.m_jitter;
+   m_bFading = l.m_bFading;
+   m_fadeDistance = l.m_fadeDistance;
+   m_fadePower = l.m_fadePower;
+   m_bMediaInteraction = l.m_bMediaInteraction;
+   m_bMediaAttenuation = l.m_bMediaAttenuation;
+}
+
 PMLight::~PMLight( )
 {
 }
@@ -88,55 +127,6 @@ QString PMLight::description( ) const
    return i18n( "light" );
 }
 
-void PMLight::serialize( PMOutputDevice& dev ) const
-{
-   dev.objectBegin( QString( "light_source" ) );
-
-   serializeName( dev );
-   dev.writeLine( m_location.serialize( ) + ", " + m_color.serialize( ) );
-
-   if( m_type == SpotLight )
-      dev.writeLine( QString( "spotlight" ) );
-   else if( m_type == CylinderLight )
-      dev.writeLine( QString( "cylinder" ) );
-   else if( m_type == ShadowlessLight )
-      dev.writeLine( QString( "shadowless" ) );
-
-   if( ( m_type == SpotLight ) || ( m_type == CylinderLight ) )
-   {
-      dev.writeLine( QString( "radius %1" ).arg( m_radius ) );
-      dev.writeLine( QString( "falloff %1" ).arg( m_falloff ) );
-      if( m_tightness != tightnessDefault )
-         dev.writeLine( QString( "tightness %1" ).arg( m_tightness ) );
-      dev.writeLine( QString( "point_at " ) + m_pointAt.serialize( ) );
-   }
-
-   if( m_bAreaLight )
-   {
-      dev.writeLine( QString( "area_light " ) + m_areaAxis1.serialize( )
-                     + QString( ", " ) + m_areaAxis2.serialize( )
-                     + QString( ", %1, %2" ).arg( m_areaSize1 ).arg( m_areaSize2 ) );
-      if( m_adaptive != adaptiveDefault )
-         dev.writeLine( QString( "adaptive %1" ).arg( m_adaptive ) );
-      if( m_jitter )
-         dev.writeLine( QString( "jitter" ) );
-   }
-
-   if( m_bFading )
-   {
-      dev.writeLine( QString( "fade_distance %1" ).arg( m_fadeDistance ) );
-      dev.writeLine( QString( "fade_power %1" ).arg( m_fadePower ) );
-   }
-
-   if( !m_bMediaInteraction )
-      dev.writeLine( QString( "media_interaction off" ) );
-   if( !m_bMediaAttenuation )
-      dev.writeLine( QString( "media_attenuation off" ) );
-
-   Base::serialize( dev );
-   dev.objectEnd( );
-}
-
 void PMLight::serialize( QDomElement& e, QDomDocument& doc ) const
 {
    e.setAttribute( "location", m_location.serializeXML( ) );
@@ -166,15 +156,30 @@ void PMLight::serialize( QDomElement& e, QDomDocument& doc ) const
       e.setAttribute( "point_at", m_pointAt.serializeXML( ) );
    }
 
+   if ( m_parallel )
+      e.setAttribute( "parallel", "1" );
+   else
+      e.setAttribute( "parallel", "0" );
+
    if( m_bAreaLight )
    {
+      if ( m_areaType == Rectangular )
+         e.setAttribute( "areatype", "rectangular" );
+      else
+         e.setAttribute( "areatype", "circular" );
+
       e.setAttribute( "area_light", "1" );
       e.setAttribute( "area_light_a", m_areaAxis1.serializeXML( ) );
       e.setAttribute( "area_light_b", m_areaAxis2.serializeXML( ) );
       e.setAttribute( "area_size_a", m_areaSize1 );
       e.setAttribute( "area_size_b", m_areaSize2 );
       e.setAttribute( "adaptive", m_adaptive );
-      
+
+      if( m_orient )
+         e.setAttribute( "orient", "1" );
+      else
+         e.setAttribute( "orient", "0" );
+
       if( m_jitter )
          e.setAttribute( "jitter", "1" );
       else
@@ -196,7 +201,7 @@ void PMLight::serialize( QDomElement& e, QDomDocument& doc ) const
       e.setAttribute( "media_interaction", "1" );
    else
       e.setAttribute( "media_interaction", "0" );
-   
+
    if( m_bMediaAttenuation )
       e.setAttribute( "media_attenuation", "1" );
    else
@@ -208,7 +213,7 @@ void PMLight::serialize( QDomElement& e, QDomDocument& doc ) const
 void PMLight::readAttributes( const PMXMLHelper& h )
 {
    QString str;
-   
+
    m_location = h.vectorAttribute( "location", locationDefault );
    m_color = h.colorAttribute( "color", colorDefault );
 
@@ -232,14 +237,23 @@ void PMLight::readAttributes( const PMXMLHelper& h )
       m_pointAt = h.vectorAttribute( "point_at", pointAtDefault );
    }
 
+   m_parallel = h.boolAttribute( "parallel", parallelDefault );
+
    m_bAreaLight = h.boolAttribute( "area_light", false );
    if( m_bAreaLight )
    {
+      str = h.stringAttribute( "areatype", "rectangular" );
+      if ( str == "circular" )
+         m_areaType = Circular;
+      else
+         m_areaType = Rectangular;
+
       m_areaAxis1 = h.vectorAttribute( "area_light_a", areaAxis1Default );
       m_areaAxis2 = h.vectorAttribute( "area_light_b", areaAxis2Default );
       m_areaSize1 = h.intAttribute( "area_size_a", areaSize1Default );
       m_areaSize2 = h.intAttribute( "area_size_b", areaSize2Default );
       m_adaptive = h.intAttribute( "adaptive", adaptiveDefault );
+      m_orient = h.boolAttribute( "orient", orientDefault );
       m_jitter = h.boolAttribute( "jitter", jitterDefault );
    }
    m_bFading = h.boolAttribute( "fading", false );
@@ -250,110 +264,71 @@ void PMLight::readAttributes( const PMXMLHelper& h )
    }
    m_bMediaInteraction = h.boolAttribute( "media_interaction", true );
    m_bMediaAttenuation = h.boolAttribute( "media_attenuation", true );
-   
+
    Base::readAttributes( h );
 }
 
-bool PMLight::isA( PMObjectType t ) const
+PMMetaObject* PMLight::metaObject( ) const
 {
-   if( t == PMTLight )
-      return true;
-   return Base::isA( t );
-}
-
-bool PMLight::containsLooksLike( ) const
-{
-   bool c = false;
-   PMObject* o = firstChild( );
-   while( o && !c )
+   if( !s_pMetaObject )
    {
-      if( o->type( ) == PMTLooksLike )
-         c = true;
-      o = o->nextSibling( );
+      s_pMetaObject = new PMMetaObject( "Light", Base::metaObject( ),
+                                        createNewLight );
+      PMTypeProperty* p = new PMTypeProperty( "lightType", &PMLight::setLightType,
+                                              &PMLight::lightType );
+      p->addEnumValue( "PointLight", PointLight );
+      p->addEnumValue( "SpotLight", SpotLight );
+      p->addEnumValue( "CylinderLight", CylinderLight );
+      p->addEnumValue( "ShadowlessLight", ShadowlessLight );
+      s_pMetaObject->addProperty( p );
+
+      s_pMetaObject->addProperty(
+         new PMProperty( "location", &PMLight::setLocation, &PMLight::location ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "color", &PMLight::setColor, &PMLight::color ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "radius", &PMLight::setRadius, &PMLight::radius ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "falloff", &PMLight::setFalloff, &PMLight::falloff ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "tightness", &PMLight::setTightness, &PMLight::tightness ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "pointAt", &PMLight::setPointAt, &PMLight::pointAt ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "parallel", &PMLight::setParallel, &PMLight::parallel ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "areaLight", &PMLight::setAreaLight, &PMLight::isAreaLight ) );
+
+      PMAreaProperty* p2 = new PMAreaProperty( "areaType", &PMLight::setAreaType,
+                                               &PMLight::areaType );
+      p2->addEnumValue( "Rectangular", Rectangular );
+      p2->addEnumValue( "Circular", Circular );
+      s_pMetaObject->addProperty( p2 );
+
+      s_pMetaObject->addProperty(
+         new PMProperty( "axis1", &PMLight::setAxis1, &PMLight::axis1 ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "axis2", &PMLight::setAxis2, &PMLight::axis2 ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "adaptive", &PMLight::setAdaptive, &PMLight::adaptive ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "orient", &PMLight::setOrient, &PMLight::orient ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "jitter", &PMLight::setJitter, &PMLight::jitter ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "fading", &PMLight::setFading, &PMLight::fading ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "fadeDistance", &PMLight::setFadeDistance, &PMLight::fadeDistance ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "fadePower", &PMLight::setFadePower, &PMLight::fadePower ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "mediaInteraction", &PMLight::setMediaInteraction,
+                         &PMLight::mediaInteraction ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "mediaAttenuation", &PMLight::setMediaAttenuation,
+                         &PMLight::mediaAttenuation ) );
    }
-   return c;
-}
-
-bool PMLight::canInsert( PMObjectType t, const PMObject*,
-                         const PMObjectList* ) const
-{
-   switch( t )
-   {
-      case PMTRotate:
-      case PMTScale:
-      case PMTTranslate:
-      case PMTMatrix:
-      case PMTComment:
-      case PMTRaw:
-         return true;
-      case PMTLooksLike:
-         return !containsLooksLike( );
-      default:
-         break;
-   }
-   return false;
-}
-
-int PMLight::canInsert( const PMObjectList& list,
-                        const PMObject* ) const
-{
-   bool c = containsLooksLike( );
-   int num = 0;
-   PMObjectListIterator it( list );
-
-   for( ; it.current( ); ++it )
-   {
-      switch( it.current( )->type( ) )
-      {
-         case PMTRotate:
-         case PMTScale:
-         case PMTTranslate:
-         case PMTMatrix:
-         case PMTComment:
-         case PMTRaw:
-            num++;
-            break;
-         case PMTLooksLike:
-            if( !c )
-               num++;
-            c = true;
-            break;
-         default:
-            break;
-      }
-   }
-   return num;   
-}
-
-int PMLight::canInsert( const QValueList<PMObjectType>& list,
-                        const PMObject* ) const
-{
-   bool c = containsLooksLike( );
-   int num = 0;
-   QValueList<PMObjectType>::ConstIterator it;
-
-   for( it = list.begin( ); it != list.end( ); ++it )
-   {
-      switch( *it )
-      {
-         case PMTRotate:
-         case PMTScale:
-         case PMTTranslate:
-         case PMTMatrix:
-         case PMTComment:
-         case PMTRaw:
-            num++;
-            break;
-         case PMTLooksLike:
-            if( !c )
-               num++;
-            c = true;
-            break;
-         default:
-            break;
-      }
-   }
-   return num;   
+   return s_pMetaObject;
 }
 
 void PMLight::setLocation( const PMVector& p )
@@ -361,7 +336,7 @@ void PMLight::setLocation( const PMVector& p )
    if( p != m_location )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMLocationID, m_location );
+         m_pMemento->addData( s_pMetaObject, PMLocationID, m_location );
       m_location = p;
       m_location.resize( 3 );
       setViewStructureChanged( );
@@ -373,7 +348,7 @@ void PMLight::setColor( const PMColor& c )
    if( c != m_color )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMColorID, m_color );
+         m_pMemento->addData( s_pMetaObject, PMColorID, m_color );
       m_color = c;
    }
 }
@@ -383,7 +358,7 @@ void PMLight::setLightType( PMLightType t )
    if( t != m_type )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMTypeID, m_type );
+         m_pMemento->addData( s_pMetaObject, PMTypeID, m_type );
       m_type = t;
       setViewStructureChanged( );
    }
@@ -394,7 +369,7 @@ void PMLight::setRadius( double r )
    if( !approx( r, m_radius ) )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMRadiusID, m_radius );
+         m_pMemento->addData( s_pMetaObject, PMRadiusID, m_radius );
       m_radius = r;
       setViewStructureChanged( );
    }
@@ -405,7 +380,7 @@ void PMLight::setFalloff( double f )
    if( !approx( f, m_falloff ) )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMFalloffID, m_falloff );
+         m_pMemento->addData( s_pMetaObject, PMFalloffID, m_falloff );
       m_falloff = f;
       setViewStructureChanged( );
    }
@@ -416,7 +391,7 @@ void PMLight::setTightness( double t )
    if( !approx( t, m_tightness ) )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMTightnessID, m_tightness );
+         m_pMemento->addData( s_pMetaObject, PMTightnessID, m_tightness );
       m_tightness = t;
    }
 }
@@ -426,29 +401,50 @@ void PMLight::setPointAt( const PMVector& v )
    if( !m_pointAt.approxEqual( v ) )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMPointAtID, m_pointAt );
+         m_pMemento->addData( s_pMetaObject, PMPointAtID, m_pointAt );
       m_pointAt = v;
       setViewStructureChanged( );
    }
 }
 
+void PMLight::setParallel( bool p )
+{
+   if ( p != m_parallel )
+   {
+      if ( m_pMemento )
+         m_pMemento->addData( s_pMetaObject, PMParallelID, m_parallel );
+      m_parallel = p;
+   }
+}
+
 void PMLight::setAreaLight( bool yes )
 {
    if( yes != m_bAreaLight )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMAreaLightID, m_bAreaLight );
+         m_pMemento->addData( s_pMetaObject, PMAreaLightID, m_bAreaLight );
       m_bAreaLight = yes;
       setViewStructureChanged( );
    }
 }
 
+void PMLight::setAreaType( PMAreaType at )
+{
+   if ( at != m_areaType )
+   {
+      if ( m_pMemento )
+         m_pMemento->addData( s_pMetaObject, PMAreaTypeID, m_areaType );
+      m_areaType = at;
+      setViewStructureChanged( );
+   }
+}
+
 void PMLight::setAxis1( const PMVector& v )
 {
    if( !m_areaAxis1.approxEqual( v ) )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMAreaAxis1ID, m_areaAxis1 );
+         m_pMemento->addData( s_pMetaObject, PMAreaAxis1ID, m_areaAxis1 );
       m_areaAxis1 = v;
       setViewStructureChanged( );
    }
@@ -459,7 +455,7 @@ void PMLight::setAxis2( const PMVector& v )
    if( !m_areaAxis2.approxEqual( v ) )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMAreaAxis2ID, m_areaAxis2 );
+         m_pMemento->addData( s_pMetaObject, PMAreaAxis2ID, m_areaAxis2 );
       m_areaAxis2 = v;
       setViewStructureChanged( );
    }
@@ -470,7 +466,7 @@ void PMLight::setSize1( int s )
    if( s != m_areaSize1 )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMAreaSize1ID, m_areaSize1 );
+         m_pMemento->addData( s_pMetaObject, PMAreaSize1ID, m_areaSize1 );
       m_areaSize1 = s;
       setViewStructureChanged( );
    }
@@ -481,7 +477,7 @@ void PMLight::setSize2( int s )
    if( s != m_areaSize2 )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMAreaSize2ID, m_areaSize2 );
+         m_pMemento->addData( s_pMetaObject, PMAreaSize2ID, m_areaSize2 );
       m_areaSize2 = s;
       setViewStructureChanged( );
    }
@@ -492,17 +488,28 @@ void PMLight::setAdaptive( int a )
    if( a != m_adaptive )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMAdaptiveID, m_adaptive );
+         m_pMemento->addData( s_pMetaObject, PMAdaptiveID, m_adaptive );
       m_adaptive = a;
    }
 }
 
+void PMLight::setOrient( bool o )
+{
+   if( o != m_orient )
+   {
+      if ( m_pMemento )
+         m_pMemento->addData( s_pMetaObject, PMOrientID, m_orient );
+      m_orient = o;
+      setViewStructureChanged( );
+   }
+}
+
 void PMLight::setJitter( bool j )
 {
    if( j != m_jitter )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMJitterID, m_jitter );
+         m_pMemento->addData( s_pMetaObject, PMJitterID, m_jitter );
       m_jitter = j;
    }
 }
@@ -512,7 +519,7 @@ void PMLight::setFading( bool y )
    if( y != m_bFading )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMFadingID, m_bFading );
+         m_pMemento->addData( s_pMetaObject, PMFadingID, m_bFading );
       m_bFading = y;
    }
 }
@@ -522,7 +529,7 @@ void PMLight::setFadeDistance( double d )
    if( !approx( d, m_fadeDistance ) )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMFadeDistanceID, m_fadeDistance );
+         m_pMemento->addData( s_pMetaObject, PMFadeDistanceID, m_fadeDistance );
       m_fadeDistance = d;
    }
 }
@@ -532,7 +539,7 @@ void PMLight::setFadePower( int p )
    if( p != m_fadePower )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMFadePowerID, m_fadePower );
+         m_pMemento->addData( s_pMetaObject, PMFadePowerID, m_fadePower );
       m_fadePower = p;
    }
 }
@@ -542,7 +549,7 @@ void PMLight::setMediaInteraction( bool y )
    if( y != m_bMediaInteraction )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMInteractionID, m_bMediaInteraction );
+         m_pMemento->addData( s_pMetaObject, PMInteractionID, m_bMediaInteraction );
       m_bMediaInteraction = y;
    }
 }
@@ -552,7 +559,7 @@ void PMLight::setMediaAttenuation( bool y )
    if( y != m_bMediaAttenuation )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTLight, PMAttenuationID, m_bMediaAttenuation );
+         m_pMemento->addData( s_pMetaObject, PMAttenuationID, m_bMediaAttenuation );
       m_bMediaAttenuation = y;
    }
 }
@@ -570,7 +577,7 @@ void PMLight::restoreMemento( PMMemento* s )
    for( ; it.current( ); ++it )
    {
       data = it.current( );
-      if( data->objectType( ) == PMTLight )
+      if( data->objectType( ) == s_pMetaObject )
       {
          switch( data->valueID( ) )
          {
@@ -595,9 +602,15 @@ void PMLight::restoreMemento( PMMemento* s )
             case PMPointAtID:
                setPointAt( data->vectorData( ) );
                break;
+            case PMParallelID:
+               setParallel( data->boolData( ) );
+               break;
             case PMAreaLightID:
                setAreaLight( data->boolData( ) );
                break;
+            case PMAreaTypeID:
+               setAreaType( ( PMAreaType ) ( data->intData( ) ) );
+               break;
             case PMAreaAxis1ID:
                setAxis1( data->vectorData( ) );
                break;
@@ -613,6 +626,9 @@ void PMLight::restoreMemento( PMMemento* s )
             case PMAdaptiveID:
                setAdaptive( data->intData( ) );
                break;
+            case PMOrientID:
+               setOrient( data->boolData( ) );
+               break;
             case PMJitterID:
                setJitter( data->boolData( ) );
                break;
@@ -643,7 +659,7 @@ void PMLight::restoreMemento( PMMemento* s )
 void PMLight::createViewStructure( )
 {
    if( ( m_type == PointLight ) || ( m_type == ShadowlessLight ) )
-   { 
+   {
       if( !m_pViewStructure )
       {
          m_pViewStructure = new PMViewStructure( defaultPointStructure( ) );
@@ -655,21 +671,21 @@ void PMLight::createViewStructure( )
             defaultPointStructure( )->points( ).size( ) );
          m_pViewStructure->lines( ) = defaultPointStructure( )->lines( );
       }
-         
+
       PMPointArray& points = m_pViewStructure->points( );
-         
+
       int i;
       double c = s_pointLightSize / sqrt( 3 );
       for( i = 0; i < 14; i++ )
          points[i] = PMPoint( m_location );
-         
-      points[0][0] += s_pointLightSize;   
+
+      points[0][0] += s_pointLightSize;
       points[1][0] -= s_pointLightSize;
       points[2][1] += s_pointLightSize;
       points[3][1] -= s_pointLightSize;
       points[4][2] += s_pointLightSize;
       points[5][2] -= s_pointLightSize;
-         
+
       for( i = 0; i < 4; i++ )
       {
          points[6+2*i][0] += c;
@@ -693,11 +709,11 @@ void PMLight::createViewStructure( )
             defaultSpotStructure( )->points( ).size( ) );
          m_pViewStructure->lines( ) = defaultSpotStructure( )->lines( );
       }
-         
+
       PMPointArray& points = m_pViewStructure->points( );
-         
+
       points[0] = PMPoint( m_location );
-         
+
       PMVector pointAtVector = m_pointAt - m_location;
       double pl = pointAtVector.abs( );
       if( approxZero( pl ) )
@@ -735,11 +751,11 @@ void PMLight::createViewStructure( )
          r = 1;
       else
          r = r1 / r2;
-      
+
       PMVector circleCenter = m_location + length * pointAtVector;
       points[1] = PMPoint( circleCenter + endPoint );
       points[s_nSpotLines + 1] = PMPoint( circleCenter + endPoint * r );
-         
+
       int i;
       for( i = 2; i < ( s_nSpotLines + 1 ); i++ )
       {
@@ -762,12 +778,12 @@ void PMLight::createViewStructure( )
             defaultCylindricalStructure( )->points( ).size( ) );
          m_pViewStructure->lines( ) = defaultCylindricalStructure( )->lines( );
       }
-         
+
       PMPointArray& points = m_pViewStructure->points( );
-         
+
       points[s_nCylinderLines*4] = PMPoint( m_location );
       points[s_nCylinderLines*4+1] = PMPoint( m_pointAt );
-         
+
       PMVector pointAtVector = m_pointAt - m_location;
       double pl = pointAtVector.abs( );
       if( approxZero( pl ) )
@@ -790,13 +806,13 @@ void PMLight::createViewStructure( )
          r = 1;
       else
          r = r1 / r2;
-      
+
       PMVector circleCenter = m_location + s_length * pointAtVector;
       points[0] = PMPoint( circleCenter + endPoint );
       points[s_nCylinderLines] = PMPoint( m_location + endPoint );
       points[2*s_nCylinderLines] = PMPoint( circleCenter + endPoint * r );
       points[3*s_nCylinderLines] = PMPoint( m_location + endPoint * r );
-         
+
       int i;
       for( i = 1; i < s_nCylinderLines; i++ )
       {
@@ -822,12 +838,12 @@ void PMLight::createViewStructure( )
          int x, y, h;
          int ps, ls;
          PMVector bp;
-         
+
          PMPointArray& points = m_pViewStructure->points( );
          PMLineArray& lines = m_pViewStructure->lines( );
          points.detach( );
          lines.detach( );
-         
+
          ps = points.size( );
          ls = lines.size( );
          points.resize( ps + s1*s2 );
@@ -854,12 +870,49 @@ void PMLight::createViewStructure( )
          else
          {
             bp = m_location - m_areaAxis1/2 - m_areaAxis2/2;
-            for( x = 0; x < s1; x++ )
-               for( y = 0; y < s2; y++ )
-                  points[ps + y*s1 + x] =
-                     PMPoint( bp + m_areaAxis1 * ( (double)x/(double)(s1-1) )
-                              + m_areaAxis2 * ( (double)y/(double)(s2-1) ) );
-            
+
+            if ( m_areaType == Rectangular || s1 < 2 || s2 < 2 )
+            {
+               for( x = 0; x < s1; x++ )
+                  for( y = 0; y < s2; y++ )
+                     points[ps + y*s1 + x] =
+                        PMPoint( bp + m_areaAxis1 * ( (double)x/(double)(s1-1) )
+                                 + m_areaAxis2 * ( (double)y/(double)(s2-1) ) );
+            }
+            else
+            {
+               double stepX = ( 2.0 / (double)(s1-1) );
+               double stepY = ( 2.0 / (double)(s2-1) );
+               double doubleX, doubleY, xSqr, scaleFactor;
+
+               for ( x = 0; x < s1; ++x )
+               {
+                  doubleX = ( (double)x * stepX ) - 1.0;
+                  xSqr = doubleX * doubleX;
+                  for ( y = 0; y < s2; ++y )
+                  {
+                     doubleY = ( (double)y * stepY ) - 1.0;
+
+                     if ( doubleX == 0.0 && doubleY == 0.0 )
+                        scaleFactor = 1.0;
+                     else
+                     {
+                        if ( fabs( doubleX ) > fabs( doubleY ) )
+                           scaleFactor = fabs( doubleX );
+                        else
+                           scaleFactor = fabs( doubleY );
+                        scaleFactor /= sqrt( xSqr + doubleY * doubleY );
+                     }
+
+                     points[ps + y*s1 + x] =
+                        PMPoint( bp + m_areaAxis1 *
+                                 ( ( ( doubleX * scaleFactor ) / 2.0 ) + 0.5 )
+                                 + m_areaAxis2 *
+                                 ( ( ( doubleY * scaleFactor ) / 2.0 ) + 0.5 ) );
+                  }
+               }
+            }
+
             for( x = 0; x < s1; x++ )
             {
                for( y = 0; y < (s2-1); y++ )
@@ -868,7 +921,7 @@ void PMLight::createViewStructure( )
                   lines[ls + x*(s2-1) + y] = PMLine( h, h+s1 );
                }
             }
-            
+
             ls += s1*(s2-1);
             for( y = 0; y < s2; y++ )
             {
@@ -879,7 +932,7 @@ void PMLight::createViewStructure( )
                }
             }
          }
-      }     
+      }
    }
 }
 
@@ -1002,5 +1055,10 @@ void PMLight::cleanUp( ) const
    if( s_pDefaultCylindricalStructure )
       delete s_pDefaultCylindricalStructure;
    s_pDefaultCylindricalStructure = 0;
+   if( s_pMetaObject )
+   {
+      delete s_pMetaObject;
+      s_pMetaObject = 0;
+   }
    Base::cleanUp( );
 }
