@@ -19,13 +19,10 @@
 #include "pmmaterialmapedit.h"
 #include "pmmaterialmap.h"
 
-#include "pmoutputdevice.h"
 #include "pmxmlhelper.h"
 #include "pmcompositeobject.h"
 #include "pmmemento.h"
-
-#include <kdebug.h>
-#include "pmglobals.h"
+#include "pmenumproperty.h"
 
 #include <klocale.h>
 
@@ -39,8 +36,22 @@ const bool onceDefault = false;
 const PMMaterialMap::PMMapType mapTypeDefault = PMMaterialMap::MapPlanar;
 const PMMaterialMap::PMInterpolateType interpolateTypeDefault = PMMaterialMap::InterpolateNone;
 
-PMMaterialMap::PMMaterialMap( )
-      : Base( )
+PMDefinePropertyClass( PMMaterialMap, PMProperty );
+PMDefineEnumPropertyClass( PMMaterialMap, PMMaterialMap::PMBitmapType,
+                           PMBitmapTypeProperty );
+PMDefineEnumPropertyClass( PMMaterialMap, PMMaterialMap::PMInterpolateType,
+                           PMInterpolateTypeProperty );
+PMDefineEnumPropertyClass( PMMaterialMap, PMMaterialMap::PMMapType,
+                           PMMapTypeProperty );
+
+PMMetaObject* PMMaterialMap::s_pMetaObject = 0;
+PMObject* createNewMaterialMap( PMPart* part )
+{
+   return new PMMaterialMap( part );
+}
+
+PMMaterialMap::PMMaterialMap( PMPart* part )
+      : Base( part )
 {
    m_bitmapType = bitmapTypeDefault;
    m_bitmapFile = bitmapFileDefault;
@@ -49,85 +60,20 @@ PMMaterialMap::PMMaterialMap( )
    m_interpolateType = interpolateTypeDefault;
 }
 
+PMMaterialMap::PMMaterialMap( const PMMaterialMap& m )
+      : Base( m )
+{
+   m_bitmapType = m.m_bitmapType;
+   m_bitmapFile = m.m_bitmapFile;
+   m_once = m.m_once;
+   m_mapType = m.m_mapType;
+   m_interpolateType = m.m_interpolateType;
+}
 
 PMMaterialMap::~PMMaterialMap( )
 {
 }
 
-void PMMaterialMap::serialize( PMOutputDevice& dev ) const
-{
-   QString str1;
-
-   dev.objectBegin( "material_map" );
-
-   switch( m_bitmapType )
-   {
-      case PMMaterialMap::BitmapGif:
-         dev.writeLine( "gif" );
-         break;
-      case PMMaterialMap::BitmapTga:
-         dev.writeLine( "tga" );
-         break;
-      case PMMaterialMap::BitmapIff:
-         dev.writeLine( "iff" );
-         break;
-      case PMMaterialMap::BitmapPpm:
-         dev.writeLine( "ppm" );
-         break;
-      case PMMaterialMap::BitmapPgm:
-         dev.writeLine( "pgm" );
-         break;
-      case PMMaterialMap::BitmapPng:
-         dev.writeLine( "png" );
-         break;
-      case PMMaterialMap::BitmapJpeg:
-         dev.writeLine( "jpeg" );
-         break;
-      case PMMaterialMap::BitmapTiff:
-         dev.writeLine( "tiff" );
-         break;
-      case PMMaterialMap::BitmapSys:
-         dev.writeLine( "sys" );
-         break;
-   }
-
-   dev.writeLine( "\"" + m_bitmapFile + "\"" );
-
-   if( m_once )
-      dev.writeLine( "once" );
-
-   switch( m_mapType )
-   {
-      case PMMaterialMap::MapPlanar:
-         dev.writeLine( "map_type 0" );
-         break;
-      case PMMaterialMap::MapSpherical:
-         dev.writeLine( "map_type 1" );
-         break;
-      case PMMaterialMap::MapCylindrical:
-         dev.writeLine( "map_type 2" );
-         break;
-      case PMMaterialMap::MapToroidal:
-         dev.writeLine( "map_type 5" );
-         break;
-   }
-   switch( m_interpolateType )
-   { 
-      case PMMaterialMap::InterpolateBilinear:
-         dev.writeLine( "interpolate 2" );
-	 break;
-      case PMMaterialMap::InterpolateNormalized:
-         dev.writeLine( "interpolate 4" );
-	 break;
-      default:
-         break;
-   }
-
-   Base::serialize( dev );
-
-   dev.objectEnd( );
-}
-
 void PMMaterialMap::serialize( QDomElement& e, QDomDocument& doc ) const
 {
    switch( m_bitmapType )
@@ -240,24 +186,59 @@ void PMMaterialMap::readAttributes( const PMXMLHelper& h )
    Base::readAttributes( h );
 }
 
-bool PMMaterialMap::canInsert( PMObjectType t, const PMObject*,
-                               const PMObjectList* ) const
+PMMetaObject* PMMaterialMap::metaObject( ) const
 {
-   switch( t )
+   if( !s_pMetaObject )
    {
-      case PMTComment:
-      case PMTTexture:
-         return true;
-      default:
-         return false;
+      s_pMetaObject = new PMMetaObject( "MaterialMap", Base::metaObject( ),
+                                        createNewMaterialMap );
+
+      PMBitmapTypeProperty* bp = new PMBitmapTypeProperty(
+         "bitmapType", &PMMaterialMap::setBitmapType, &PMMaterialMap::bitmapType );
+      bp->addEnumValue( "Gif", BitmapGif );
+      bp->addEnumValue( "Tga", BitmapTga );
+      bp->addEnumValue( "Iff", BitmapIff );
+      bp->addEnumValue( "Ppm", BitmapPpm );
+      bp->addEnumValue( "Pgm", BitmapPgm );
+      bp->addEnumValue( "Png", BitmapPng );
+      bp->addEnumValue( "Jpeg", BitmapJpeg );
+      bp->addEnumValue( "Tiff", BitmapTiff );
+      bp->addEnumValue( "Sys", BitmapSys );
+      s_pMetaObject->addProperty( bp );
+
+      PMInterpolateTypeProperty* ip = new PMInterpolateTypeProperty(
+         "interpolateType", &PMMaterialMap::setInterpolateType,
+         &PMMaterialMap::interpolateType );
+      ip->addEnumValue( "None", InterpolateNone );
+      ip->addEnumValue( "Bilinear", InterpolateBilinear );
+      ip->addEnumValue( "Normalized", InterpolateNormalized );
+      s_pMetaObject->addProperty( ip );
+
+      PMMapTypeProperty* mp = new PMMapTypeProperty(
+         "mapType", &PMMaterialMap::setMapType, &PMMaterialMap::mapType );
+      mp->addEnumValue( "Planar", MapPlanar );
+      mp->addEnumValue( "Spherical", MapSpherical );
+      mp->addEnumValue( "Cylindrical", MapCylindrical );
+      mp->addEnumValue( "Toroidal", MapToroidal );
+      s_pMetaObject->addProperty( mp );
+
+      s_pMetaObject->addProperty(
+         new PMProperty( "bitmapFile", &PMMaterialMap::setBitmapFileName,
+                         &PMMaterialMap::bitmapFile ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "once", &PMMaterialMap::enableOnce, &PMMaterialMap::isOnceEnabled ) );
    }
+   return s_pMetaObject;
 }
-   
-bool PMMaterialMap::isA( PMObjectType t ) const
+
+void PMMaterialMap::cleanUp( ) const
 {
-   if( t == PMTMaterialMap )
-      return true;
-   return Base::isA( t );
+   if( s_pMetaObject )
+   {
+      delete s_pMetaObject;
+      s_pMetaObject = 0;
+   }
+   Base::cleanUp( );
 }
 
 QString PMMaterialMap::description( ) const
@@ -270,9 +251,9 @@ void PMMaterialMap::setBitmapType( PMBitmapType c )
    if( c != m_bitmapType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTMaterialMap, PMBitmapTypeID, m_bitmapType );
+         m_pMemento->addData( s_pMetaObject, PMBitmapTypeID, m_bitmapType );
       m_bitmapType = c;
-   } 
+   }
 }
 
 void PMMaterialMap::setBitmapFileName( const QString& c )
@@ -280,9 +261,9 @@ void PMMaterialMap::setBitmapFileName( const QString& c )
    if( c != m_bitmapFile )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTMaterialMap, PMBitmapFileID, m_bitmapFile );
+         m_pMemento->addData( s_pMetaObject, PMBitmapFileID, m_bitmapFile );
       m_bitmapFile = c;
-   } 
+   }
 }
 
 void PMMaterialMap::setMapType( PMMapType c )
@@ -290,7 +271,7 @@ void PMMaterialMap::setMapType( PMMapType c )
    if( c != m_mapType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTMaterialMap, PMMapTypeID, m_mapType );
+         m_pMemento->addData( s_pMetaObject, PMMapTypeID, m_mapType );
       m_mapType = c;
    }
 }
@@ -300,7 +281,7 @@ void PMMaterialMap::setInterpolateType( PMInterpolateType c )
    if( c != m_interpolateType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTMaterialMap, PMInterpolateID, m_interpolateType );
+         m_pMemento->addData( s_pMetaObject, PMInterpolateID, m_interpolateType );
       m_interpolateType = c;
    }
 }
@@ -310,9 +291,9 @@ void PMMaterialMap::enableOnce( bool c )
    if( c != m_once )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTMaterialMap, PMOnceID, m_once );
+         m_pMemento->addData( s_pMetaObject, PMOnceID, m_once );
       m_once = c;
-   } 
+   }
 }
 
 PMDialogEditBase* PMMaterialMap::editWidget( QWidget* parent ) const
@@ -328,7 +309,7 @@ void PMMaterialMap::restoreMemento( PMMemento* s )
    for( ; it.current( ); ++it )
    {
       data = it.current( );
-      if( data->objectType( ) == PMTMaterialMap )
+      if( data->objectType( ) == s_pMetaObject )
       {
          switch( data->valueID( ) )
          {
