@@ -40,6 +40,7 @@
 #include "pmcommandmanager.h"
 
 class PMView;
+class PMShell;
 class PMObjectDrag;
 class PMScene;
 class PMCamera;
@@ -50,17 +51,16 @@ class PMBrowserExtension;
 class PMSymbolTable;
 class PMDeclare;
 
+class PMPrototypeManager;
+class PMInsertRuleSystem;
+class PMIOManager;
+
 class QMimeSource;
 class KAction;
 class PMComboAction;
 class PMSpinBoxAction;
 class PMLabelAction;
 class KProcess;
-class PMShell;
-
-class PMTreeView;
-class PMDialogView;
-class PMGLView;
 
 /**
  * The part for kpovmodeler (povray document)
@@ -77,30 +77,59 @@ public:
            QObject* parent, const char* name, bool readWrite,
            PMShell* shell = 0 );
 
-   /** Destructor */
+   /**
+    * construtor of PMPart, calls all init functions to create the
+    * application. It does not create the main widget.
+    */
+   PMPart( QWidget* parentWidget, const char* widgetName,
+           QObject* parent, const char* name, bool readWrite,
+           bool onlyCutPaste, PMShell* shell = 0 );
+
+   /**
+    * Destructor
+    */
    ~PMPart( );
-   /** deletes the document's contents */
+   /**
+    * deletes the document's contents
+    */
    void deleteContents( );
-   /** initializes the document generally */
+   /**
+    * initializes the document generally
+    */
    bool newDocument( );
-   /** closes the actual document */
+   /**
+    * closes the actual document
+    */
    void closeDocument( );
-   /** Exports the scene as povray code */
+   /**
+    * Exports the scene as povray code
+    */
    bool exportPovray( const KURL& url );
 
-   /** returns the toplevel object */
+   /**
+    * returns the toplevel object
+    */
    PMScene* scene( ) const { return m_pScene; }
-   /** Returns a pointer to the shell if there is one */
+   /**
+    * Returns a pointer to the shell if there is one
+    */
    PMShell* shell( ) const { return m_pShell; }
-   /** saves settings */
+   /**
+    * saves settings
+    */
    void saveConfig( KConfig* cfg );
-   /** loads settings */
+   /**
+    * loads settings
+    */
    void restoreConfig( KConfig* cfg );
 
-   /** Updates the sorted list of selected objects if necessary and
-    * returns a reference to the list. */
+   /**
+    * Updates the sorted list of selected objects if necessary and
+    * returns a reference to the list.
+    */
    const PMObjectList& selectedObjects( );
-   /** Executes the command cmd.
+   /**
+    * Executes the command cmd.
     *
     * All changes to the document have to be made
     * with @ref PMCommand objects for undo/redo. Each slot that changes the
@@ -110,43 +139,62 @@ public:
     * and the command object was deleted.
     */
    bool executeCommand( PMCommand* cmd );
-   /** Checks, where the objects in list can be inserted, as child
+   /**
+    * Checks, where the objects in list can be inserted, as child
     * or as sibling of obj and asks the user if there is more than one
     * possibility
     *
     * Returns PMIFirstChild, PMILastChild, PMISibling or 0, if the objects
     * should not be inserted.*/
    int whereToInsert( PMObject* obj, const PMObjectList& list );
-   /** Checks, where the objects in list can be inserted, as child
+   /**
+    * Checks, where the objects in list can be inserted, as child
     * or as sibling of obj and asks the user if there is more than one
     * possibility
     *
     * Returns PMIFirstChild, PMILastChild, PMISibling or 0, if the objects
     * should not be inserted.*/
-   int whereToInsert( PMObject* obj, const QValueList<PMObjectType>& list );
-   /** Asks the user, where to insert new objects.
+   int whereToInsert( PMObject* obj, const QStringList& list );
+   /**
+    * Asks the user, where to insert new objects.
     *
     * Returns PMIFirstChild, PMILastChild, PMISibling or 0, if the objects
     * should not be inserted.*/
    int whereToInsert( PMObject* obj );
 
-   /** Returns the symbol table of this document */
+   /**
+    * Returns the symbol table of this document
+    */
    PMSymbolTable* symbolTable( ) const { return m_pSymbolTable; }
-   /** Returns the first camera */
+   /**
+    * Returns the first camera
+    */
    PMCamera* firstCamera( );
-   /** Returns an iterator to the list of cameras */
+   /**
+    * Returns an iterator to the list of cameras
+    */
    QPtrListIterator<PMCamera> cameras( );
-   /** The active object */
+   /**
+    * The active object
+    */
    PMObject* activeObject( ) const { return m_pActiveObject; }
+   /**
+    * List of control points of the active object
+    */
+   PMControlPointList activeControlPoints( ) const { return m_controlPoints; }
 
-   /** Called when the user moves some objects in the tree view.
+   /**
+    * Called when the user moves some objects in the tree view.
     * If obj is 0, the user moved the objects to another document (remove)
     *
-    * Returns true if successful. */
+    * Returns true if successful.
+    */
    bool dragMoveSelectionTo( PMObject* obj );
-   /** Called when the user drags some objects onto the tree view
+   /**
+    * Called when the user drags some objects onto the tree view
     *
-    * Returns true if successful. */
+    * Returns true if successful.
+    */
    bool drop( PMObject* obj, QMimeSource* e );
    /**
     * Tries to parse the data and insert the parsed objects
@@ -156,271 +204,534 @@ public:
     */
    bool insertFromParser( const QString& type, PMParser* parser, PMObject* obj );
 
-
-
-   /** Returns the list of tree views for this part */
-   const QPtrList<PMTreeView>& treeViews( ) const { return m_treeViews; }
-   /** Returns the list of dialog views for this part */
-   const QPtrList<PMDialogView>& dialogViews( ) const { return m_dialogViews; }
-   /** Returns the list of gl views for this part */
-   const QPtrList<PMGLView>& glViews( ) const { return m_glViews; }
+   /**
+    * Returns a pointer to the prototype manager
+    */
+   PMPrototypeManager* prototypeManager( ) const { return m_pPrototypeManager; }
+   /**
+    * Returns a pointer to the insert rules system
+    */
+   PMInsertRuleSystem* insertRuleSystem( ) const { return m_pInsertRuleSystem; }
+   /**
+    * Returns a pointer to the IO formats manager for this part
+    */
+   PMIOManager* ioManager( ) const { return m_pIOManager; }
 
 public slots:
-   /** Imports the povray file */
-   void slotImportPovray( );
+   /**
+    * Opens the import file dialog
+    */
+   void slotFileImport( );
+   /**
+    * Opens the export file dialog
+    */
+   void slotFileExport( );
 
-   /** puts the marked text/object into the clipboard and removes the objects */
+   /**
+    * puts the marked text/object into the clipboard and removes the objects
+    */
    void slotEditCut( );
-   /** removes the selected object */
+   /**
+    * removes the selected object
+    */
    void slotEditDelete( );
-   /** puts the marked text/object into the clipboard */
+   /**
+    * puts the marked text/object into the clipboard
+    */
    void slotEditCopy( );
-   /** paste the clipboard into the document */
+   /**
+    * paste the clipboard into the document
+    */
    void slotEditPaste( );
-   /** undoes the last change */
+   /**
+    * undoes the last change
+    */
    void slotEditUndo( );
-   /** redoes the last undone command */
+   /**
+    * redoes the last undone command
+    */
    void slotEditRedo( );
 
-   /** Called when an object is changed.
-    * Mode is a bit combination of @ref PMChange constants */
+   /**
+    * Called when an object is changed.
+    * Mode is a bit combination of @ref PMChange constants
+    */
    void slotObjectChanged( PMObject* obj, const int mode, QObject* sender );
-   /** Called when an id is changed */
+   /**
+    * Called when an id is changed
+    */
    void slotIDChanged( PMObject* obj, const QString& oldID );
 
-   /** Inserts a new PMObject of type type */
-   void slotNewObject( PMObjectType type );
-   /** Inserts the new PMObject. The object will be deleted if it can't be
-    * inserted or the user aborts the action */
+   /**
+    * Inserts a new PMObject of type type
+    */
+   void slotNewObject( const QString& type );
+   /**
+    * Inserts the new PMObject. The object will be deleted if it can't be
+    * inserted or the user aborts the action
+    */
    void slotNewObject( PMObject* newObject );
 
-   /** Adds transformations to the object and calls slotNewObject
-    * for it */
+   /**
+    * Adds transformations to the object and calls slotNewObject
+    * for it
+    */
    void slotNewTransformedObject( PMObject* o );
 
-   /** Inserts a new PMGlobalSettings */
+   /**
+    * Inserts a new PMGlobalSettings
+    */
    void slotNewGlobalSettings( );
-   /** Inserts a new PMSkySphere */
+   /**
+    * Inserts a new PMSkySphere
+    */
    void slotNewSkySphere( );
-   /** Inserts a new PMRainbow */
+   /**
+    * Inserts a new PMRainbow
+    */
    void slotNewRainbow( );
-   /** Inserts a new PMFog */
+   /**
+    * Inserts a new PMFog
+    */
    void slotNewFog( );
-   /** Inserts a new PMInterior */
+   /**
+    * Inserts a new PMInterior
+    */
    void slotNewInterior( );
-   /** Inserts a new PMMedia */
+   /**
+    * Inserts a new PMMedia
+    */
    void slotNewMedia( );
-   /** Inserts a new PMDensity */
+   /**
+    * Inserts a new PMDensity
+    */
    void slotNewDensity( );
-   /** Inserts a new PMMaterial */
+   /**
+    * Inserts a new PMMaterial
+    */
    void slotNewMaterial( );
-   /** Inserts a new PMBox */
+   /**
+    * Inserts a new PMBox
+    */
    void slotNewBox( );
-   /** Inserts a new PMSphere */
+   /**
+    * Inserts a new PMSphere
+    */
    void slotNewSphere( );
-   /** Inserts a new PMCylinder */
+   /**
+    * Inserts a new PMCylinder
+    */
    void slotNewCylinder( );
-   /** Inserts a new PMCone */
+   /**
+    * Inserts a new PMCone
+    */
    void slotNewCone( );
-   /** Inserts a new PMTorus */
+   /**
+    * Inserts a new PMTorus
+    */
    void slotNewTorus( );
-   /** Inserts a new PMLathe */
+   /**
+    * Inserts a new PMLathe
+    */
    void slotNewLathe( );
-   /** Inserts a new PMPrism */
+   /**
+    * Inserts a new PMPrism
+    */
    void slotNewPrism( );
-   /** Inserts a new PMSurfaceOfRevolution */
+   /**
+    * Inserts a new PMSurfaceOfRevolution
+    */
    void slotNewSurfaceOfRevolution( );
-   /** Inserts a new PMSuperquadricEllipsoid */
+   /**
+    * Inserts a new PMSuperquadricEllipsoid
+    */
    void slotNewSuperquadricEllipsoid( );
-   /** Inserts a new PMJuliaFractal */
+   /**
+    * Inserts a new PMJuliaFractal
+    */
    void slotNewJuliaFractal( );
-   /** Inserts a new PMHeightField */
+   /**
+    * Inserts a new PMHeightField
+    */
    void slotNewHeightField( );
-   /** Inserts a new text object */
+   /**
+    * Inserts a new text object
+    */
    void slotNewText( );
 
-   /** Inserts a new PMBlob */
+   /**
+    * Inserts a new PMBlob
+    */
    void slotNewBlob( );
-   /** Inserts a new PMBlobSphere */
+   /**
+    * Inserts a new PMBlobSphere
+    */
    void slotNewBlobSphere( );
-   /** Inserts a new PMBlobCylinder */
+   /**
+    * Inserts a new PMBlobCylinder
+    */
    void slotNewBlobCylinder( );
 
-   /** Inserts a new PMPlane */
+   /**
+    * Inserts a new PMPlane
+    */
    void slotNewPlane( );
-   /** Inserts a new PMPolynom */
+   /**
+    * Inserts a new PMPolynom
+    */
    void slotNewPolynom( );
-
-   /** Inserts a new PMDeclare */
+   /**
+    * Inserts a new PMDeclare
+    */
    void slotNewDeclare( );
-   /** Inserts a new PMObjectLink */
+   /**
+    * Inserts a new PMObjectLink
+    */
    void slotNewObjectLink( );
 
-   /** Inserts a new PMCSG ( union ) */
+   /**
+    * Inserts a new PMCSG ( union )
+    */
    void slotNewUnion( );
-   /** Inserts a new PMCSG ( intersection ) */
+   /**
+    * Inserts a new PMCSG ( intersection )
+    */
    void slotNewIntersection( );
-   /** Inserts a new PMCSG ( difference ) */
+   /**
+    * Inserts a new PMCSG ( difference )
+    */
    void slotNewDifference( );
-   /** Inserts a new PMCSG ( merge ) */
+   /**
+    * Inserts a new PMCSG ( merge )
+    */
    void slotNewMerge( );
 
-   /** Inserts a new PMDisc */
+   /**
+    * Inserts a new PMDisc
+    */
    void slotNewDisc( );
-   /** Inserts a new PMBicubicPatch */
+   /**
+    * Inserts a new PMBicubicPatch
+    */
    void slotNewBicubicPatch( );
-   /** Inserts a new PMTriangle */
+   /**
+    * Inserts a new PMTriangle
+    */
    void slotNewTriangle( );
 
-   /** Inserts a new PMBoundedBy */
+   /**
+    * Inserts a new PMBoundedBy
+    */
    void slotNewBoundedBy( );
-   /** Inserts a new PMClippedBy */
+   /**
+    * Inserts a new PMClippedBy
+    */
    void slotNewClippedBy( );
 
-   /** Inserts a new PMLight object */
+   /**
+    * Inserts a new PMLight object
+    */
    void slotNewLight( );
-   /** Inserts a new PMLooksLike object */
+   /**
+    * Inserts a new PMLooksLike object
+    */
    void slotNewLooksLike( );
+   /**
+    * Inserts a new PMProjectedThrough object
+    */
+   void slotNewProjectedThrough( );
 
-   /** Inserts a new PMCamera */
+   /**
+    * Inserts a new PMCamera
+    */
    void slotNewCamera( );
 
-   /** Inserts a new PMTexture */
+   /**
+    * Inserts a new PMTexture
+    */
    void slotNewTexture( );
-   /** Inserts a new PMPigment */
+   /**
+    * Inserts a new PMPigment
+    */
    void slotNewPigment( );
-   /** Inserts a new PMNormal */
+   /**
+    * Inserts a new PMNormal
+    */
    void slotNewNormal( );
-   /** Inserts a new PMSolidColor */
+   /**
+    * Inserts a new PMSolidColor
+    */
    void slotNewSolidColor( );
-   /** Inserts a new PMTextureList */
+   /**
+    * Inserts a new PMTextureList
+    */
    void slotNewTextureList( );
-   /** Inserts a new PMColorList */
+   /**
+    * Inserts a new PMColorList
+    */
    void slotNewColorList( );
-   /** Inserts a new PMPigmentList */
+   /**
+    * Inserts a new PMPigmentList
+    */
    void slotNewPigmentList( );
-   /** Inserts a new PMNormalList */
+   /**
+    * Inserts a new PMNormalList
+    */
    void slotNewNormalList( );
-   /** Inserts a new PMDensityList */
+   /**
+    * Inserts a new PMDensityList
+    */
    void slotNewDensityList( );
-   /** Inserts a new PMFinish */
+   /**
+    * Inserts a new PMFinish
+    */
    void slotNewFinish( );
-   /** Inserts a new PMPattern */
+   /**
+    * Inserts a new PMPattern
+    */
    void slotNewPattern( );
-   /** Inserts a new PMBlendMapModifiers */
+   /**
+    * Inserts a new PMBlendMapModifiers
+    */
    void slotNewBlendMapModifiers( );
-   /** Inserts a new PMTextureMap */
+   /**
+    * Inserts a new PMTextureMap
+    */
    void slotNewTextureMap( );
-   /** Inserts a new PMMaterialMap */
+   /**
+    * Inserts a new PMMaterialMap
+    */
    void slotNewMaterialMap( );
-   /** Inserts a new PMColorMap */
+   /**
+    * Inserts a new PMColorMap
+    */
    void slotNewColorMap( );
-   /** Inserts a new PMPigmentMap */
+   /**
+    * Inserts a new PMPigmentMap
+    */
    void slotNewPigmentMap( );
-   /** Inserts a new PMNormalMap */
+   /**
+    * Inserts a new PMNormalMap
+    */
    void slotNewNormalMap( );
-   /** Inserts a new PMBumpMap */
+   /**
+    * Inserts a new PMBumpMap
+    */
    void slotNewBumpMap( );
-   /** Inserts a new PMSlopeMap */
+   /**
+    * Inserts a new PMSlopeMap
+    */
    void slotNewSlopeMap( );
-   /** Inserts a new PMDensityMap */
+   /**
+    * Inserts a new PMDensityMap
+    */
    void slotNewDensityMap( );
-   /** Inserts a new PMSlope */
+   /**
+    * Inserts a new PMSlope
+    */
    void slotNewSlope( );
-   /** Inserts a new PMWarp */
+   /**
+    * Inserts a new PMWarp
+    */
    void slotNewWarp( );
-   /** Inserts a new PMImageMap */
+   /**
+    * Inserts a new PMImageMap
+    */
    void slotNewImageMap( );
-   /** Inserts a new PMQuickColor */
+   /**
+    * Inserts a new PMQuickColor
+    */
    void slotNewQuickColor( );
 
-   /** Inserts a new PMTranslate */
+   /**
+    * Inserts a new PMTranslate
+    */
    void slotNewTranslate( );
-   /** Inserts a new PMRotate */
+   /**
+    * Inserts a new PMRotate
+    */
    void slotNewRotate( );
-   /** Inserts a new PMScale */
+   /**
+    * Inserts a new PMScale
+    */
    void slotNewScale( );
-   /** Inserts a new PMPovrayMatrix */
+   /**
+    * Inserts a new PMPovrayMatrix
+    */
    void slotNewMatrix( );
-   /** Inserts a new PMComment */
+   /**
+    * Inserts a new PMComment
+    */
    void slotNewComment( );
-   /** Inserts a new PMRaw */
+   /**
+    * Inserts a new PMRaw
+    */
    void slotNewRaw( );
 
-   /** Called when the clipboard contents changed */
+   // POV-Ray 3.5 objects
+   /**
+    * Inserts a new PMIsoSurfate
+    */
+   void slotNewIsoSurface( );
+   /**
+    * Inserts a new PMRadiosity
+    */
+   void slotNewRadiosity( );
+   /**
+    * Inserts a new PMGlobalPhotons
+    */
+   void slotNewGlobalPhotons( );
+   /**
+    * Inserts a new PMPhotons
+    */
+   void slotNewPhotons( );
+   /**
+    * Inserts a new PMLightGroup
+    */
+   void slotNewLightGroup( );
+   /**
+    * Inserts a new PMInteriorTexture
+    */
+   void slotNewInteriorTexture( );
+   /**
+    * Inserts a new PMSphereSweep
+    */
+   void slotNewSphereSweep( );
+   /**
+    * Inserts a new PMMesh
+    */
+   void slotNewMesh( );
+
+
+   /**
+    * Called when the clipboard contents changed
+    */
    void slotClipboardDataChanged( );
-   /** updates the undo/redo menu items */
+   /**
+    * updates the undo/redo menu items
+    */
    void slotUpdateUndoRedo( const QString& undo, const QString& redo );
 
-   /** Starts rendering with povray */
+   /**
+    * Starts rendering with povray
+    */
    void slotRender( );
-   /** Opens the render settings */
+   /**
+    * Opens the render settings
+    */
    void slotRenderSettings( );
-   /** Called when a render mode is selected */
+   /**
+    * Called when a render mode is selected
+    */
    void slotRenderMode( int index );
-   /** Called when the render modes combo action is plugged into the toolbar */
+   /**
+    * Called when the render modes combo action is plugged into the toolbar
+    */
    void slotRenderModeActionPlugged( );
-   /** Shows the render window */
+   /**
+    * Shows the render window
+    */
    void slotViewRenderWindow( );
-   /** Called when the visibility level has changed */
+   /**
+    * Called when the visibility level has changed
+    */
    void slotVisibilityLevelChanged( int );
-   /** Called when the visibility level action is plugged into the toolbar */
+   /**
+    * Called when the visibility level action is plugged into the toolbar
+    */
    void slotVisibilityActionPlugged( );
 
+   /**
+    * Opens the Import library object dialog
+    */
+   void slotImportLibraryObject( );
+   /**
+    * Opens the search library object dialog
+    */
+   void slotSearchLibraryObject( );
+
    /** */
    virtual void setModified( );
    /** */
    virtual void setModified( bool modified );
 
-   /** Called when a tree view is added */
-   void addTreeView( PMTreeView* v );
-   /** Called when a tree view is destroyed */
-   void removeTreeView( PMTreeView* v );
-   /** Called when a dialog view is added */
-   void addDialogView( PMDialogView* v );
-   /** Called when a dialog view is destroyed */
-   void removeDialogView( PMDialogView* v );
-   /** Called when a gl view is added */
-   void addGLView( PMGLView* v );
-   /** Called when a gl view is destroyed */
-   void removeGLView( PMGLView* v );
+   /** Set the scene object. To be used with extreme care. */
+   void setScene( PMScene* scene );
 
 signals:
-   /** Signal that is emitted when an object is changed.
-    * Mode is a bit combination of @ref PMChange constants. */
+   /**
+    * Signal that is emitted when an object is changed.
+    * Mode is a bit combination of @ref PMChange constants.
+    */
    void objectChanged( PMObject* obj, const int mode, QObject* sender );
-   /** Signal that is emitted when the views have to be refreshed.
-    * Usually on newDocument or openDocument */
+   /**
+    * Signal that is emitted when the views have to be refreshed.
+    * Usually on newDocument or openDocument
+    */
    void refresh( );
-   /** Emitted when all views should delete all data */
+   /**
+    * Emitted when all views should delete all data
+    */
    void clear( );
-   /** Emitted, when the modified flag changes */
+   /**
+    * Emitted, when the modified flag changes
+    */
    void modified( );
-   /** Emitted when the mouse is over a control point */
+   /**
+    * Emitted when the mouse is over a control point
+    */
    void controlPointMessage( const QString& msg );
-   /** Emitted when the active render mode has changed */
+   /**
+    * Emitted when the active render mode has changed
+    */
    void activeRenderModeChanged( );
-
+   /**
+    * Emitted before the scene is rendered.
+    *
+    * Views should ask the user to save pending changes.
+    */
+   void aboutToRender( );
 protected:
-   /** reimplemented from @ref KParts::ReadOnlyPart */
+   /**
+    * reimplemented from @ref KParts::ReadOnlyPart
+    */
    virtual bool openFile( );
-   /** reimplemented from @ref KParts::ReadOnlyPart */
+   /**
+    * reimplemented from @ref KParts::ReadOnlyPart
+    */
    virtual bool saveFile( );
-   /** Inits all actions */
+   /**
+    * Inits all actions
+    */
    void initActions( );
-   /** creates the widget of the part instance and sets
+   /**
+    * Inits only actions related to copy&paste.
+    * Required by the library browser.
+    */
+   void initCopyPasteActions( );
+
+   /**
+    * creates the widget of the part instance and sets
     * it as the view
     */
    void initView( QWidget* parent, const char* name );
-   /** initializes the documents contents */
+   /**
+    * initializes the documents contents
+    */
    void initDocument( );
-   /** clears the selection */
+   /**
+    * clears the selection
+    */
    void clearSelection( );
 
 private:
-   /** Disables all actions, that modify the part */
+   /**
+    * Disables all actions, that modify the part
+    */
    void disableReadWriteActions( );
-   /** Updates all "new object" actions */
+   /**
+    * Updates all "new object" actions
+    */
    void updateNewObjectActions( );
    /**
     * Finds a free id of the format <prefix><number>.
@@ -431,57 +742,112 @@ private:
     */
    unsigned int findNewID( const QString& prefix, unsigned int firstNumber,
                            PMDeclare* obj );
-   /** Updates the list of cameras */
+   /**
+    * Updates the list of cameras
+    */
    void updateCameraList( );
 
-   /** Generic drop/paste function */
+   /**
+    * Generic drop/paste function
+    */
    bool pasteOrDrop( const QString& type, QMimeSource* mime, PMObject* obj );
-   /** Generic cut/delete/remove function */
+   /**
+    * Generic cut/delete/remove function
+    */
    bool removeSelection( const QString& type );
-   /** Updates the render mode combo action */
+   /**
+    * Updates the render mode combo action
+    */
    void updateRenderModes( );
-   /** Updates the visibility level action */
+   /**
+    * Updates the visibility level action
+    */
    void updateVisibilityLevel( );
+   /**
+    * Updates the control point list
+    */
+   void updateControlPoints( PMObject* oldActive );
 
    PMView* m_pView;
    PMShell* m_pShell;
    PMBrowserExtension* m_pExtension;
 
-   /** the selected objects, unsorted! */
+   /**
+    * the selected objects, unsorted!
+    */
    PMObjectList m_selectedObjects;
-   /** the selected objects, sorted. This list is only created if necessary. */
+   /**
+    * the selected objects, sorted. This list is only created if necessary.
+    */
    PMObjectList m_sortedSelectedObjects;
-   /** true if the list m_sortedSelectedObjects is up to date */
+   /**
+    * true if the list m_sortedSelectedObjects is up to date
+    */
    bool m_sortedListUpToDate;
-   /** the active object */
+   /**
+    * the active object
+    */
    PMObject* m_pActiveObject;
-   /** the new selection after a command was executed */
+   /**
+    * the new selection after a command was executed
+    */
    PMObject* m_pNewSelection;
-   /** List of all cameras */
+   /**
+    * List of all cameras
+    */
    QPtrList<PMCamera> m_cameras;
-   /** true if the m_cameras list is up to date */
+   /**
+    * true if the m_cameras list is up to date
+    */
    bool m_bCameraListUpToDate;
-   /** true if the clipboard data can be decoded */
+   /**
+    * true if the clipboard data can be decoded
+    */
    bool m_canDecode;
-   /** Commands stack for undo and redo */
+   /**
+    * Commands stack for undo and redo
+    */
    PMCommandManager m_commandManager;
-   /** The povray scene, top level object */
+   /**
+    * The povray scene, top level object
+    */
    PMScene* m_pScene;
-   /** Number of added objects during the last executed command */
+   /**
+    * Number of added objects during the last executed command
+    */
    unsigned int m_numAddedObjects;
-   /** Number of insert errors during the last executed command */
+   /**
+    * Number of insert errors during the last executed command
+    */
    unsigned int m_numInsertErrors;
-   /** Details of insert errors */
+   /**
+    * Details of insert errors
+    */
    QStringList m_insertErrorDetails;
-   /** The symbol table for this document */
+   /**
+    * The symbol table for this document
+    */
    PMSymbolTable* m_pSymbolTable;
-   /** The povray render window */
+   /**
+    * The povray render window
+    */
    PMPovrayWidget* m_pPovrayWidget;
-   /** true if the new object actions have to be updated */
+   /**
+    * true if the new object actions have to be updated
+    */
    bool m_updateNewObjectActions;
+   /**
+    * Control points of the active object
+    */
+   PMControlPointList m_controlPoints;
+   /**
+    * true if only copy'n'paste actions are available
+    */
+   bool m_onlyCopyPaste;
 
    // the actions
-   KAction* m_pImportPovrayAction;
+   KAction* m_pImportAction;
+   KAction* m_pExportAction;
 
    KAction* m_pCutAction;
    KAction* m_pCopyAction;
@@ -535,6 +901,7 @@ private:
 
    KAction* m_pNewLightAction;
    KAction* m_pNewLooksLikeAction;
+   KAction* m_pNewProjectedThroughAction;
 
    KAction* m_pNewCameraAction;
 
@@ -571,6 +938,20 @@ private:
    KAction* m_pNewCommentAction;
    KAction* m_pNewRawAction;
 
+   // POV-Ray 3.5 objects
+   KAction* m_pNewIsoSurfaceAction;
+   KAction* m_pNewRadiosityAction;
+   KAction* m_pNewGlobalPhotonsAction;
+   KAction* m_pNewPhotonsAction;
+   KAction* m_pNewLightGroupAction;
+   KAction* m_pNewInteriorTextureAction;
+   KAction* m_pNewSphereSweepAction;
+   KAction* m_pNewMeshAction;
+
+   KAction* m_pImportLibraryObjectAction;
+   KAction* m_pExportLibraryObjectAction;
+   KAction* m_pSearchLibraryObjectAction;
+
    PMComboAction* m_pRenderComboAction;
    KAction* m_pRenderSettingsAction;
    KAction* m_pViewRenderWindowAction;
@@ -580,9 +961,9 @@ private:
 
    QPtrList<KAction> m_readWriteActions;
 
-   QPtrList<PMTreeView> m_treeViews;
-   QPtrList<PMDialogView> m_dialogViews;
-   QPtrList<PMGLView> m_glViews;
+   PMPrototypeManager* m_pPrototypeManager;
+   PMInsertRuleSystem* m_pInsertRuleSystem;
+   PMIOManager* m_pIOManager;
 };
 
 class PMBrowserExtension : public KParts::BrowserExtension
