@@ -2,7 +2,7 @@
 **************************************************************************
                                  description
                              --------------------
-    copyright            : (C) 2001 by Luis Carvalho
+    copyright            : (C) 2001-2002 by Luis Carvalho
     email                : lpassos@mail.telepac.pt
 **************************************************************************
 
@@ -19,14 +19,11 @@
 #include "pmpatternedit.h"
 #include "pmpattern.h"
 
-#include "pmoutputdevice.h"
 #include "pmxmlhelper.h"
 #include "pmlistpatternedit.h"
 #include "pmcompositeobject.h"
 #include "pmmemento.h"
-
-#include <kdebug.h>
-#include "pmglobals.h"
+#include "pmenumproperty.h"
 
 #include <klocale.h>
 
@@ -38,6 +35,8 @@ const int maxIterationsDefault = 10;
 const double quiltControl0Default = 1.0;
 const double quiltControl1Default = 1.0;
 const int spiralNumberArmsDefault = 0;
+const PMPattern::PMNoiseType noiseGeneratorDefault = PMPattern::GlobalSetting;
+const QString noiseGeneratorDefaultText = QString( "global_setting" );
 const QString densityFileDefault = QString( "" );
 const PMVector valueVectorDefault = PMVector( 0.0, 0.0, 0.0 );
 const int octavesDefault = 6;
@@ -45,9 +44,20 @@ const double omegaDefault = 0.5;
 const double lambdaDefault = 2.0;
 const double depthDefault = 0.0;
 
+PMDefinePropertyClass( PMPattern, PMProperty );
+PMDefineEnumPropertyClass( PMPattern, PMPattern::PMPatternType,
+                           PMPatternTypeProperty );
+PMDefineEnumPropertyClass( PMPattern, PMPattern::PMNoiseType,
+                           PMNoiseProperty );
 
-PMPattern::PMPattern( )
-      : Base( )
+PMMetaObject* PMPattern::s_pMetaObject = 0;
+PMObject* createNewPattern( PMPart* part )
+{
+   return new PMPattern( part );
+}
+
+PMPattern::PMPattern( PMPart* part )
+      : Base( part )
 {
    m_patternType = patternTypeDefault;
    m_agateTurbulence = agateTurbulenceDefault;
@@ -57,6 +67,7 @@ PMPattern::PMPattern( )
    m_quiltControl0 = quiltControl0Default;
    m_quiltControl1 = quiltControl1Default;
    m_spiralNumberArms = spiralNumberArmsDefault;
+   m_noiseGenerator = noiseGeneratorDefault;
    m_enableTurbulence = false;
    m_valueVector = valueVectorDefault;
    m_octaves = octavesDefault;
@@ -65,147 +76,28 @@ PMPattern::PMPattern( )
    m_depth = depthDefault;
 }
 
-PMPattern::~PMPattern( )
+PMPattern::PMPattern( const PMPattern& p )
+      : Base( p )
 {
+   m_patternType = p.m_patternType;
+   m_agateTurbulence = p.m_agateTurbulence;
+   m_densityInterpolate = p.m_densityInterpolate;
+   m_gradient = p.m_gradient;
+   m_maxIterations = p.m_maxIterations;
+   m_quiltControl0 = p.m_quiltControl0;
+   m_quiltControl1 = p.m_quiltControl1;
+   m_spiralNumberArms = p.m_spiralNumberArms;
+   m_noiseGenerator = p.m_noiseGenerator;
+   m_enableTurbulence = p.m_enableTurbulence;
+   m_valueVector = p.m_valueVector;
+   m_octaves = p.m_octaves;
+   m_omega = p.m_omega;
+   m_lambda = p.m_lambda;
+   m_depth = p.m_depth;
 }
 
-void PMPattern::serialize( PMOutputDevice& dev ) const
+PMPattern::~PMPattern( )
 {
-   QString str;
-
-   // pattern type
-   switch( m_patternType )
-   {
-      case PatternAgate:
-         dev.writeLine( "agate" );
-         break;
-      case PatternAverage:
-         dev.writeLine( "average" );
-         break;
-      case PatternBoxed:
-         dev.writeLine( "boxed" );
-         break;
-      case PatternBozo:
-         dev.writeLine( "bozo" );
-         break;
-      case PatternBumps:
-         dev.writeLine( "bumps" );
-         break;
-      case PatternCrackle:
-         dev.writeLine( "crackle" );
-         break;
-      case PatternCylindrical:
-         dev.writeLine( "cylindrical" );
-         break;
-      case PatternDensity:
-         dev.writeLine( "density_file df3 \"" + m_densityFile + "\"");  
-         break;
-      case PatternDents:
-         dev.writeLine( "dents" );
-         break;
-      case PatternGradient:
-         dev.writeLine( "gradient " + m_gradient.serialize( ) );
-         break;
-      case PatternGranite:
-         dev.writeLine( "granite" );
-         break;
-      case PatternLeopard:
-         dev.writeLine( "leopard" );
-         break;
-      case PatternMandel:
-         str.setNum( m_maxIterations );
-         dev.writeLine( "mandel " + str );
-         break;
-      case PatternMarble:
-         dev.writeLine( "marble" );
-         break;
-      case PatternOnion:
-         dev.writeLine( "onion" );
-         break;
-      case PatternPlanar:
-         dev.writeLine( "planar" );
-         break;
-      case PatternQuilted:
-         dev.writeLine( "quilted" );
-         break;
-      case PatternRadial:
-         dev.writeLine( "radial" );
-         break;
-      case PatternRipples:
-         dev.writeLine( "ripples" );
-         break;
-      case PatternSpherical:
-         dev.writeLine( "spherical" );
-         break;
-      case PatternSpiral1:
-         str.setNum( m_spiralNumberArms );
-         dev.writeLine( "spiral1 " + str );
-         break;
-      case PatternSpiral2:
-         str.setNum( m_spiralNumberArms );
-         dev.writeLine( "spiral2 " + str );
-         break;
-      case PatternSpotted:
-         dev.writeLine( "spotted" );
-         break;
-      case PatternWaves:
-         dev.writeLine( "waves" );
-         break;
-      case PatternWood:
-         dev.writeLine( "wood" );
-         break;
-      case PatternWrinkles:
-         dev.writeLine( "wrinkles" );
-         break;
-   }
-   // depth
-   if( parent( ) )
-   {
-      if( m_depth && parent( )->type( ) == PMTNormal )
-      {
-         str.setNum( m_depth );
-         dev.writeLine( str );
-      }
-   }
-   // modifiers
-   switch( m_patternType )
-   {
-      case PatternAgate:
-         str.setNum( m_agateTurbulence );
-         dev.writeLine( "agate_turb " + str );
-         break;
-      case PatternDensity:
-         str.setNum( m_densityInterpolate );
-         dev.writeLine( "interpolate " + str );
-         break;
-      case PatternQuilted:
-         str.setNum( m_quiltControl0 );
-         dev.writeLine( "control0 " + str );
-         str.setNum( m_quiltControl1 );
-         dev.writeLine( "control1 " + str );
-         break;
-      default:
-         break;
-   }
-   if( m_enableTurbulence )
-   {
-      dev.writeLine( "turbulence " + m_valueVector.serialize( ) );
-      if( m_octaves != octavesDefault )
-      {
-         str.setNum(m_octaves);
-         dev.writeLine( "octaves " + str );
-      }
-      if( m_omega != omegaDefault )
-      {
-         str.setNum(m_omega);
-         dev.writeLine( "omega " + str );
-      }
-      if( m_lambda != lambdaDefault )
-      {
-         str.setNum(m_lambda);
-         dev.writeLine( "lambda " + str );
-      }
-   }
 }
 
 void PMPattern::serialize( QDomElement& e, QDomDocument& ) const
@@ -299,6 +191,21 @@ void PMPattern::serialize( QDomElement& e, QDomDocument& ) const
    e.setAttribute( "densityinterpolate", m_densityInterpolate );
    e.setAttribute( "densityfile", m_densityFile );
    e.setAttribute( "gradient", m_gradient.serializeXML( ) );
+   switch ( m_noiseGenerator )
+   {
+      case GlobalSetting:
+         e.setAttribute( "noise_generator", "global_setting" );
+         break;
+      case Original:
+         e.setAttribute( "noise_generator", "original" );
+         break;
+      case RangeCorrected:
+         e.setAttribute( "noise_generator", "range_corrected" );
+         break;
+      case Perlin:
+         e.setAttribute( "noise_generator", "perlin" );
+         break;
+   }
    e.setAttribute( "enable_turbulence", m_enableTurbulence );
    e.setAttribute( "turbulence", m_valueVector.serializeXML( ) );
    e.setAttribute( "octaves", m_octaves );
@@ -372,6 +279,15 @@ void PMPattern::readAttributes( const PMXMLHelper& h )
    m_quiltControl1 = h.doubleAttribute( "quiltcontrol1", quiltControl1Default );
    m_densityInterpolate = h.intAttribute( "densityinterpolate", densityInterpolateDefault );
    m_densityFile = h.stringAttribute( "densityfile", densityFileDefault );
+   str = h.stringAttribute( "noise_generator", noiseGeneratorDefaultText );
+   if ( str == "original" )
+      m_noiseGenerator = Original;
+   else if ( str == "range_corrected" )
+      m_noiseGenerator = RangeCorrected;
+   else if ( str == "perlin" )
+      m_noiseGenerator = Perlin;
+   else
+      m_noiseGenerator = GlobalSetting;
    m_enableTurbulence = h.boolAttribute( "enable_turbulence" , false );
    m_valueVector = h.vectorAttribute( "turbulence", valueVectorDefault );
    m_octaves = h.intAttribute( "octaves", octavesDefault );
@@ -382,11 +298,88 @@ void PMPattern::readAttributes( const PMXMLHelper& h )
    Base::readAttributes( h );
 }
 
-bool PMPattern::isA( PMObjectType t ) const
+PMMetaObject* PMPattern::metaObject( ) const
 {
-   if( t == PMTPattern )
-      return true;
-   return Base::isA( t );
+   if( !s_pMetaObject )
+   {
+      s_pMetaObject = new PMMetaObject( "Pattern", Base::metaObject( ),
+                                        createNewPattern );
+      s_pMetaObject->addProperty(
+         new PMProperty( "agateTurbulence", &PMPattern::setAgateTurbulence, &PMPattern::agateTurbulence ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "spiralNumberArms", &PMPattern::setSpiralNumberArms, &PMPattern::spiralNumberArms ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "maxIterations", &PMPattern::setMaxIterations, &PMPattern::maxIterations ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "gradient", &PMPattern::setGradient, &PMPattern::gradient ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "quiltControl0", &PMPattern::setQuiltControl0, &PMPattern::quiltControl0 ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "quiltControl1", &PMPattern::setQuiltControl1, &PMPattern::quiltControl1 ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "densityFile", &PMPattern::setDensityFile, &PMPattern::densityFile ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "densityInterpolate", &PMPattern::setDensityInterpolate, &PMPattern::densityInterpolate ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "valueVector", &PMPattern::setValueVector, &PMPattern::valueVector ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "octaves", &PMPattern::setOctaves, &PMPattern::octaves ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "omega", &PMPattern::setOmega, &PMPattern::omega ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "lambda", &PMPattern::setLambda, &PMPattern::lambda ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "turbulence", &PMPattern::enableTurbulence, &PMPattern::isTurbulenceEnabled ) );
+
+      PMPatternTypeProperty* p = new PMPatternTypeProperty(
+         "patternType", &PMPattern::setPatternType, &PMPattern::patternType );
+      p->addEnumValue( "Agate", PatternAgate );
+      p->addEnumValue( "Average", PatternAverage );
+      p->addEnumValue( "Boxed", PatternBoxed );
+      p->addEnumValue( "Bozo", PatternBozo );
+      p->addEnumValue( "Bumps", PatternBumps );
+      p->addEnumValue( "Crackle", PatternCrackle );
+      p->addEnumValue( "Cylindrical", PatternCylindrical );
+      p->addEnumValue( "Density", PatternDensity );
+      p->addEnumValue( "Dents", PatternDents );
+      p->addEnumValue( "Gradient", PatternGradient );
+      p->addEnumValue( "Granite", PatternGranite );
+      p->addEnumValue( "Leopard", PatternLeopard );
+      p->addEnumValue( "Mandel", PatternMandel );
+      p->addEnumValue( "Marble", PatternMarble );
+      p->addEnumValue( "Onion", PatternOnion );
+      p->addEnumValue( "Planar", PatternPlanar );
+      p->addEnumValue( "Quilted", PatternQuilted );
+      p->addEnumValue( "Radial", PatternRadial );
+      p->addEnumValue( "Ripples", PatternRipples );
+      p->addEnumValue( "Spherical", PatternSpherical );
+      p->addEnumValue( "Spiral1", PatternSpiral1 );
+      p->addEnumValue( "Spiral2", PatternSpiral2 );
+      p->addEnumValue( "Spotted", PatternSpotted );
+      p->addEnumValue( "Waves", PatternWaves );
+      p->addEnumValue( "Wood", PatternWood );
+      p->addEnumValue( "Wrinkles", PatternWrinkles );
+      s_pMetaObject->addProperty( p );
+
+      PMNoiseProperty* p2 = new PMNoiseProperty(
+         "noiseGenerator", &PMPattern::setNoiseGenerator, &PMPattern::noiseGenerator );
+      p2->addEnumValue( "GlobalSetting", GlobalSetting );
+      p2->addEnumValue( "Original", Original );
+      p2->addEnumValue( "RangeCorrected", RangeCorrected );
+      p2->addEnumValue( "Perlin", Perlin );
+      s_pMetaObject->addProperty( p2 );
+   }
+   return s_pMetaObject;
+}
+
+void PMPattern::cleanUp( ) const
+{
+   if( s_pMetaObject )
+   {
+      delete s_pMetaObject;
+      s_pMetaObject = 0;
+   }
+   Base::cleanUp( );
 }
 
 QString PMPattern::description( ) const
@@ -399,9 +392,9 @@ void PMPattern::setPatternType( PMPatternType c )
    if( c != m_patternType )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMPatternTypeID, m_patternType );
+         m_pMemento->addData( s_pMetaObject, PMPatternTypeID, m_patternType );
       m_patternType = c;
-   } 
+   }
 }
 
 void PMPattern::setAgateTurbulence( double c )
@@ -409,9 +402,9 @@ void PMPattern::setAgateTurbulence( double c )
    if( c != m_agateTurbulence )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMAgateTurbulenceID, m_agateTurbulence );
+         m_pMemento->addData( s_pMetaObject, PMAgateTurbulenceID, m_agateTurbulence );
       m_agateTurbulence = c;
-   } 
+   }
 }
 
 void PMPattern::setSpiralNumberArms( int c )
@@ -419,9 +412,9 @@ void PMPattern::setSpiralNumberArms( int c )
    if( c != m_spiralNumberArms )
    {
       if(m_pMemento )
-         m_pMemento->addData( PMTPattern, PMSpiralNumberArmsID, m_spiralNumberArms );
+         m_pMemento->addData( s_pMetaObject, PMSpiralNumberArmsID, m_spiralNumberArms );
       m_spiralNumberArms = c;
-   } 
+   }
 }
 
 void PMPattern::setMaxIterations( int c )
@@ -429,9 +422,9 @@ void PMPattern::setMaxIterations( int c )
    if( c != m_maxIterations )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMMaxIterationsID, m_maxIterations );
+         m_pMemento->addData( s_pMetaObject, PMMaxIterationsID, m_maxIterations );
       m_maxIterations = c;
-   } 
+   }
 }
 
 void PMPattern::setQuiltControl0( double c )
@@ -439,9 +432,9 @@ void PMPattern::setQuiltControl0( double c )
    if( c != m_quiltControl0 )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMQuiltControl0ID, m_quiltControl0 );
+         m_pMemento->addData( s_pMetaObject, PMQuiltControl0ID, m_quiltControl0 );
       m_quiltControl0 = c;
-   } 
+   }
 }
 
 void PMPattern::setQuiltControl1( double c )
@@ -449,9 +442,9 @@ void PMPattern::setQuiltControl1( double c )
    if( c != m_quiltControl1 )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMQuiltControl1ID, m_quiltControl1 );
+         m_pMemento->addData( s_pMetaObject, PMQuiltControl1ID, m_quiltControl1 );
       m_quiltControl1 = c;
-   } 
+   }
 }
 
 void PMPattern::setDensityInterpolate( int c )
@@ -459,9 +452,9 @@ void PMPattern::setDensityInterpolate( int c )
    if( c != m_densityInterpolate )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMDensityInterpolateID, m_densityInterpolate );
+         m_pMemento->addData( s_pMetaObject, PMDensityInterpolateID, m_densityInterpolate );
       m_densityInterpolate = c;
-   } 
+   }
 }
 
 void PMPattern::setGradient( const PMVector& v )
@@ -469,9 +462,9 @@ void PMPattern::setGradient( const PMVector& v )
    if( v != m_gradient )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMGradientID, m_gradient );
+         m_pMemento->addData( s_pMetaObject, PMGradientID, m_gradient );
       m_gradient = v;
-   } 
+   }
 }
 
 void PMPattern::setDensityFile( const QString& s )
@@ -479,9 +472,19 @@ void PMPattern::setDensityFile( const QString& s )
    if( s != m_densityFile )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMDensityFileID, m_densityFile );
+         m_pMemento->addData( s_pMetaObject, PMDensityFileID, m_densityFile );
       m_densityFile = s;
-   } 
+   }
+}
+
+void PMPattern::setNoiseGenerator( PMNoiseType c )
+{
+   if( c != m_noiseGenerator )
+   {
+      if( m_pMemento )
+         m_pMemento->addData( s_pMetaObject, PMNoiseGeneratorID, m_noiseGenerator );
+      m_noiseGenerator = c;
+   }
 }
 
 void PMPattern::enableTurbulence( bool c )
@@ -489,7 +492,7 @@ void PMPattern::enableTurbulence( bool c )
    if( c != m_enableTurbulence )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMEnableTurbulenceID, m_enableTurbulence );
+         m_pMemento->addData( s_pMetaObject, PMEnableTurbulenceID, m_enableTurbulence );
       m_enableTurbulence = c;
    }
 }
@@ -499,7 +502,7 @@ void PMPattern::setValueVector( const PMVector& c )
    if( c != m_valueVector )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMValueVectorID, m_valueVector );
+         m_pMemento->addData( s_pMetaObject, PMValueVectorID, m_valueVector );
       m_valueVector = c;
    }
 }
@@ -509,7 +512,7 @@ void PMPattern::setOctaves( const int c )
    if( c != m_octaves )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMOctavesID, m_octaves );
+         m_pMemento->addData( s_pMetaObject, PMOctavesID, m_octaves );
       m_octaves = c;
    }
 }
@@ -519,7 +522,7 @@ void PMPattern::setOmega( const double c )
    if( c != m_omega )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMOmegaID, m_omega );
+         m_pMemento->addData( s_pMetaObject, PMOmegaID, m_omega );
       m_omega = c;
    }
 }
@@ -529,7 +532,7 @@ void PMPattern::setLambda( const double c )
    if( c != m_lambda )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMLambdaID, m_lambda );
+         m_pMemento->addData( s_pMetaObject, PMLambdaID, m_lambda );
       m_lambda = c;
    }
 }
@@ -539,7 +542,7 @@ void PMPattern::setDepth( const double c )
    if( c != m_depth )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTPattern, PMDepthID, m_depth );
+         m_pMemento->addData( s_pMetaObject, PMDepthID, m_depth );
       m_depth = c;
    }
 }
@@ -557,7 +560,7 @@ void PMPattern::restoreMemento( PMMemento* s )
    for( ; it.current( ); ++it )
    {
       data = it.current( );
-      if( data->objectType( ) == PMTPattern )
+      if( data->objectType( ) == s_pMetaObject )
       {
          switch( data->valueID( ) )
          {
@@ -588,6 +591,9 @@ void PMPattern::restoreMemento( PMMemento* s )
             case PMDensityFileID:
                setDensityFile( data->stringData( ) );
                break;
+            case PMNoiseGeneratorID:
+               setNoiseGenerator( ( PMNoiseType ) ( data->intData( ) ) );
+               break;
             case PMEnableTurbulenceID:
                enableTurbulence( data->boolData( ) );
                break;
