@@ -24,9 +24,6 @@
 #include "pmlineedits.h"
 #include "pmvector.h"
 
-#include <kdebug.h>
-#include "pmglobals.h"
-
 #include <qwidget.h>
 #include <qlayout.h>
 #include <qcombobox.h>
@@ -84,7 +81,7 @@ void PMPatternEdit::createTopWidgets( )
    m_pTypeCombo->insertItem( i18n( "Wood" ) );
    m_pTypeCombo->insertItem( i18n( "Wrinkles" ) );
 
-   layout = new QGridLayout( topLayout( ), 8, 2 );
+   layout = new QGridLayout( topLayout( ), 9, 2 );
    hl = new QHBoxLayout( KDialog::spacingHint( ) );
    hl->addWidget( label );
    hl->addWidget( m_pTypeCombo );
@@ -95,7 +92,7 @@ void PMPatternEdit::createTopWidgets( )
    m_pAgateTurbulenceEdit = new PMFloatEdit( this );
    layout->addWidget( m_pAgateTurbulenceLabel, 1, 0 );
    layout->addWidget( m_pAgateTurbulenceEdit, 1, 1, AlignLeft );
-   
+
    m_pGradientLabel = new QLabel( i18n( "Gradient:" ), this );
    m_pGradientEdit = new PMVectorEdit( "x", "y", "z", this );
    layout->addWidget( m_pGradientLabel, 2, 0 );
@@ -125,7 +122,7 @@ void PMPatternEdit::createTopWidgets( )
    m_pDepthEdit = new PMFloatEdit( this );
    layout->addWidget( m_pDepthLabel, 6, 0 );
    layout->addWidget( m_pDepthEdit, 6, 1, AlignLeft );
-   
+
    m_pDensityWidget = new QWidget( this );
    vl = new QVBoxLayout( m_pDensityWidget, 0, KDialog::spacingHint( ) );
    hl = new QHBoxLayout( vl );
@@ -146,6 +143,16 @@ void PMPatternEdit::createTopWidgets( )
    hl->addStretch( 1 );
    layout->addMultiCellWidget( m_pDensityWidget, 7, 7, 0, 1 );
 
+
+   m_pNoiseGeneratorLabel = new QLabel( i18n( "Noise generator:" ), this );
+   m_pNoiseGenerator = new QComboBox( false, this );
+   m_pNoiseGenerator->insertItem( i18n( "Use Global Setting" ) );
+   m_pNoiseGenerator->insertItem( i18n( "Original" ) );
+   m_pNoiseGenerator->insertItem( i18n( "Range Corrected" ) );
+   m_pNoiseGenerator->insertItem( i18n( "Perlin" ) );
+   layout->addWidget( m_pNoiseGeneratorLabel, 8, 0 );
+   layout->addWidget( m_pNoiseGenerator, 8, 1 );
+
    m_pEnableTurbulenceEdit = new QCheckBox( i18n( "Turbulence" ), this );
    topLayout( )->addWidget( m_pEnableTurbulenceEdit );
 
@@ -179,6 +186,7 @@ void PMPatternEdit::createTopWidgets( )
    connect( m_pQuiltControl0Edit, SIGNAL( dataChanged( ) ), SIGNAL( dataChanged( ) ) );
    connect( m_pQuiltControl1Edit, SIGNAL( dataChanged( ) ), SIGNAL( dataChanged( ) ) );
    connect( m_pMaxIterationsEdit, SIGNAL( dataChanged( ) ), SIGNAL( dataChanged( ) ) );
+   connect( m_pNoiseGenerator, SIGNAL( activated( int ) ), SIGNAL( dataChanged( ) ) );
    connect( m_pEnableTurbulenceEdit, SIGNAL( clicked( ) ), SLOT( slotTurbulenceClicked( ) ) );
    connect( m_pValueVectorEdit, SIGNAL( dataChanged( ) ), SIGNAL( dataChanged( ) ) );
    connect( m_pOctavesEdit, SIGNAL( dataChanged( ) ), SIGNAL( dataChanged( ) ) );
@@ -186,7 +194,7 @@ void PMPatternEdit::createTopWidgets( )
    connect( m_pLambdaEdit, SIGNAL( dataChanged( ) ), SIGNAL( dataChanged( ) ) );
    connect( m_pDepthEdit, SIGNAL( dataChanged( ) ), SIGNAL( dataChanged( ) ) );
    connect( m_pDensityInterpolate, SIGNAL( activated( int ) ), SLOT( slotInterpolateChanged( int ) ) );
-   connect( m_pDensityFile, SIGNAL( textChanged( const QString& ) ), 
+   connect( m_pDensityFile, SIGNAL( textChanged( const QString& ) ),
                                           SLOT( slotDensityFileNameChanged( const QString& ) ) );
    connect( m_pDensityFileBrowse, SIGNAL( clicked( ) ), SLOT( slotDensityFileBrowseClicked( ) ) );
 }
@@ -196,7 +204,7 @@ void PMPatternEdit::displayObject( PMObject* o )
    QString str;
    bool readOnly;
 
-   if( o->isA( PMTPattern ) )
+   if( o->isA( "Pattern" ) )
    {
       m_pDisplayedObject = ( PMPattern* ) o;
       readOnly = m_pDisplayedObject->isReadOnly( );
@@ -299,6 +307,8 @@ void PMPatternEdit::displayObject( PMObject* o )
       m_pDensityFile->setEnabled( !readOnly );
       m_pDensityInterpolate->setCurrentItem( m_pDisplayedObject->densityInterpolate( ) );
       m_pDensityInterpolate->setEnabled( !readOnly );
+      m_pNoiseGenerator->setCurrentItem( m_pDisplayedObject->noiseGenerator( ) );
+      m_pNoiseGenerator->setEnabled( !readOnly );
 
       m_pEnableTurbulenceEdit->setChecked( m_pDisplayedObject->isTurbulenceEnabled( ) );
       m_pEnableTurbulenceEdit->setEnabled( !readOnly );
@@ -310,8 +320,8 @@ void PMPatternEdit::displayObject( PMObject* o )
       m_pOmegaEdit->setReadOnly( readOnly );
       m_pLambdaEdit->setValue( m_pDisplayedObject->lambda( ) );
       m_pLambdaEdit->setReadOnly( readOnly );
-      
-      if( o->parent( ) && ( o->parent( )->type( ) == PMTNormal ) )
+
+      if( o->parent( ) && ( o->parent( )->type( ) == "Normal" ) )
       {
          m_pDepthEdit->setValue( m_pDisplayedObject->depth( ) );
          m_pDepthEdit->setReadOnly( readOnly );
@@ -359,9 +369,13 @@ void PMPatternEdit::saveContents( )
             break;
          case 3:
             m_pDisplayedObject->setPatternType( PMPattern::PatternBozo );
+            m_pDisplayedObject->setNoiseGenerator(
+               ( PMPattern::PMNoiseType ) m_pNoiseGenerator->currentItem( ) );
             break;
          case 4:
             m_pDisplayedObject->setPatternType( PMPattern::PatternBumps );
+            m_pDisplayedObject->setNoiseGenerator(
+               ( PMPattern::PMNoiseType ) m_pNoiseGenerator->currentItem( ) );
             break;
          case 5:
             m_pDisplayedObject->setPatternType( PMPattern::PatternCrackle );
@@ -383,6 +397,8 @@ void PMPatternEdit::saveContents( )
             break;
          case 10:
             m_pDisplayedObject->setPatternType( PMPattern::PatternGranite );
+            m_pDisplayedObject->setNoiseGenerator(
+               ( PMPattern::PMNoiseType ) m_pNoiseGenerator->currentItem( ) );
             break;
          case 11:
             m_pDisplayedObject->setPatternType( PMPattern::PatternLeopard );
@@ -432,6 +448,8 @@ void PMPatternEdit::saveContents( )
             break;
          case 25:
             m_pDisplayedObject->setPatternType( PMPattern::PatternWrinkles );
+            m_pDisplayedObject->setNoiseGenerator(
+               ( PMPattern::PMNoiseType ) m_pNoiseGenerator->currentItem( ) );
             break;
       }
       m_pDisplayedObject->enableTurbulence( m_pEnableTurbulenceEdit->isChecked( ) );
@@ -441,7 +459,7 @@ void PMPatternEdit::saveContents( )
       m_pDisplayedObject->setLambda( m_pLambdaEdit->value( ) );
 
       if( m_pDisplayedObject->parent( ) && 
-          ( m_pDisplayedObject->parent( )->type( ) == PMTNormal ) )
+          ( m_pDisplayedObject->parent( )->type( ) == "Normal" ) )
          m_pDisplayedObject->setDepth( m_pDepthEdit->value( ) );
    }
 }
@@ -482,7 +500,7 @@ bool PMPatternEdit::isDataValid( )
       if( !m_pLambdaEdit->isDataValid( ) ) return false;
    }
    if( m_pDisplayedObject->parent( ) &&
-       m_pDisplayedObject->parent( )->type( ) == PMTNormal && 
+       m_pDisplayedObject->parent( )->type( ) == "Normal" && 
        !m_pDepthEdit->isDataValid( ) ) 
       return false;
 
@@ -506,6 +524,25 @@ void PMPatternEdit::slotComboChanged( int c )
          m_pMaxIterationsEdit->hide( );
          m_pMaxIterationsLabel->hide( );
          m_pDensityWidget->hide( );
+         m_pNoiseGeneratorLabel->hide( );
+         m_pNoiseGenerator->hide( );
+         break;
+      case 3:  /* Bozo */
+      case 4:  /* Bumps */
+         m_pAgateTurbulenceEdit->hide( );
+         m_pAgateTurbulenceLabel->hide( );
+         m_pSpiralNumberEdit->hide( );
+         m_pSpiralNumberLabel->hide( );
+         m_pGradientEdit->hide( );
+         m_pGradientLabel->hide( );
+         m_pQuiltControlsLabel->hide( );
+         m_pQuiltControl0Edit->hide( );
+         m_pQuiltControl1Edit->hide( );
+         m_pMaxIterationsEdit->hide( );
+         m_pMaxIterationsLabel->hide( );
+         m_pDensityWidget->hide( );
+         m_pNoiseGeneratorLabel->show( );
+         m_pNoiseGenerator->show( );
          break;
       case 7:  /* Density File */
          m_pAgateTurbulenceEdit->hide( );
@@ -520,6 +557,8 @@ void PMPatternEdit::slotComboChanged( int c )
          m_pMaxIterationsEdit->hide( );
          m_pMaxIterationsLabel->hide( );
          m_pDensityWidget->show( );
+         m_pNoiseGeneratorLabel->hide( );
+         m_pNoiseGenerator->hide( );
          break;
       case 9:  /* Gradient */
          m_pAgateTurbulenceEdit->hide( );
@@ -534,6 +573,24 @@ void PMPatternEdit::slotComboChanged( int c )
          m_pMaxIterationsEdit->hide( );
          m_pMaxIterationsLabel->hide( );
          m_pDensityWidget->hide( );
+         m_pNoiseGeneratorLabel->hide( );
+         m_pNoiseGenerator->hide( );
+         break;
+      case 10:  /* Granite */
+         m_pAgateTurbulenceEdit->hide( );
+         m_pAgateTurbulenceLabel->hide( );
+         m_pSpiralNumberEdit->hide( );
+         m_pSpiralNumberLabel->hide( );
+         m_pGradientEdit->hide( );
+         m_pGradientLabel->hide( );
+         m_pQuiltControlsLabel->hide( );
+         m_pQuiltControl0Edit->hide( );
+         m_pQuiltControl1Edit->hide( );
+         m_pMaxIterationsEdit->hide( );
+         m_pMaxIterationsLabel->hide( );
+         m_pDensityWidget->hide( );
+         m_pNoiseGeneratorLabel->show( );
+         m_pNoiseGenerator->show( );
          break;
       case 12:  /* Mandel */
          m_pAgateTurbulenceEdit->hide( );
@@ -548,6 +605,8 @@ void PMPatternEdit::slotComboChanged( int c )
          m_pMaxIterationsEdit->show( );
          m_pMaxIterationsLabel->show( );
          m_pDensityWidget->hide( );
+         m_pNoiseGeneratorLabel->hide( );
+         m_pNoiseGenerator->hide( );
          break;
       case 16:  /* Quilted */
          m_pAgateTurbulenceEdit->hide( );
@@ -562,6 +621,8 @@ void PMPatternEdit::slotComboChanged( int c )
          m_pMaxIterationsEdit->hide( );
          m_pMaxIterationsLabel->hide( );
          m_pDensityWidget->hide( );
+         m_pNoiseGeneratorLabel->hide( );
+         m_pNoiseGenerator->hide( );
          break;
       case 20:  /* Spiral1 */
       case 21:  /* Spiral2 */
@@ -577,6 +638,24 @@ void PMPatternEdit::slotComboChanged( int c )
          m_pMaxIterationsEdit->hide( );
          m_pMaxIterationsLabel->hide( );
          m_pDensityWidget->hide( );
+         m_pNoiseGeneratorLabel->hide( );
+         m_pNoiseGenerator->hide( );
+         break;
+      case 25:  /* Wrinkles */
+         m_pAgateTurbulenceEdit->hide( );
+         m_pAgateTurbulenceLabel->hide( );
+         m_pSpiralNumberEdit->hide( );
+         m_pSpiralNumberLabel->hide( );
+         m_pGradientEdit->hide( );
+         m_pGradientLabel->hide( );
+         m_pQuiltControlsLabel->hide( );
+         m_pQuiltControl0Edit->hide( );
+         m_pQuiltControl1Edit->hide( );
+         m_pMaxIterationsEdit->hide( );
+         m_pMaxIterationsLabel->hide( );
+         m_pDensityWidget->hide( );
+         m_pNoiseGeneratorLabel->show( );
+         m_pNoiseGenerator->show( );
          break;
       default:
          m_pAgateTurbulenceEdit->hide( );
@@ -591,8 +670,10 @@ void PMPatternEdit::slotComboChanged( int c )
          m_pMaxIterationsEdit->hide( );
          m_pMaxIterationsLabel->hide( );
          m_pDensityWidget->hide( );
+         m_pNoiseGeneratorLabel->hide( );
+         m_pNoiseGenerator->hide( );
          break;
-   }  
+   }
    emit dataChanged( );
    emit sizeChanged( );
 }
@@ -639,7 +720,7 @@ void PMPatternEdit::slotDensityFileBrowseClicked( )
 {
    QString str = KFileDialog::getOpenFileName( QString::null, QString::null );
 
-   if( str != "" && str != QString::null )
+   if( !str.isEmpty() )
    {
       m_pDensityFile->setText( str );
       emit dataChanged( );
