@@ -20,9 +20,6 @@
 #include "pmpigment.h"
 #include "pmlinkedit.h"
 
-#include <kdebug.h>
-#include "pmglobals.h"
-
 #include <qlayout.h>
 #include <qlabel.h>
 #include <klocale.h>
@@ -36,7 +33,7 @@ PMPigmentEdit::PMPigmentEdit( QWidget* parent, const char* name )
 
 void PMPigmentEdit::displayObject( PMObject* o )
 {
-   if( o->isA( PMTPigment ) )
+   if( o->isA( "Pigment" ) )
    {
       Base::displayObject( o );
    }
