@@ -20,9 +20,8 @@
 #include "pmprism.h"
 #include "pmvectoredit.h"
 #include "pmlineedits.h"
-
-#include <kdebug.h>
-#include "pmglobals.h"
+#include "pmvectorlistedit.h"
+#include "pmpart.h"
 
 #include <qlayout.h>
 #include <qlabel.h>
@@ -39,17 +38,11 @@ PMPrismEdit::PMPrismEdit( QWidget* parent, const char* name )
       : Base( parent, name )
 {
    m_pDisplayedObject = 0;
-   m_edits.setAutoDelete( true );
-   m_addButtons.setAutoDelete( true );
-   m_removeButtons.setAutoDelete( true );
    m_lastSplineType = 0;
 }
 
 PMPrismEdit::~PMPrismEdit( )
 {
-   m_edits.clear( );
-   m_addButtons.clear( );
-   m_removeButtons.clear( );
 }
 
 void PMPrismEdit::createTopWidgets( )
@@ -93,7 +86,6 @@ void PMPrismEdit::createTopWidgets( )
 
 void PMPrismEdit::createBottomWidgets( )
 {
-   //topLayout( )->addWidget( new QLabel( i18n( "Prism Points:" ), this ) );
    m_pEditWidget = new QWidget( this );
    topLayout( )->addWidget( m_pEditWidget );
    m_pOpen = new QCheckBox( i18n( "Open" ), this );
@@ -108,7 +100,7 @@ void PMPrismEdit::createBottomWidgets( )
 
 void PMPrismEdit::displayObject( PMObject* o )
 {
-   if( o->isA( PMTPrism ) )
+   if( o->isA( "Prism" ) )
    {
       bool readOnly = o->isReadOnly( );
       m_pDisplayedObject = ( PMPrism* ) o;
@@ -159,107 +151,65 @@ void PMPrismEdit::displayPoints( const QValueList< QValueList<PMVector> >& sp )
 {
    bool readOnly = m_pDisplayedObject->isReadOnly( );
 
-   // (re)create the edit widget if neccessary
+   // (re)create the edit widget if necessary
    createEdits( sp );
    
    QValueList< QValueList<PMVector> >::ConstIterator spit = sp.begin( );
-   QPtrListIterator< QPtrList<PMVectorEdit> > seit( m_edits );
+   QPtrListIterator< PMVectorListEdit > seit( m_points );
+   QPtrListIterator< QPushButton > sbit1( m_removeButtons );
 
    // display the points
-   for( ; ( spit != sp.end( ) ) && *seit; ++spit, ++seit )
+   for( ; ( spit != sp.end( ) ) && *seit; ++spit, ++seit, ++sbit1 )
    {
-      QValueList<PMVector>::ConstIterator pit = ( *spit ).begin( );
-      QPtrListIterator<PMVectorEdit> eit( **seit );
-
-      for( ; ( pit != ( *spit ).end( ) ) && *eit; ++pit, ++eit )
-      {
-         ( *eit )->setVector( *pit );
-         ( *eit )->setReadOnly( readOnly );
-      }
+      ( *seit )->setVectors( *spit );
+      ( *seit )->setReadOnly( readOnly );
+      ( *sbit1 )->setEnabled( !readOnly && ( *spit ).size( ) > 3 );
    }
 
-   // enable/disable the edits
-   QPtrListIterator< QPtrList<QPushButton> > sbit1( m_addButtons );
-   for( ; *sbit1; ++sbit1 )
-   {
-      QPtrListIterator<QPushButton> bit( **sbit1 );
-      for( ; *bit; ++bit )
-         ( *bit )->setEnabled( !readOnly );
-   }
-   QPtrListIterator< QPtrList<QPushButton> > sbit2( m_removeButtons );
+   QPtrListIterator< QPushButton > sbit2( m_addAboveButtons );
+   QPtrListIterator< QPushButton > sbit3( m_addBelowButtons );
    for( ; *sbit2; ++sbit2 )
-   {
-      QPtrListIterator<QPushButton> bit( **sbit2 );
-      for( ; *bit; ++bit )
-         ( *bit )->setEnabled( !readOnly );
-   }
+      ( *sbit2 )->setEnabled( !readOnly );
+   for( ; *sbit3; ++sbit3 )
+      ( *sbit3 )->setEnabled( !readOnly );
+
    QPtrListIterator<QPushButton> bit1( m_subPrismAddButtons );
    for( ; *bit1; ++bit1 )
       ( *bit1 )->setEnabled( !readOnly );
    QPtrListIterator<QPushButton> bit2( m_subPrismRemoveButtons );
    for( ; *bit2; ++bit2 )
-      ( *bit2 )->setEnabled( !readOnly );
+      ( *bit2 )->setEnabled( !readOnly && sp.size( ) > 1 );
+   updateControlPointSelection( );
 }
 
 void PMPrismEdit::createEdits( const QValueList< QValueList<PMVector> >& sp )
 {
-   // test if the current edits can be reused
-   bool reuse = true;
    int st = m_pSplineType->currentItem( );
-
-   if( m_labels.count( ) == 0 )               // no edits created
-      reuse = false;
-   else if( m_lastSplineType != st )          // labels %1=%2 would be wrong
-      reuse = false;
-   else if( sp.count( ) != m_edits.count( ) ) // different number of sub prisms
-      reuse = false;
-   else
-   {
-      QValueList< QValueList<PMVector> >::ConstIterator spit = sp.begin( );
-      QPtrListIterator< QPtrList<PMVectorEdit> > eit( m_edits );
-      for( ; ( spit != sp.end( ) ) && *eit && reuse; ++spit, ++eit )
-         if( ( *spit ).count( ) != ( *eit )->count( ) )
-            reuse = false; // different number of points in sub prism
-   }
-   m_lastSplineType = st;
-
-   if( !reuse )
+   
+   if( sp.size( ) != m_points.count( ) )
    {
       deleteEdits( );
-      
-      // count number of needed lines
-      QValueList< QValueList<PMVector> >::ConstIterator spit = sp.begin( );
-      int lines = 1;
-      for( ; spit != sp.end( ); ++spit )
-      {
-         if( st != 3 )
-            lines += ( *spit ).count( ) + 4;
-         else
-            lines += ( *spit ).count( ) / 3 * 4 + 3;
-      }
-      
-      QGridLayout* gl = new QGridLayout( m_pEditWidget, lines, 4,
-                                         0, KDialog::spacingHint( ) );
+
       QPixmap addPixmap = SmallIcon( "pmaddpoint" );
       QPixmap removePixmap = SmallIcon( "pmremovepoint" );
       QPixmap addPrismPixmap = SmallIcon( "pmaddsubprism" );
-      QLabel* label;
-      QPushButton* button;
-      int lnr = 0;
+      QVBoxLayout* tvl = new QVBoxLayout( m_pEditWidget,
+                                          0, KDialog::spacingHint( ) );
+      QHBoxLayout* hl = 0;
+      QVBoxLayout* vl;
+      QLabel* label = 0;
+      QPushButton* button = 0;
+      PMVectorListEdit* vle;
       int spnr = 0;
       
-      for( spit = sp.begin( ); spit != sp.end( ); ++spit, spnr++ )
+      for( spnr = 0; spnr < ( signed ) sp.size( ); spnr++ )
       {
          // create all edits for one sub prism
-         int pnr;
-         QValueList<PMVector>::ConstIterator pit = ( *spit ).begin( );
-         QPtrList<PMVectorEdit>* editList = new QPtrList<PMVectorEdit>;
-         QPtrList<QPushButton>* addList = new QPtrList<QPushButton>;
-         QPtrList<QPushButton>* removeList = new QPtrList<QPushButton>;
-         
+         hl = new QHBoxLayout( tvl );
          label = new QLabel( i18n( "Sub prism %1:" ).arg( spnr + 1 ),
                              m_pEditWidget );
-         gl->addMultiCellWidget( label, lnr, lnr, 0, 1 );
+         hl->addWidget( label );
+         hl->addStretch( 1 );
          m_labels.append( label );
          label->show( );
          
@@ -267,120 +217,60 @@ void PMPrismEdit::createEdits( const QValueList< QValueList<PMVector> >& sp )
          button->setPixmap( addPrismPixmap );
          m_subPrismAddButtons.append( button );
          connect( button, SIGNAL( clicked( ) ), SLOT( slotAddSubPrism( ) ) );
-         gl->addWidget( button, lnr, 2 );
+         hl->addWidget( button );
          button->show( );
-         QToolTip::add( button, i18n( "Add Sub Prism" ) );
+         QToolTip::add( button, i18n( "Add sub prism" ) );
          
          button = new QPushButton( m_pEditWidget );
          button->setPixmap( removePixmap );
          m_subPrismRemoveButtons.append( button );
          connect( button, SIGNAL( clicked( ) ), SLOT( slotRemoveSubPrism( ) ) );
-         gl->addWidget( button, lnr, 3 );
+         hl->addWidget( button );
          button->show( );
-         QToolTip::add( button, i18n( "Remove Sub Prism" ) );
+         if( sp.size( ) < 2 )
+            button->setEnabled( false );
+         QToolTip::add( button, i18n( "Remove sub prism" ) );
+
+         hl = new QHBoxLayout( tvl );
          
-         lnr++;
+         vle = new PMVectorListEdit( "x", "z", m_pEditWidget );
+         m_points.append( vle );
+         connect( vle, SIGNAL( dataChanged( ) ), SIGNAL( dataChanged( ) ) );
+         connect( vle, SIGNAL( selectionChanged( ) ),
+                  SLOT( slotSelectionChanged( ) ) );
+         hl->addWidget( vle, 2 );
+         vle->show( );
+
+         vl = new QVBoxLayout( hl );
          
          button = new QPushButton( m_pEditWidget );
-         button->setPixmap( addPixmap );
-         addList->append( button );
-         connect( button, SIGNAL( clicked( ) ), SLOT( slotAddPoint( ) ) );
-         gl->addWidget( button, lnr, 2 );
+         button->setPixmap( SmallIcon( "pmaddpointabove" ) );
+         connect( button, SIGNAL( clicked( ) ), SLOT( slotAddPointAbove( ) ) );
+         m_addAboveButtons.append( button );
          button->show( );
-         QToolTip::add( button, i18n( "Add new point" ) );
-         
-         lnr++;
-         
-         int refa = 0, refb = ( *spit ).count( ) - 1;
-         switch( m_pSplineType->currentItem( ) )
-         {
-            case 0:
-               break;
-            case 1:
-               refa++;
-               break;
-            case 2:
-               refa++;
-               refb--;
-               break;
-            case 3:
-               refb = ( refb + 1 ) / 3 * 4;
-               break;
-         }
+         vl->addWidget( button );
+         button = new QPushButton( m_pEditWidget );
+         button->setPixmap( SmallIcon( "pmaddpoint" ) );
+         connect( button, SIGNAL( clicked( ) ), SLOT( slotAddPointBelow( ) ) );
+         m_addBelowButtons.append( button );
+         button->show( );
+         vl->addWidget( button );
+         button = new QPushButton( m_pEditWidget );
+         button->setPixmap( SmallIcon( "pmremovepoint" ) );
+         connect( button, SIGNAL( clicked( ) ), SLOT( slotRemovePoint( ) ) );
+         m_removeButtons.append( button );
+         button->show( );
+         vl->addWidget( button );
 
-         for( pnr = 0; pit != ( *spit ).end( ); ++pit )
-         {
-            // create the edits for one point
-            PMVectorEdit* edit;
-            
-            label = new QLabel( QString( "%1:" ).arg( pnr + 1 ), m_pEditWidget );
-            gl->addWidget( label, lnr, 0 );
-            label->show( );
-            m_labels.append( label );
-            
-            edit = new PMVectorEdit( "x", "z", m_pEditWidget );
-            connect( edit, SIGNAL( dataChanged( ) ), SIGNAL( dataChanged( ) ) );
-            editList->append( edit );
-            gl->addWidget( edit, lnr, 1 );
-            edit->show( );
-            
-            button = new QPushButton( m_pEditWidget );
-            button->setPixmap( addPixmap );
-            addList->append( button );
-            connect( button, SIGNAL( clicked( ) ), SLOT( slotAddPoint( ) ) );
-            gl->addWidget( button, lnr, 2 );
-            button->show( );
-            QToolTip::add( button, i18n( "Add new point" ) );
+         vl->addStretch( 1 );
          
-            button = new QPushButton( m_pEditWidget );
-            button->setPixmap( removePixmap );
-            removeList->append( button );
-            connect( button, SIGNAL( clicked( ) ), SLOT( slotRemovePoint( ) ) );
-            gl->addWidget( button, lnr, 3 );
-            button->show( );
-            QToolTip::add( button, i18n( "Remove point" ) );
-
-            if( st != 3 )
-            {
-               if( pnr == refb )
-               {
-                  pnr++;
-                  lnr++;
-                  label = new QLabel( QString( "%1 = %2" ).arg( pnr + 1 )
-                                      .arg( refa + 1 ), m_pEditWidget );
-                  gl->addMultiCellWidget( label, lnr, lnr, 0, 1 );
-                  label->show( );
-                  m_labels.append( label );
-               }
-            }
-            else
-            {
-               if( ( ( pnr + 2 ) %4 ) == 0 )
-               {
-                  pnr++;
-                  lnr++;
-                  label = new QLabel( QString( "%1 = %2" ).arg( pnr + 1 )
-                                      .arg( ( pnr + 1 ) % refb + 1 ),
-                                      m_pEditWidget );
-                  gl->addMultiCellWidget( label, lnr, lnr, 0, 1 );
-                  label->show( );
-                  m_labels.append( label );
-               }
-            }
-            pnr++;
-            lnr++;
-         }
-         m_edits.append( editList );
-         m_addButtons.append( addList );
-         m_removeButtons.append( removeList );
-
-         // One extra line
-         gl->addRowSpacing( lnr, 8 );
-         lnr++;
+         tvl->addSpacing( KDialog::spacingHint( ) );
       }
-
+      
+      hl = new QHBoxLayout( tvl );
       label = new QLabel( i18n( "New sub prism" ), m_pEditWidget );
-      gl->addMultiCellWidget( label, lnr, lnr, 0, 1 );
+      hl->addWidget( label );
+      hl->addStretch( 1 );
       m_labels.append( label );
       label->show( );
          
@@ -388,36 +278,38 @@ void PMPrismEdit::createEdits( const QValueList< QValueList<PMVector> >& sp )
       button->setPixmap( addPrismPixmap );
       m_subPrismAddButtons.append( button );
       connect( button, SIGNAL( clicked( ) ), SLOT( slotAddSubPrism( ) ) );
-      gl->addWidget( button, lnr, 2 );
+      hl->addWidget( button );
       button->show( );
-      QToolTip::add( button, i18n( "Append Sub Prism" ) );
-      lnr++;
+      QToolTip::add( button, i18n( "Append sub prism" ) );
    }
+
+   QPtrListIterator< PMVectorListEdit > vlit( m_points );
+   QValueList< QValueList< PMVector > >::ConstIterator spit;
+   PMVectorListEdit* vle = 0;
+   bool newSize = false;
+   
+   for( spit = sp.begin( ); spit != sp.end( ); ++spit, ++vlit )
+   {
+      int lines = ( *spit ).count( );
+      
+      vle = *vlit;
+      if( ( vle->size( ) != lines ) /*|| ( st != m_lastSplineType )*/ )
+      {
+         newSize = true;
+         vle->setSize( lines );
+      }
+   }
+   if( newSize )
+   {
+      m_pEditWidget->updateGeometry( );
+      emit sizeChanged( );
+   }
+
+   m_lastSplineType = st;
 }
 
 void PMPrismEdit::deleteEdits( )
 {
-   QPtrListIterator< QPtrList<PMVectorEdit> > elit( m_edits );
-   for( ; elit.current( ); ++elit )
-   {
-      elit.current( )->setAutoDelete( true );
-      elit.current( )->clear( );
-   }
-   m_edits.clear( );
-   QPtrListIterator< QPtrList<QPushButton> > blit( m_addButtons );
-   for( ; blit.current( ); ++blit )
-   {
-      blit.current( )->setAutoDelete( true );
-      blit.current( )->clear( );
-   }
-   m_addButtons.clear( );
-   QPtrListIterator< QPtrList<QPushButton> > rlit( m_removeButtons );
-   for( ; rlit.current( ); ++rlit )
-   {
-      rlit.current( )->setAutoDelete( true );
-      rlit.current( )->clear( );
-   }
-   m_removeButtons.clear( );
    m_labels.setAutoDelete( true );
    m_labels.clear( );
    m_labels.setAutoDelete( false );
@@ -427,6 +319,18 @@ void PMPrismEdit::deleteEdits( )
    m_subPrismRemoveButtons.setAutoDelete( true );
    m_subPrismRemoveButtons.clear( );
    m_subPrismRemoveButtons.setAutoDelete( false );
+   m_addAboveButtons.setAutoDelete( true );
+   m_addAboveButtons.clear( );
+   m_addAboveButtons.setAutoDelete( false );
+   m_addBelowButtons.setAutoDelete( true );
+   m_addBelowButtons.clear( );
+   m_addBelowButtons.setAutoDelete( false );
+   m_removeButtons.setAutoDelete( true );
+   m_removeButtons.clear( );
+   m_removeButtons.setAutoDelete( false );
+   m_points.setAutoDelete( true );
+   m_points.clear( );
+   m_points.setAutoDelete( false );
    
    if( m_pEditWidget->layout( ) )
       delete m_pEditWidget->layout( );
@@ -434,18 +338,11 @@ void PMPrismEdit::deleteEdits( )
 
 QValueList< QValueList<PMVector> > PMPrismEdit::splinePoints( )
 {
-   QPtrListIterator< QPtrList<PMVectorEdit> > it( m_edits );
+   QPtrListIterator< PMVectorListEdit > it( m_points );
    QValueList< QValueList<PMVector> > values;
-   QValueList<PMVector> subPrism;
-   
+
    for( ; it.current( ); ++it )
-   {
-      subPrism.clear( );
-      QPtrListIterator<PMVectorEdit> sit( *it.current( ) );
-      for( ; sit.current( ); ++sit )
-         subPrism.append( sit.current( )->vector( ) );
-      values.append( subPrism );
-   }
+      values.append( ( *it )->vectors( ) );
    
    return values;
 }
@@ -490,24 +387,20 @@ void PMPrismEdit::saveContents( )
 
 bool PMPrismEdit::isDataValid( )
 {
-   QPtrListIterator< QPtrList<PMVectorEdit> > it( m_edits );
+   QPtrListIterator< PMVectorListEdit > it( m_points );
    for( ; it.current( ); ++it )
-   {
-      QPtrListIterator<PMVectorEdit> sit( *it.current( ) );
-      for( ; sit.current( ); ++sit )
-         if( !sit.current( )->isDataValid( ) )
-            return false;
-   }
-
+      if( !it.current( )->isDataValid( ) )
+         return false;
+   
    for( it.toFirst( ); it.current( ); ++it )
    {
-      int np = it.current( )->count( );
+      int np = it.current( )->size( );
       switch( m_pSplineType->currentItem( ) )
       {
          case 0:
             if( np < 3 )
             {
-               KMessageBox::error( this, i18n( "Linear splines need at least 4 points." ),
+               KMessageBox::error( this, i18n( "Linear splines need at least 3 points." ),
                                    i18n( "Error" ) );
                return false;
             }
@@ -515,7 +408,7 @@ bool PMPrismEdit::isDataValid( )
          case 1:
             if( np < 4 )
             {
-               KMessageBox::error( this, i18n( "Quadratic splines need at least 5 points." ),
+               KMessageBox::error( this, i18n( "Quadratic splines need at least 4 points." ),
                                    i18n( "Error" ) );
                return false;
             }
@@ -523,7 +416,7 @@ bool PMPrismEdit::isDataValid( )
          case 2:
             if( np < 5 )
             {
-               KMessageBox::error( this, i18n( "Cubic splines need at least 6 points." ),
+               KMessageBox::error( this, i18n( "Cubic splines need at least 5 points." ),
                                    i18n( "Error" ) );
                return false;
             }
@@ -531,13 +424,14 @@ bool PMPrismEdit::isDataValid( )
          case 3:
             if( ( np < 3 ) || ( ( np % 3 ) != 0 ) )
             {
-               KMessageBox::error( this, i18n( "Bezier splines need 4 points for each segment." ),
+               KMessageBox::error( this, i18n( "Bezier splines need 3 points for each segment." ),
                                    i18n( "Error" ) );
                return false;
             }
             break;
       }
    }
+   
    return Base::isDataValid( );
 }
 
@@ -553,72 +447,103 @@ void PMPrismEdit::slotSweepChanged( int )
    emit dataChanged( );
 }
 
-void PMPrismEdit::slotAddPoint( )
+void PMPrismEdit::slotAddPointAbove( )
 {
-   QPushButton* button = ( QPushButton* ) sender( );
-   if( button )
+   QPushButton* bt = ( QPushButton* ) sender( );
+   if( bt )
    {
-      QValueList< QValueList<PMVector> > points = splinePoints( );
-      QPtrListIterator< QPtrList<QPushButton> > seit( m_addButtons );
-      QValueList<QValueList<PMVector> >::Iterator spit = points.begin( );
-      bool found = false;
-
-      for( ; ( spit != points.end( ) ) && *seit && !found; ++spit, ++seit )
-      {  
-         int index = ( *seit )->findRef( button );
-         if( index >= 0 )
+      int subIndex = m_addAboveButtons.findRef( bt );
+      if( subIndex >= 0 )
+      {
+         PMVectorListEdit* ed = m_points.at( subIndex );
+         int index = ed->currentRow( );
+         if( index >= 0 && index < ed->size( ) )
          {
-            found = true;
-            QValueListIterator<PMVector> it = ( *spit ).at( index );
-            PMVector newPoint( 2 );
-            if( index == 0 )
-               newPoint = ( *spit ).first( );
-            else
+            QValueList<PMVector> points = ed->vectors( );
+            QValueListIterator<PMVector> it = points.at( index );
+            
+            PMVector newPoint = *it;
+            if( index != 0 )
             {
                --it;
-               newPoint = *it;
+               newPoint = ( newPoint + *it ) / 2;
                ++it;
-               if( it != ( *spit ).end( ) )
-                  newPoint = ( newPoint + *it ) / 2;
             }
-            ( *spit ).insert( it, newPoint );
+            points.insert( it, newPoint );
+
+            ed->setSize( points.size( ) );
+            ed->setVectors( points );
+            if( points.size( ) > 3 )
+               m_removeButtons.at( subIndex )->setEnabled( true );
+
+            emit dataChanged( );
+            emit sizeChanged( );
          }
       }
-      if( found )
+   }
+}
+
+void PMPrismEdit::slotAddPointBelow( )
+{
+   QPushButton* bt = ( QPushButton* ) sender( );
+   if( bt )
+   {
+      int subIndex = m_addBelowButtons.findRef( bt );
+      if( subIndex >= 0 )
       {
-         displayPoints( points );
-         emit dataChanged( );
-         emit sizeChanged( );
+         PMVectorListEdit* ed = m_points.at( subIndex );
+         int index = ed->currentRow( );
+         if( index >= 0 && index < ed->size( ) )
+         {
+            QValueList<PMVector> points = ed->vectors( );
+            QValueListIterator<PMVector> it = points.at( index );
+            
+            PMVector newPoint = *it;
+            ++it;
+            
+            if( it != points.end( ) )
+               newPoint = ( newPoint + *it ) / 2;
+            
+            points.insert( it, newPoint );
+
+            ed->setSize( points.size( ) );
+            ed->setVectors( points );
+            ed->setCurrentCell( index + 1, ed->currentColumn( ) );
+            if( points.size( ) > 3 )
+               m_removeButtons.at( subIndex )->setEnabled( true );
+
+            emit dataChanged( );
+            emit sizeChanged( );
+         }
       }
    }
 }
 
 void PMPrismEdit::slotRemovePoint( )
 {
-   QPushButton* button = ( QPushButton* ) sender( );
-   if( button )
+   QPushButton* bt = ( QPushButton* ) sender( );
+   if( bt )
    {
-      QValueList< QValueList<PMVector> > points = splinePoints( );
-      QPtrListIterator< QPtrList<QPushButton> > seit( m_removeButtons );
-      QValueList<QValueList<PMVector> >::Iterator spit = points.begin( );
-      bool found = false;
-
-      for( ; ( spit != points.end( ) ) && *seit && !found; ++spit, ++seit )
-      {  
-         int index = ( *seit )->findRef( button );
-         if( index >= 0 )
-         {
-            found = true;
-            QValueListIterator<PMVector> it = ( *spit ).at( index );
-            if( ( *spit ).count( ) > 1 )
-               ( *spit ).remove( it );
-         }
-      }
-      if( found )
+      int subIndex = m_removeButtons.findRef( bt );
+      if( subIndex >= 0 )
       {
-         displayPoints( points );
-         emit dataChanged( );
-         emit sizeChanged( );
+         PMVectorListEdit* ed = m_points.at( subIndex );
+         int index = ed->currentRow( );
+         if( index >= 0 && index < ed->size( ) )
+         {
+            QValueList<PMVector> points = ed->vectors( );
+            QValueListIterator<PMVector> it = points.at( index );
+            
+            points.remove( it );
+
+            ed->setSize( points.size( ) );
+            ed->setVectors( points );
+            if( points.size( ) <= 3 )
+               m_removeButtons.at( subIndex )->setEnabled( false );
+
+            emit dataChanged( );
+            emit sizeChanged( );
+         }
       }
    }
 }
@@ -628,7 +553,7 @@ void PMPrismEdit::slotAddSubPrism( )
    if( m_pSplineType->currentItem( ) == 3 )
    {
       KMessageBox::information( this, i18n( "Sub prisms do not work with "
-                                            "bezier splines in pov-ray 3.1" ),
+                                            "bezier splines in POV-Ray 3.1" ),
                                 i18n( "Warning" ), "subPrismWithBezierSplines" );
    }
    
@@ -692,4 +617,80 @@ void PMPrismEdit::slotRemoveSubPrism( )
    }
 }
 
+void PMPrismEdit::slotSelectionChanged( )
+{
+   PMVectorListEdit* edit = ( PMVectorListEdit* ) sender( );
+   if( edit )
+   {
+      QValueList< QValueList< PMVector > > points = m_pDisplayedObject->points( );
+
+      if( m_points.count( ) == points.size( ) )
+      {
+         int i;
+         bool changed = false;
+         QValueList< QValueList< PMVector > >::Iterator spit;
+         PMControlPointList cp = part( )->activeControlPoints( );
+         PMControlPointListIterator it( cp );  ++it; ++it;
+         QPtrListIterator<PMVectorListEdit> edit( m_points );
+         
+         for( spit = points.begin( ); spit != points.end( ) && it.current( );
+              ++spit, ++edit )
+         {
+            int np = ( *spit ).size( );
+            
+            if( ( *edit )->size( ) == np )
+            {
+               for( i = 0; i < np && it.current( ); i++, ++it )
+                  ( *it )->setSelected( ( *edit )->isSelected( i ) );
+               changed = true;
+            }
+            else
+               for( i = 0; i < np; i++ )
+                  ++it;
+         }
+         if( changed )
+            emit controlPointSelectionChanged( );
+      }
+   }
+}
+
+void PMPrismEdit::updateControlPointSelection( )
+{
+   QValueList< QValueList< PMVector > > points = m_pDisplayedObject->points( );
+   
+   if( m_points.count( ) == points.size( ) )
+   {
+      QValueList< QValueList< PMVector > >::Iterator spit;
+      PMControlPointList cp = part( )->activeControlPoints( );
+      PMControlPointListIterator it( cp );  ++it; ++it;
+      QPtrListIterator<PMVectorListEdit> edit( m_points );
+      
+      for( spit = points.begin( ); spit != points.end( ) && it.current( );
+           ++spit, ++edit )
+      {
+         PMVectorListEdit* vl = *edit;
+         int np = ( *spit ).size( );
+         int i;
+         
+         if( vl->size( ) == np )
+         {
+            vl->blockSelectionUpdates( true );
+            bool sb = vl->signalsBlocked( );
+            vl->blockSignals( true );
+
+            vl->clearSelection( );
+            for( i = 0; i < np && it.current( ); i++, ++it )
+               if( ( *it )->selected( ) )
+                  vl->select( i );
+            
+            vl->blockSignals( sb );
+            vl->blockSelectionUpdates( false );
+         }
+         else
+            for( i = 0; i < np; i++ )
+               ++it;
+      }
+   }
+}
+
 #include "pmprismedit.moc"
