@@ -47,12 +47,13 @@ PMRenderModesDialog::PMRenderModesDialog( PMRenderModeList* modes, QWidget* pare
    for( ; it.current( ); ++it )
       m_workingModes.append( new PMRenderMode( *( it.current( ) ) ) );
    m_workingModes.setAutoDelete( true );
-   
+
    QVBox* mainPage = makeVBoxMainWidget( );
    m_pListBox = new QListBox( mainPage );
    connect( m_pListBox, SIGNAL( highlighted( int ) ), SLOT( slotModeSelected( int ) ) );
 
    QHBox* buttons = new QHBox( mainPage );
+   buttons->setSpacing( KDialog::spacingHint( ) );
    m_pAddButton = new QPushButton( i18n( "Add" ), buttons );
    connect( m_pAddButton, SIGNAL( clicked( ) ), SLOT( slotAdd( ) ) );
    m_pRemoveButton = new QPushButton( i18n( "Remove" ), buttons );
@@ -69,9 +70,10 @@ PMRenderModesDialog::PMRenderModesDialog( PMRenderModeList* modes, QWidget* pare
    m_pDownButton->setEnabled( false );
 
    enableButtonOK( false );
-   
+
    resize( s_size );
    displayList( );
+   connect( m_pListBox,  SIGNAL( doubleClicked ( QListBoxItem *) ), this, SLOT(slotEdit( ) ) );
 }
 
 void PMRenderModesDialog::slotChanged( )
@@ -96,7 +98,7 @@ void PMRenderModesDialog::displayList( )
    for( ; it.current( ); ++it )
       m_pListBox->insertItem( it.current( )->description( ) );
    m_pListBox->setSelected( m_selectionIndex, true );
-   
+
    m_pListBox->blockSignals( b );
 
    checkButtons( );
@@ -147,7 +149,7 @@ void PMRenderModesDialog::slotAdd( )
    if( m_selectionIndex < 0 )
       m_selectionIndex = 0;
    m_workingModes.insert( ( uint ) m_selectionIndex, new PMRenderMode( ) );
-   
+
    displayList( );
    slotChanged( );
 }
@@ -185,13 +187,15 @@ void PMRenderModesDialog::slotDown( )
    if( m_selectionIndex > num )
       m_selectionIndex = num;
    m_workingModes.insert( m_selectionIndex, mode );
-   
+
    displayList( );
    slotChanged( );
 }
 
 void PMRenderModesDialog::slotEdit( )
 {
+    if (  m_selectionIndex==-1 )
+        return;
    PMRenderModeDialog dlg( m_workingModes.at( m_selectionIndex ) );
    bool changed = ( dlg.exec( ) == QDialog::Accepted );
    if( changed )
@@ -208,10 +212,10 @@ void PMRenderModesDialog::slotOk( )
    m_pOriginalModes->setAutoDelete( false );
    *m_pOriginalModes = m_workingModes;
    m_pOriginalModes->at( m_selectionIndex );
-   
+
    m_workingModes.setAutoDelete( false );
    m_workingModes.clear( );
-   
+
    accept( );
 }
 
@@ -243,7 +247,7 @@ PMRenderModeDialog::PMRenderModeDialog( PMRenderMode* mode, QWidget* parent, con
    int i;
 
    // main page
-   QWidget* page = new QWidget( this ); 
+   QWidget* page = new QWidget( this );
    setMainWidget( page );
    QVBoxLayout* topLayout = new QVBoxLayout( page, 0, spacingHint( ) );
 
@@ -313,7 +317,7 @@ PMRenderModeDialog::PMRenderModeDialog( PMRenderMode* mode, QWidget* parent, con
    quHelpLayout->addWidget( m_pQualityCombo );
    for( i = 0; i < numQuality; i++ )
       m_pQualityCombo->insertItem( i18n( qualityString[i] ) );
-   
+
    m_pAntialiasingBox = new QCheckBox( i18n( "Antialiasing" ), tab );
    tabLayout->addWidget( m_pAntialiasingBox );
 
@@ -322,18 +326,18 @@ PMRenderModeDialog::PMRenderModeDialog( PMRenderMode* mode, QWidget* parent, con
    aaGridLayout->addWidget( new QLabel( i18n( "Method:" ), tab ), 0, 0 );
    m_pSamplingCombo = new QComboBox( tab );
    aaGridLayout->addWidget( m_pSamplingCombo, 0, 1 );
-   m_pSamplingCombo->insertItem( i18n( "Non recursive" ) );
+   m_pSamplingCombo->insertItem( i18n( "Non Recursive" ) );
    m_pSamplingCombo->insertItem( i18n( "Recursive" ) );
 
    aaGridLayout->addWidget( new QLabel( i18n( "Threshold:" ), tab ), 1, 0 );
    m_pThresholdEdit = new PMFloatEdit( tab );
    aaGridLayout->addWidget( m_pThresholdEdit, 1, 1 );
-   
+
    aaGridLayout->addWidget( new QLabel( i18n( "Depth:" ), tab ), 2, 0 );
    m_pAntialiasDepthEdit = new PMIntEdit( tab );
    m_pAntialiasDepthEdit->setValidation( true, 1, true, 9 );
    aaGridLayout->addWidget( m_pAntialiasDepthEdit, 2, 1 );
-   
+
    m_pJitterBox = new QCheckBox( i18n( "Jitter" ), tab );
    aaGridLayout->addMultiCellWidget( m_pJitterBox, 3, 3, 0, 1 );
 
@@ -345,27 +349,27 @@ PMRenderModeDialog::PMRenderModeDialog( PMRenderMode* mode, QWidget* parent, con
 
    m_pRadiosityBox = new QCheckBox( i18n( "Radiosity" ), tab );
    tabLayout->addWidget( m_pRadiosityBox );
-   
+
    tabLayout->addStretch( 1 );
 
    // output options tab
    tab = new QWidget( );
    m_pTabWidget->addTab( tab, i18n( "Output" ) );
    tabLayout = new QVBoxLayout( tab, marginHint( ), spacingHint( ) );
-   
+
    m_pAlphaBox = new QCheckBox( i18n( "Alpha" ), tab );
    tabLayout->addWidget( m_pAlphaBox );
 
    tabLayout->addStretch( 1 );
 
-   
+
    resize( s_size );
-   
+
    // display the mode BEFORE the signals are connected!!!
    displayMode( );
 
    enableButtonOK( false );
-   
+
    // connect signals
    connect( m_pDescriptionEdit, SIGNAL( textChanged( const QString& ) ), SLOT( slotTextChanged( const QString& ) ) );
    connect( m_pHeightEdit, SIGNAL( dataChanged( ) ), SLOT( slotChanged( ) ) );
@@ -477,7 +481,7 @@ bool PMRenderModeDialog::validate( )
       if( m_pThresholdEdit->isDataValid( ) )
          if( m_pAntialiasDepthEdit->isDataValid( ) )
             error = false;
-      
+
       if( m_pJitterBox->isChecked( ) && !error )
          error = !m_pJitterAmountEdit->isDataValid( );
 
@@ -489,7 +493,7 @@ bool PMRenderModeDialog::validate( )
    }
 
    // tab 2
-   
+
    return true;
 }
 
@@ -545,7 +549,7 @@ int PMRenderModeDialog::qualityToIndex( int quality )
       quality = 0;
    if( quality > 11 )
       quality = 11;
-   
+
    return c_qualityToIndex[quality];
 }
 
@@ -555,7 +559,7 @@ int PMRenderModeDialog::indexToQuality( int index )
       index = 0;
    if( index >= numQuality )
       index = numQuality - 1;
-   
+
    return c_indexToQuality[index];
 }
 
