@@ -2,7 +2,7 @@
 **************************************************************************
                                  description
                              --------------------
-    copyright            : (C) 2000-2002 by Andreas Zehender
+    copyright            : (C) 2000-2003 by Andreas Zehender
     email                : zehender@kde.org
 **************************************************************************
 
@@ -17,13 +17,10 @@
 
 
 #include <ctype.h>
-//#include <alloc.h>
 #include <stdlib.h>
-#include <kdebug.h>
 #include <klocale.h>
 
-
-#include "pmglobals.h"
+#include "pmdebug.h"
 #include "pmscanner.h"
 #include "pmtokens.h"
 
@@ -36,10 +33,11 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
    switch( mode )
    {
       case PMDReservedWords:
-         insert( "aa_level", new int( AA_LEVEL_TOK ) ); 
+         insert( "aa_level", new int( AA_LEVEL_TOK ) );
          insert( "aa_threshold", new int( AA_THRESHOLD_TOK ) );
          insert( "abs", new int( ABS_TOK ) );
          insert( "absorption", new int( ABSORPTION_TOK ) );
+         insert( "accuracy", new int( ACCURACY_TOK ) );
          insert( "acos", new int( ACOS_TOK ) );
          insert( "acosh", new int( ACOSH_TOK ) );
          insert( "adaptive", new int( ADAPTIVE_TOK ) );
@@ -47,13 +45,17 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "agate", new int( AGATE_TOK ) );
          insert( "agate_turb", new int( AGATE_TURB_TOK ) );
          insert( "all", new int( ALL_TOK ) );
+         insert( "all_intersections", new int( ALL_INTERSECTIONS_TOK ) );
          insert( "alpha", new int( ALPHA_TOK ) );
+         insert( "always_sample", new int ( ALWAYS_SAMPLE_TOK ) );
          insert( "ambient", new int( AMBIENT_TOK ) );
          insert( "ambient_light", new int( AMBIENT_LIGHT_TOK ) );
          insert( "angle", new int( ANGLE_TOK ) );
          insert( "aperture", new int( APERTURE_TOK ) );
          insert( "arc_angle", new int( ARC_ANGLE_TOK ) );
          insert( "area_light", new int( AREA_LIGHT_TOK ) );
+         insert( "autostop", new int ( AUTOSTOP_TOK ) );
+         insert( "circular", new int( AREA_CIRCULAR_TOK ) );
          insert( "asc", new int( ASC_TOK ) );
          insert( "asin", new int( ASIN_TOK ) );
          insert( "asinh", new int( ASINH_TOK ) );
@@ -65,6 +67,7 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "atmospheric_attenuation", new int( ATMOSPHERIC_ATTENUATION_TOK ) );
          insert( "attenuating", new int( ATTENUATING_TOK ) );
          insert( "average", new int( AVERAGE_TOK ) );
+         insert( "b_spline", new int( B_SPLINE_TOK ) );
          insert( "background", new int( BACKGROUND_TOK ) );
          insert( "bezier_spline", new int( BEZIER_SPLINE_TOK ) );
          insert( "bicubic_patch", new int( BICUBIC_PATCH_TOK ) );
@@ -94,6 +97,7 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "clipped_by", new int( CLIPPED_BY_TOK ) );
          insert( "clock", new int( CLOCK_TOK ) );
          insert( "clock_delta", new int( CLOCK_DELTA_TOK ) );
+         insert( "collect", new int( COLLECT_TOK ) );
          insert( "color", new int( COLOR_TOK ) );
          insert( "color_map", new int( COLOR_MAP_TOK ) );
          insert( "colour", new int( COLOUR_TOK ) );
@@ -104,7 +108,9 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "cone", new int( CONE_TOK ) );
          insert( "confidence", new int( CONFIDENCE_TOK ) );
          insert( "conic_sweep", new int( CONIC_SWEEP_TOK ) );
+         insert( "conserve_energy", new int( CONSERVE_ENERGY_TOK ) );
          insert( "constant", new int( CONSTANT_TOK ) );
+         insert( "contained_by", new int( CONTAINED_BY_TOK ) );
          insert( "control0", new int( CONTROL0_TOK ) );
          insert( "control1", new int( CONTROL1_TOK ) );
          insert( "cos", new int( COS_TOK ) );
@@ -128,6 +134,9 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "diffuse", new int( DIFFUSE_TOK ) );
          insert( "direction", new int( DIRECTION_TOK ) );
          insert( "disc", new int( DISC_TOK ) );
+         insert( "dispersion", new int ( DISPERSION_TOK ) );
+         insert( "dispersion_samples", new int ( DISPERSION_SAMPLES_TOK ) );
+         insert( "dist_exp", new int ( DIST_EXP_TOK ) );
          insert( "distance", new int( DISTANCE_TOK ) );
          insert( "distance_maximum", new int( DISTANCE_MAXIMUM_TOK ) );
          insert( "div", new int( DIV_TOK ) );
@@ -138,7 +147,9 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "emitting", new int( EMITTING_TOK ) );
          insert( "error", new int( ERROR_TOK ) );
          insert( "error_bound", new int( ERROR_BOUND_TOK ) );
+         insert( "evaluate", new int( EVALUATE_TOK ) );
          insert( "exp", new int( EXP_TOK ) );
+         insert( "expand_thresholds", new int (EXPAND_THRESHOLDS_TOK) );
          insert( "exponent", new int( EXPONENT_TOK ) );
          insert( "extinction", new int( EXTINCTION_TOK ) );
          insert( "fade_distance", new int( FADE_DISTANCE_TOK ) );
@@ -159,8 +170,12 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "fog_offset", new int( FOG_OFFSET_TOK ) );
          insert( "fog_type", new int( FOG_TYPE_TOK ) );
          insert( "fractal", new int( FRACTAL_TOK ) );
+         insert( "fresnel", new int( FRESNEL_TOK ) );
          insert( "frequency", new int( FREQUENCY_TOK ) );
+         insert( "function", new int( FUNCTION_TOK ) );
+         insert( "gather", new int( GATHER_TOK ) );
          insert( "gif", new int( GIF_TOK ) );
+         insert( "global_lights", new int ( GLOBAL_LIGHTS_TOK ) );
          insert( "global_settings", new int( GLOBAL_SETTINGS_TOK ) );
          insert( "glowing", new int( GLOWING_TOK ) );
          insert( "gradient", new int( GRADIENT_TOK ) );
@@ -177,8 +192,10 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "iff", new int( IFF_TOK ) );
          insert( "image_map", new int( IMAGE_MAP_TOK ) );
          insert( "incidence", new int( INCIDENCE_TOK ) );
+         insert( "inside_vector", new int( INSIDE_VECTOR_TOK ) );
          insert( "int", new int( INT_TOK ) );
          insert( "interior", new int( INTERIOR_TOK ) );
+         insert( "interior_texture", new int( INTERIOR_TEXTURE_TOK ) );
          insert( "interpolate", new int( INTERPOLATE_TOK ) );
          insert( "intersection", new int( INTERSECTION_TOK ) );
          insert( "intervals", new int( INTERVALS_TOK ) );
@@ -186,11 +203,13 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "ior", new int( IOR_TOK ) );
          insert( "irid", new int( IRID_TOK ) );
          insert( "irid_wavelength", new int( IRID_WAVELENGTH_TOK ) );
+         insert( "isosurface", new int( ISOSURFACE_TOK ) );
          insert( "jitter", new int( JITTER_TOK ) );
          insert( "julia_fractal", new int( JULIA_FRACTAL_TOK ) );
          insert( "lambda", new int( LAMBDA_TOK ) );
          insert( "lathe", new int( LATHE_TOK ) );
          insert( "leopard", new int( LEOPARD_TOK ) );
+         insert( "light_group", new int ( LIGHT_GROUP_TOK ) );
          insert( "light_source", new int( LIGHT_SOURCE_TOK ) );
          insert( "linear", new int( LINEAR_TOK ) );
          insert( "linear_spline", new int( LINEAR_SPLINE_TOK ) );
@@ -200,6 +219,7 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "looks_like", new int( LOOKS_LIKE_TOK ) );
          insert( "look_at", new int( LOOK_AT_TOK ) );
          insert( "low_error_factor", new int( LOW_ERROR_FACTOR_TOK ) );
+         insert( "major_radius", new int( MAJOR_RADIUS_TOK ) );
          insert( "mandel", new int( MANDEL_TOK ) );
          insert( "map_type", new int( MAP_TYPE_TOK ) );
          insert( "marble", new int( MARBLE_TOK ) );
@@ -207,8 +227,11 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "material_map", new int( MATERIAL_MAP_TOK ) );
          insert( "matrix", new int( MATRIX_TOK ) );
          insert( "max", new int( MAX_TOK ) );
+         insert( "max_gradient", new int( MAX_GRADIENT_TOK ) );
          insert( "max_intersections", new int( MAX_INTERSECTIONS_TOK ) );
          insert( "max_iteration", new int( MAX_ITERATION_TOK ) );
+         insert( "max_sample", new int( MAX_SAMPLE_TOK ) );
+         insert( "max_trace", new int( MAX_TRACE_TOK ) );
          insert( "max_trace_level", new int( MAX_TRACE_LEVEL_TOK ) );
          insert( "max_value", new int( MAX_VALUE_TOK ) );
          insert( "media", new int( MEDIA_TOK ) );
@@ -223,6 +246,7 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "mortar", new int( MORTAR_TOK ) );
          insert( "nearest_count", new int( NEAREST_COUNT_TOK ) );
          insert( "no", new int( NO_TOK ) );
+         insert( "noise_generator", new int( NOISE_GENERATOR_TOK ) );
          insert( "normal", new int( NORMAL_TOK ) );
          insert( "normal_map", new int( NORMAL_MAP_TOK ) );
          insert( "no_shadow", new int( NO_SHADOW_TOK ) );
@@ -237,8 +261,11 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "once", new int( ONCE_TOK ) );
          insert( "onion", new int( ONION_TOK ) );
          insert( "open", new int( OPEN_TOK ) );
+         insert( "orient", new int( ORIENT_TOK ) );
          insert( "orthographic", new int( ORTHOGRAPHIC_TOK ) );
          insert( "panoramic", new int( PANORAMIC_TOK ) );
+         insert( "parallel", new int( PARALLEL_TOK ) );
+         insert( "pass_through", new int ( PASS_THROUGH_TOK ) );
          insert( "pattern1", new int( PATTERN1_TOK ) );
          insert( "pattern2", new int( PATTERN2_TOK ) );
          insert( "pattern3", new int( PATTERN3_TOK ) );
@@ -247,6 +274,7 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "phase", new int( PHASE_TOK ) );
          insert( "phong", new int( PHONG_TOK ) );
          insert( "phong_size", new int( PHONG_SIZE_TOK ) );
+         insert( "photons", new int ( PHOTONS_TOK ) );
          insert( "pi", new int( PI_TOK ) );
          insert( "pigment", new int( PIGMENT_TOK ) );
          insert( "pigment_map", new int( PIGMENT_MAP_TOK ) );
@@ -261,7 +289,10 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "pow", new int( POW_TOK ) );
          insert( "ppm", new int( PPM_TOK ) );
          insert( "precision", new int( PRECISION_TOK ) );
+         insert( "pretrace_end", new int( PRETRACE_END_TOK ) );
+         insert( "pretrace_start", new int( PRETRACE_START_TOK ) );
          insert( "prism", new int( PRISM_TOK ) );
+         insert( "projected_through", new int( PROJECTED_THROUGH_TOK ) );
          insert( "pwr", new int( PWR_TOK ) );
          insert( "quadratic_spline", new int( QUADRATIC_SPLINE_TOK ) );
          insert( "quadric", new int( QUADRIC_TOK ) );
@@ -282,6 +313,7 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "recursion_limit", new int( RECURSION_LIMIT_TOK ) );
          insert( "red", new int( RED_TOK ) );
          insert( "reflection", new int( REFLECTION_TOK ) );
+         insert( "reflection_exponent", new int( REFLECTION_EXPONENT_TOK ) );
          insert( "refraction", new int( REFRACTION_TOK ) );
          insert( "repeat", new int( REPEAT_TOK ) );
          insert( "rgb", new int( RGB_TOK ) );
@@ -308,8 +340,10 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "smooth", new int( SMOOTH_TOK ) );
          insert( "smooth_triangle", new int( SMOOTH_TRIANGLE_TOK ) );
          insert( "sor", new int( SOR_TOK ) );
+         insert( "spacing", new int ( SPACING_TOK ) );
          insert( "specular", new int( SPECULAR_TOK ) );
          insert( "sphere", new int( SPHERE_TOK ) );
+         insert( "sphere_sweep", new int ( SPHERE_SWEEP_TOK ) );
          insert( "spherical", new int( SPHERICAL_TOK ) );
          insert( "spiral", new int( SPIRAL_TOK ) );
          insert( "spiral1", new int( SPIRAL1_TOK ) );
@@ -331,6 +365,7 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "t", new int( T_TOK ) );
          insert( "tan", new int( TAN_TOK ) );
          insert( "tanh", new int( TANH_TOK ) );
+         insert( "target", new int( TARGET_TOK ) );
          insert( "test_camera_1", new int( TEST_CAMERA_1_TOK ) );
          insert( "test_camera_2", new int( TEST_CAMERA_2_TOK ) );
          insert( "test_camera_3", new int( TEST_CAMERA_3_TOK ) );
@@ -344,6 +379,8 @@ PMReservedWordDict::PMReservedWordDict( PMDictMode mode )
          insert( "tightness", new int( TIGHTNESS_TOK ) );
          insert( "tile2", new int( TILE2_TOK ) );
          insert( "tiles", new int( TILES_TOK ) );
+         insert( "tolerance", new int( TOLERANCE_TOK ) );
+         insert( "toroidal", new int( TOROIDAL_TOK ) );
          insert( "torus", new int( TORUS_TOK ) );
          insert( "track", new int( TRACK_TOK ) );
          insert( "transform", new int( TRANSFORM_TOK ) );
@@ -435,7 +472,7 @@ PMScanner::PMScanner( QIODevice* device )
    m_svalue[0] = '\0';
    m_lastAlloc = m_svalue + m_svalueAlloc;
    m_lastChar = m_svalue;
-   
+
    m_ivalue = 0;
    m_fvalue = 0;
    m_pDevice = device;
@@ -443,6 +480,7 @@ PMScanner::PMScanner( QIODevice* device )
    m_char = 0;
    m_indentation = 0;
    m_rawIndentation = 0;
+   m_bFunctionMode = false;
 
 //   m_lineData = "";
 //   m_lineDataPos = 100;
@@ -481,9 +519,9 @@ void PMScanner::addChar( char c )
       m_svalueAlloc += 64;
       m_svalue = ( char* ) realloc( m_svalue, m_svalueAlloc );
       m_lastAlloc = m_svalue + m_svalueAlloc;
-      m_lastChar = m_lastAlloc - 64; 
+      m_lastChar = m_lastAlloc - 64;
    }
-   
+
    *m_lastChar = '\0';
 }
 
@@ -547,8 +585,40 @@ int PMScanner::nextToken( )
    m_ivalue = 0;
    m_fvalue = 0;
 //   m_error = "";
-   
-   while( status != TOKEN_END_ST )
+
+   if( m_bFunctionMode )
+   {
+      m_bFunctionMode = false;
+      // FIXME: TODO brackets in comments will not be scanned correctly
+      int count = 1;
+
+      while( count > 0 )
+      {
+         if( m_char < 0 )
+            count = 0;
+         else if( m_char == '{' )
+            count++;
+         else if( m_char == '}' )
+            count--;
+         if( count > 0 )
+         {
+            addChar( m_char );
+            nextChar( );
+         }
+      }
+
+      if( m_char != '}' )
+      {
+         m_error = i18n( "Function statement not terminated" );
+#ifdef PMSCAN_DEBUG
+         kdDebug( PMArea ) << "Line " << m_line << ": Error " << m_error << "\n";
+#endif
+         m_token = SCANNER_ERROR_TOK;
+      }
+      else
+         m_token = FUNCTION_TOK;
+   }
+   else while( status != TOKEN_END_ST )
    {
       consumed = true;
       if( m_char < 0 )
@@ -589,20 +659,20 @@ int PMScanner::nextToken( )
             if( isalpha( m_char ) || ( m_char == '_' ) )
             {
                status = ID_ENDST;
-               addChar( m_char ); 
+               addChar( m_char );
                break;
             }
             if( isdigit( m_char ) )
             {
                status = INTEGER_ENDST;
-               addChar( m_char ); 
+               addChar( m_char );
                break;
             }
             switch( m_char )
             {
                case '.':
                   status = POINT_ST;
-                  addChar( m_char ); 
+                  addChar( m_char );
                   break;
                case '#':
                   status = DIRECTIVE1_ST;
@@ -617,7 +687,7 @@ int PMScanner::nextToken( )
                   addChar( m_char );
                   m_token = m_char;
                   status = TOKEN_END_ST;
-                  
+
 #ifdef PMSCAN_DEBUG
                   kdDebug( PMArea ) << "Line " << m_line << ": Single char '"
                                     << QString( QChar( ( char ) m_char ) ) << "'\n";
@@ -647,7 +717,7 @@ int PMScanner::nextToken( )
                                     << m_svalue << "\"\n";
 #endif
 
-                  
+
                status = TOKEN_END_ST;
             }
             else
@@ -809,7 +879,7 @@ int PMScanner::nextToken( )
             }
             break;
          case DIRECTIVE1_ST:
-            if( m_char == ' ' ) 
+            if( m_char == ' ' )
             {
                // special treatment for povray inc files
                // "#   debug" directives
@@ -848,7 +918,7 @@ int PMScanner::nextToken( )
 #ifdef PMSCAN_DEBUG
                   kdDebug( PMArea ) << "Line " << m_line << ": Error " << m_error << "\n";
 #endif
-                  
+
                   m_token = SCANNER_ERROR_TOK;
                }
                status = TOKEN_END_ST;
@@ -871,14 +941,14 @@ int PMScanner::nextToken( )
                   status = TOKEN_END_ST;
 #ifdef PMSCAN_DEBUG
                   kdDebug( PMArea ) << "Line " << m_line << ": Error " << m_error << "\n";
-#endif                  
+#endif
                   break;
                case '"':
 #ifdef PMSCAN_DEBUG
                   kdDebug( PMArea ) << "Line " << m_line << ": String: \""
                                     << m_svalue << "\"\n";
 #endif
-                  
+
                   m_token = STRING_TOK;
                   status = TOKEN_END_ST;
                   break;
@@ -898,7 +968,7 @@ int PMScanner::nextToken( )
                m_error = i18n( "String not terminated" );
 #ifdef PMSCAN_DEBUG
                kdDebug( PMArea ) << "Line " << m_line << ": Error " << m_error << "\n";
-#endif                  
+#endif
                m_token = SCANNER_ERROR_TOK;
                status = TOKEN_END_ST;
                break;
@@ -932,7 +1002,7 @@ int PMScanner::nextToken( )
             {
 #ifdef PMSCAN_DEBUG
                kdDebug( PMArea ) << "Line " << m_line << ": Empty line comment\n";
-#endif                  
+#endif
                consumed = false;
                m_token = LINE_COMMENT_TOK;
                status = TOKEN_END_ST;
@@ -948,7 +1018,7 @@ int PMScanner::nextToken( )
 #ifdef PMSCAN_DEBUG
                kdDebug( PMArea ) << "Line " << m_line << ": Line comment: \""
                                  << m_svalue << "\"\n";
-#endif                  
+#endif
                consumed = false;
                m_token = LINE_COMMENT_TOK;
                status = TOKEN_END_ST;
@@ -985,7 +1055,7 @@ int PMScanner::nextToken( )
                m_error = i18n( "Comment not terminated" );
 #ifdef PMSCAN_DEBUG
                kdDebug( PMArea ) << "Line " << m_line << ": Error " << m_error << "\n";
-#endif                  
+#endif
                m_token = SCANNER_ERROR_TOK;
                status = TOKEN_END_ST;
             }
@@ -1007,7 +1077,7 @@ int PMScanner::nextToken( )
                m_error = i18n( "Comment not terminated" );
 #ifdef PMSCAN_DEBUG
                kdDebug( PMArea ) << "Line " << m_line << ": Error " << m_error << "\n";
-#endif                  
+#endif
                m_token = SCANNER_ERROR_TOK;
                status = TOKEN_END_ST;
             }
@@ -1039,7 +1109,7 @@ int PMScanner::nextToken( )
                m_error = i18n( "Comment not terminated" );
 #ifdef PMSCAN_DEBUG
                kdDebug( PMArea ) << "Line " << m_line << ": Error " << m_error << "\n";
-#endif                  
+#endif
                m_token = SCANNER_ERROR_TOK;
                status = TOKEN_END_ST;
             }
@@ -1090,7 +1160,7 @@ int PMScanner::nextToken( )
                m_error = i18n( "Comment not terminated" );
 #ifdef PMSCAN_DEBUG
                kdDebug( PMArea ) << "Line " << m_line << ": Error " << m_error << "\n";
-#endif                  
+#endif
                m_token = SCANNER_ERROR_TOK;
                status = TOKEN_END_ST;
             }
@@ -1123,7 +1193,7 @@ int PMScanner::nextToken( )
 #ifdef PMSCAN_DEBUG
                kdDebug( PMArea ) << "Line " << m_line << ": PMName: \""
                                  << m_svalue << "\"\n";
-#endif                  
+#endif
                consumed = false;
                m_token = PMNAME_TOK;
                status = TOKEN_END_ST;
@@ -1142,7 +1212,7 @@ int PMScanner::nextToken( )
                m_error = i18n( "Raw povray not terminated" );
 #ifdef PMSCAN_DEBUG
                kdDebug( PMArea ) << "Line " << m_line << ": Error " << m_error << "\n";
-#endif                  
+#endif
                m_token = SCANNER_ERROR_TOK;
                status = TOKEN_END_ST;
             }
@@ -1185,7 +1255,7 @@ int PMScanner::nextToken( )
                m_error = i18n( "Raw povray not terminated" );
 #ifdef PMSCAN_DEBUG
                kdDebug( PMArea ) << "Line " << m_line << ": Error " << m_error << "\n";
-#endif                  
+#endif
                m_token = SCANNER_ERROR_TOK;
                status = TOKEN_END_ST;
             }
@@ -1211,7 +1281,7 @@ int PMScanner::nextToken( )
                m_error = i18n( "Raw povray not terminated" );
 #ifdef PMSCAN_DEBUG
                kdDebug( PMArea ) << "Line " << m_line << ": Error " << m_error << "\n";
-#endif                  
+#endif
                m_token = SCANNER_ERROR_TOK;
                status = TOKEN_END_ST;
             }
@@ -1259,6 +1329,11 @@ int PMScanner::nextToken( )
          kdError( PMArea ) << "Error in scanner: No TOKEN_END_ST after EOF\n";
       }
    }
-   
+
    return m_token;
 }
+
+void PMScanner::scanFunction( )
+{
+   m_bFunctionMode = true;
+}
