@@ -2,7 +2,7 @@
 **************************************************************************
                                  description
                              --------------------
-    copyright            : (C) 2000-2002 by Andreas Zehender
+    copyright            : (C) 2000-2003 by Andreas Zehender
     email                : zehender@kde.org
 **************************************************************************
 
@@ -22,10 +22,10 @@
 #include <kstatusbar.h>
 #include <kfiledialog.h>
 #include <kmessagebox.h>
+#include <kiconloader.h>
 #include <kedittoolbar.h>
 #include <kkeydialog.h>
 #include <kaboutdialog.h>
-#include <kiconloader.h>
 #include <kpopupmenu.h>
 #include <qtimer.h>
 
@@ -34,11 +34,11 @@
 #include "pmfactory.h"
 #include "pmsettingsdialog.h"
 #include "pmdefaults.h"
+#include "pmdockwidget.h"
 
-#include "pmtreeview.h"
-#include "pmdialogview.h"
-#include "pmglviewconnector.h"
-#include "pmglview.h"
+#include "pmviewfactory.h"
+#include "pmviewbase.h"
+#include "pmunknownview.h"
 #include "pmviewlayoutmanager.h"
 
 const int c_statusBarInfo = 0;
@@ -47,13 +47,12 @@ const int c_statusBarControlPoints = 1;
 PMShell::PMShell( const KURL& url )
       : PMDockMainWindow( 0, "mainwindow" )
 {
-   setInstance( PMFactory::instance( ) );
+   setPluginLoadingMode( DoNotLoadPlugins );
+   setInstance( PMFactory::instance( ), false );
+   
    m_pPart = new PMPart( this, "part", this, "part", true, this );
    m_pPart->setReadWrite( ); // read-write mode
-   m_pGLConnector = 0;
-   m_numGLViews = 0;
-   m_numTreeViews = 0;
-   m_numDialogViews = 0;
+   m_viewNumber = 0;
    m_objectsToDelete.setAutoDelete( true );
 
    setupActions( );
@@ -97,9 +96,6 @@ void PMShell::setupActions( )
                                        actionCollection( ) );
    KStdAction::save( this, SLOT( slotFileSave( ) ), actionCollection( ) );
    KStdAction::saveAs( this, SLOT( slotFileSaveAs( ) ), actionCollection( ) );
-   m_pExportPovrayAction = new KAction( i18n( "&Export Povray..." ), 0, this,
-                               SLOT( slotFileExportPovray( ) ), actionCollection( ),
-                               "file_export_povray" );
 
    KStdAction::revert( this, SLOT( slotFileRevert( ) ), actionCollection( ) );
    KStdAction::print( this, SLOT( slotFilePrint( ) ), actionCollection( ) );
@@ -111,13 +107,9 @@ void PMShell::setupActions( )
                                SLOT( slotShowPath( ) ), actionCollection( ),
                                "options_show_path" );
 
-   
-//   m_pToolbarAction = KStdAction::showToolbar( this, SLOT( slotShowToolbar( ) ),
-//                                               actionCollection( ) );
    m_pStatusbarAction = KStdAction::showStatusbar( this, SLOT( slotShowStatusbar( ) ),
-                                            actionCollection( ) );
-   
-   
+                                                   actionCollection( ) );
+
    KStdAction::saveOptions( this, SLOT( saveOptions( ) ), actionCollection( ) );
    
    KStdAction::keyBindings( this, SLOT( slotConfigureKeys( ) ),
@@ -170,14 +162,62 @@ void PMShell::setupActions( )
 
 void PMShell::setupView( )
 {
-   m_pGLConnector = new PMGLViewConnector( m_pPart, m_pPart,
-                                           "GLViewConnector" );
    PMViewLayoutManager::theManager( )->displayDefaultLayout( this );
 }
 
+
+PMDockWidget* PMShell::createView( const QString& t, PMViewOptions* o,
+                                   bool initPosition )
+{
+   PMDockWidget* dock = 0;
+   PMViewBase* contents = 0;
+   
+   PMViewTypeFactory* factory =
+      PMViewFactory::theFactory( )->viewFactory( t );
+
+   m_viewNumber++;
+   QString name = QString( "View (%1)" ).arg( m_viewNumber );
+   
+   if( factory )
+   {
+      QString desc;
+      // Create the appropriate dock widget
+      if( o )
+         desc = factory->description( o );
+      else
+         desc = factory->description( );
+         
+      dock = createDockWidget( name, SmallIcon( factory->iconName( ) ), 0L,
+                               desc, desc );
+      contents = factory->newInstance( dock, m_pPart );
+      if( o )
+         contents->restoreViewConfig( o );
+   }
+   else
+   {
+      // unknown view type
+      dock = createDockWidget( name, SmallIcon( "unknown" ), 0L,
+                               i18n( "Unknown" ), i18n( "Unknown" ) );
+      contents = new PMUnknownView( t, dock );
+   }
+
+   dock->setWidget( contents );
+   connect( dock, SIGNAL( headerCloseButtonClicked( ) ),
+            SLOT( slotDockWidgetClosed( ) ) );
+   
+   if( initPosition )
+   {
+      dock->resize( 300, 400 );
+      dock->manualDock( 0, PMDockWidget::DockDesktop, 50,
+                        mapToGlobal( QPoint( 50, 50 ) ) );
+   }
+   return dock;
+}
+
+/*
 PMDockWidget* PMShell::createTreeView( )
 {
-   PMDockWidget* dock;
+   PMDockWidget* dock = 0;
    m_numTreeViews++;
    QString name = QString( "Object Tree (%1)" ).arg( m_numTreeViews );
    dock = createDockWidget( name, SmallIcon( "pmtreeview" ),
@@ -194,7 +234,7 @@ PMDockWidget* PMShell::createTreeView( )
 
 PMDockWidget* PMShell::createDialogView( )
 {
-   PMDockWidget* dock;
+   PMDockWidget* dock = 0;
    m_numDialogViews++;
    QString name = QString( "Object Properties (%1)" ).arg( m_numDialogViews );
    dock = createDockWidget( name, SmallIcon( "pmdialogview" ),
@@ -211,37 +251,30 @@ PMDockWidget* PMShell::createDialogView( )
 
 PMDockWidget* PMShell::create3DView( PMGLView::PMViewType t )
 {
-   if( m_pGLConnector )
-   {
-      PMDockWidget* dock;
-      m_numGLViews++;
-      QString name = QString( "3D View (%1)" ).arg( m_numGLViews );
-      dock = createDockWidget( name, SmallIcon( "pmglview" ),
-                               0L, i18n( "3D View" ), i18n( "3D View" ) );
-      dock->setDockSite( PMDockWidget::DockFullSite );
-      PMGLView* vgl = new PMGLView( m_pPart, m_pGLConnector, t, dock );
-      dock->setWidget( vgl );
-      connect( vgl, SIGNAL( viewTypeChanged( const QString& ) ),
-               dock, SLOT( slotSetCaption( const QString& ) ) );
-      dock->slotSetCaption( PMGLView::viewTypeAsString( t ) );
-         
-      connect( dock, SIGNAL( headerCloseButtonClicked( ) ),
-               SLOT( slotDockWidgetClosed( ) ) );
-
-      return dock;
-   }
-   return 0;
+   PMDockWidget* dock = 0;
+   m_numGLViews++;
+   QString name = QString( "3D View (%1)" ).arg( m_numGLViews );
+   dock = createDockWidget( name, SmallIcon( "pmglview" ),
+                            0L, i18n( "3D View" ), i18n( "3D View" ) );
+   dock->setDockSite( PMDockWidget::DockFullSite );
+   PMGLView* vgl = new PMGLView( m_pPart, t, dock );
+   dock->setWidget( vgl );
+   connect( vgl, SIGNAL( viewTypeChanged( const QString& ) ),
+            dock, SLOT( slotSetCaption( const QString& ) ) );
+   dock->slotSetCaption( PMGLView::viewTypeAsString( t ) );
+   
+   connect( dock, SIGNAL( headerCloseButtonClicked( ) ),
+            SLOT( slotDockWidgetClosed( ) ) );
+   
+   return dock;
 }
+*/
 
 void PMShell::slotNewGraphicalView( PMGLView::PMViewType t )
 {
-   PMDockWidget* dock = create3DView( t );
-   if( dock )
-   {
-      dock->resize( 300, 300 );
-      dock->manualDock( 0, PMDockWidget::DockDesktop, 50,
-                        mapToGlobal( QPoint( 50, 50 ) ) );
-   }
+   PMGLViewOptions* o = new PMGLViewOptions( t );
+   createView( "glview", o );
+   delete o;
 }
 
 void PMShell::slotNewTopView( )
@@ -281,18 +314,12 @@ void PMShell::slotNewCameraView( )
 
 void PMShell::slotNewDialogView( )
 {
-   PMDockWidget* dock = createDialogView( );
-   dock->resize( 300, 400 );
-   dock->manualDock( 0, PMDockWidget::DockDesktop, 50,
-                     mapToGlobal( QPoint( 50, 50 ) ) );
+   createView( "dialogview" );
 }
 
 void PMShell::slotNewTreeView( )
 {
-   PMDockWidget* dock = createTreeView( );
-   dock->resize( 300, 400 );
-   dock->manualDock( 0, PMDockWidget::DockDesktop, 50,
-                     mapToGlobal( QPoint( 50, 50 ) ) );
+   createView( "treeview" );
 }
 
 void PMShell::slotDockWidgetClosed( )
@@ -409,34 +436,6 @@ void PMShell::slotFileSaveAs( )
    }
 }
 
-void PMShell::slotFileExportPovray( )
-{
-   KFileDialog dlg( QString::null,
-                    QString( "*.pov|" ) + i18n( "POV-Ray Files (*.pov)" ) +
-                    QString( "\n*.ini|" ) + i18n( "POV-Ray Include Files (*.ini)" ) +
-                    QString( "\n*|" ) + i18n( "All Files" ),
-                    0, "filedialog", true );
-   dlg.setCaption( i18n( "Save As" ) );
-   dlg.setOperationMode( KFileDialog::Saving );
-   dlg.exec( );
-
-   KURL url = dlg.selectedURL( );
-
-   if( !url.isEmpty( ) )
-   {
-      if( dlg.currentFilter( ) == QString( "*.pov" ) )
-         if( QFileInfo( url.path( ) ).extension( ).isEmpty( ) )
-            url.setPath( url.path( ) + ".pov" );
-      if( dlg.currentFilter( ) == QString( "*.ini" ) )
-         if( QFileInfo( url.path( ) ).extension( ).isEmpty( ) )
-            url.setPath( url.path( ) + ".ini" );
-      
-      if( overwriteURL( url ) )
-         if( !m_pPart->exportPovray( url ) )
-            KMessageBox::sorry( this, i18n( "Couldn't export the file." ) );
-   }
-}
-
 void PMShell::slotFileRevert( )
 {
    KURL url = m_pPart->url( );
@@ -522,6 +521,15 @@ void PMShell::slotNewToolbarConfig( )
    applyMainWindowSettings( KGlobal::config( ), "Appearance" );
 }
 
+void PMShell::updateGUI( )
+{
+   saveMainWindowSettings( KGlobal::config( ), "Appearance" );
+   createGUI( 0 );
+   createShellGUI( false );
+   createGUI( m_pPart );
+   applyMainWindowSettings( KGlobal::config( ), "Appearance" );
+}
+
 void PMShell::saveOptions( )
 {
    kdDebug( PMArea ) << "Saving configuration" << endl;
@@ -529,13 +537,7 @@ void PMShell::saveOptions( )
 
    // set group
    config->setGroup( "Appearance" );
-   
-//   config->writeEntry( "ShowToolbar", m_pToolbarAction->isChecked( ) );
    config->writeEntry( "ShowStatusbar", m_pStatusbarAction->isChecked( ) );
-   
-//   config->writeEntry( "Width", width( ) );
-//   config->writeEntry( "Height", height( ) );
-
    saveMainWindowSettings( config ); 
    m_pRecent->saveEntries( config );
 
@@ -563,9 +565,6 @@ void PMShell::restoreOptions( )
    else
       statusBar( )->hide( );
 
-//   resize( config->readNumEntry( "Width", 640 ),
-//           config->readNumEntry( "Height", 480 ) );
-
    m_pRecent->loadEntries( config );
 }
 
