@@ -18,7 +18,6 @@
 
 #include "pmsor.h"
 
-#include "pmoutputdevice.h"
 #include "pmxmlhelper.h"
 #include "pmsoredit.h"
 #include "pmmemento.h"
@@ -27,9 +26,7 @@
 #include "pmsplinememento.h"
 #include "pmsorsegment.h"
 #include "pmdefaults.h"
-
-#include <kdebug.h>
-#include "pmglobals.h"
+#include "pmobjectaction.h"
 
 #include <klocale.h>
 
@@ -48,9 +45,75 @@ const bool defaultOpen = false;
 int PMSurfaceOfRevolution::s_rSteps = c_defaultSurfaceOfRevolutionRSteps;
 int PMSurfaceOfRevolution::s_sSteps = c_defaultSurfaceOfRevolutionSSteps;
 int PMSurfaceOfRevolution::s_parameterKey = 0;
+PMMetaObject* PMSurfaceOfRevolution::s_pMetaObject = 0;
+PMObject* createNewSurfaceOfRevolution( PMPart* part )
+{
+   return new PMSurfaceOfRevolution( part );
+}
 
-PMSurfaceOfRevolution::PMSurfaceOfRevolution( )
-      : Base( )
+PMDefinePropertyClass( PMSurfaceOfRevolution, PMProperty );
+
+class PMPointProperty : public PMPropertyBase
+{
+public:
+   PMPointProperty( )
+         : PMPropertyBase( "controlPoints", PMVariant::Vector )
+   {
+      m_index = 0;
+   }
+   virtual int dimensions( ) const { return 1; }
+   virtual void setIndex( int /*dimension*/, int index )
+   {
+      m_index = index;
+   }
+   virtual int size( PMObject* object, int /*dimension*/ ) const
+   {
+      return ( ( PMSurfaceOfRevolution* ) object )->numberOfPoints( );
+   }
+protected:
+   virtual bool setProtected( PMObject* obj, const PMVariant& var )
+   {
+      PMSurfaceOfRevolution* p = ( PMSurfaceOfRevolution* ) obj;
+      QValueList<PMVector> list = p->points( );
+      QValueList<PMVector>::Iterator it = list.begin( );
+      int i;
+      PMVector v = var.vectorData( );
+      v.resize( 2 );
+
+      for( i = 0; i < m_index && it != list.end( ); i++ )
+         ++it;
+      // expand the list if necessary
+      for( ; i < m_index; i++ )
+         list.insert( it, v );
+      if( it == list.end( ) )
+         it = list.insert( it, v );
+      else
+         *it = v;
+
+      p->setPoints( list );
+      return true;
+   }
+   virtual PMVariant getProtected( const PMObject* obj )
+   {
+      PMSurfaceOfRevolution* p = ( PMSurfaceOfRevolution* ) obj;
+      QValueList<PMVector> list = p->points( );
+      QValueList<PMVector>::ConstIterator it = list.at( m_index );
+
+      if( it == list.end( ) )
+      {
+         kdError( PMArea ) << "Range error in PMSurfaceOfRevolution::PointProperty::get" << endl;
+         return PMVariant( );
+      }
+
+      return PMVariant( *it );
+   }
+
+private:
+   int m_index;
+};
+
+PMSurfaceOfRevolution::PMSurfaceOfRevolution( PMPart* part )
+      : Base( part )
 {
    int i;
 
@@ -60,6 +123,14 @@ PMSurfaceOfRevolution::PMSurfaceOfRevolution( )
    m_open = defaultOpen;
 }
 
+PMSurfaceOfRevolution::PMSurfaceOfRevolution( const PMSurfaceOfRevolution& s )
+      : Base( s )
+{
+   m_points = s.m_points;
+   m_sturm = s.m_sturm;
+   m_open = s.m_open;
+}
+
 PMSurfaceOfRevolution::~PMSurfaceOfRevolution( )
 {
 }
@@ -69,35 +140,6 @@ QString PMSurfaceOfRevolution::description( ) const
    return i18n( "surface of revolution" );
 }
 
-void PMSurfaceOfRevolution::serialize( PMOutputDevice& dev ) const
-{
-   dev.objectBegin( "sor" );
-
-   serializeName( dev );
-   
-   int num = m_points.count( );
-   dev.writeLine( QString( "%1," ).arg( num ) );
-   
-   bool first = true;
-   QValueList<PMVector>::ConstIterator it = m_points.begin( );
-   for( ; it != m_points.end( ); ++it )
-   {
-      if( !first )
-         dev.write( ", " );
-      dev.write( ( *it ).serialize( ) );
-      first = false;
-   }
-   dev.writeLine( "" );
-
-   if( m_open )
-      dev.writeLine( "open" );
-   if( m_sturm )
-      dev.writeLine( "sturm" );
-   
-   Base::serialize( dev );
-   dev.objectEnd( );
-}
-
 void PMSurfaceOfRevolution::serialize( QDomElement& e, QDomDocument& doc ) const
 {
    QDomElement data = doc.createElement( "extra_data" );
@@ -152,11 +194,31 @@ void PMSurfaceOfRevolution::readAttributes( const PMXMLHelper& h )
    Base::readAttributes( h );
 }
 
-bool PMSurfaceOfRevolution::isA( PMObjectType t ) const
+PMMetaObject* PMSurfaceOfRevolution::metaObject( ) const
 {
-   if( t == PMTSurfaceOfRevolution )
-      return true;
-   return Base::isA( t );
+   if( !s_pMetaObject )
+   {
+      s_pMetaObject = new PMMetaObject( "SurfaceOfRevolution", Base::metaObject( ),
+                                        createNewSurfaceOfRevolution );
+      s_pMetaObject->addProperty(
+         new PMProperty( "sturm", &PMSurfaceOfRevolution::setSturm,
+                         &PMSurfaceOfRevolution::sturm ) );
+      s_pMetaObject->addProperty(
+         new PMProperty( "open", &PMSurfaceOfRevolution::setOpen,
+                         &PMSurfaceOfRevolution::open ) );
+      s_pMetaObject->addProperty( new PMPointProperty( ) );
+   }
+   return s_pMetaObject;
+}
+
+void PMSurfaceOfRevolution::cleanUp( ) const
+{
+   if( s_pMetaObject )
+   {
+      delete s_pMetaObject;
+      s_pMetaObject = 0;
+   }
+   Base::cleanUp( );
 }
 
 void PMSurfaceOfRevolution::setSturm( bool s )
@@ -164,7 +226,7 @@ void PMSurfaceOfRevolution::setSturm( bool s )
    if( m_sturm != s )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTSurfaceOfRevolution, PMSturmID, m_sturm );
+         m_pMemento->addData( s_pMetaObject, PMSturmID, m_sturm );
       m_sturm = s;
    }
 }
@@ -174,7 +236,7 @@ void PMSurfaceOfRevolution::setOpen( bool o )
    if( m_open != o )
    {
       if( m_pMemento )
-         m_pMemento->addData( PMTSurfaceOfRevolution, PMOpenID, m_open );
+         m_pMemento->addData( s_pMetaObject, PMOpenID, m_open );
       m_open = o;
    }
 }
@@ -185,7 +247,7 @@ void PMSurfaceOfRevolution::setPoints( const QValueList<PMVector>& points )
    {
       if( m_pMemento )
          ( ( PMSplineMemento* ) m_pMemento )->setSplinePoints( m_points );
-      
+
       setViewStructureChanged( );
       m_points = points;
    }
@@ -212,7 +274,7 @@ void PMSurfaceOfRevolution::restoreMemento( PMMemento* s )
    for( ; it.current( ); ++it )
    {
       data = it.current( );
-      if( data->objectType( ) == PMTSurfaceOfRevolution )
+      if( data->objectType( ) == s_pMetaObject )
       {
          switch( data->valueID( ) )
          {
@@ -230,7 +292,7 @@ void PMSurfaceOfRevolution::restoreMemento( PMMemento* s )
    }
    if( m->splinePointsSaved( ) )
       setPoints( m->splinePoints( ) );
-   
+
    Base::restoreMemento( s );
 }
 
@@ -258,7 +320,7 @@ void PMSurfaceOfRevolution::createViewStructure( )
       if( m_pViewStructure->points( ).size( ) != ( unsigned ) vsp )
          m_pViewStructure->points( ).resize( vsp );
       if( m_pViewStructure->lines( ).size( ) != ( unsigned ) vsl )
-         m_pViewStructure->lines( ).resize( vsl );      
+         m_pViewStructure->lines( ).resize( vsl );
    }
    else
       m_pViewStructure = new PMViewStructure( vsp, vsl );
@@ -268,7 +330,7 @@ void PMSurfaceOfRevolution::createViewStructure( )
    QValueList<PMSorSegment> segments;
    QValueList<PMVector>::Iterator it1, it2, it3, it4;
    it1 = m_points.begin( );
-   it2 = it1; ++it2; 
+   it2 = it1; ++it2;
    it3 = it2; ++it3;
    it4 = it3; ++it4;
 
@@ -300,7 +362,7 @@ void PMSurfaceOfRevolution::createViewStructure( )
    // calculate the points
    PMVector point2, point3;
    QValueList<PMSorSegment>::Iterator sit = segments.begin( );
-   
+
    double poffset = 1.0 / s_sSteps;
    PMMatrix rot = PMMatrix::rotation( 0.0, M_PI * 2.0 / s_rSteps, 0.0 );
    PMPointArray& points = m_pViewStructure->points( );
@@ -314,7 +376,7 @@ void PMSurfaceOfRevolution::createViewStructure( )
          point3[0] = point2[0];
          point3[1] = point2[1];
          point3[2] = 0.0;
-         
+
          for( r = 0; r < s_rSteps; r++ )
          {
             points[pi] = PMPoint( point3 );
@@ -329,7 +391,7 @@ void PMSurfaceOfRevolution::createViewStructure( )
          point3[0] = point2[0];
          point3[1] = point2[1];
          point3[2] = 0.0;
-         
+
          for( r = 0; r < s_rSteps; r++ )
          {
             points[pi] = PMPoint( point3 );
@@ -344,27 +406,44 @@ void PMSurfaceOfRevolution::createViewStructure( )
 void PMSurfaceOfRevolution::controlPoints( PMControlPointList& list )
 {
    QValueList<PMVector>::Iterator it;
-   int i, d;
+   QPtrList<PMSorControlPoint> tmp1, tmp2;
+   int i;
 
-   PMSorControlPoint::CPType type = PMSorControlPoint::PM2DXY;
    PMSorControlPoint* cp = 0;
-   
-   for( d = 0; d < 2; d++ )
-   {
-      PMSorControlPoint* lastPoint = 0;
-      cp = 0;
 
-      for( it = m_points.begin( ), i = 0; it != m_points.end( ); ++it, i++ )
-      {
-         lastPoint = cp;
-         cp = new PMSorControlPoint( lastPoint, *it, type, i,
-                                    i18n( "Point %1 (xy)" ).arg( i + 1 ) );
-         
-         list.append( cp );
-      }
-      
-      type = PMSorControlPoint::PM2DZY;
+   PMSorControlPoint* lastPoint = 0;
+   cp = 0;
+
+   for( it = m_points.begin( ), i = 0; it != m_points.end( ); ++it, i++ )
+   {
+      lastPoint = cp;
+      cp = new PMSorControlPoint( lastPoint, *it, PMSorControlPoint::PM2DXY, i,
+                                  i18n( "Point %1 (xy)" ).arg( i + 1 ) );
+      tmp1.append( cp );
    }
+
+   lastPoint = 0;
+   cp = 0;
+
+   for( it = m_points.begin( ), i = 0; it != m_points.end( ); ++it, i++ )
+   {
+      lastPoint = cp;
+      cp = new PMSorControlPoint( lastPoint, *it, PMSorControlPoint::PM2DZY, i,
+                                  i18n( "Point %1 (yz)" ).arg( i + 1 ) );
+      tmp2.append( cp );
+   }
+
+   QPtrListIterator<PMSorControlPoint> cit1( tmp1 ), cit2( tmp2 );
+
+   for( ; cit1.current( ) && cit2.current( ); ++cit1, ++cit2 )
+   {
+      ( *cit1 )->setSorLink( *cit2 );
+      ( *cit2 )->setSorLink( *cit1 );
+   }
+   for( cit1.toFirst( ); cit1.current( ); ++cit1 )
+      list.append( *cit1 );
+   for( cit2.toFirst( ); cit2.current( ); ++cit2 )
+      list.append( *cit2 );
 }
 
 void PMSurfaceOfRevolution::controlPointsChanged( PMControlPointList& list )
@@ -377,12 +456,12 @@ void PMSurfaceOfRevolution::controlPointsChanged( PMControlPointList& list )
    PMVector lastPoint( 2 );
    int num = list.count( ) / 2;
    int pnr = 0;
-   
+
    for( it2 += num; it2.current( ); ++it1, ++it2, ++pit, ++pnr )
    {
       p1 = ( PMSorControlPoint* ) it1.current( );
       p2 = ( PMSorControlPoint* ) it2.current( );
-      
+
       if( p1->changed( ) )
       {
          if( firstChange )
@@ -429,15 +508,15 @@ void PMSurfaceOfRevolution::controlPointsChanged( PMControlPointList& list )
       {
          QValueList<PMVector>::Iterator hit = pit;
          --hit; --hit;
-         
+
          if( approxZero( ( *hit )[1] - ( *pit )[1], c_sorTolerance ) )
          {
             ( *pit )[1] = ( *hit )[1] + c_sorTolerance;
             p1->setPoint( *pit );
-            p2->setPoint( *pit );            
+            p2->setPoint( *pit );
          }
       }
-      
+
       lastPoint = *pit;
    }
 }
@@ -447,14 +526,14 @@ void PMSurfaceOfRevolution::addObjectActions( const PMControlPointList& /*cp*/,
 {
    PMObjectAction* a;
 
-   a = new PMObjectAction( PMTSurfaceOfRevolution, PMSplitSegmentID,
-                           i18n( "Split Segment" ) );
+   a = new PMObjectAction( s_pMetaObject, PMSplitSegmentID,
+                           i18n( "Add Point" ) );
    actions.append( a );
-   
-   a = new PMObjectAction( PMTSurfaceOfRevolution, PMJoinSegmentsID,
-                           i18n( "Join Segments" ) );
+
+   a = new PMObjectAction( s_pMetaObject, PMJoinSegmentsID,
+                           i18n( "Remove Point" ) );
    int np = m_points.count( );
-   
+
    if( np < 5 )
       a->setEnabled( false );
    actions.append( a );
@@ -465,7 +544,7 @@ void PMSurfaceOfRevolution::objectActionCalled( const PMObjectAction* action,
                                   const QPtrList<PMVector>& cpViewPosition,
                                   const PMVector& clickPosition )
 {
-   if( action->objectType( ) == PMTSurfaceOfRevolution )
+   if( action->objectType( ) == s_pMetaObject )
    {
       switch( action->actionID( ) )
       {
@@ -510,7 +589,7 @@ void PMSurfaceOfRevolution::splitSegment( const PMControlPointList& /*cp*/,
          dist[1] = mid[1];
          dist -= clickPosition;
          abs = dist.abs( );
-         
+
          if( ( minabs > abs ) || ( ns < 0 ) )
          {
             minabs = abs;
@@ -564,7 +643,7 @@ void PMSurfaceOfRevolution::joinSegments( const PMControlPointList& /*cp*/,
       kdError( PMArea ) << "Not enough points in PMSurfaceOfRevolution::joinSegments\n";
       return;
    }
-   
+
    double abs = 0.0, minabs = 1e10;
    int ns = -1;
    int i, j;
@@ -582,7 +661,7 @@ void PMSurfaceOfRevolution::joinSegments( const PMControlPointList& /*cp*/,
          dist[1] = (*p)[1];
          dist -= clickPosition;
          abs = dist.abs( );
-         
+
          if( ( minabs > abs ) || ( ns < 0 ) )
          {
             minabs = abs;
