@@ -20,9 +20,6 @@
 #include "pmtexture.h"
 #include "pmlinkedit.h"
 
-#include <kdebug.h>
-#include "pmglobals.h"
-
 #include <qlayout.h>
 #include <qlabel.h>
 #include <klocale.h>
@@ -36,7 +33,7 @@ PMTextureEdit::PMTextureEdit( QWidget* parent, const char* name )
 
 void PMTextureEdit::displayObject( PMObject* o )
 {
-   if( o->isA( PMTTexture ) )
+   if( o->isA( "Texture" ) )
    {
       Base::displayObject( o );
    }
