@@ -4,6 +4,8 @@
                              --------------------
     copyright            : (C) 2002 by Luis Carvalho
     email                : lpassos@mail.telepac.pt
+    copyright            : (C) 2003 by Andreas Zehender
+    email                : zehender@kde.org
 **************************************************************************
 
 **************************************************************************
@@ -23,6 +25,7 @@
 #include <klocale.h>
 #include <kfilterdev.h>
 #include <kpopupmenu.h>
+#include <kiconloader.h>
 
 #include <qfile.h>
 #include <qdom.h>
@@ -32,9 +35,9 @@
 #include <qlayout.h>
 #include <qlabel.h>
 
-#include <kdebug.h>
-#include "pmglobals.h"
 #include "pmshell.h"
+#include "pmdebug.h"
+#include "pmviewfactory.h"
 
 PMViewLayoutManager* PMViewLayoutManager::s_pInstance = 0;
 KStaticDeleter<PMViewLayoutManager> PMViewLayoutManager::s_staticDeleter;
@@ -44,8 +47,7 @@ KStaticDeleter<PMViewLayoutManager> PMViewLayoutManager::s_staticDeleter;
 
 PMViewLayoutEntry::PMViewLayoutEntry( )
 {
-   m_viewType = PMTreeView;
-   m_glViewType = PMGLView::PMViewCamera;
+   m_viewType = "treeview";
    m_dockPosition = PMDockWidget::DockRight;
    m_columnWidth = 33;
    m_height = 50;
@@ -53,6 +55,29 @@ PMViewLayoutEntry::PMViewLayoutEntry( )
    m_floatingHeight = 400;
    m_floatingPositionX = 200;
    m_floatingPositionY = 200;
+   m_pCustomOptions = 0;
+}
+
+PMViewLayoutEntry::PMViewLayoutEntry( const PMViewLayoutEntry& e )
+{
+   m_viewType = e.m_viewType;
+   m_dockPosition = e.m_dockPosition;
+   m_columnWidth = e.m_columnWidth;
+   m_height = e.m_height;
+   m_floatingWidth = e.m_floatingWidth;
+   m_floatingHeight = e.m_floatingHeight;
+   m_floatingPositionX = e.m_floatingPositionX;
+   m_floatingPositionY = e.m_floatingPositionY;
+   if( e.m_pCustomOptions )
+      m_pCustomOptions = e.m_pCustomOptions->copy( );
+   else
+      m_pCustomOptions = 0;
+}
+
+PMViewLayoutEntry::~PMViewLayoutEntry( )
+{
+   if( m_pCustomOptions )
+      delete m_pCustomOptions;
 }
 
 void PMViewLayoutEntry::loadData( QDomElement& e )
@@ -62,14 +87,8 @@ void PMViewLayoutEntry::loadData( QDomElement& e )
 
    // Read the view type
    s = e.tagName( );
-   if( s == "treeview" ) m_viewType = PMTreeView;
-   else if ( s == "dialogview" ) m_viewType = PMDialogView;
-   else if ( s == "glview" ) m_viewType = PMGLView;
-   else {
-      kdError( PMArea ) << i18n( "Unknown view type." ) << endl;
-      return;
-   }
-
+   m_viewType = s;
+   
    // Read dock position
    s = e.attribute( "position", "Right" );
    if( s == "Right" ) m_dockPosition = PMDockWidget::DockRight;
@@ -101,64 +120,19 @@ void PMViewLayoutEntry::loadData( QDomElement& e )
    m_floatingPositionY = s.toInt( &ok );
    if( !ok ) m_floatingPositionY = 200;
 
-   // Read the gl view type
-   s = e.attribute( "type", "Camera" );
-   if( s == "Camera" ) m_glViewType = PMGLView::PMViewCamera;
-   else if( s == "X" ) m_glViewType = PMGLView::PMViewPosX;
-   else if( s == "Y" ) m_glViewType = PMGLView::PMViewPosY;
-   else if( s == "Z" ) m_glViewType = PMGLView::PMViewPosZ;
-   else if( s == "NegX" ) m_glViewType = PMGLView::PMViewNegX;
-   else if( s == "NegY" ) m_glViewType = PMGLView::PMViewNegY;
-   else if( s == "NegZ" ) m_glViewType = PMGLView::PMViewNegZ;
+   // Load custom options
+   if( m_pCustomOptions )
+      delete m_pCustomOptions;
+   m_pCustomOptions =
+      PMViewFactory::theFactory( )->newOptionsInstance( m_viewType );
+   if( m_pCustomOptions )
+      m_pCustomOptions->loadData( e );
 }
 
 void PMViewLayoutEntry::saveData( QDomElement& e ) const
 {
    QString s;
-   switch( m_viewType )
-   {
-      case PMTreeView:
-         e.setTagName( "treeview" );
-         break;
-      case PMDialogView:
-         e.setTagName( "dialogview" );
-         break;
-      case PMGLView:
-         e.setTagName( "glview" );
-         switch( m_glViewType )
-         {
-            case PMGLView::PMViewCamera:
-               e.setAttribute( "type", "Camera" );
-               break;
-            case PMGLView::PMViewPosX:
-               e.setAttribute( "type", "X" );
-               break;
-            case PMGLView::PMViewPosY:
-               e.setAttribute( "type", "Y" );
-               break;
-            case PMGLView::PMViewPosZ:
-               e.setAttribute( "type", "Z" );
-               break;
-            case PMGLView::PMViewNegX:
-               e.setAttribute( "type", "NegX" );
-               break;
-            case PMGLView::PMViewNegY:
-               e.setAttribute( "type", "NegY" );
-               break;
-            case PMGLView::PMViewNegZ:
-               e.setAttribute( "type", "NegZ" );
-               break;
-            default:
-               kdError( PMArea ) << i18n( "Unknown GL view type." )
-                                 << endl;
-               break;
-         }
-         break;
-      default:
-         kdError( PMArea ) << i18n( "Unknown view type." )
-                           << endl;
-         break;
-   }
+   e.setTagName( m_viewType );
    switch( m_dockPosition )
    {
       case PMDockWidget::DockRight:
@@ -190,16 +164,15 @@ void PMViewLayoutEntry::saveData( QDomElement& e ) const
    e.setAttribute( "floatingPosX", s );
    s.setNum( m_floatingPositionY );
    e.setAttribute( "floatingPosY", s );
+
+   // save custom options
+   if( m_pCustomOptions )
+      m_pCustomOptions->saveData( e );
 }
 
-void PMViewLayoutEntry::setViewType( PMViewEntryType i )
+void PMViewLayoutEntry::setViewType( const QString& v )
 {
-   m_viewType = i;
-}
-
-void PMViewLayoutEntry::setGLViewType( PMGLView::PMViewType i )
-{
-   m_glViewType = i;
+   m_viewType = v;
 }
 
 void PMViewLayoutEntry::setDockPosition( PMDockWidget::DockPosition i )
@@ -221,54 +194,19 @@ void PMViewLayoutEntry::setHeight( int i )
       m_height = 1;
 }
 
-const QString PMViewLayoutEntry::extendedViewTypeAsString( )
-{
-   if( m_viewType == PMGLView )
-      return viewTypeAsString( ) + "(" + glViewTypeAsString( ) + ")";
-   else
-      return viewTypeAsString( );
-}
-
 const QString PMViewLayoutEntry::viewTypeAsString( )
 {
-   switch( m_viewType )
+   PMViewTypeFactory* f = PMViewFactory::theFactory( )->viewFactory( m_viewType );
+   if( f )
    {
-      case PMTreeView:
-         return i18n( "Object Tree" );
-      case PMDialogView:
-         return i18n( "Properties View" );
-      case PMGLView:
-         return i18n( "3D View" );
-      default:
-         kdError( PMArea ) << i18n( "Unknown view type." )
-                           << endl;
-         return i18n( "unknown" );
-   }
-}
-
-const QString PMViewLayoutEntry::glViewTypeAsString( )
-{
-   switch( m_glViewType )
-   {
-      case PMGLView::PMViewCamera:
-         return i18n( "Camera" );
-      case PMGLView::PMViewPosX:
-         return i18n( "Left" );
-      case PMGLView::PMViewPosY:
-         return i18n( "Bottom" );
-      case PMGLView::PMViewPosZ:
-         return i18n( "Front" );
-      case PMGLView::PMViewNegX:
-         return i18n( "Right" );
-      case PMGLView::PMViewNegY:
-         return i18n( "Top" );
-      case PMGLView::PMViewNegZ:
-         return i18n( "Back" );
-      default:
-         kdError( PMArea ) << i18n( "Unknown GL view type." )
-                           << endl;
-         return i18n( "unknown" );
+      if( m_pCustomOptions )
+         return f->description( m_pCustomOptions );
+      else
+         return f->description( );
    }
+   
+   kdError( PMArea ) << i18n( "Unknown view type." )<< endl;
+   return i18n( "Unknown" );
 }
 
 const QString PMViewLayoutEntry::dockPositionAsString( )
@@ -290,6 +228,13 @@ const QString PMViewLayoutEntry::dockPositionAsString( )
    }
 }
 
+void PMViewLayoutEntry::setCustomOptions( PMViewOptions* o )
+{
+   if( m_pCustomOptions && m_pCustomOptions != o )
+      delete m_pCustomOptions;
+   m_pCustomOptions = o;
+}
+
 //==================  PMViewLayout ======================
 
 PMViewLayout::PMViewLayout( )
@@ -373,25 +318,9 @@ void PMViewLayout::displayLayout( PMShell* shell )
 
    for( it = m_entries.begin( ); it != m_entries.end( ) ; ++it )
    {
-      PMDockWidget* dock;
-
-      // Create the appropriate dock widget
-      switch( ( *it ).viewType( ) )
-      {
-         case PMViewLayoutEntry::PMTreeView:
-            dock = shell->createTreeView( );
-            break;
-         case PMViewLayoutEntry::PMGLView:
-            dock = shell->create3DView( ( *it ).glViewType( ) );
-            break;
-         case PMViewLayoutEntry::PMDialogView:
-            dock = shell->createDialogView( );
-            break;
-         default:
-            dock = shell->createTreeView( );
-            break;
-      }
-      
+      PMDockWidget* dock = shell->createView( ( *it ).viewType( ),
+                                              ( *it ).customOptions( ),
+                                              false );
       // Dock the widget
       if( ( *it ).dockPosition( ) == PMDockWidget::DockNone )
       {
@@ -549,15 +478,17 @@ PMViewLayout PMViewLayout::extractViewLayout( PMShell* shell )
          e.setFloatingWidth( it.current( )->width( ) );
          e.setFloatingHeight( it.current( )->height( ) );
          
-         if( w->inherits( "PMDialogView" ) )
-            e.setViewType( PMViewLayoutEntry::PMDialogView );
-         else if( w->inherits( "PMTreeView" ) )
-            e.setViewType( PMViewLayoutEntry::PMTreeView );
-         else if( w->inherits( "PMGLView" ) )
+         if( w->inherits( "PMViewBase" ) )
          {
-            PMGLView* gl = ( PMGLView* ) w;
-            e.setViewType( PMViewLayoutEntry::PMGLView );
-            e.setGLViewType( gl->type( ) );
+            PMViewBase* v = ( PMViewBase* ) w;
+            ( *eit ).setViewType( v->viewType( ) );
+            PMViewOptions* vo =
+               PMViewFactory::theFactory( )->newOptionsInstance( v->viewType( ) );
+            if( vo )
+            {
+               v->saveViewConfig( vo );
+               ( *eit ).setCustomOptions( vo );
+            }
          }
          else
             invalid = true;
@@ -675,22 +606,22 @@ void PMViewLayout::recursiveExtractOneColumn(
                }
             }
          }
-         else if( w->inherits( "PMDialogView" ) )
+         else
          {
-            ( *eit ).setViewType( PMViewLayoutEntry::PMDialogView );
-            kdDebug( PMArea ) << widget << endl;
-         }
-         else if( w->inherits( "PMTreeView" ) )
-         {
-            ( *eit ).setViewType( PMViewLayoutEntry::PMTreeView );
-            kdDebug( PMArea ) << widget << endl;
-         }
-         else if( w->inherits( "PMGLView" ) )
-         {
-            PMGLView* gl = ( PMGLView* ) w;
-            ( *eit ).setViewType( PMViewLayoutEntry::PMGLView );
-            ( *eit ).setGLViewType( gl->type( ) );
-            kdDebug( PMArea ) << widget << endl;
+            // a kpovmodeler view???
+            QString cn( w->className( ) );
+            if( cn.left( 2 ) == "PM" && cn.right( 4 ) == "View" )
+            {
+               PMViewBase* v = ( PMViewBase* ) w;
+               ( *eit ).setViewType( v->viewType( ) );
+               PMViewOptions* vo =
+                  PMViewFactory::theFactory( )->newOptionsInstance( v->viewType( ) );
+               if( vo )
+               {
+                  v->saveViewConfig( vo );
+                  ( *eit ).setCustomOptions( vo );
+               }
+            }
          }
       }
    }
@@ -741,31 +672,31 @@ void PMViewLayoutManager::loadData( )
       PMViewLayout a;
       a.setName( i18n( "Default" ) );
       PMViewLayoutEntry p;
-      p.setViewType( PMViewLayoutEntry::PMTreeView );
+      p.setViewType( "treeview" );
       p.setDockPosition( PMDockWidget::DockRight );
       p.setHeight( 50 );
       p.setColumnWidth( 33 );
       a.addEntry( p );
-      p.setViewType( PMViewLayoutEntry::PMDialogView );
+      p.setViewType( "dialogview" );
       p.setDockPosition( PMDockWidget::DockBottom );
       p.setHeight( 50 );
       a.addEntry( p );
-      p.setViewType( PMViewLayoutEntry::PMGLView );
-      p.setGLViewType( PMGLView::PMViewPosX );
+      p.setViewType( "glview" );
+      p.setCustomOptions( new PMGLViewOptions( PMGLView::PMViewPosX ) );
       p.setDockPosition( PMDockWidget::DockRight );
       p.setHeight( 50 );
       p.setColumnWidth( 33 );
       a.addEntry( p );
-      p.setGLViewType( PMGLView::PMViewNegY );
+      p.setCustomOptions( new PMGLViewOptions( PMGLView::PMViewNegY ) );
       p.setDockPosition( PMDockWidget::DockBottom );
       p.setHeight( 50 );
       a.addEntry( p );
-      p.setGLViewType( PMGLView::PMViewPosZ );
+      p.setCustomOptions( new PMGLViewOptions( PMGLView::PMViewPosZ ) );
       p.setDockPosition( PMDockWidget::DockRight );
       p.setHeight( 50 );
       p.setColumnWidth( 33 );
       a.addEntry( p );
-      p.setGLViewType( PMGLView::PMViewCamera );
+      p.setCustomOptions( new PMGLViewOptions( PMGLView::PMViewCamera ) );
       p.setDockPosition( PMDockWidget::DockBottom );
       p.setHeight( 50 );
       a.addEntry( p );
