@@ -21,6 +21,7 @@
 #include <kcursor.h>
 #include <kdebug.h>
 #include <kglobal.h>
+#include <kglobalsettings.h>
 #include <kiconloader.h>
 #include <kimageeffect.h>
 #include <klocale.h>
@@ -29,7 +30,7 @@
 #include <kpopupmenu.h>
 #include <kstandarddirs.h>
 #include <kwin.h>
-#include <kglobalsettings.h>
+#include <kstdguiitem.h>
 
 #include <qbitmap.h>
 #include <qcursor.h>
@@ -46,6 +47,7 @@
 #define CFG_KEY_SCALE_FONT "ScaleFont"
 #define CFG_KEY_LENGTH "Length"
 #define CFG_GROUP_SETTINGS "StoredSettings"
+#define DEFAULT_RULER_COLOR QColor(255, 200, 80)
 /**
 * this is our cursor bitmap:
 * a line 48 pixels long with an arrow pointing down
@@ -91,7 +93,7 @@ KLineal::KLineal(QWidget*parent,const char* name):KMainWindow(parent,name){
   setMinimumSize(60,60);
   setMaximumSize(8000,8000);
   KConfig *cfg = kapp->config();
-  QColor defaultColor(255, 200, 80);
+  QColor defaultColor = DEFAULT_RULER_COLOR;
   QFont defaultFont(KGlobalSettings::generalFont().family(), 8);
   defaultFont.setPixelSize(8);
   if (cfg) {
@@ -139,7 +141,7 @@ KLineal::KLineal(QWidget*parent,const char* name):KMainWindow(parent,name){
   _clicked = false;
   setOrientation(South);
   // setMediumLength();
-  mMenu = new KPopupMenu();
+  mMenu = new KPopupMenu(this);
   mMenu->insertTitle(i18n("KRuler"));
   KPopupMenu *oriMenu = new KPopupMenu(this);
   oriMenu->insertItem(UserIconSet("kruler-north"), i18n("&North"), this, SLOT(setNorth()), Key_N);
@@ -155,12 +157,12 @@ KLineal::KLineal(QWidget*parent,const char* name):KMainWindow(parent,name){
   lenMenu->insertItem(i18n("&Tall"), this, SLOT(setTallLength()), CTRL+Key_T);
   lenMenu->insertItem(i18n("&Full Screen Width"), this, SLOT(setFullLength()), CTRL+Key_F);
   mMenu->insertItem(i18n("&Length"), lenMenu);
-  mMenu->insertItem(i18n("&Choose Color..."), this, SLOT(choseColor()), CTRL+Key_C);
-  mMenu->insertItem(i18n("Choose &Font..."), this, SLOT(choseFont()), Key_F);
+  mMenu->insertItem(SmallIcon("colorscm"),i18n("&Choose Color..."), this, SLOT(choseColor()), CTRL+Key_C);
+  mMenu->insertItem(SmallIcon("font"), i18n("Choose &Font..."), this, SLOT(choseFont()), Key_F);
   mMenu->insertSeparator();
-  mMenu->insertItem(SmallIcon( "help" ), i18n("&Help"), helpMenu());
+  mMenu->insertItem(SmallIcon( "help" ), KStdGuiItem::help().text(), helpMenu());
   mMenu->insertSeparator();
-  mMenu->insertItem(SmallIcon( "exit" ), i18n("&Quit"), kapp, SLOT(quit()), CTRL+Key_Q);
+  mMenu->insertItem(SmallIcon( "exit" ), KStdGuiItem::quit().text(), kapp, SLOT(quit()), CTRL+Key_Q);
   mLastClickPos = geometry().topLeft()+QPoint(width()/2, height()/2);
 }
 
@@ -241,14 +243,14 @@ void KLineal::setOrientation(int inOrientation) {
   QRect r = frameGeometry();
   int nineties = (int)inOrientation - (int)mOrientation;
   QPoint center = mLastClickPos;
-	
+
   if (_clicked) {
     center = mLastClickPos;
     _clicked = false;
    } else {
      center = r.topLeft()+QPoint(width()/2, height()/2);
    }
-	
+
   rotateRect(r, center, nineties);
   setGeometry(r);
   mOrientation = (inOrientation + 4) % 4;
@@ -300,12 +302,13 @@ void KLineal::reLength(int percentOfScreen) {
   if (percentOfScreen < 10) {
     return;
   }
-  int scnum = QApplication::desktop()->screenNumber(this);
+  QRect r = KGlobalSettings::desktopGeometry(this);
+
   if (mOrientation == North || mOrientation == South) {
-    mLongEdgeLen = QApplication::desktop()->screenGeometry(scnum).width() * percentOfScreen / 100;
+    mLongEdgeLen = r.width() * percentOfScreen / 100;
     resize(mLongEdgeLen, height());
   } else {
-    mLongEdgeLen = QApplication::desktop()->screenGeometry(scnum).height() * percentOfScreen / 100;
+    mLongEdgeLen = r.height() * percentOfScreen / 100;
     resize(width(), mLongEdgeLen);
   }
   if (x()+width() < 10) {
@@ -329,19 +332,20 @@ void KLineal::setFullLength() {
   reLength(100);
 }
 void KLineal::choseColor() {
-	QPoint pos = QCursor::pos();
-	QSize screen(QApplication::desktop()->size());
-	if (pos.x() + mColorSelector.width() > screen.width()) {
-		pos.setX(screen.width() - mColorSelector.width());
-	}
-	if (pos.y() + mColorSelector.height() > screen.height()) {
-		pos.setY(screen.height() - mColorSelector.height());
-	}
+  QRect r = KGlobalSettings::desktopGeometry(this);
+
+  QPoint pos = QCursor::pos();
+  if (pos.x() + mColorSelector.width() > r.width()) {
+    pos.setX(r.width() - mColorSelector.width());
+  }
+  if (pos.y() + mColorSelector.height() > r.height()) {
+    pos.setY(r.height() - mColorSelector.height());
+  }
   mStoredColor = mColor;
   mColorSelector.move(pos);
-  mColorSelector.show();
   mColorSelector.setColor(mColor);
-
+  mColorSelector.setDefaultColor( DEFAULT_RULER_COLOR );
+  mColorSelector.show();
 
   connect(&mColorSelector, SIGNAL(okClicked()), this, SLOT(setColor()));
   connect(&mColorSelector, SIGNAL(yesClicked()), this, SLOT(setColor()));
@@ -388,6 +392,8 @@ void KLineal::setColor() {
 */
 void KLineal::setColor(const QColor &color) {
   mColor = color;
+  if ( !mColor.isValid() )
+    mColor = DEFAULT_RULER_COLOR;
   setupBackground();
 }
 
@@ -497,7 +503,7 @@ void KLineal::keyPressEvent(QKeyEvent *e) {
   	dist *= 10;
   }
   move(pos()+dist);
-  KNotifyClient::event("cursormove");
+  KNotifyClient::event(0, "cursormove",  QString::null);
 }
 /**
 * overwritten to handle the line cursor which is a seperate widget outside the main
@@ -510,13 +516,13 @@ void KLineal::mouseMoveEvent(QMouseEvent * /*inEvent*/) {
   	QPoint p = QCursor::pos();
   	switch (mOrientation) {
     case North:
-    	p.setY(p.y()-46);    	
+    	p.setY(p.y()-46);
       break;
     case East:
-    	p.setX(p.x()+46);    	   	
+    	p.setX(p.x()+46);
       break;
     case West:
-    	p.setX(p.x()-46);    	   	
+    	p.setX(p.x()-46);
       break;
     case South:
     	p.setY(p.y()+46);
@@ -533,7 +539,7 @@ void KLineal::mouseMoveEvent(QMouseEvent * /*inEvent*/) {
   	} else {
   		v = 0;
    	}
-   	color.setHsv(h, s, v);		
+   	color.setHsv(h, s, v);
   	QPalette palette = mColorLabel->palette();
   	palette.setColor(QColorGroup::Foreground, color);
   	mColorLabel->setPalette(palette);
@@ -552,7 +558,7 @@ void KLineal::mousePressEvent(QMouseEvent *inEvent) {
   mDragOffset = mLastClickPos - QPoint(gr.left(), gr.top());
   if (inEvent->button() == LeftButton) {
     if (!mDragging) {
-      grabMouse(KCursor::handCursor());
+      grabMouse(KCursor::sizeAllCursor());
       mDragging = TRUE;
     }
   } else if (inEvent->button() == MidButton) {
