@@ -2,13 +2,19 @@
 
 #ifndef KSNAPSHOT_H
 #define KSNAPSHOT_H
-#include "ksnapshotbase.h"
 #include "ksnapshotiface.h"
+
 #include <qlabel.h>
 #include <qpixmap.h>
 #include <qtimer.h>
+
 #include <dcopclient.h>
 #include <kglobalsettings.h>
+#include <kdialogbase.h>
+#include <kurl.h>
+
+class RegionGrabber;
+class KSnapshotWidget;
 
 class KSnapshotThumb : public QLabel
 {
@@ -17,12 +23,14 @@ class KSnapshotThumb : public QLabel
     public:
         KSnapshotThumb(QWidget *parent, const char *name = 0)
             : QLabel(parent, name)
-        {}
+        {
+            setAlignment(AlignHCenter | AlignVCenter);
+        }
         virtual ~KSnapshotThumb() {}
-    
+
     signals:
         void startDrag();
-    
+
     protected:
         void mousePressEvent(QMouseEvent * e)
         {
@@ -39,15 +47,15 @@ class KSnapshotThumb : public QLabel
             }
         }
 
-    void mouseReleaseEvent(QMouseEvent * /*e*/)
+        void mouseReleaseEvent(QMouseEvent * /*e*/)
         {
             mClickPt = QPoint(0, 0);
         }
-        
+
         QPoint mClickPt;
 };
 
-class KSnapshot : public KSnapshotBase , virtual public KSnapshotIface
+class KSnapshot : public KDialogBase, virtual public KSnapshotIface
 {
   Q_OBJECT
 
@@ -55,20 +63,23 @@ public:
   KSnapshot(QWidget *parent= 0, const char *name= 0);
   ~KSnapshot();
 
-  bool save( const QString &filename );
-  QString url() const { return filename; }
+  enum CaptureMode { FullScreen=0, WindowUnderCursor=1, Region=2 };
 
+  bool save( const QString &filename );
+  QString url() const { return filename.url(); }
+
+protected slots:
   void slotGrab();
   void slotSave();
   void slotSaveAs();
   void slotCopy();
   void slotPrint();
   void slotMovePointer( int x, int y );
-  void exit();
 
   void setTime(int newTime);
   void setURL(const QString &newURL);
-  void setGrabPointer(bool grab);
+  void setGrabMode( int m );
+  void exit();
 
 protected:
     void reject() { close(); }
@@ -80,8 +91,10 @@ private slots:
     void grabTimerDone();
     void slotDragSnapshot();
     void updateCaption();
+    void slotRegionGrabbed( const QPixmap & );
 
 private:
+    bool save( const KURL& url );
     void updatePreview();
     void performGrab();
     void autoincFilename();
@@ -89,8 +102,11 @@ private:
     QPixmap snapshot;
     QTimer grabTimer;
     QWidget* grabber;
-    QString filename;
+    KURL filename;
+    KSnapshotWidget *mainWidget;
+    RegionGrabber *rgnGrab;
     bool modified;
+    bool haveXShape;
 };
 
 #endif // KSNAPSHOT_H
