@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 1998-2003 Carsten Pfeiffer <pfeiffer@kde.org>
+   Copyright (C) 1998-2004 Carsten Pfeiffer <pfeiffer@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -14,7 +14,7 @@
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
- */
+*/
 
 #include <stdlib.h>
 
@@ -34,12 +34,12 @@
 #include <qstring.h>
 #include <qstringlist.h>
 #include <qtimer.h>
-#include <qdragobject.h>
 
 #include <kapplication.h>
 #include <kconfig.h>
 #include <kcursor.h>
 #include <kdebug.h>
+#include <kdeversion.h>
 #ifdef KDE_USE_FINAL
 #undef Unsorted
 #endif
@@ -52,9 +52,12 @@
 #include <kpropertiesdialog.h>
 #include <kstdaccel.h>
 #include <kstandarddirs.h>
+#include <kglobalsettings.h>
 #include <ktempfile.h>
 #include <kwin.h>
 #include <netwm.h>
+#include <kurldrag.h>
+#include <kio/netaccess.h>
 
 #include "imagewindow.h"
 #include "kuick.h"
@@ -115,11 +118,8 @@ void ImageWindow::init()
 
     transWidget    = 0L;
     myIsFullscreen = false;
-    initialFullscreen = kdata->fullScreen;
-    ignore_resize_hack = false;
 
     xpos = 0, ypos = 0;
-    m_width = width(); m_height = height();
     m_numHeads = ScreenCount( x11Display() );
 
     setAcceptDrops( true );
@@ -157,13 +157,13 @@ void ImageWindow::setupActions()
                  this, SLOT( maximize() ),
                  m_actions, "maximize" );
 
-    new KAction( i18n("Rotate 90 degrees"), Key_9,
+    new KAction( i18n("Rotate 90 Degrees"), Key_9,
                  this, SLOT( rotate90() ),
                  m_actions, "rotate90" );
-    new KAction( i18n("Rotate 180 degrees"), Key_8,
+    new KAction( i18n("Rotate 180 Degrees"), Key_8,
                  this, SLOT( rotate180() ),
                  m_actions, "rotate180" );
-    new KAction( i18n("Rotate 270 degrees"), Key_7,
+    new KAction( i18n("Rotate 270 Degrees"), Key_7,
                  this, SLOT( rotate270() ),
                  m_actions, "rotate270" );
 
@@ -177,12 +177,10 @@ void ImageWindow::setupActions()
     new KAction( i18n("Print Image..."), KStdAccel::print(),
                  this, SLOT( printImage() ),
                  m_actions, "print_image" );
-    new KAction( i18n("Save As..."), KStdAccel::save(),
-                 this, SLOT( saveImage() ),
+    KStdAction::saveAs( this, SLOT( saveImage() ),
                  m_actions, "save_image_as" );
 
-    new KAction( i18n("Close"), Key_Q,
-                 this, SLOT( close() ),
+    KStdAction::close( this, SLOT( close() ),
                  m_actions, "close_image" );
     // --------
     new KAction( i18n("More Brightness"), Key_B,
@@ -218,9 +216,12 @@ void ImageWindow::setupActions()
                  this, SLOT( scrollRight() ),
                  m_actions, "scroll_right" );
 
-    new KAction( i18n("Toggle Fullscreen mode"), Key_Return,
-                 this, SLOT( toggleFullscreen() ),
-                 m_actions, "toggle_fullscreen" );
+    KShortcut cut(Key_Return);
+    cut.append(KStdAccel::shortcut(KStdAccel::FullScreen));
+
+    KAction *action = KStdAction::fullScreen(this, SLOT( toggleFullscreen() ), m_actions, 0 );
+    action->setShortcut(cut);
+
     new KAction( i18n("Reload Image"), Key_Enter,
                  this, SLOT( reload() ),
                  m_actions, "reload_image" );
@@ -238,43 +239,10 @@ void ImageWindow::setFullscreen( bool enable )
     xpos = 0; ypos = 0;
 
     if ( enable && !myIsFullscreen ) { // set Fullscreen
-        KWin::Info info = KWin::info( winId() );
-        oldGeometry = info.frameGeometry;
-
-	// qDebug("** oldGeometry: %i, %i, %i, %i",
-	// oldGeometry.x(), oldGeometry.y(),
-	// oldGeometry.width(), oldGeometry.height());
-
-        int scnum = QApplication::desktop()->screenNumber(this);
-        setFixedSize( QApplication::desktop()->screenGeometry(scnum).size() );
-
-        KWin::setType( winId(), NET::Override );
-        KWin::setState( winId(), NET::StaysOnTop );
-
-        setGeometry( QApplication::desktop()->screenGeometry(scnum) );
-        // qApp->processEvents(); // not necessary anymore
-
+        showFullScreen();
     }
-
     else if ( !enable && myIsFullscreen ) { // go into window mode
-        bool wasInitialFullscreen = initialFullscreen;
-        initialFullscreen = false;
-	
-        ignore_resize_hack = true; //ignore the resizeEvent triggered by move()
-        move( oldGeometry.topLeft() );
-        setMinimumSize(0,0);
-        myIsFullscreen = false; // we want resizeOptimal to use window-mode
-        resizeOptimal( imageWidth(), imageHeight() ); // resizeEvent centers
-
-        KWin::setType( winId(), NET::Normal );
-        KWin::clearState( winId(), NET::StaysOnTop );
-
-        // hack around kwin not giving us a decoration, when going into window
-        // mode and initially started in fullscreen mode
-        if ( wasInitialFullscreen ) {
-            hide();
-            show();
-        }
+        showNormal();
     }
 
     myIsFullscreen = enable;
@@ -288,7 +256,7 @@ void ImageWindow::updateGeometry( int imWidth, int imHeight )
     //  XMoveWindow( x11Display(), win, 0, 0 );
     XResizeWindow( x11Display(), win, imWidth, imHeight );
 
-    if ( imWidth != (int) m_width || imHeight != (int) m_height ) {
+    if ( imWidth != width() || imHeight != height() ) {
 	if ( myIsFullscreen ) {
 	    centerImage();
 	}
@@ -302,26 +270,45 @@ void ImageWindow::updateGeometry( int imWidth, int imHeight )
 	XMoveWindow( x11Display(), win, 0, 0 );
     }
 
+    updateCursor();
+
     QString caption = i18n( "Filename (Imagewidth x Imageheight)",
-                            "%1 (%2 x %3)" );
-    caption = caption.arg( m_kuim->filename() ).
-              arg( m_kuim->originalWidth() ).
-              arg( m_kuim->originalHeight() );
+                            "%3 (%1 x %2)" );
+    caption = caption.arg( m_kuim->originalWidth() ).
+              arg( m_kuim->originalHeight() ).arg( m_kuim->filename() );
     setCaption( kapp->makeStdCaption( caption ) );
 }
 
 
 void ImageWindow::centerImage()
 {
+    int w, h;
+    if ( myIsFullscreen )
+    {
+        QRect desktopRect = KGlobalSettings::desktopGeometry( this );
+        w = desktopRect.width();
+        h = desktopRect.height();
+    }
+    else
+    {
+        w = width();
+        h = height();
+    }
+
+    xpos = w/2 - imageWidth()/2;
+    ypos = h/2 - imageHeight()/2;
+
+    XMoveWindow( x11Display(), win, xpos, ypos );
+
     // Modified by Evan for his Multi-Head (2 screens)
     // This should center on the first head
-    if ( myIsFullscreen && m_numHeads > 1 && ((m_numHeads % 2) == 0) )
-        xpos = ((m_width/m_numHeads) / 2) - imageWidth()/2;
-    else
-        xpos = m_width/2 - imageWidth()/2;
+//     if ( myIsFullscreen && m_numHeads > 1 && ((m_numHeads % 2) == 0) )
+//         xpos = ((width()/m_numHeads) / 2) - imageWidth()/2;
+//     else
+//         xpos = width()/2 - imageWidth()/2;
 
-    ypos = m_height/2 - imageHeight()/2;
-    XMoveWindow( x11Display(), win, xpos, ypos );
+//     ypos = height()/2 - imageHeight()/2;
+//     XMoveWindow( x11Display(), win, xpos, ypos );
 }
 
 
@@ -330,16 +317,16 @@ void ImageWindow::scrollImage( int x, int y, bool restrict )
     xpos += x;
     ypos += y;
 
-    int cwlocal = m_width;
-    int chlocal = m_height;
+    int cwlocal = width();
+    int chlocal = height();
 
     int iw = imageWidth();
     int ih = imageHeight();
 
-    if ( myIsFullscreen || m_width > desktopWidth() )
+    if ( myIsFullscreen || width() > desktopWidth() )
 	cwlocal = desktopWidth();
 
-    if ( myIsFullscreen || m_height > desktopHeight() )
+    if ( myIsFullscreen || height() > desktopHeight() )
 	chlocal = desktopHeight();
 
     if ( restrict ) { // don't allow scrolling in certain cases
@@ -437,22 +424,22 @@ void ImageWindow::addGamma( int factor )
 
 void ImageWindow::scrollUp()
 {
-    scrollImage( 0, kdata->scrollSteps );
+    scrollImage( 0, 20 * kdata->scrollSteps );
 }
 
 void ImageWindow::scrollDown()
 {
-    scrollImage( 0, - kdata->scrollSteps );
+    scrollImage( 0, - 20 * kdata->scrollSteps );
 }
 
 void ImageWindow::scrollLeft()
 {
-    scrollImage( kdata->scrollSteps, 0 );
+    scrollImage( 20 * kdata->scrollSteps, 0 );
 }
 
 void ImageWindow::scrollRight()
 {
-    scrollImage( - kdata->scrollSteps, 0 );
+    scrollImage( - 20 * kdata->scrollSteps, 0 );
 }
 
 ///
@@ -527,14 +514,17 @@ void ImageWindow::keyPressEvent( QKeyEvent *e )
 {
     uint key = e->key();
 
+    if ( key == Key_Shift )
+        updateCursor( ZoomCursor );
+
     if ( key == Key_Escape || KStdAccel::close().contains( KKey( e ) ) )
-	close( true );
+        close( true );
     else if ( KStdAccel::save().contains( KKey( e ) ) )
-	saveImage();
+        saveImage();
 
     else {
- 	e->ignore();
- 	return;
+        e->ignore();
+        return;
     }
 
     e->accept();
@@ -543,11 +533,11 @@ void ImageWindow::keyPressEvent( QKeyEvent *e )
 void ImageWindow::keyReleaseEvent( QKeyEvent *e )
 {
     if ( e->state() & ShiftButton ) { // Shift-key released
-	setCursor( arrowCursor );
-	if ( transWidget ) {
-	    delete transWidget;
-	    transWidget = 0L;
-	}
+        updateCursor();
+        if ( transWidget ) {
+            delete transWidget;
+            transWidget = 0L;
+        }
     }
 
     e->accept();
@@ -566,9 +556,9 @@ void ImageWindow::mousePressEvent( QMouseEvent *e )
 
     if ( e->button() == LeftButton ) {
         if ( e->state() & ShiftButton )
-            setCursor( arrowCursor ); // need a magnify-cursor
+            updateCursor( ZoomCursor );
         else
-            setCursor( *s_handCursor );
+            updateCursor( MoveCursor );
     }
 
     ImlibWidget::mousePressEvent( e );
@@ -584,6 +574,26 @@ void ImageWindow::contextMenuEvent( QContextMenuEvent *e )
     viewerMenu->popup( e->globalPos() );
 }
 
+void ImageWindow::updateCursor( KuickCursor cursor )
+{
+    switch ( cursor )
+    {
+        case ZoomCursor:
+            setCursor( arrowCursor ); // need a magnify-cursor
+            break;
+        case MoveCursor:
+            setCursor( *s_handCursor );
+            break;
+        case DefaultCursor:
+        default:
+            if ( imageWidth() > width() || imageHeight() > height() )
+                setCursor( *s_handCursor );
+            else
+                setCursor( arrowCursor );
+            break;
+    }
+}
+
 void ImageWindow::mouseMoveEvent( QMouseEvent *e )
 {
     if ( !(e->state() & LeftButton) ) { // only handle LeftButton actions
@@ -591,10 +601,10 @@ void ImageWindow::mouseMoveEvent( QMouseEvent *e )
     }
 
     if ( e->state() & ShiftButton ) {
-	
+
 	if ( !transWidget ) {
 	    transWidget = new QWidget( this );
-	    transWidget->setGeometry( 0, 0, m_width, m_height );
+	    transWidget->setGeometry( 0, 0, width(), height() );
 	    transWidget->setBackgroundMode( NoBackground );
 	}
 
@@ -604,10 +614,10 @@ void ImageWindow::mouseMoveEvent( QMouseEvent *e )
  	p.eraseRect( transWidget->rect() );
 	transWidget->show();
 	qApp->processOneEvent();
-	
+
 	int width  = e->x() - xposPress;
 	int height = e->y() - yposPress;
-	
+
 	if ( width < 0 ) {
 	    width = abs( width );
 	    xzoom = e->x();
@@ -638,7 +648,7 @@ void ImageWindow::mouseMoveEvent( QMouseEvent *e )
 
 void ImageWindow::mouseReleaseEvent( QMouseEvent *e )
 {
-    setCursor( arrowCursor );
+    updateCursor();
 
     if ( transWidget ) {
        // destroy the transparent widget, used for showing the rectangle (zoom)
@@ -670,7 +680,7 @@ void ImageWindow::mouseReleaseEvent( QMouseEvent *e )
     }
 
     if ( yposPress > y ) {
-	topY = y;	
+	topY = y;
 	botY = yposPress;
     }
     else {
@@ -681,8 +691,8 @@ void ImageWindow::mouseReleaseEvent( QMouseEvent *e )
     neww = botX - topX;
     newh = botY - topY;
 
-    factorx = ((float) m_width / (float) neww);
-    factory = ((float) m_height / (float) newh);
+    factorx = ((float) width() / (float) neww);
+    factory = ((float) height() / (float) newh);
 
     if ( factorx < factory ) // use the smaller factor
 	factor = factorx;
@@ -701,9 +711,9 @@ void ImageWindow::mouseReleaseEvent( QMouseEvent *e )
     int xtmp = - (int) (factor * abs(xpos - topX) );
     int ytmp = - (int) (factor * abs(ypos - topY) );
 
-    // if image has different ratio (m_width/m_height), center it
-    int xcenter = (m_width  - (int) (neww * factor)) / 2;
-    int ycenter = (m_height - (int) (newh * factor)) / 2;
+    // if image has different ratio (width()/height()), center it
+    int xcenter = (width()  - (int) (neww * factor)) / 2;
+    int ycenter = (height() - (int) (newh * factor)) / 2;
 
     xtmp += xcenter;
     ytmp += ycenter;
@@ -729,25 +739,8 @@ void ImageWindow::resizeEvent( QResizeEvent *e )
 {
     ImlibWidget::resizeEvent( e );
 
-    // to save a lot of calls in scrollImage() for example
-    m_width  = width();
-    m_height = height();
-
-    if ( ignore_resize_hack ) {
-	ignore_resize_hack = false;
-	
-	int w = width();
-	int h = height();
-        int scnum = QApplication::desktop()->screenNumber(this);
-	if ( w == QApplication::desktop()->screenGeometry(scnum).width() &&
-	     h == QApplication::desktop()->screenGeometry(scnum).height() &&
-	     imageWidth() < w && imageHeight() < h ) {
-	
-	    return;
-	}
-    }
-
     centerImage();
+    updateCursor();
 }
 
 
@@ -764,9 +757,15 @@ void ImageWindow::dragEnterEvent( QDragEnterEvent *e )
 void ImageWindow::dropEvent( QDropEvent *e )
 {
     // FIXME - only preliminary drop-support for now
-    QStringList list;
-    if ( QUriDrag::decodeLocalFiles( e, list ) ) {
-	loadImage( list.first() );
+    KURL::List list;
+    if ( KURLDrag::decode( e, list ) && !list.isEmpty()) {
+        QString tmpFile;
+        const KURL &url = list.first();
+        if (KIO::NetAccess::download( url, tmpFile, this ) )
+        {
+	    loadImage( tmpFile );
+	    KIO::NetAccess::removeTempFile( tmpFile );
+	}
 	updateWidget();
 	e->accept();
     }
@@ -844,11 +843,18 @@ void ImageWindow::saveImage()
         return;
 
     KuickData tmp;
-    QCheckBox *keepSize = new QCheckBox( i18n("Keep original Image Size"), 0L);
+    QCheckBox *keepSize = new QCheckBox( i18n("Keep original image size"), 0L);
     keepSize->setChecked( true );
-    KFileDialog dlg( QString::null, tmp.fileFilter, this, "filedialog", true,
-                     keepSize );
-    dlg.setSelection( m_kuim->filename() );
+    KFileDialog dlg( m_saveDirectory, tmp.fileFilter, this, "filedialog", true
+#if KDE_VERSION >= 310
+                     ,keepSize
+#endif
+                   );
+
+    QString selection = m_saveDirectory.isEmpty() ?
+                            m_kuim->filename() :
+                            KURL::fromPathOrURL( m_kuim->filename() ).fileName();
+    dlg.setSelection( selection );
     dlg.setOperationMode( KFileDialog::Saving );
     dlg.setCaption( i18n("Save As") );
     if ( dlg.exec() == QDialog::Accepted )
@@ -869,6 +875,14 @@ void ImageWindow::saveImage()
             }
         }
     }
+
+    QString lastDir = dlg.baseURL().path(+1);
+    if ( lastDir != m_saveDirectory )
+        m_saveDirectory = lastDir;
+
+#if KDE_VERSION < 310
+    delete keepSize;
+#endif
 }
 
 bool ImageWindow::saveImage( const QString& filename, bool keepOriginalSize ) const
@@ -885,8 +899,8 @@ bool ImageWindow::saveImage( const QString& filename, bool keepOriginalSize ) co
     if ( saveIm ) {
         Imlib_apply_modifiers_to_rgb( id, saveIm );
         success = Imlib_save_image( id, saveIm,
-                                         QFile::encodeName( filename ).data(),
-                                         NULL );
+                                    QFile::encodeName( filename ).data(),
+                                    NULL );
         Imlib_kill_image( id, saveIm );
     }
 
@@ -1004,8 +1018,7 @@ int ImageWindow::desktopWidth( bool totalScreen ) const
 {
     if ( myIsFullscreen || totalScreen )
     {
-        int scnum = QApplication::desktop()->screenNumber(topLevelWidget());
-	return QApplication::desktop()->screenGeometry(scnum).width();
+        return KGlobalSettings::desktopGeometry(topLevelWidget()).width();
     } else
 	return Kuick::workArea().width();
 }
@@ -1014,8 +1027,7 @@ int ImageWindow::desktopWidth( bool totalScreen ) const
 int ImageWindow::desktopHeight( bool totalScreen ) const
 {
     if ( myIsFullscreen || totalScreen ) {
-        int scnum = QApplication::desktop()->screenNumber(topLevelWidget());
-	return QApplication::desktop()->screenGeometry(scnum).height();
+        return KGlobalSettings::desktopGeometry(topLevelWidget()).height();
     } else {
 	return Kuick::workArea().height();
     }
@@ -1023,9 +1035,8 @@ int ImageWindow::desktopHeight( bool totalScreen ) const
 
 QSize ImageWindow::maxImageSize() const
 {
-    if ( myIsFullscreen || initialFullscreen ) {
-        int scnum = QApplication::desktop()->screenNumber(topLevelWidget());
-	return QApplication::desktop()->screenGeometry(scnum).size();
+    if ( myIsFullscreen ) {
+        return KGlobalSettings::desktopGeometry(topLevelWidget()).size();
     }
     else {
 	return Kuick::workArea().size() - Kuick::frameSize( winId() );
@@ -1040,7 +1051,7 @@ void ImageWindow::resizeOptimal( int w, int h )
     int neww = (w >= mw) ? mw : w;
     int newh = (h >= mh) ? mh : h;
 
-    if ( neww == m_width && newh == m_height )
+    if ( neww == width() && newh == height() )
 	centerImage();
     else
 	resize( neww, newh ); // also centers the image
@@ -1074,5 +1085,4 @@ void ImageWindow::slotProperties()
     (void) new KPropertiesDialog( url, this, "props dialog", true );
 }
 
-
 #include "imagewindow.moc"
