@@ -14,7 +14,7 @@
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
- */
+*/
 
 #ifndef IMAGEWINDOW_H
 #define IMAGEWINDOW_H
@@ -104,8 +104,10 @@ protected:
   virtual void 	resizeEvent( QResizeEvent * );
   virtual void 	dragEnterEvent( QDragEnterEvent * );
   virtual void 	dropEvent( QDropEvent * );
-  virtual void contextMenuEvent( QContextMenuEvent * );
+  virtual void  contextMenuEvent( QContextMenuEvent * );
 
+  enum KuickCursor { DefaultCursor = 0, ZoomCursor, MoveCursor };
+  void 	updateCursor( KuickCursor cursor = DefaultCursor );
 
 
   // popupmenu entries
@@ -124,8 +126,6 @@ protected:
   int 		xzoom, yzoom;  // used for zooming the image with the mouse
   uint 		xposPress, yposPress;
 
-  QRect 	oldGeometry;
-
 
   QPopupMenu    *viewerMenu, *gammaMenu, *brightnessMenu, *contrastMenu;
   QWidget       *transWidget;
@@ -147,17 +147,11 @@ private:
   void 		setPopupMenu();
 
   bool 		myIsFullscreen;
-  bool 		initialFullscreen;
-  int 		m_width;
-  int 		m_height;
   int           m_numHeads;
+  QString   m_saveDirectory;
 
   KActionCollection *m_actions;
 
-  // Qt resizes us even if we just request a move(). This sucks when
-  // switching from fullscreen to window mode, as we will be resized twice.
-  bool 		ignore_resize_hack;
-
   static QCursor * s_handCursor;
 };
 
