@@ -8,21 +8,23 @@
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
+   General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
- */
+*/
 
 #include <stdio.h>
 #include <assert.h>
 
 #include <qdir.h>
+#include <qdesktopwidget.h>
 #include <qdialog.h>
 #include <qglobal.h>
 #include <qkeycode.h>
+#include <qlayout.h>
 #include <qsize.h>
 #include <qstring.h>
 
@@ -33,6 +35,7 @@
 #include <kcmdlineargs.h>
 #include <kconfig.h>
 #include <kcursor.h>
+#include <kdeversion.h>
 #include <kfiledialog.h>
 #include <kfilemetainfo.h>
 #include <kglobal.h>
@@ -50,8 +53,11 @@
 #include <kstandarddirs.h>
 #include <kstartupinfo.h>
 #include <ktoolbar.h>
+#include <kurlcombobox.h>
+#include <kurlcompletion.h>
 #include <kurldrag.h>
 #include <kwin.h>
+#include <kstdguiitem.h>
 
 #include "aboutwidget.h"
 #include "filewidget.h"
@@ -81,7 +87,6 @@ static const int META_ITEM = 1;
 
 QValueList<ImageWindow*> KuickShow::s_viewers;
 
-
 KuickShow::KuickShow( const char *name )
     : KMainWindow( 0L, name ),
       m_slideshowCycle( 1 ),
@@ -93,90 +98,94 @@ KuickShow::KuickShow( const char *name )
       m_accel( 0L ),
       m_delayedRepeatItem( 0L )
 {
-  kdata = new KuickData;
-  kdata->load();
+    aboutWidget = 0L;
+    kdata = new KuickData;
+    kdata->load();
 
-  initImlib();
-  resize( 400, 500 );
+    initImlib();
+    resize( 400, 500 );
 
-  m_slideTimer = new QTimer( this );
-  connect( m_slideTimer, SIGNAL( timeout() ), SLOT( nextSlide() ));
+    m_slideTimer = new QTimer( this );
+    connect( m_slideTimer, SIGNAL( timeout() ), SLOT( nextSlide() ));
 
 
-  KConfig *kc = KGlobal::config();
+    KConfig *kc = KGlobal::config();
 
-  bool isDir = false; // true if we get a directory on the commandline
+    bool isDir = false; // true if we get a directory on the commandline
 
-  // parse commandline options
-  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
-		
-  // files to display
-  // either a directory to display, an absolute path, a relative path, or a URL
-  KURL startDir;
-  startDir.setPath( QDir::currentDirPath() + '/' );
-  for ( int i = 0; i < args->count(); i++ ) {
-      KURL url = args->url( i );
-      KFileItem item( KFileItem::Unknown, KFileItem::Unknown, url, false );
+    // parse commandline options
+    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 
-      // for remote URLs, we don't know if it's a file or directory, but
-      // FileWidget::isImage() should correct in most cases.
-      // For non-local non-images, we just assume directory.
+    // files to display
+    // either a directory to display, an absolute path, a relative path, or a URL
+    KURL startDir;
+    startDir.setPath( QDir::currentDirPath() + '/' );
+    for ( int i = 0; i < args->count(); i++ ) {
+        KURL url = args->url( i );
+        KFileItem item( KFileItem::Unknown, KFileItem::Unknown, url, false );
 
-      if ( FileWidget::isImage( &item ) )
-      {
-	  showImage( &item, true, false, true ); // show in new window, not fullscreen-forced and move to 0,0
-// 	  showImage( &item, true, false, false ); // show in new window, not fullscreen-forced and not moving to 0,0
-      }
-      else if ( item.isDir() )
-      {
-          startDir = url;
-          isDir = true;
-      }
+        // for remote URLs, we don't know if it's a file or directory, but
+        // FileWidget::isImage() should correct in most cases.
+        // For non-local non-images, we just assume directory.
 
-      // need to check remote files
-      else if ( !url.isLocalFile() )
-      {
-          KMimeType::Ptr mime = KMimeType::findByURL( url );
-          QString name = mime->name();
-          if ( name == "application/octet-stream" ) // unknown -> stat()
-              name = KIO::NetAccess::mimetype( url );
+        if ( FileWidget::isImage( &item ) )
+        {
+            showImage( &item, true, false, true ); // show in new window, not fullscreen-forced and move to 0,0
+//    showImage( &item, true, false, false ); // show in new window, not fullscreen-forced and not moving to 0,0
+        }
+        else if ( item.isDir() )
+        {
+            startDir = url;
+            isDir = true;
+        }
 
-          if ( name.startsWith( "image/" ) )
-          {
-              FileWidget::setImage( item, true );
-              showImage( &item, true, false, true );
-          }
-          else // assume directory, KDirLister will tell us if we can't list
-          {
-	      startDir = url;
-	      isDir = true;
-	  }
-      }
-      // else // we don't handle local non-images
-  }
+        // need to check remote files
+        else if ( !url.isLocalFile() )
+        {
+            KMimeType::Ptr mime = KMimeType::findByURL( url );
+            QString name = mime->name();
+#if KDE_VERSION >= 310
+            if ( name == "application/octet-stream" ) // unknown -> stat()
+                name = KIO::NetAccess::mimetype( url, this );
+#endif
 
-  if ( args->isSet( "lastdir" ))
-      startDir = kc->readEntry( "CurrentDirectory", startDir.url() );
+            if ( name.startsWith( "image/" ) )
+            {
+                FileWidget::setImage( item, true );
+                showImage( &item, true, false, true );
+            }
+            else // assume directory, KDirLister will tell us if we can't list
+            {
+                startDir = url;
+                isDir = true;
+            }
+        }
+        // else // we don't handle local non-images
+    }
 
-  if ( s_viewers.isEmpty() || isDir ) {
-      initGUI( startDir );
-      show();
-  }
+    if ( (kdata->startInLastDir && args->count() == 0) || args->isSet( "lastfolder" )) {
+        kc->setGroup( "SessionSettings");
+        startDir = kc->readPathEntry( "CurrentDirectory", startDir.url() );
+    }
 
-  else { // don't show browser, when image on commandline
-      hide();
-      KStartupInfo::appStarted();
-  }
+    if ( s_viewers.isEmpty() || isDir ) {
+        initGUI( startDir );
+        show();
+    }
+
+    else { // don't show browser, when image on commandline
+        hide();
+        KStartupInfo::appStarted();
+    }
 }
 
 
 KuickShow::~KuickShow()
 {
-    if ( fileWidget )
-	saveSettings();
+    saveSettings();
 
     if ( m_viewer )
-	m_viewer->close( true );
+        m_viewer->close( true );
 
     free( id );
     kapp->quit();
@@ -184,7 +193,6 @@ KuickShow::~KuickShow()
     delete kdata;
 }
 
-
 void KuickShow::initGUI( const KURL& startDir )
 {
     fileWidget = new FileWidget( startDir, this, "MainWidget" );
@@ -193,40 +201,45 @@ void KuickShow::initGUI( const KURL& startDir )
     KActionCollection *coll = fileWidget->actionCollection();
 
     connect( fileWidget, SIGNAL( fileSelected( const KFileItem * ) ),
-	     this, SLOT( slotSelected( const KFileItem * ) ));
+             this, SLOT( slotSelected( const KFileItem * ) ));
 
     connect( fileWidget, SIGNAL( fileHighlighted( const KFileItem * )),
-	     this, SLOT( slotHighlighted( const KFileItem * ) ));
+             this, SLOT( slotHighlighted( const KFileItem * ) ));
 
     connect( fileWidget, SIGNAL( urlEntered( const KURL&  )),
-	     this, SLOT( dirSelected( const KURL& )) );
+             this, SLOT( dirSelected( const KURL& )) );
+
+
+    fileWidget->setAcceptDrops(true);
+    connect( fileWidget, SIGNAL( dropped( const KFileItem *, QDropEvent *, const KURL::List & )),
+             this, SLOT( slotDropped( const KFileItem *, QDropEvent *, const KURL::List &)) );
 
     // setup actions
     KAction *open = KStdAction::open( this, SLOT( slotOpenURL() ),
                                       coll, "openURL" );
 
     KAction *print = KStdAction::print( this, SLOT( slotPrint() ),
-					coll, "kuick_print" );
+                                        coll, "kuick_print" );
     print->setText( i18n("Print Image...") );
 
     KAction *configure = new KAction( i18n("Configure %1...").arg( KGlobal::instance()->aboutData()->programName() ), "configure",
                                       KShortcut(),
-				      this, SLOT( configuration() ),
-				      coll, "kuick_configure" );
+                                      this, SLOT( configuration() ),
+                                      coll, "kuick_configure" );
     KAction *slide = new KAction( i18n("Start Slideshow" ), "ksslide",
                                   KShortcut( Key_F2 ),
-				  this, SLOT( startSlideShow() ),
-				  coll, "kuick_slideshow" );
+                                  this, SLOT( startSlideShow() ),
+                                  coll, "kuick_slideshow" );
     KAction *about = new KAction( i18n( "About KuickShow" ), "about",
                                   KShortcut(),
-				  this, SLOT( about() ), coll, "about" );
+                                  this, SLOT( about() ), coll, "about" );
 
-    oneWindowAction = new KToggleAction( i18n("Open only one Image Window"),
-					 "window_new",
+    oneWindowAction = new KToggleAction( i18n("Open Only One Image Window"),
+                                         "window_new",
                                          KShortcut( CTRL+Key_N ), coll,
-					 "kuick_one window" );
+                                         "kuick_one window" );
 
-    m_toggleBrowserAction = new KToggleAction( i18n("Show/Hide the File Browser"), KShortcut( Key_Space ), coll, "toggleBrowser" );
+    m_toggleBrowserAction = new KToggleAction( i18n("Show/Hide File Browser"), KShortcut( Key_Space ), coll, "toggleBrowser" );
     connect( m_toggleBrowserAction, SIGNAL( toggled( bool ) ),
              SLOT( toggleBrowser() ));
 
@@ -315,7 +328,7 @@ void KuickShow::initGUI( const KURL& startDir )
     about->plug( tBar );
 
     QPopupMenu *help = helpMenu( QString::null, false );
-    mBar->insertItem( i18n("&Help"), help );
+    mBar->insertItem( KStdGuiItem::help().text() , help );
 
 
     KStatusBar* sBar = statusBar();
@@ -332,6 +345,25 @@ void KuickShow::initGUI( const KURL& startDir )
 
     tBar->show();
 
+    // Address box in address tool bar
+    KToolBar *addressToolBar = toolBar( "address_bar" );
+    const int ID_ADDRESSBAR = 1;
+
+    cmbPath = new KURLComboBox( KURLComboBox::Directories,
+                                true, addressToolBar, "address_combo_box" );
+    KURLCompletion *cmpl = new KURLCompletion( KURLCompletion::DirCompletion );
+    cmbPath->setCompletionObject( cmpl );
+    cmbPath->setAutoDeleteCompletionObject( true );
+
+    addressToolBar->insertWidget( ID_ADDRESSBAR, 1, cmbPath);
+    addressToolBar->setItemAutoSized( ID_ADDRESSBAR );
+
+    connect( cmbPath, SIGNAL( urlActivated( const KURL& )),
+             this, SLOT( slotSetURL( const KURL& )));
+    connect( cmbPath, SIGNAL( returnPressed()),
+             this, SLOT( slotURLComboReturnPressed()));
+
+
     fileWidget->initActions();
     fileWidget->clearHistory();
     dirSelected( fileWidget->url() );
@@ -353,6 +385,16 @@ void KuickShow::initGUI( const KURL& startDir )
     fileWidget->resize( size() );
 }
 
+void KuickShow::slotSetURL( const KURL& url )
+{
+    fileWidget->setURL( url, true );
+}
+
+void KuickShow::slotURLComboReturnPressed()
+{
+    KURL where = KURL::fromPathOrURL( cmbPath->currentText() );
+    slotSetURL( where );
+}
 
 void KuickShow::viewerDeleted()
 {
@@ -362,14 +404,13 @@ void KuickShow::viewerDeleted()
         m_viewer = 0L;
 
     if ( !haveBrowser() && s_viewers.isEmpty() ) {
-	if ( fileWidget )
-	    saveSettings();
+        saveSettings();
 
-	::exit(0);
+        ::exit(0);
     }
 
     else if ( haveBrowser() ) {
-	setActiveWindow();
+        setActiveWindow();
         // This setFocus() call causes problems in the combiview (always the
         // directory view on the left gets the focus, which is not desired)
         // fileWidget->setFocus();
@@ -387,23 +428,26 @@ void KuickShow::slotHighlighted( const KFileItem *fi )
 {
     KFileItem *item = const_cast<KFileItem *>( fi );
     statusBar()->changeItem( item->getStatusBarInfo(), URL_ITEM );
+    bool image = FileWidget::isImage( fi );
 
     QString meta;
-    KFileMetaInfo info = item->metaInfo();
-    if ( info.isValid() )
+    if ( image )
     {
-        meta = info.item( KFileMimeTypeInfo::Size ).string();
-        KFileMetaInfoGroup group = info.group( "Technical" );
-        if ( group.isValid() )
+        KFileMetaInfo info = item->metaInfo();
+        if ( info.isValid() )
         {
-            QString bpp = group.item( "BitDepth" ).string();
-            if ( !bpp.isEmpty() )
-                meta.append( ", " ).append( bpp );
+            meta = info.item( KFileMimeTypeInfo::Size ).string();
+            KFileMetaInfoGroup group = info.group( "Technical" );
+            if ( group.isValid() )
+            {
+                QString bpp = group.item( "BitDepth" ).string();
+                if ( !bpp.isEmpty() )
+                    meta.append( ", " ).append( bpp );
+            }
         }
     }
     statusBar()->changeItem( meta, META_ITEM );
 
-    bool image = FileWidget::isImage( fi );
     fileWidget->actionCollection()->action("kuick_print")->setEnabled( image );
     fileWidget->actionCollection()->action("kuick_showInSameWindow")->setEnabled( image );
     fileWidget->actionCollection()->action("kuick_showInOtherWindow")->setEnabled( image );
@@ -412,10 +456,11 @@ void KuickShow::slotHighlighted( const KFileItem *fi )
 void KuickShow::dirSelected( const KURL& url )
 {
     if ( url.isLocalFile() )
-	setCaption( url.path() );
+        setCaption( url.path() );
     else
-	setCaption( url.prettyURL() );
-	
+        setCaption( url.prettyURL() );
+
+    cmbPath->setURL( url );
     statusBar()->changeItem( url.prettyURL(), URL_ITEM );
 }
 
@@ -425,8 +470,8 @@ void KuickShow::slotSelected( const KFileItem *item )
 }
 
 // downloads item if necessary
-void KuickShow::showFileItem( ImageWindow */*view*/,
-			      const KFileItem */*item*/ )
+void KuickShow::showFileItem( ImageWindow * /*view*/,
+                              const KFileItem * /*item*/ )
 {
 
 }
@@ -436,28 +481,27 @@ void KuickShow::showImage( const KFileItem *fi,
 {
     newWindow  |= !m_viewer;
     fullscreen |= (newWindow && kdata->fullScreen);
-
     if ( FileWidget::isImage( fi ) ) {
 
-	if ( newWindow ) {
-	    m_viewer = new ImageWindow( kdata->idata, id, 0L, "image window" );
-	    s_viewers.append( m_viewer );
+        if ( newWindow ) {
+            m_viewer = new ImageWindow( kdata->idata, id, 0L, "image window" );
+            s_viewers.append( m_viewer );
 
-	    connect( m_viewer, SIGNAL( destroyed() ), SLOT( viewerDeleted() ));
-	    connect( m_viewer, SIGNAL( sigFocusWindow( ImageWindow *) ),
-		     this, SLOT( slotSetActiveViewer( ImageWindow * ) ));
-	    connect( m_viewer, SIGNAL( sigBadImage(const QString& ) ),
-		     this, SLOT( messageCantLoadImage(const QString &) ));
+            connect( m_viewer, SIGNAL( destroyed() ), SLOT( viewerDeleted() ));
+            connect( m_viewer, SIGNAL( sigFocusWindow( ImageWindow *) ),
+                     this, SLOT( slotSetActiveViewer( ImageWindow * ) ));
+            connect( m_viewer, SIGNAL( sigBadImage(const QString& ) ),
+                     this, SLOT( messageCantLoadImage(const QString &) ));
             connect( m_viewer, SIGNAL( requestImage( ImageWindow *, int )),
                      this, SLOT( slotAdvanceImage( ImageWindow *, int )));
-	    if ( s_viewers.count() == 1 && moveToTopLeft ) {
-		// we have to move to 0x0 before showing _and_
-		// after showing, otherwise we get some bogus geometry()
-		m_viewer->move( Kuick::workArea().topLeft() );
-	    }
+            if ( s_viewers.count() == 1 && moveToTopLeft ) {
+                // we have to move to 0x0 before showing _and_
+                // after showing, otherwise we get some bogus geometry()
+                m_viewer->move( Kuick::workArea().topLeft() );
+            }
 
-	    m_viewer->installEventFilter( this );
-	}
+            m_viewer->installEventFilter( this );
+        }
 
         // for some strange reason, m_viewer sometimes changes during the
         // next few lines of code, so as a workaround, we use safeViewer here.
@@ -467,46 +511,48 @@ void KuickShow::showImage( const KFileItem *fi,
         // Imlib can't load).
         ImageWindow *safeViewer = m_viewer;
 
-	QString filename;
-	KIO::NetAccess::download(fi->url(), filename);
+        QString filename;
+        KIO::NetAccess::download(fi->url(), filename, this);
 
-	if ( !safeViewer->showNextImage( filename ) ) {
+        if ( !safeViewer->showNextImage( filename ) ) {
             m_viewer = safeViewer;
-	    safeViewer->close( true ); // couldn't load image, close window
+            safeViewer->close( true ); // couldn't load image, close window
         }
-	else {
+        else {
             safeViewer->setFullscreen( fullscreen );
 
-	    if ( newWindow ) {
-		safeViewer->show();
-		
-		if ( !fullscreen && s_viewers.count() == 1 && moveToTopLeft ) {
-		    // the WM might have moved us after showing -> strike back!
-		    // move the first image to 0x0 workarea coord
-		    safeViewer->move( Kuick::workArea().topLeft() );
-		}
-	    }
+            if ( newWindow ) {
+                safeViewer->show();
 
-  	    if ( kdata->preloadImage && fileWidget ) {
-  		KFileItem *item = 0L;                 // don't move cursor
-  		item = fileWidget->getItem( FileWidget::Next, true );
-  		if ( item )
-  		    safeViewer->cacheImage( item->url().path() ); // FIXME
-  	    }
+                if ( !fullscreen && s_viewers.count() == 1 && moveToTopLeft ) {
+                    // the WM might have moved us after showing -> strike back!
+                    // move the first image to 0x0 workarea coord
+                    safeViewer->move( Kuick::workArea().topLeft() );
+                }
+            }
+
+            if ( kdata->preloadImage && fileWidget ) {
+                KFileItem *item = 0L;                 // don't move cursor
+                item = fileWidget->getItem( FileWidget::Next, true );
+                if ( item )
+                    safeViewer->cacheImage( item->url().path() ); // FIXME
+            }
 
             m_viewer = safeViewer;
-	} // m_viewer created successfully
+        } // m_viewer created successfully
     } // isImage
 }
 
 void KuickShow::startSlideShow()
 {
-    KFileItem *item = fileWidget->gotoFirstImage();
+    KFileItem *item = kdata->slideshowStartAtFirst ?
+                      fileWidget->gotoFirstImage() :
+                      fileWidget->getCurrentItem(false);
 
     if ( item ) {
         m_slideshowCycle = 1;
-	fileWidget->actionCollection()->action("kuick_slideshow")->setEnabled( false );
-	showImage( item, !oneWindowAction->isChecked(),
+        fileWidget->actionCollection()->action("kuick_slideshow")->setEnabled( false );
+        showImage( item, !oneWindowAction->isChecked(),
                    kdata->slideshowFullscreen );
         m_slideTimer->start( kdata->slideDelay );
     }
@@ -516,14 +562,14 @@ void KuickShow::nextSlide()
 {
     if ( !m_viewer ) {
         m_slideshowCycle = 1;
-	fileWidget->actionCollection()->action("kuick_slideshow")->setEnabled( true );
-	return;
+        fileWidget->actionCollection()->action("kuick_slideshow")->setEnabled( true );
+        return;
     }
 
     KFileItem *item = fileWidget->getNext( true );
     if ( !item ) { // last image
         if ( m_slideshowCycle < kdata->slideshowCycles
-           || kdata->slideshowCycles == 0 ) {
+             || kdata->slideshowCycles == 0 ) {
             item = fileWidget->gotoFirstImage();
             if ( item ) {
                 nextSlide( item );
@@ -532,9 +578,9 @@ void KuickShow::nextSlide()
             }
         }
 
-	m_viewer->close( true );
-	fileWidget->actionCollection()->action("kuick_slideshow")->setEnabled( true );
-	return;
+        m_viewer->close( true );
+        fileWidget->actionCollection()->action("kuick_slideshow")->setEnabled( true );
+        return;
     }
 
     nextSlide( item );
@@ -552,7 +598,7 @@ void KuickShow::slotPrint()
 {
     const KFileItemList *items = fileWidget->selectedItems();
     if ( !items )
-	return;
+        return;
 
     KFileItemListIterator it( *items );
 
@@ -560,9 +606,9 @@ void KuickShow::slotPrint()
     ImageWindow *iw = new ImageWindow( 0, id, this, "printing image" );
     KFileItem *item;
     while ( (item = it.current()) ) {
-	if (FileWidget::isImage( item ) && iw->loadImage( item->url().path()))
-	    iw->printImage();
-	++it;
+        if (FileWidget::isImage( item ) && iw->loadImage( item->url().path()))
+            iw->printImage();
+        ++it;
     }
 
     iw->close( true );
@@ -578,33 +624,16 @@ void KuickShow::slotShowInSameWindow()
     showImage( fileWidget->getCurrentItem( false ), false );
 }
 
-void KuickShow::dropEvent( QDropEvent *e )
+void KuickShow::slotDropped( const KFileItem *, QDropEvent *, const KURL::List &urls)
 {
-    KURL dir; // in case we get a directory dropped
-    KURL::List urls;
-    KURLDrag::decode( e, urls );
-    bool hasRemote = false;
-
-    KURL::List::Iterator it = urls.begin();
-    while ( it != urls.end() ) {
-	KURL u = *it;
-	if ( u.isLocalFile() ) {
-	    if ( !u.fileName().isEmpty() )
-		dir = u;
-	    else {
-		KFileItem item( KFileItem::Unknown, KFileItem::Unknown, u, false );
-		showImage( &item, true );
-	    }
-	}
-	else
-	    hasRemote = true;
-    }
-
-
-    if ( hasRemote ) {
-	QString tmp( i18n("You can only drop local files "
-                          "onto the image viewer!"));
-	KMessageBox::sorry( this, tmp, i18n("KuickShow Drop Error") );
+    KURL::List::ConstIterator it = urls.begin();
+    for ( ; it != urls.end(); ++it )
+    {
+        KFileItem item( KFileItem::Unknown, KFileItem::Unknown, *it );
+        if ( FileWidget::isImage( &item ) )
+            showImage( &item, true );
+        else
+            fileWidget->setURL( *it, true );
     }
 }
 
@@ -662,11 +691,11 @@ void KuickShow::slotAdvanceImage( ImageWindow *view, int steps )
     }
 
     if ( FileWidget::isImage( item ) ) {
-	QString filename;
-	KIO::NetAccess::download(item->url(), filename);
+        QString filename;
+        KIO::NetAccess::download(item->url(), filename, this);
         view->showNextImage( filename );
         if (m_slideTimer->isActive())
-	  m_slideTimer->start( kdata->slideDelay );
+            m_slideTimer->start( kdata->slideDelay );
 
         if ( kdata->preloadImage && item_next && item_next->url().isLocalFile() ) // preload next image
             if ( FileWidget::isImage( item_next ) )
@@ -677,24 +706,24 @@ void KuickShow::slotAdvanceImage( ImageWindow *view, int steps )
 bool KuickShow::eventFilter( QObject *o, QEvent *e )
 {
     if ( m_delayedRepeatItem ) // we probably need to install an eventFilter over
-	return true;    // kapp, to make it really safe
+        return true;    // kapp, to make it really safe
 
     bool ret = false;
     int eventType = e->type();
     QKeyEvent *k = 0L;
     if ( eventType == QEvent::KeyPress )
-	k = static_cast<QKeyEvent *>( e );
+        k = static_cast<QKeyEvent *>( e );
 
     if ( k ) {
         if ( KStdAccel::quit().contains( KKey( k ) ) ) {
-	    saveSettings();
+            saveSettings();
             deleteAllViewers();
-	    ::exit(0);
+            ::exit(0);
         }
         else if ( KStdAccel::help().contains( KKey( k ) ) ) {
-	    appHelpActivated();
-	    return true;
-	}
+            appHelpActivated();
+            return true;
+        }
     }
 
 
@@ -707,18 +736,18 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
         if ( eventType != QEvent::Leave && eventType != QEvent::Enter )
             KCursor::autoHideEventFilter( o, e );
 
-	m_viewer = window;
-	QString img;
-	KFileItem *item = 0L;      // the image to be shown
-	KFileItem *item_next = 0L; // the image to be cached
+        m_viewer = window;
+        QString img;
+        KFileItem *item = 0L;      // the image to be shown
+        KFileItem *item_next = 0L; // the image to be cached
 
-	if ( k ) { // keypress
-	    ret = true;
-	    int key = k->key();
+        if ( k ) { // keypress
+            ret = true;
+            int key = k->key();
 
-	    // Key_Shift shouldn't load the browser in nobrowser mode, it
-	    // is used for zooming in the imagewindow
-	    if ( !fileWidget )
+            // Key_Shift shouldn't load the browser in nobrowser mode, it
+            // is used for zooming in the imagewindow
+            if ( !fileWidget )
             {
                 if ( key != Key_Escape && key != Key_Shift )
                 {
@@ -762,30 +791,30 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
                 }
 
                 return KMainWindow::eventFilter( o, e );
-	    }
+            }
 
             // we definitely have a fileWidget here!
 
             KKey kkey( k );
             if ( key == Key_Home || KStdAccel::home().contains( kkey ) )
             {
-		item = fileWidget->gotoFirstImage();
-		item_next = fileWidget->getNext( false );
-	    }
+                item = fileWidget->gotoFirstImage();
+                item_next = fileWidget->getNext( false );
+            }
 
             else if ( key == Key_End || KStdAccel::end().contains( kkey ) )
             {
-		item = fileWidget->gotoLastImage();
-		item_next = fileWidget->getPrevious( false );
-	    }
+                item = fileWidget->gotoLastImage();
+                item_next = fileWidget->getPrevious( false );
+            }
 
             else if ( fileWidget->actionCollection()->action("delete")->shortcut().contains( key ))
             {
-// 		KFileItem *cur = fileWidget->getCurrentItem( false );
- 		(void) fileWidget->getCurrentItem( false );
-		item = fileWidget->getNext( false ); // don't move
-		if ( !item )
-		    item = fileWidget->getPrevious( false );
+//      KFileItem *cur = fileWidget->getCurrentItem( false );
+                (void) fileWidget->getCurrentItem( false );
+                item = fileWidget->getNext( false ); // don't move
+                if ( !item )
+                    item = fileWidget->getPrevious( false );
                 KFileItem it( KFileItem::Unknown, KFileItem::Unknown,
                               m_viewer->url() );
                 KFileItemList list;
@@ -796,30 +825,30 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
 
                 // ### check failure asynchronously and restore old item?
                 fileWidget->setCurrentItem( item );
-	    }
+            }
 
-	    else if ( m_toggleBrowserAction->shortcut().contains( key ) )
+            else if ( m_toggleBrowserAction->shortcut().contains( key ) )
             {
                 toggleBrowser();
-		return true; // don't pass keyEvent
-	    }
+                return true; // don't pass keyEvent
+            }
 
             else
-		ret = false;
+                ret = false;
 
 
-	    if ( FileWidget::isImage( item ) ) {
+            if ( FileWidget::isImage( item ) ) {
                 QString filename;
-                KIO::NetAccess::download(item->url(), filename);
-		m_viewer->showNextImage( filename );
-		
-		if ( kdata->preloadImage && item_next && item_next->url().isLocalFile() ) // preload next image
-		    if ( FileWidget::isImage( item_next ) )
-			m_viewer->cacheImage( item_next->url().path() ); // ###
+                KIO::NetAccess::download(item->url(), filename, this);
+                m_viewer->showNextImage( filename );
 
-		ret = true; // don't pass keyEvent
-	    }
-	} // keyPressEvent on ImageWindow
+                if ( kdata->preloadImage && item_next && item_next->url().isLocalFile() ) // preload next image
+                    if ( FileWidget::isImage( item_next ) )
+                        m_viewer->cacheImage( item_next->url().path() ); // ###
+
+                ret = true; // don't pass keyEvent
+            }
+        } // keyPressEvent on ImageWindow
 
 
         // doubleclick closes image window
@@ -853,7 +882,7 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
 
 
     if ( ret )
-	return true;
+        return true;
 
     return KMainWindow::eventFilter( o, e );
 }
@@ -862,9 +891,9 @@ bool KuickShow::eventFilter( QObject *o, QEvent *e )
 void KuickShow::configuration()
 {
     if ( !m_accel ) {
-	KURL start;
-	start.setPath( QDir::homeDirPath() );
-	initGUI( QDir::homeDirPath() );
+        KURL start;
+        start.setPath( QDir::homeDirPath() );
+        initGUI( QDir::homeDirPath() );
     }
 
     dialog = new KuickConfigDialog( fileWidget->actionCollection(), 0L,
@@ -873,11 +902,11 @@ void KuickShow::configuration()
     dialog->setIcon( kapp->miniIcon() );
 
     connect( dialog, SIGNAL( okClicked() ),
-	     this, SLOT( slotConfigApplied() ) );
+             this, SLOT( slotConfigApplied() ) );
     connect( dialog, SIGNAL( applyClicked() ),
-	     this, SLOT( slotConfigApplied() ) );
+             this, SLOT( slotConfigApplied() ) );
     connect( dialog, SIGNAL( finished() ),
-	     this, SLOT( slotConfigClosed() ) );
+             this, SLOT( slotConfigClosed() ) );
 
     fileWidget->actionCollection()->action( "kuick_configure" )->setEnabled( false );
     dialog->show();
@@ -895,8 +924,8 @@ void KuickShow::slotConfigApplied()
     QValueListIterator<ImageWindow*> it = s_viewers.begin();
     while ( it != s_viewers.end() ) {
         viewer = *it;
-	viewer->updateActions();
-	++it;
+        viewer->updateActions();
+        ++it;
     }
 
     fileWidget->reloadConfiguration();
@@ -915,47 +944,60 @@ void KuickShow::about()
         aboutWidget = new AboutWidget( 0L, "about" );
 
     aboutWidget->adjustSize();
+
+#if KDE_VERSION >= 310
     KDialog::centerOnScreen( aboutWidget );
+#else
+// Not fixed because it must be dead code now.
+    QDesktopWidget *desktop = QApplication::desktop();
+    int screen = desktop->screenNumber( aboutWidget );
+    if ( screen == -1 )
+        screen = desktop->primaryScreen();
+
+    QRect r = desktop->screenGeometry( screen );
+    aboutWidget->move( r.center().x() - aboutWidget->width()/2,
+                       r.center().y() - aboutWidget->height()/2 );
+#endif
+
     aboutWidget->show();
 }
 
-
 // ------ sessionmanagement - load / save current directory -----
 void KuickShow::readProperties( KConfig *kc )
 {
     assert( fileWidget ); // from SM, we should always have initGUI on startup
-    QString dir = kc->readEntry( "CurrentDirectory" );
+    QString dir = kc->readPathEntry( "CurrentDirectory" );
     if ( !dir.isEmpty() ) {
-	fileWidget->setURL( dir, true );
-	fileWidget->clearHistory();
+        fileWidget->setURL( dir, true );
+        fileWidget->clearHistory();
     }
 
-    QStringList images = kc->readListEntry( "Images shown" );
+    QStringList images = kc->readPathListEntry( "Images shown" );
     QStringList::Iterator it;
     for ( it = images.begin(); it != images.end(); ++it ) {
-	KFileItem item( KFileItem::Unknown, KFileItem::Unknown, *it, false );
-	if ( item.isReadable() )
-	    showImage( &item, true );
+        KFileItem item( KFileItem::Unknown, KFileItem::Unknown, *it, false );
+        if ( item.isReadable() )
+            showImage( &item, true );
     }
 
     if ( !s_viewers.isEmpty() ) {
-	bool visible = kc->readBoolEntry( "Browser visible", true );
-	if ( !visible )
-	    hide();
+        bool visible = kc->readBoolEntry( "Browser visible", true );
+        if ( !visible )
+            hide();
     }
 }
 
 void KuickShow::saveProperties( KConfig *kc )
 {
-    kc->writeEntry( "CurrentDirectory", fileWidget->url().url() );
+    kc->writePathEntry( "CurrentDirectory", fileWidget->url().url() );
     kc->writeEntry( "Browser visible", fileWidget->isVisible() );
 
     QStringList urls;
     QValueListIterator<ImageWindow*> it;
     for ( it = s_viewers.begin(); it != s_viewers.end(); ++it )
-	urls.append( (*it)->filename() );
-	
-    kc->writeEntry( "Images shown", urls );
+        urls.append( (*it)->filename() );
+
+    kc->writePathEntry( "Images shown", urls );
 }
 
 // --------------------------------------------------------------
@@ -965,11 +1007,13 @@ void KuickShow::saveSettings()
     KConfig *kc = KGlobal::config();
 
     kc->setGroup("SessionSettings");
-    kc->writeEntry( "OpenImagesInActiveWindow", oneWindowAction->isChecked() );
-    kc->writeEntry( "CurrentDirectory", fileWidget->url().url() );
+    if ( oneWindowAction )
+        kc->writeEntry( "OpenImagesInActiveWindow", oneWindowAction->isChecked() );
 
-    if ( fileWidget )
-	fileWidget->writeConfig( kc, "Filebrowser" );
+    if ( fileWidget ) {
+        kc->writePathEntry( "CurrentDirectory", fileWidget->url().url() );
+        fileWidget->writeConfig( kc, "Filebrowser" );
+    }
 
     kc->sync();
 }
@@ -979,8 +1023,8 @@ void KuickShow::messageCantLoadImage( const QString& filename )
 {
     m_viewer->clearFocus();
     QString tmp = i18n("Unable to load the image %1.\n"
-	    "Perhaps the file format is unsupported or "
-                      "your Imlib is not installed properly.").arg(filename);
+                       "Perhaps the file format is unsupported or "
+                       "your Imlib is not installed properly.").arg(filename);
     KMessageBox::sorry( m_viewer, tmp, i18n("Image Error") );
 }
 
@@ -992,28 +1036,28 @@ void KuickShow::initImlib()
 
     id = Imlib_init_with_params( x11Display(), &par );
     if ( !id ) {
-	initImlibParams( idata, &par );
+        initImlibParams( idata, &par );
 
-	qWarning("*** KuickShow: Whoops, can't initialize imlib, trying my own palettefile now.");
-	QString paletteFile = locate( "data", "kuickshow/im_palette.pal" );
-	// FIXME - does the qstrdup() cure the segfault in imlib eventually?
-	char *file = qstrdup( paletteFile.local8Bit() );
-	par.palettefile = file;
-	par.flags |= PARAMS_PALETTEFILE;
+        qWarning("*** KuickShow: Whoops, can't initialize imlib, trying my own palettefile now.");
+        QString paletteFile = locate( "data", "kuickshow/im_palette.pal" );
+        // FIXME - does the qstrdup() cure the segfault in imlib eventually?
+        char *file = qstrdup( paletteFile.local8Bit() );
+        par.palettefile = file;
+        par.flags |= PARAMS_PALETTEFILE;
 
-	qWarning("Palettefile: %s", par.palettefile );
+        qWarning("Palettefile: %s", par.palettefile );
 
-	id = Imlib_init_with_params( x11Display(), &par );
+        id = Imlib_init_with_params( x11Display(), &par );
 
-	if ( !id ) {
-	    QString tmp = i18n("Can't initialize \"Imlib\".\n"
-		    "Start kuickshow on the command line "
-		    "and look for error messages.\n"
-		    "I will quit now.");
-	    KMessageBox::error( this, tmp, i18n("Fatal Imlib error") );
+        if ( !id ) {
+            QString tmp = i18n("Unable to initialize \"Imlib\".\n"
+                               "Start kuickshow from the command line "
+                               "and look for error messages.\n"
+                               "The program will now quit.");
+            KMessageBox::error( this, tmp, i18n("Fatal Imlib Error") );
 
-	    ::exit(1);
-	}
+            ::exit(1);
+        }
     }
 }
 
@@ -1021,8 +1065,8 @@ void KuickShow::initImlib()
 void KuickShow::initImlibParams( ImData *idata, ImlibInitParams *par )
 {
     par->flags = ( PARAMS_REMAP |
-		   PARAMS_FASTRENDER | PARAMS_HIQUALITY | PARAMS_DITHER |
-		   PARAMS_IMAGECACHESIZE | PARAMS_PIXMAPCACHESIZE );
+                   PARAMS_FASTRENDER | PARAMS_HIQUALITY | PARAMS_DITHER |
+                   PARAMS_IMAGECACHESIZE | PARAMS_PIXMAPCACHESIZE );
 
     par->paletteoverride = idata->ownPalette  ? 1 : 0;
     par->remap           = idata->fastRemap   ? 1 : 0;
@@ -1049,7 +1093,7 @@ void KuickShow::delayedRepeatEvent( ImageWindow *w, QKeyEvent *e )
 void KuickShow::slotReplayEvent()
 {
     disconnect( fileWidget, SIGNAL( finished() ),
-		this, SLOT( slotReplayEvent() ));
+                this, SLOT( slotReplayEvent() ));
 
     DelayedRepeatEvent *e = m_delayedRepeatItem;
     m_delayedRepeatItem = 0L; // otherwise, eventFilter aborts
@@ -1101,7 +1145,7 @@ void KuickShow::toggleBrowser()
         fileWidget->resize( size() ); // ### somehow fileWidget isn't resized!?
         show();
         raise();
-        KWin::setActiveWindow( winId() ); // ### this should not be necessary
+        KWin::activateWindow( winId() ); // ### this should not be necessary
 //         setFocus();
     }
     else if ( !s_viewers.isEmpty() )
@@ -1112,7 +1156,7 @@ void KuickShow::slotOpenURL()
 {
     KFileDialog dlg(QString::null, kdata->fileFilter, this, "filedialog", true);
     dlg.setMode( KFile::Files | KFile::Directory );
-    dlg.setCaption( i18n("Select Files or Directory to Open") );
+    dlg.setCaption( i18n("Select Files or Folder to Open") );
 
     if ( dlg.exec() == QDialog::Accepted )
     {
@@ -1133,7 +1177,7 @@ void KuickShow::deleteAllViewers()
 {
     QValueListIterator<ImageWindow*> it = s_viewers.begin();
     for ( ; it != s_viewers.end(); ++it ) {
-        (*it)->blockSignals( true ); // don't call viewerDeleted()
+        (*it)->disconnect( SIGNAL( destroyed() ), this, SLOT( viewerDeleted() ));
         (*it)->close( true );
     }
 
@@ -1141,4 +1185,12 @@ void KuickShow::deleteAllViewers()
     m_viewer = 0L;
 }
 
+KActionCollection * KuickShow::actionCollection() const
+{
+    if ( fileWidget )
+        return fileWidget->actionCollection();
+
+    return KMainWindow::actionCollection();
+}
+
 #include "kuickshow.moc"
