@@ -1,5 +1,5 @@
 /*  This file is part of the KDE project
-    Copyright (C) 2001-2002 Matthias Kretz <kretz@kde.org>
+    Copyright (C) 2001-2003 Matthias Kretz <kretz@kde.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -12,17 +12,16 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
-// $Id: kview.cpp,v 1.58.2.2 2003/02/11 13:29:34 mkretz Exp $
 
 #include "kview.h"
 #include "kimageviewer/viewer.h"
 #include "kimageviewer/canvas.h"
-#include "kviewconfmodules.h"
 
-#include "kpreferencesdialog.h"
+#include <ksettings/dialog.h>
+#include <ksettings/dispatcher.h>
 #include <kurl.h>
 #include <klibloader.h>
 #include <kmessagebox.h>
@@ -39,7 +38,6 @@
 #include <kmenubar.h>
 #include <kstatusbar.h>
 #include <kdebug.h>
-#include <kdebugclasses.h>
 #include <kedittoolbar.h>
 #include <kkeydialog.h>
 #include <kstdaccel.h>
@@ -56,15 +54,15 @@
 #include <qdockarea.h>
 
 #include <assert.h>
+#include <kplugininfo.h>
 
 KView::KView()
     : KParts::MainWindow( 0, "KView" )
 	, m_pViewer( 0 )
 	, m_pCanvas( 0 )
-	, m_pWinModule( new KWinModule( this ) )
-	, m_pPreferencesDialog( new KPreferencesDialog( this ) )
-	, m_bFullScreen( false )
+	, m_pWinModule( new KWinModule( this, KWinModule::INFO_DESKTOP ) )
 	, m_bImageSizeChangedBlocked( false )
+	, m_bFullscreen( false )
 {
 	KParts::ReadWritePart * part = KParts::ComponentFactory::createPartInstanceFromLibrary<KParts::ReadWritePart>(
 			"libkviewviewer", this, "KViewViewer Widget", this, "KImageViewer Part" );
@@ -106,29 +104,36 @@ KView::KView()
 		connect( m_pViewer, SIGNAL( imageOpened( const KURL & ) ),
 				m_paRecent, SLOT( addURL( const KURL & ) ) );
 
+		connect( m_pCanvas->widget(), SIGNAL( cursorPos( const QPoint & ) ), SLOT( cursorPos( const QPoint & ) ) );
+
 		m_paRecent->loadEntries( KGlobal::config() );
 		readSettings();
+		m_pViewer->widget()->installEventFilter( this );
 
 		// reload configuration when it's changed by the conf dlg
-		// create conf dlg modules
-		connect( new KViewGeneralConfig( this ), SIGNAL( configChanged() ), SLOT( readSettings() ) );
-		connect( new KViewPluginConfig( this ) , SIGNAL( configChanged() ), SLOT( loadPlugins() ) );
+		KSettings::Dispatcher::self()->registerInstance( instance(), this, SLOT( readSettings() ) );
 
 		setPluginLoadingMode( LoadPluginsIfEnabled );
 		createGUI( part );
 
 		// create status bar (hidden by default)
-		statusBar()->insertItem( "", STATUSBAR_SPEED_ID );
-		statusBar()->setItemFixed( STATUSBAR_SPEED_ID, 8 + fontMetrics().width( i18n( "%1/s" ).arg( KIO::convertSize( 999000 ) ) ) );
+		statusBar()->insertItem( "", STATUSBAR_SPEED_ID, 0, true );
+		statusBar()->setItemFixed( STATUSBAR_SPEED_ID,
+				8 + fontMetrics().width( i18n( "%1/s" ).arg( KIO::convertSize( 999000 ) ) ) );
+		statusBar()->insertItem( "", STATUSBAR_CURSOR_ID, 0, true );
+		statusBar()->setItemFixed( STATUSBAR_CURSOR_ID, 8 + fontMetrics().width( "8888, 8888" ) );
+		statusBar()->insertItem( "", STATUSBAR_SIZE_ID, 0, true );
+		statusBar()->setItemFixed( STATUSBAR_SIZE_ID, 8 + fontMetrics().width( "8888 x 8888" ) );
+
+		statusBar()->insertItem( QString::null, STATUSBAR_SELECTION_ID );
+
 		m_pProgressBar = new KProgress( statusBar() );
 		m_pProgressBar->setFixedSize( 140, fontMetrics().height() );
 		statusBar()->addWidget( m_pProgressBar, 0, true );
 		m_pProgressBar->hide();
 
 		setAutoSaveSettings();
-
 		m_paShowMenubar->setChecked( ! menuBar()->isHidden() );
-		m_paShowStatusbar->setChecked( ! statusBar()->isHidden() );
 
 		// show progress info dialog if the statusbar is hidden
 		m_pViewer->setProgressInfoEnabled( statusBar()->isHidden() );
@@ -152,7 +157,40 @@ KView::~KView()
 void KView::load( const KURL & url )
 {
 	if( m_pViewer )
+	{
 		m_pViewer->openURL( url );
+		if( url.isLocalFile() )
+		{
+			// XXX: this code is what
+			//KRecentDirs::add( QString::fromLatin1( ":load_image" ), url.directory() );
+			// would do:
+			QString directory = url.directory();
+			QString key = QString::fromLatin1( "load_image" );
+			KConfig * config = KGlobal::config();
+
+			config->setGroup( QString::fromLatin1( "Recent Dirs" ) );
+			QStringList result = config->readPathListEntry( key );
+			// make sure the dir is first in history
+			result.remove( directory );
+			result.prepend( directory );
+			while( result.count() > 3 )
+				result.remove( result.fromLast() );
+			config->writePathEntry( key, result );
+			config->sync();
+		}
+	}
+}
+
+void KView::loadFromStdin()
+{
+	if( m_pViewer )
+	{
+		QFile file;
+		file.open( IO_ReadOnly, stdin );
+		QImage image( file.readAll() );
+		file.close();
+		m_pViewer->newImage( image );
+	}
 }
 
 QSize KView::sizeForCentralWidgetSize( QSize size )
@@ -206,17 +244,24 @@ void KView::readSettings() // KConfig * config )
 	kdDebug( 4600 ) << k_funcinfo << endl;
 	KConfigGroup cfgGroup( KGlobal::config(), "KView General" );
 	m_nResizeMode = cfgGroup.readNumEntry( "Resize Mode", 2 );
+	kdDebug( 4600 ) << "m_nResizeMode = " << m_nResizeMode << endl;
+	loadPlugins();
 }
 
-void KView::resizeEvent( QResizeEvent * ev )
+bool KView::eventFilter( QObject * obj, QEvent * ev )
 {
-	KParts::MainWindow::resizeEvent( ev );
-	if( m_nResizeMode == ResizeImage )
-		handleResize();
+	if( obj == m_pViewer->widget() && ev->type() == QEvent::Resize )
+	{
+		if( m_nResizeMode == ResizeImage )
+			handleResize();
+	}
+	return KParts::MainWindow::eventFilter( obj, ev );
 }
 
 void KView::imageSizeChanged( const QSize & /*size*/ )
 {
+	QSize size = m_pCanvas->imageSize();
+	statusBar()->changeItem( QString( "%1 x %2" ).arg( size.width() ).arg( size.height() ), STATUSBAR_SIZE_ID );
 	if( m_bImageSizeChangedBlocked )
 		return;
 
@@ -225,6 +270,11 @@ void KView::imageSizeChanged( const QSize & /*size*/ )
 
 void KView::selectionChanged( const QRect & rect )
 {
+	kdDebug( 4600 ) << k_funcinfo << rect << endl;
+	if( rect.isNull() )
+		statusBar()->changeItem( QString::null, STATUSBAR_SELECTION_ID );
+	else
+		statusBar()->changeItem( QString( "%1, %2 - %3 x %4" ).arg( rect.x() ).arg( rect.y() ).arg( rect.width() ).arg( rect.height() ), STATUSBAR_SELECTION_ID );
 	action( "crop" )->setEnabled( ! rect.isNull() );
 }
 
@@ -293,26 +343,23 @@ void KView::slotCrop()
 	m_pViewer->setModified( true );
 }
 
-void KView::slotToggleFullScreen()
+void KView::slotUpdateFullScreen( bool set )
 {
-	m_bFullScreen = ! m_bFullScreen;
-	if( m_bFullScreen )
+	m_bFullscreen = set;
+	if( set )
 	{ // switch to FullScreen mode
 		saveMainWindowSettings( KGlobal::config(), "nonFullScreen MainWindow" );
 		showFullScreen();
 		applyMainWindowSettings( KGlobal::config(), "FullScreen MainWindow" );
-		m_paShowFullScreen->setIcon( "window_nofullscreen" );
 		m_paShowMenubar->setChecked( ! menuBar()->isHidden() );
-		m_paShowStatusbar->setChecked( ! statusBar()->isHidden() );
 	}
 	else
 	{ // leave FullScreen mode
 		saveMainWindowSettings( KGlobal::config(), "FullScreen MainWindow" );
 		showNormal();
 		applyMainWindowSettings( KGlobal::config(), "nonFullScreen MainWindow" );
-		m_paShowFullScreen->setIcon( "window_fullscreen" );
 		m_paShowMenubar->setChecked( ! menuBar()->isHidden() );
-		m_paShowStatusbar->setChecked( ! statusBar()->isHidden() );
+		handleResize();
 	}
 }
 
@@ -325,24 +372,17 @@ void KView::slotToggleMenubar()
 	handleResize();
 }
 
-void KView::slotToggleStatusbar()
-{
-	if( statusBar()->isVisible() )
-	{
-		statusBar()->hide();
-		m_pViewer->setProgressInfoEnabled( true );
-	}
-	else
-	{
-		statusBar()->show();
-		m_pViewer->setProgressInfoEnabled( false );
-	}
-	handleResize();
-}
-
 void KView::slotPreferences()
 {
-	m_pPreferencesDialog->show();
+	// construct KCD for KView app
+	static KSettings::Dialog * dlg = 0;
+	if( ! dlg )
+	{
+		dlg = new KSettings::Dialog( this );
+		//dlg = new KConfigureDialog( KConfigureDialog::Configurable, this );
+		//dlg->addPluginInfos( KPluginInfo::fromKPartsInstanceName( instance()->instanceName(), KGlobal::config(), "KParts Plugins" ) );
+	}
+	dlg->show();
 }
 
 void KView::slotKeyBindings()
@@ -448,6 +488,11 @@ void KView::slotSetStatusBarText( const QString & msg )
 		KMessageBox::information( this, msg );
 }
 
+void KView::cursorPos( const QPoint & pos )
+{
+	statusBar()->changeItem( QString( "%1, %2" ).arg( pos.x() ).arg( pos.y() ), STATUSBAR_CURSOR_ID );
+}
+
 void KView::setupActions( QObject * partobject )
 {
 	// File
@@ -479,15 +524,19 @@ void KView::setupActions( QObject * partobject )
 	KAction * aCrop = new KAction( i18n( "Cr&op" ), Key_C, this, SLOT( slotCrop() ), actionCollection(), "crop" );
 	aCrop->setEnabled( false );
 
-	KAction * aReload = new KAction( i18n( "&Reload" ), "reload", KStdAccel::key( KStdAccel::Reload ), partobject,
+	KAction * aReload = new KAction( i18n( "&Reload" ), "reload", KStdAccel::shortcut( KStdAccel::Reload ), partobject,
 			SLOT( reload() ), actionCollection(), "reload" );
 	aReload->setEnabled( false );
 	connect( m_pViewer->widget(), SIGNAL( hasImage( bool ) ), aReload, SLOT( setEnabled( bool ) ) );
 
 	// Settings
 	m_paShowMenubar = KStdAction::showMenubar( this, SLOT( slotToggleMenubar() ), actionCollection() );
-	m_paShowStatusbar = KStdAction::showStatusbar( this, SLOT( slotToggleStatusbar() ), actionCollection() );
-	m_paShowFullScreen = new KToggleAction( i18n( "Show FullScreen" ), "window_fullscreen", CTRL+SHIFT+Key_F, this, SLOT( slotToggleFullScreen() ), actionCollection(), "show_fullscreen" );
+	createStandardStatusBarAction();
+	m_paShowStatusBar = ::qt_cast<KToggleAction*>( action( "options_show_statusbar" ) );
+	if( m_paShowStatusBar )
+		connect( m_paShowStatusBar, SIGNAL( toggled( bool ) ), SLOT( statusbarToggled( bool ) ) );
+	m_paShowFullScreen = KStdAction::fullScreen( 0, 0, actionCollection(), this );
+        connect( m_paShowFullScreen, SIGNAL( toggled( bool )), this, SLOT( slotUpdateFullScreen( bool )));
 	KStdAction::preferences( this, SLOT( slotPreferences() ), actionCollection() );
 	KStdAction::keyBindings( this, SLOT( slotKeyBindings() ), actionCollection() );
 	KStdAction::configureToolbars( this, SLOT( slotConfigureToolbars() ), actionCollection() );
@@ -506,9 +555,7 @@ void KView::handleResize()
 			break;
 		case ResizeImage:
 			m_bImageSizeChangedBlocked = true;
-			m_pCanvas->setMaximumImageSize( m_pViewer->widget()->size() );
-			m_pCanvas->setZoom( 10000.0 );
-			m_pCanvas->setMaximumImageSize( QSize( 0, 0 ) );
+			m_pCanvas->boundImageTo( m_pViewer->widget()->size() );
 			m_bImageSizeChangedBlocked = false;
 			break;
 	}
@@ -516,6 +563,9 @@ void KView::handleResize()
 
 void KView::fitWindowToImage()
 {
+	if( m_bFullscreen ) // don't do anything in fullscreen mode
+		return;
+
 	bool centeredOrig = m_pCanvas->centered();
 	m_pCanvas->setCentered( false );
 
@@ -617,6 +667,12 @@ void KView::loadPlugins()
 	createGUI( m_pViewer );
 }
 
+void KView::statusbarToggled( bool sbvisible )
+{
+	kdDebug( 4600 ) << k_funcinfo << sbvisible << endl;
+	m_pViewer->setProgressInfoEnabled( !sbvisible );
+}
+
 // vim:sw=4:ts=4
 
 #include "kview.moc"
