@@ -1,7 +1,7 @@
 /*  This file is part of the KDE project
 
     Copyright (C) 1999 Simon Hausmann <hausmann@kde.org>
-                  2001-2002 Matthias Kretz <kretz@kde.org>
+                  2001-2003 Matthias Kretz <kretz@kde.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,51 +15,41 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
 #include "kviewviewer.h"
-#include "kviewkonqextension.h"
 #include "kimageviewer/canvas.h"
-#include "confmodules.h"
 #include "version.h"
-#include "kpreferencesdialog.h"
+
+#include <ksettings/dispatcher.h>
 
 #include <unistd.h>
 #include <assert.h>
 
-#include <qimage.h>
-#include <qpainter.h>
 #include <qbuffer.h>
-#include <qcursor.h>
-#include <qdir.h>
-#include <qhbox.h>
+#include <qlayout.h>
 #include <qvbox.h>
 #include <qlabel.h>
 #include <qregexp.h>
-#include <qdragobject.h>
 
 #include <kpushbutton.h>
 #include <kpassivepopup.h>
-#include <kaction.h>
-#include <kstdaction.h>
-#include <klocale.h>
 #include <kdebug.h>
 #include <kio/job.h>
-#include <kinstance.h>
 #include <kxmlguifactory.h>
 #include <kfiledialog.h>
 #include <kparts/genericfactory.h>
 #include <kparts/componentfactory.h>
+#include <kparts/plugin.h>
 #include <kmessagebox.h>
 #include <ktempfile.h>
-#include <kdirlister.h>
 #include <kapplication.h>
 #include <kimageio.h>
 #include <kstdaccel.h>
 #include <kdirwatch.h>
-#include <kstdguiitem.h>
+#include <kurldrag.h>
 #include <krecentdocument.h>
 
 typedef KParts::GenericFactory<KViewViewer> KViewViewerFactory;
@@ -85,9 +75,9 @@ KViewViewer::KViewViewer( QWidget *parentWidget, const char * /*widgetName*/,
 	if( ! ( widget && m_pCanvas ) )
 	{
 		if( ! widget )
-			KMessageBox::error( m_pParentWidget, i18n( "Could not find a suitable Image Canvas! This probably means, that you didn't install KView properly." ) );
+			KMessageBox::error( m_pParentWidget, i18n( "Unable to find a suitable Image Canvas. This probably means that KView was not installed properly." ) );
 		else
-			KMessageBox::error( m_pParentWidget, i18n( "Accessing the KImageViewer interface of the Image Canvas failed. Something in your setup is broken (a component claims to be a KImageViewer::Canvas but it's not)." ) );
+			KMessageBox::error( m_pParentWidget, i18n( "Accessing the KImageViewer interface of the Image Canvas failed. Something in your setup is broken (a component claims to be a KImageViewer::Canvas but it is not)." ) );
 	}
 	else
 	{
@@ -106,7 +96,7 @@ KViewViewer::KViewViewer( QWidget *parentWidget, const char * /*widgetName*/,
 		// Drag and Drop for the Canvas
 		widget->setAcceptDrops( true );
 		widget->installEventFilter( this );
-		
+
 		setupActions();
 
 		if( isReadWrite() )
@@ -128,16 +118,13 @@ KViewViewer::KViewViewer( QWidget *parentWidget, const char * /*widgetName*/,
 		connect( m_pFileWatch, SIGNAL( dirty( const QString & ) ),
 				this, SLOT( slotFileDirty( const QString & ) ) );
 
+		KSettings::Dispatcher::self()->registerInstance( instance(), this, SLOT( readSettings() ) );
+
 		// by default disable progress info (so it won't open the dialog in Konqueror)
 		setProgressInfoEnabled( false );
 
 		m_popupDoc = KXMLGUIFactory::readConfigFile( "kviewpopup.rc", true, instance() );
 
-		// create preferences modules
-		connect( new GeneralConfig( m_pCanvas, instance(), this ), SIGNAL( configChanged() ), SLOT( readSettings() ) );
-		connect( new PluginConfig( instance(), this )            , SIGNAL( configChanged() ), SLOT( loadPlugins() ) );
-		//connect( KPreferencesDialog::self(), SIGNAL( reloadConfig() ), SLOT( reloadConfig() ) );
-
 		KConfigGroup cfgGroup( instance()->config(), "Settings" );
 		bool hideBars = cfgGroup.readBoolEntry( "hideScrollbars", false );
 		m_pCanvas->hideScrollbars( hideBars );
@@ -151,6 +138,7 @@ KViewViewer::KViewViewer( QWidget *parentWidget, const char * /*widgetName*/,
 KViewViewer::~KViewViewer()
 {
 	kdDebug( 4610 ) << k_funcinfo << endl;
+
 	writeSettings();
 	instance()->config()->sync();
 
@@ -203,27 +191,27 @@ void KViewViewer::setReadWrite( bool readwrite )
 bool KViewViewer::saveAs( const KURL & kurl )
 {
 	kdDebug( 4610 ) << k_funcinfo << endl;
-	if( kurl.isMalformed() )
+	if( !kurl.isValid() )
 		return KParts::ReadWritePart::saveAs( kurl ); // sets m_bClosing = false
 
 	// if the image wasn't modified and should be saved in the same format we just copy the file - no need
 	// to lose some quality or information by calling QImage::save()
-	if( ! ( isModified() && isReadWrite() ) && KImageIO::type( m_file ) == KImageIO::type( kurl.fileName() ) )
+	if( ! ( isModified() && isReadWrite() ) && m_mimeType == m_newMimeType )
 	{
 		kdDebug( 4610 ) << "copy image from " << m_file << " to " << kurl.prettyURL() << endl;
-		m_url = kurl;
-		m_sCaption = m_url.prettyURL();
 
 		KIO::Job * job = KIO::copy( m_file, kurl, isProgressInfoEnabled() );
 		emit started( job );
 		connect( job, SIGNAL( result( KIO::Job * ) ),
 				this, SLOT( slotResultSaveAs( KIO::Job * ) ) );
-		emit setWindowCaption( m_sCaption );
 		return true;
 	}
 	kdDebug( 4610 ) << "call KParts::ReadWritePart::saveAs( " << kurl.prettyURL() << " )" << endl;
 
-	return KParts::ReadWritePart::saveAs( kurl );
+	bool ret = KParts::ReadWritePart::saveAs( kurl );
+	if( ret == false )
+		KMessageBox::error( m_pParentWidget, i18n( "The image could not be saved to disk. A possible causes is that you don't have permission to write to that file." ) );
+	return ret;
 }
 
 void KViewViewer::setModified( bool modified )
@@ -235,7 +223,7 @@ void KViewViewer::setModified( bool modified )
 bool KViewViewer::openURL( const KURL & url )
 {
 	kdDebug( 4610 ) << k_funcinfo << endl;
-	if ( url.isMalformed() )
+	if ( !url.isValid() )
 	{
 		kdWarning( 4610 ) << "malformed URL " << url.prettyURL() << endl;
 		return false;
@@ -245,8 +233,10 @@ bool KViewViewer::openURL( const KURL & url )
 		kdDebug( 4610 ) << "closeURL didn't work out" << endl;
 		return false;
 	}
+	setModified( false );
 	m_url = url;
-	m_mimeType = m_pExtension->urlArgs().serviceType;
+	m_mimeType = m_pExtension->urlArgs().serviceType; // if this fails to
+	// find the right mime type it'll be determined in openFile()
 	if ( m_url.isLocalFile() )
 	{
 		emit started( 0 );
@@ -315,7 +305,15 @@ void KViewViewer::newImage( const QImage & newimg )
 void KViewViewer::reload()
 {
 	kdDebug( 4610 ) << k_funcinfo << endl;
-	openURL( m_url );
+
+	if( isReadWrite() && isModified() )
+		if( ! queryClose() )
+			return;
+
+	//load from file
+	QImage image( m_file );
+	m_pCanvas->setImage( image );
+	setModified( false );
 }
 
 bool KViewViewer::eventFilter( QObject * o, QEvent * e )
@@ -332,7 +330,7 @@ bool KViewViewer::eventFilter( QObject * o, QEvent * e )
 				kdDebug( 4610 ) << "DragEnter Event in the Canvas: " << endl;
 				for( int i = 0; ev->format( i ); ++i )
 					kdDebug( 4610 ) << " - " << ev->format( i ) << endl;
-				ev->accept( QUriDrag::canDecode( ev ) || QImageDrag::canDecode( ev ) );
+				ev->accept( KURLDrag::canDecode( ev ) || QImageDrag::canDecode( ev ) );
 				return true;
 			}
 			case QEvent::Drop:
@@ -341,7 +339,7 @@ bool KViewViewer::eventFilter( QObject * o, QEvent * e )
 				kdDebug( 4610 ) << "Drop Event in the Canvas" << endl;
 				QStringList l;
 				QImage image;
-				if( QUriDrag::decodeToUnicodeUris( ev, l ) )
+				if( KURLDrag::decodeToUnicodeUris( ev, l ) )
 					openURL( l.first() );
 				else if( QImageDrag::decode( ev, image ) )
 					newImage( image );
@@ -376,19 +374,32 @@ bool KViewViewer::openFile()
 			m_pTempFile->dataStream()->writeRawBytes( m_pBuffer->buffer().data(), m_pBuffer->buffer().size() );
 			m_pTempFile->close();
 		}
+
+		// determine Mime Type
+		if( m_mimeType.isNull() )
+		{
+			m_mimeType = KImageIO::mimeType( m_url.fileName() );
+			kdDebug( 4610 ) << "MimeType: " << m_mimeType << endl;
+			if( m_mimeType.isNull() )
+			{
+				m_mimeType = KMimeType::findByContent( m_pBuffer->buffer() )->name();
+				kdDebug( 4610 ) << "MimeType: " << m_mimeType << endl;
+			}
+		}
+
 		QImage image( m_pBuffer->buffer() );
-		if( ! image.isNull() )
-			m_pCanvas->setImage( QImage( m_pBuffer->buffer() ) );
 		delete m_pBuffer;
 		m_pBuffer = 0;
-		if( image.isNull() )
+		if( ! image.isNull() )
+			m_pCanvas->setImage( image );
+		else
 		{
 			emit setStatusBarText( i18n( "Unknown image format: %1" ).arg( m_url.prettyURL() ) );
 			return false;
 		}
 	}
 	else
-	{
+	{ //load from local file
 		kdDebug( 4610 ) << k_funcinfo << " load from file: " << m_file << endl;
 		if( ! QFile::exists( m_file ) )
 		{
@@ -400,12 +411,18 @@ bool KViewViewer::openFile()
 			emit setStatusBarText( i18n( "Unknown image format: %1" ).arg( m_file ) );
 			return false;
 		}
+		// determine Mime Type
+		if( m_mimeType.isNull() )
+		{
+			m_mimeType = KImageIO::mimeType( m_file );
+			kdDebug( 4610 ) << "MimeType: " << m_mimeType << endl;
+		}
+
 		//load from file
-		m_pCanvas->setImage( QImage( m_file ) );
+		QImage image( m_file );
+		m_pCanvas->setImage( image );
 	}
-	kdDebug( 4610 ) << "m_pFileWatch->addFile( m_file );\n";
 	m_pFileWatch->addFile( m_file );
-	kdDebug( 4610 ) << "m_pFileWatch->addFile( m_file ); done\n";
 	emit imageOpened( m_url );
 	return true;
 }
@@ -416,7 +433,7 @@ bool KViewViewer::saveFile()
 	// two possibilities:
 	//
 	// 1. The image should be saved as an exact copy of the loaded one (this
-	// is usefull if you downloaded an image from the net and want to save it
+	// is useful if you downloaded an image from the net and want to save it
 	// locally) - this can happen only in saveAs() so only 2. applies here:
 	//
 	// 2. The image should be saved in a different format or was modified, so
@@ -426,15 +443,27 @@ bool KViewViewer::saveFile()
 	if( ! image )
 		return false;
 
-	kdDebug( 4610 ) << "save m_pCanvas->image() to " << m_file << " as " << KImageIO::type( m_file ) << endl;
-	return image->save( m_file, KImageIO::type( m_file ).latin1() );
+	if( ! m_newMimeType.isNull() )
+	{
+		m_mimeType = m_newMimeType;
+		m_newMimeType = QString::null;
+	}
+	QString type = KImageIO::typeForMime( m_mimeType );
+	kdDebug( 4610 ) << "save m_pCanvas->image() to " << m_file << " as " << type << endl;
+	int quality = 100; // TODO: ask user for quality. Keep it at 100 so that our
+	// users don't lose quality when working with KView.
+	
+	m_pFileWatch->removeFile( m_file );
+	bool ret = image->save( m_file, type.latin1(), quality );
+	m_pFileWatch->addFile( m_file );
+	return ret;
 }
 
 void KViewViewer::setupActions()
 {
-	m_paZoomIn = new KAction( i18n( "Zoom In 10%" ), "viewmag+", KStdAccel::key( KStdAccel::ZoomIn ), this,
+	m_paZoomIn = new KAction( i18n( "Zoom In 10%" ), "viewmag+", KStdAccel::shortcut( KStdAccel::ZoomIn ), this,
 			SLOT( slotZoomIn() ), actionCollection(), "zoomin" );
-	m_paZoomOut = new KAction( i18n( "Zoom Out 10%" ), "viewmag-", KStdAccel::key( KStdAccel::ZoomOut ), this,
+	m_paZoomOut = new KAction( i18n( "Zoom Out 10%" ), "viewmag-", KStdAccel::shortcut( KStdAccel::ZoomOut ), this,
 			SLOT( slotZoomOut() ), actionCollection(), "zoomout" );
 
 	m_paZoom = new KSelectAction( i18n( "Zoom" ), "viewmag", 0, actionCollection(), "view_zoom" );
@@ -450,9 +479,11 @@ void KViewViewer::setupActions()
 	m_paFlipMenu->insert( m_paFlipV );
 	m_paFlipMenu->insert( m_paFlipH );
 
-	m_paRotate = new KAction( i18n( "Ro&tate Counter-Clockwise" ), "rotate", 0, this,
-			SLOT( slotRotate() ), actionCollection(), "rotate" );
-	m_paSave = KStdAction::save( this, SLOT( save() ), actionCollection() );
+	m_paRotateCCW = new KAction( i18n( "Ro&tate Counter-Clockwise" ), "rotate_ccw", 0, this,
+			SLOT( slotRotateCCW() ), actionCollection(), "rotateCCW" );
+	m_paRotateCW = new KAction( i18n( "Rotate Clockwise" ), "rotate_cw", 0, this,
+			SLOT( slotRotateCW() ), actionCollection(), "rotateCW" );
+	m_paSave = KStdAction::save( this, SLOT( slotSave() ), actionCollection() );
 	m_paSave->setEnabled( false );
 	m_paSaveAs = KStdAction::saveAs( this, SLOT( slotSaveAs() ), actionCollection() );
 
@@ -461,7 +492,8 @@ void KViewViewer::setupActions()
 	m_paZoomIn->setEnabled( false );
 	m_paZoomOut->setEnabled( false );
 	m_paZoom->setEnabled( false );
-	m_paRotate->setEnabled( false );
+	m_paRotateCCW->setEnabled( false );
+	m_paRotateCW->setEnabled( false );
 	m_paSaveAs->setEnabled( false );
 	m_paFitToWin->setEnabled( false );
 	m_paFlipMenu->setEnabled( false );
@@ -470,7 +502,8 @@ void KViewViewer::setupActions()
 	connect( widget(), SIGNAL( hasImage( bool ) ), m_paZoomIn, SLOT( setEnabled( bool ) ) );
 	connect( widget(), SIGNAL( hasImage( bool ) ), m_paZoomOut, SLOT( setEnabled( bool ) ) );
 	connect( widget(), SIGNAL( hasImage( bool ) ), m_paZoom, SLOT( setEnabled( bool ) ) );
-	connect( widget(), SIGNAL( hasImage( bool ) ), m_paRotate, SLOT( setEnabled( bool ) ) );
+	connect( widget(), SIGNAL( hasImage( bool ) ), m_paRotateCCW, SLOT( setEnabled( bool ) ) );
+	connect( widget(), SIGNAL( hasImage( bool ) ), m_paRotateCW, SLOT( setEnabled( bool ) ) );
 	connect( widget(), SIGNAL( hasImage( bool ) ), m_paSaveAs, SLOT( setEnabled( bool ) ) );
 	connect( widget(), SIGNAL( hasImage( bool ) ), m_paFitToWin, SLOT( setEnabled( bool ) ) );
 	connect( widget(), SIGNAL( hasImage( bool ) ), m_paFlipMenu, SLOT( setEnabled( bool ) ) );
@@ -496,14 +529,16 @@ void KViewViewer::guiActivateEvent( KParts::GUIActivateEvent * event )
 void KViewViewer::readSettings()
 {
 	KConfigGroup cfgGroup( instance()->config(), "Settings" );
-	m_pCanvas->setFastScale( ! cfgGroup.readBoolEntry( "Smooth Scaling", true ) );
-	m_pCanvas->setKeepAspectRatio( cfgGroup.readBoolEntry( "Keep Aspect Ratio", true ) );
-	m_pCanvas->setCentered( cfgGroup.readBoolEntry( "Center Image", true ) );
+	m_pCanvas->setFastScale( ! cfgGroup.readBoolEntry( "Smooth Scaling", ! m_pCanvas->fastScale() ) );
+	m_pCanvas->setKeepAspectRatio( cfgGroup.readBoolEntry( "Keep Aspect Ratio", m_pCanvas->keepAspectRatio() ) );
+	m_pCanvas->setCentered( cfgGroup.readBoolEntry( "Center Image", m_pCanvas->centered() ) );
 
-	m_pCanvas->setMinimumImageSize( QSize( cfgGroup.readNumEntry( "Minimum Width", 1 ),
-				cfgGroup.readNumEntry( "Minimum Height", 1 ) ) );
-	m_pCanvas->setMaximumImageSize( QSize( cfgGroup.readNumEntry( "Maximum Width", 10000 ),
-				cfgGroup.readNumEntry( "Maximum Height", 10000 ) ) );
+	m_pCanvas->setBgColor( cfgGroup.readColorEntry( "Background Color", &m_pCanvas->bgColor() ) );
+
+	m_pCanvas->setMinimumImageSize( QSize( cfgGroup.readNumEntry( "Minimum Width", m_pCanvas->minimumImageSize().width() ),
+				cfgGroup.readNumEntry( "Minimum Height", m_pCanvas->minimumImageSize().height() ) ) );
+	m_pCanvas->setMaximumImageSize( QSize( cfgGroup.readNumEntry( "Maximum Width", m_pCanvas->maximumImageSize().width() ),
+				cfgGroup.readNumEntry( "Maximum Height", m_pCanvas->maximumImageSize().height() ) ) );
 
 	KConfigGroup blendConfig( instance()->config(), "Blend Effects" );
 	m_vEffects.clear();
@@ -514,6 +549,7 @@ void KViewViewer::readSettings()
 	}
 	// and now tell the canvas what blend effect to use
 	switchBlendEffect();
+	loadPlugins();
 }
 
 void KViewViewer::writeSettings()
@@ -558,22 +594,43 @@ void KViewViewer::slotData( KIO::Job *, const QByteArray & data )
 	//m_pCanvas->setImage( QImage( m_pBuffer->buffer() ) );
 }
 
+void KViewViewer::slotSave()
+{
+	kdDebug( 4610 ) << k_funcinfo << endl;
+	if( ! save() )
+		KMessageBox::error( m_pParentWidget, i18n( "The image could not be saved to disk. A possible causes is that you don't have permission to write to that file." ) );
+}
+
 void KViewViewer::slotSaveAs()
 {
 	kdDebug( 4610 ) << k_funcinfo << endl;
-	KURL url = KFileDialog::getSaveURL( ":save_image", KImageIO::pattern( KImageIO::Writing ), widget() );
+	KFileDialog dlg( QString::null, QString::null, widget(), "filedialog", true );
+	dlg.setMimeFilter( KImageIO::mimeTypes( KImageIO::Writing ) );
+	dlg.setSelection( m_url.fileName() );
+	dlg.setCaption( i18n( "Save As" ) );
+	dlg.setOperationMode( KFileDialog::Saving );
+	dlg.exec();
+	KURL url = dlg.selectedURL();
+	m_newMimeType = dlg.currentMimeFilter();
+	if( m_newMimeType.isEmpty() )
+		m_newMimeType = KImageIO::mimeType( url.path() );
+	kdDebug( 4610 ) << k_funcinfo << "m_newMimeType = " << m_newMimeType << endl;
+	if( url.isValid() )
+		KRecentDocument::add( url );
 	saveAs( url );
 }
 
 void KViewViewer::slotZoomIn()
 {
 	double zoom = m_pCanvas->zoom() * 1.1;
+	if (zoom < 0.01) zoom=0.01 ;
 	m_pCanvas->setZoom( zoom );
 }
 
 void KViewViewer::slotZoomOut()
 {
 	double zoom = m_pCanvas->zoom() / 1.1;
+	if (zoom < 0.01) zoom=0.01 ;
 	m_pCanvas->setZoom( zoom );
 }
 
@@ -650,19 +707,19 @@ void KViewViewer::slotFlipV()
 	m_pCanvas->flipHorizontal( isReadWrite() );
 }
 
-void KViewViewer::slotRotate()
+void KViewViewer::slotRotateCCW()
 {
 	m_pCanvas->rotate( -90.0, isReadWrite() );
 }
 
+void KViewViewer::slotRotateCW()
+{
+	m_pCanvas->rotate( 90.0, isReadWrite() );
+}
+
 void KViewViewer::slotFitToWin()
 {
-	QSize maxSize = m_pCanvas->maximumImageSize();
-	m_pCanvas->setMaximumImageSize( widget()->size() );
-	// XXX: HACK: if I use a real big zoom the image has to be resized to the maximum
-	// size...
-	m_pCanvas->setZoom( 10000.0 );
-	m_pCanvas->setMaximumImageSize( maxSize );
+	m_pCanvas->boundImageTo( widget()->size() );
 }
 
 void KViewViewer::slotDel()
@@ -671,6 +728,7 @@ void KViewViewer::slotDel()
 	KURL urltodel = m_url;
 	if( closeURL() )
 	{
+		setModified( false );
 		KIO::file_delete( urltodel );
 		m_pCanvas->clear();
 	}
@@ -690,7 +748,6 @@ void KViewViewer::slotPopupMenu( const QPoint &pos )
 {
 	KXMLGUIClient *popupGUIClient = new PopupGUIClient( instance(), m_popupDoc );
 
-	// TODO:
 	(void) new KAction( i18n( "Save Image As..." ), 0, this, SLOT( slotSaveAs() ),
 						popupGUIClient->actionCollection(), "saveimageas" );
 
@@ -708,7 +765,18 @@ void KViewViewer::slotResultSaveAs( KIO::Job *job )
 		//job->showErrorDialog( widget() );
 	}
 	else
+	{
 		emit completed();
+		KIO::CopyJob * cjob = ::qt_cast<KIO::CopyJob*>( job );
+		if( cjob )
+		{
+			m_url = cjob->destURL();
+			m_sCaption = m_url.prettyURL();
+		}
+		else
+			m_sCaption = "";
+		emit setWindowCaption( m_sCaption );
+	}
 
 	// Now we have to delete the tempfile if it exists and if the current
 	// image position is on the local HD. If it's somewhere else we keep the
@@ -729,16 +797,21 @@ void KViewViewer::slotFileDirty( const QString & )
 	if( isModified() && isReadWrite() )
 	{
 		KPassivePopup * pop = new KPassivePopup( m_pParentWidget );
-		QVBox * vb = new QVBox( pop );
-		QLabel * title = new QLabel( i18n( "<b>KView</b> Load changed image?" ), vb );
-		title->setAlignment( AlignHCenter );
+		QVBox * vb = pop->standardView( i18n( "Load changed image? - %1" ).arg( kapp->aboutData()->programName() ),
+				QString::null, kapp->miniIcon() );
 		( void )new QLabel( i18n( "The image %1 which you have modified has changed on disk.\n"
 								  "Do you want to reload the file and lose your changes?\n"
 								  "If you choose No and subsequently save the image, you will lose the\n"
 								  "changes that have already been saved." ).arg( url().fileName() ), vb );
-		QHBox * hb = new QHBox( vb );
+		QWidget * hb = new QWidget( vb );
+		QHBoxLayout * layout = new QHBoxLayout( hb );
+		layout->addItem( new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Minimum ) );
 		KPushButton * yes = new KPushButton( KStdGuiItem::yes(), hb );
+		layout->addWidget( yes );
+		layout->addItem( new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Minimum ) );
 		KPushButton * no = new KPushButton( KStdGuiItem::no(), hb );
+		layout->addWidget( no );
+		layout->addItem( new QSpacerItem( 0, 0, QSizePolicy::Minimum, QSizePolicy::Minimum ) );
 		connect( yes, SIGNAL( clicked() ), this, SLOT( slotReloadUnmodified() ) );
 		connect( yes, SIGNAL( clicked() ), pop, SLOT( hide() ) );
 		connect( no, SIGNAL( clicked() ), pop, SLOT( hide() ) );
@@ -767,6 +840,13 @@ void KViewViewer::slotToggleScrollbars()
 void KViewViewer::loadPlugins()
 {
 	KImageViewer::Viewer::loadPlugins( this, this, instance() );
+	if( factory() )
+	{
+		QPtrList<KParts::Plugin> plugins = KParts::Plugin::pluginObjects( this );
+		KParts::Plugin * plugin;
+		for( plugin = plugins.first(); plugin; plugin = plugins.next() )
+			factory()->addClient( plugin );
+	}
 }
 
 void KViewViewer::switchBlendEffect()
