@@ -12,7 +12,7 @@ static const char *version = "v0.1";
 static KCmdLineOptions options[] =
 {
     { "+[URL]", I18N_NOOP( "Image to open." ), 0 },
-    { 0, 0, 0 }
+    KCmdLineLastOption
 };
 
 int main(int argc, char **argv)
