@@ -12,10 +12,10 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
-// $Id: kviewbrowser.cpp,v 1.6.2.1 2003/07/17 15:39:33 mueller Exp $
+// $Id: kviewbrowser.cpp,v 1.11 2003/07/27 00:25:16 mueller Exp $
 
 #include "kviewbrowser.h"
 #include "kmyfileitemlist.h"
@@ -28,6 +28,7 @@
 #include <kgenericfactory.h>
 #include <kdebug.h>
 #include <kimageviewer/viewer.h>
+#include <kimageviewer/canvas.h>
 #include <kparts/browserextension.h>
 #include <kapplication.h>
 #include <kimageio.h>
@@ -45,7 +46,9 @@ KViewBrowser::KViewBrowser( QObject* parent, const char* name, const QStringList
 	if( m_pViewer )
 	{
 		m_paBack    = KStdAction::back   ( this, SLOT( slotBack()    ), actionCollection(), "previous_image" );
+		m_paBack->setShortcut( SHIFT+Key_Left );
 		m_paForward = KStdAction::forward( this, SLOT( slotForward() ), actionCollection(), "next_image"     );
+		m_paForward->setShortcut( SHIFT+Key_Right );
 		m_pExtension = m_pViewer->browserExtension();
 	}
 	else
@@ -58,6 +61,21 @@ KViewBrowser::~KViewBrowser()
 	delete m_pFileItemList;
 }
 
+void KViewBrowser::openURL(const KURL &u)
+{
+	if (m_pViewer)
+	{
+		// Opening new URL resets zoom, so remember it.
+		double oldZoom = m_pViewer->canvas()->zoom();
+		m_pViewer->openURL(u);
+		m_pViewer->canvas()->setZoom(oldZoom);
+	}
+	if( m_pExtension )
+	{
+		emit m_pExtension->setLocationBarURL( u.prettyURL() );
+	}
+}
+
 void KViewBrowser::slotBack()
 {
 	setupDirLister();
@@ -70,30 +88,8 @@ void KViewBrowser::slotBack()
 	if( item )
 	{
 		kdDebug( 4630 ) << item->url().prettyURL() << endl;
-		m_pViewer->openURL( item->url() );
-		if( m_pExtension )
-			emit m_pExtension->setLocationBarURL( item->url().prettyURL() );
+		openURL( item->url() );
 	}
-#if 0
-	KFileItem * lastitem = 0;
-	for( item = m_pFileItemList->first(); item; item = m_pFileItemList->next() )
-	{
-		if( ( item->url() == m_pViewer->url() ) && ( lastitem ) )
-		{
-			m_pViewer->openURL( lastitem->url() );
-			if( m_pExtension )
-				emit m_pExtension->setLocationBarURL( lastitem->url().prettyURL() );
-			return;
-		}
-		lastitem = item;
-	}
-	if( lastitem )
-	{
-		m_pViewer->openURL( lastitem->url() );
-		if( m_pExtension )
-			emit m_pExtension->setLocationBarURL( lastitem->url().prettyURL() );
-	}
-#endif
 	else
 		kdDebug( 4630 ) << "no file found" << endl;
 	m_bShowCurrent = false;
@@ -111,34 +107,8 @@ void KViewBrowser::slotForward()
 	if( item )
 	{
 		kdDebug( 4630 ) << item->url().prettyURL() << endl;
-		m_pViewer->openURL( item->url() );
-		if( m_pExtension )
-			emit m_pExtension->setLocationBarURL( item->url().prettyURL() );
+		openURL( item->url() );
 	}
-#if 0
-	for( item = m_pFileItemList->first(); item; item = m_pFileItemList->next() )
-	{
-		kdDebug( 4630 ) << item->url().prettyURL() << endl;
-		if( item->url() == m_pViewer->url() )
-		{
-			item = m_pFileItemList->next();
-			if( item )
-			{
-				m_pViewer->openURL( item->url() );
-				if( m_pExtension )
-					emit m_pExtension->setLocationBarURL( item->url().prettyURL() );
-				return;
-			}
-		}
-	}
-	item = m_pFileItemList->first();
-	if( item )
-	{
-		m_pViewer->openURL( item->url() );
-		if( m_pExtension )
-			emit m_pExtension->setLocationBarURL( item->url().prettyURL() );
-	}
-#endif
 	else
 		kdDebug( 4630 ) << "no file found" << endl;
 	m_bShowCurrent = false;
@@ -184,7 +154,7 @@ void KViewBrowser::setupDirLister()
 	{
 		kdDebug( 4630 ) << "create new KDirLister" << endl;
 		m_pDirLister = new KDirLister();
-		m_pDirLister->setMimeFilter( KImageIO::mimeTypes( KImageIO::Reading ) ); //QStringList::split( ";", "image/gif;image/x-xpm;image/x-xbm;image/jpeg;image/bmp;image/x-bmp;image/png;image/x-png;image/x-ico;image/tiff;video/x-mng" ) );
+		m_pDirLister->setMimeFilter( KImageIO::mimeTypes( KImageIO::Reading ) );
 		m_pDirLister->setShowingDotFiles( true );
 		connect( m_pDirLister, SIGNAL( newItems( const KFileItemList & ) ), SLOT( slotNewItems( const KFileItemList & ) ) );
 		connect( m_pDirLister, SIGNAL( deleteItem( KFileItem * ) ), SLOT( slotDeleteItem( KFileItem * ) ) );
