@@ -12,21 +12,23 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
-/* $Id: kviewpresenter.cpp,v 1.20.2.1 2003/07/17 15:39:33 mueller Exp $ */
+/* $Id: kviewpresenter.cpp,v 1.29 2003/11/27 12:49:14 mkretz Exp $ */
 
 #include "kviewpresenter.h"
 #include "imagelistdialog.h"
 #include "imagelistitem.h"
-#include "kviewpresenterconfmodule.h"
 
 #include <qvbox.h>
 #include <qobjectlist.h>
 #include <qsignalslotimp.h>
 #include <qtimer.h>
+#include <qevent.h>
+#include <qdragobject.h>
+#include <qstringlist.h>
 
 #include <kpushbutton.h>
 #include <kapplication.h>
@@ -44,6 +46,7 @@
 #include <kio/netaccess.h>
 #include <kmessagebox.h>
 #include <ktempfile.h>
+#include <kurldrag.h>
 
 typedef KGenericFactory<KViewPresenter> KViewPresenterFactory;
 K_EXPORT_COMPONENT_FACTORY( kview_presenterplugin, KViewPresenterFactory( "kviewpresenterplugin" ) )
@@ -64,17 +67,29 @@ KViewPresenter::KViewPresenter( QObject* parent, const char* name, const QString
 	delete viewerList;
 	if( m_pViewer )
 	{
-		(void) new KAction( i18n( "&Image List..." ), 0, 0,
+		( void ) new KAction( i18n( "&Image List..." ), 0, 0,
 							this, SLOT( slotImageList() ),
 							actionCollection(), "plugin_presenter_imageList" );
 		m_paSlideshow = new KToggleAction( i18n( "Start &Slideshow" ), Key_S, actionCollection(), "plugin_presenter_slideshow" );
+		( void ) new KAction( i18n( "&Previous Image in List" ), "previous", ALT+Key_Left,
+							  this, SLOT( prev() ),
+							  actionCollection(), "plugin_presenter_prev" );
+		( void ) new KAction( i18n( "&Next Image in List" ), "next", ALT+Key_Right,
+							  this, SLOT( next() ),
+							  actionCollection(), "plugin_presenter_next" );
+
 		connect( m_paSlideshow, SIGNAL( toggled( bool ) ), m_pImageList->m_pSlideshow, SLOT( setOn( bool ) ) );
 		connect( m_pImageList->m_pSlideshow, SIGNAL( toggled( bool ) ), m_paSlideshow, SLOT( setChecked( bool ) ) );
 
 		// search for file_open action
 		KXMLGUIClient * parentClient = static_cast<KXMLGUIClient*>( parent->qt_cast( "KXMLGUIClient" ) );
 		if( parentClient )
+		{
 			m_paFileOpen = parentClient->actionCollection()->action( "file_open" );
+			m_paFileClose = parentClient->actionCollection()->action( "file_close" );
+		}
+		if( m_paFileClose )
+			connect( m_paFileClose, SIGNAL( activated() ), this, SLOT( slotClose() ) );
 		if( m_paFileOpen )
 		{
 			disconnect( m_paFileOpen, SIGNAL( activated() ), parent, SLOT( slotOpenFile() ) );
@@ -117,6 +132,14 @@ KViewPresenter::KViewPresenter( QObject* parent, const char* name, const QString
 	connect( m_pImageList->m_pCloseAll, SIGNAL( clicked() ),
 			this, SLOT( closeAll() ) );
 
+	// allow drop on the dialog
+	m_pImageList->installEventFilter( this );
+	m_pImageList->m_pListView->installEventFilter( this );
+	m_pImageList->m_pListView->viewport()->installEventFilter( this );
+
+	// grab drops on the main view
+	m_pViewer->widget()->installEventFilter( this );
+
 	connect( m_pSlideshowTimer, SIGNAL( timeout() ),
 			this, SLOT( next() ) );
 }
@@ -127,10 +150,67 @@ KViewPresenter::~KViewPresenter()
 	if( m_paFileOpen )
 	{
 		disconnect( m_paFileOpen, SIGNAL( activated() ), this, SLOT( slotOpenFiles() ) );
-		connect( m_paFileOpen, SIGNAL( activated() ), parent(), SLOT( slotOpenFile() ) );
+		// If the parent() doesn't exist we either leave the "File Open" action
+		// in an unusable state or KView was just shutting down and therefor we
+		// can ignore this. I've only seen the second one happening and to get
+		// rid of the QObject::connect warning we do the parent() check.
+		if( parent() )
+			connect( m_paFileOpen, SIGNAL( activated() ), parent(), SLOT( slotOpenFile() ) );
 	}
 }
 
+bool KViewPresenter::eventFilter( QObject *obj, QEvent *ev )
+{
+	if( obj == m_pImageList || obj == m_pImageList->m_pListView || obj == m_pImageList->m_pListView->viewport() || obj == m_pViewer->widget() )
+	{
+		switch( ev->type() )
+		{
+			case QEvent::DragEnter:
+			case QEvent::DragMove:
+			{
+				// drag enter event in the image list
+				//kdDebug( 4630 ) << "DragEnterEvent in the image list: " << obj->className() << endl;
+				QDragEnterEvent * e = static_cast<QDragEnterEvent*>( ev );
+				//for( int i = 0; e->format( i ); ++i )
+					//kdDebug( 4630 ) << " - " << e->format( i ) << endl;
+				if( KURLDrag::canDecode( e ) )// || QImageDrag::canDecode( e ) )
+				{
+					e->accept();
+					return true;
+				}
+			}
+			case QEvent::Drop:
+			{
+				// drop event in the image list
+				kdDebug( 4630 ) << "DropEvent in the image list: " << obj->className() << endl;
+				QDropEvent * e = static_cast<QDropEvent*>( ev );
+				QStringList l;
+				//QImage image;
+				if( KURLDrag::decodeToUnicodeUris( e, l ) )
+				{
+					for( QStringList::const_iterator it = l.begin(); it != l.end(); ++it )
+					{
+						ImageInfo * info = new ImageInfo( *it );
+						if( ! m_imagelist.contains( info ) )
+						{
+							m_imagelist.inSort( info );
+							( void )new ImageListItem( m_pImageList->m_pListView, *it );
+						}
+						else
+							delete info;
+					}
+					return true;
+				}
+				//else if( QImageDrag::decode( e, image ) )
+					//newImage( image );
+			}
+			default: // do nothing
+				break;
+		}
+	}
+	return KParts::Plugin::eventFilter( obj, ev );
+}
+
 void KViewPresenter::slotImageOpened( const KURL & url )
 {
 	kdDebug( 4630 ) << k_funcinfo << endl;
@@ -178,6 +258,21 @@ void KViewPresenter::slotOpenFiles()
 	}
 }
 
+void KViewPresenter::slotClose()
+{
+	QListViewItem * next = m_pCurrentItem->itemBelow() ? m_pCurrentItem->itemBelow() : m_pImageList->m_pListView->firstChild();
+	if( next == m_pCurrentItem )
+		next = 0;
+
+	ImageInfo info( m_pCurrentItem->url() );
+	m_imagelist.remove( &info );
+	delete m_pCurrentItem;
+	m_pCurrentItem = 0;
+
+	if( next )
+		changeItem( next );
+}
+
 void KViewPresenter::changeItem( QListViewItem * qitem )
 {
 	kdDebug( 4630 ) << k_funcinfo << endl;
@@ -189,8 +284,27 @@ void KViewPresenter::changeItem( QListViewItem * qitem )
 			if( item->url().isLocalFile() && ! QFile::exists( item->url().path() ) )
 			{
 				kdDebug( 4630 ) << "file doesn't exist. removed." << endl;
-				delete item;
-				next();
+				ImageInfo info( item->url() );
+				m_imagelist.remove( &info );
+				if( m_pCurrentItem == item )
+				{
+					QListViewItem * next = m_pCurrentItem->itemBelow() ? m_pCurrentItem->itemBelow() : m_pImageList->m_pListView->firstChild();
+					if( next->rtti() != 48294 )
+						kdWarning( 4630 ) << "unknown ListView item" << endl;
+					else
+						m_pCurrentItem = static_cast<ImageListItem*>( next );
+
+					if( m_pCurrentItem == item )
+						m_pCurrentItem = 0; // don't create a dangling pointer
+					delete item;
+					if( m_pCurrentItem )
+						changeItem( m_pCurrentItem );
+				}
+				else
+				{
+					delete item;
+					next();
+				}
 				return;
 			}
 			kdDebug( 4630 ) << "got url" << endl;
@@ -234,9 +348,14 @@ void KViewPresenter::makeCurrent( QListViewItem * item )
 {
 	if( m_pCurrentItem )
 		m_pCurrentItem->setPixmap( 0, QPixmap() );
-	m_pCurrentItem = item;
-	m_pCurrentItem->setPixmap( 0, KGlobal::iconLoader()->loadIcon( "1rightarrow", KIcon::Small ) );
-	m_pImageList->m_pListView->ensureItemVisible( m_pCurrentItem );
+	if( item->rtti() != 48294 )
+		kdWarning( 4630 ) << "unknown ListView item" << endl;
+	else
+	{
+		m_pCurrentItem = static_cast<ImageListItem*>( item );
+		m_pCurrentItem->setPixmap( 0, KGlobal::iconLoader()->loadIcon( "1rightarrow", KIcon::Small ) );
+		m_pImageList->m_pListView->ensureItemVisible( m_pCurrentItem );
+	}
 }
 
 void KViewPresenter::slideshow( bool running )
@@ -291,7 +410,7 @@ void KViewPresenter::loadList()
 		return;
 
 	QString tempfile;
-	if( ! KIO::NetAccess::download( url, tempfile ) )
+	if( ! KIO::NetAccess::download( url, tempfile, m_pViewer->widget() ) )
 	{
 		KMessageBox::error( m_pImageList, i18n( "Could not load\n%1" ).arg( url.prettyURL() ) );
 		return;
@@ -363,7 +482,7 @@ void KViewPresenter::saveList()
 
 		if( ! url.isLocalFile() )
 		{
-			KIO::NetAccess::upload( tempfile, url );
+			KIO::NetAccess::upload( tempfile, url, m_pViewer->widget() );
 			KIO::NetAccess::removeTempFile( tempfile );
 		}
 	}
