@@ -1,19 +1,19 @@
 /* This file is in the public domain */
 
-// $Id: kviewtemplate.cpp,v 1.6 2002/11/19 13:30:35 mkretz Exp $
+// $Id: kviewtemplate.cpp,v 1.8.2.1 2004/02/08 21:26:46 goossens Exp $
 
 #include "kviewtemplate.h"
 
 #include <qobjectlist.h>
 
 #include <kaction.h>
-#include <klocale.h>
+/*#include <klocale.h>*/
 #include <kgenericfactory.h>
 #include <kdebug.h>
 #include <kimageviewer/viewer.h>
 
 typedef KGenericFactory<KViewTemplate> KViewTemplateFactory;
-K_EXPORT_COMPONENT_FACTORY( kview_templateplugin, KViewTemplateFactory( "kviewtemplateplugin" ) );
+K_EXPORT_COMPONENT_FACTORY( kview_templateplugin, KViewTemplateFactory( "kviewtemplateplugin" ) )
 
 KViewTemplate::KViewTemplate( QObject* parent, const char* name, const QStringList & )
 	: Plugin( parent, name )
@@ -23,7 +23,7 @@ KViewTemplate::KViewTemplate( QObject* parent, const char* name, const QStringLi
 	delete viewerList;
 	if( m_pViewer )
 	{
-		(void) new KAction( i18n( "&do something" ), 0, 0,
+		(void) new KAction( /*i18n(*/ "&Do Something" /*)*/, 0, 0,
 							this, SLOT( yourSlot() ),
 							actionCollection(), "plugin_template" );
 	}
