@@ -25,13 +25,14 @@ KMultiPage::KMultiPage(QWidget *parentWidget, const char *widgetName, QObject *p
   // For reasons which I don't understand, the initialization of the
   // DCOPObject above does not work properly, the name is ignored. It
   // works fine if we repeat the name here. -- Stefan Kebekus
-  setObjId("kmultipage");  
+  // This is because of the virtual inheritance. Get rid of it (but it's BC, and this is a lib...) -- DF
+  setObjId("kmultipage");
   _scrollView = new CenteringScrollview(parentWidget, widgetName);
   _scrollView->setResizePolicy(QScrollView::AutoOne);
   _scrollView->setVScrollBarMode(QScrollView::Auto);
   _scrollView->setHScrollBarMode(QScrollView::Auto);
   _scrollView->viewport()->setBackgroundMode(QWidget::PaletteMid );
-  
+
   setWidget(_scrollView);
 }
 
@@ -40,7 +41,7 @@ KMultiPage::~KMultiPage()
 {
 }
 
-QString KMultiPage::name_of_current_file() 
+QString KMultiPage::name_of_current_file()
 {
   return m_file;
 }
@@ -50,6 +51,11 @@ bool KMultiPage::is_file_loaded(QString filename)
   return (filename == m_file);
 }
 
+void KMultiPage::slotSave_defaultFilename()
+{
+  slotSave();
+}
+
 void KMultiPage::slotSave()
 {
   // Try to guess the proper ending...
@@ -73,10 +79,10 @@ void KMultiPage::slotSave()
   // way.
   if (!ending.isEmpty() && fileName.find(ending) == -1)
     fileName = fileName+ending;
-  
+
   if (QFile(fileName).exists()) {
-    int r = KMessageBox::warningYesNo (0, QString(i18n("The file %1\nexists. Shall I overwrite that file?")).arg(fileName), 
-				       i18n("Overwrite file"));
+    int r = KMessageBox::warningYesNo (0, i18n("The file %1\nexists. Shall I overwrite that file?").arg(fileName),
+				       i18n("Overwrite File"));
     if (r == KMessageBox::No)
       return;
   }
@@ -87,8 +93,44 @@ void KMultiPage::slotSave()
   return;
 }
 
+void KMultiPage::slotShowScrollbars(bool status)
+{
+  if (status == true) {
+    _scrollView->setVScrollBarMode(QScrollView::Auto);
+    _scrollView->setHScrollBarMode(QScrollView::Auto);
+  } else {
+    _scrollView->setVScrollBarMode(QScrollView::AlwaysOff);
+    _scrollView->setHScrollBarMode(QScrollView::AlwaysOff);
+  }  
+
+  // Remember the state of the scrollbars so that we can restore
+  // scrollbar visibility when we switch back from fullscreen mode to
+  // normal mode
+  showScrollBars = status;
+}
+
 void KMultiPage::slotIOJobFinished ( KIO::Job *job )
 {
   if ( job->error() )
     job->showErrorDialog( 0L );
 }
+
+
+void KMultiPage::slotSetFullPage(bool fullpage)
+{
+  //  kdDebug() << "KMultiPage::slotSetFullPage() called" << endl;
+
+  if (fullpage == true) {
+    _scrollView->setVScrollBarMode(QScrollView::AlwaysOff);
+    _scrollView->setHScrollBarMode(QScrollView::AlwaysOff);
+    frameStyle = _scrollView->frameStyle();
+    _scrollView->setFrameStyle(QFrame::NoFrame);
+    backgroundColor = _scrollView->viewport()->paletteBackgroundColor();
+    _scrollView->viewport()->setPaletteBackgroundColor( Qt::black ) ;
+     //   _scrollView->repaintContents(true);
+  } else {
+    _scrollView->viewport()->setPaletteBackgroundColor( backgroundColor ) ;
+    slotShowScrollbars(showScrollBars);
+    _scrollView->setFrameStyle(frameStyle);
+  }
+}
