@@ -10,6 +10,7 @@
 
 #include "centeringScrollview.h"
 #include "kmultipageInterface.h"
+#include "pageSize.h"
 
 
 class KInstance;
@@ -18,6 +19,7 @@ class QLabel;
 class QPainter;
 
 
+// TODO remove virtual inheritance for KDE 4. It's the reason for the strange DCOPObject construction
 class KMultiPage : public KParts::ReadOnlyPart, virtual public kmultipageInterface
 {
   Q_OBJECT
@@ -27,7 +29,7 @@ public:
   KMultiPage(QWidget *parentWidget, const char *widgetName, QObject *parent, const char *name);
   virtual ~KMultiPage();
   /// returns the scrollview used for the display
-  virtual CenteringScrollview *scrollView() { return _scrollView; }
+  virtual CenteringScrollview *scrollView() { return _scrollView; };
 
   // Methods which are associated with the DCOP functionality of the
   // kmultipage. This method returns the file name (not the URL) of
@@ -37,7 +39,7 @@ public:
   // Methods which are associated with the DCOP functionality of the
   // kmultipage. This method can be implemented by the multipage,
   // e.g. to jump to a certain location.
-  ASYNC  jumpToReference(QString) {}
+  virtual ASYNC  jumpToReference(QString /*reference*/) {}
 
   // Methods which are associated with the DCOP functionality of the
   // kmultipage. This method checks if a given file is loaded.
@@ -45,7 +47,14 @@ public:
 
 // Interface definition start ------------------------------------------------
 
-  /// returns the list of supported file formats
+  /// returns the list of supported file formats. These strings will
+  /// be in the format explained in the documentation to
+  /// KFileDialog.setFilter(), e.g. ""*.cpp *.cxx *.c++|C++ Source
+  /// Files". The use of mimetype-filters is allowed, but
+  /// discouraged. Among other penalties, if a multipage gives a
+  /// mimetype-filter, e.g. "application/x-dvi", the open file dialog
+  /// in kviewshell will not allow the user to see and select
+  /// compressed dvi-files.
   virtual QStringList fileFormats() = 0;
 
   /// opens a file
@@ -75,50 +84,87 @@ public:
   virtual void setPaperSize(double w, double h) = 0;
 
   /// returns a nice little preview of the page
-  virtual bool preview(QPainter *, int, int) { return false; }
+  virtual bool preview(QPainter *, int, int) { return false; };
 
   /// prints
   virtual bool print(const QStringList &pages, int current) = 0;
 
   /// reads in settings
-  virtual void readSettings() {}
+  virtual void readSettings() {};
 
   /// writes settings
-  virtual void writeSettings() {}
-
-  /// reloads the file form disk and redisplays
-  virtual void reload() {}
+  virtual void writeSettings() {};
 
   /// redisplays the contents of the current page
-  virtual void redisplay() {}
+  virtual void redisplay() {} ;
+
+  /// multipage implementations that offer read- and write
+  /// functionality should re-implement this method and return true
+  /// here.
+  virtual bool isReadWrite() {return false;};
+
+  /// multipage implementations that offer read- and write
+  /// functionality should re-implement this method.
+  virtual bool isModified() {return false;};
 
 public slots:
+  /** reloads the file form disk and redisplays */
+  virtual void reload() {};
+
   /** Opens a file requestor and starts a basic copy KIO-Job. A
       multipage implementation that wishes to offer saving in various
       formats must re-implement this slot. */
-  void slotSave();
+  virtual void slotSave();
+  
+  /** The standard implementation just calls slotSave. */
+  virtual void slotSave_defaultFilename();
+
+  /** Tells the multipage if scrollbars should be used. */
+  void slotShowScrollbars(bool);
 
   /** Used internally. */
   void slotIOJobFinished ( KIO::Job *job );
 
-signals:
+  /** Switches to fullscreen mode and back */
+  virtual void slotSetFullPage(bool fullpage);
 
-  /// emitted to indicate the number of pages in the file
+signals:
+  /** Emitted to indicate the number of pages in the file. The
+      receiver will set the current page to zero and call the
+      gotoPage()-method. */
   void numberOfPages(int nr);
 
+  void askingToCheckActions();
+
   /// emitted when a new preview is available
   void previewChanged(bool previewAvailable);
 
+  /** This signal is emitted to indicate to the multipage that the
+      document specifies a page size. Depending on the settings, this
+      page size will then be set. */
+  void documentSpecifiedPageSize(const pageSize &size);
+
 
 // Interface definition end --------------------------------------------------
 
 
 private:
-
   QLabel *mainWidget;
-
   CenteringScrollview *_scrollView;
 
+  // This is set in slotShowScrollbars() to remember the status of the
+  // scrollbars when we switch from fullscreen mode back to normal
+  // mode
+  bool showScrollBars;
+
+  /** This int remembers the style of the frame of the centering
+      scrollview when fullscreen mode is switched on. It is then
+      restored when it is switched off */
+  int      frameStyle;
+
+  /** color of the viewport's background in _scrollView. This is also
+      stored on entering the fullscreen mode. */
+  QColor   backgroundColor;
 };
 
 
