@@ -2,27 +2,34 @@
 
 #include <qfileinfo.h>
 #include <qlayout.h>
+#include <qobjectlist.h>
 #include <qscrollview.h>
 #include <qscrollbar.h>
 #include <qtimer.h>
 #include <qpainter.h>
 #include <qprinter.h>
 #include <qprintdialog.h>
-#include <qobjectlist.h>
 
 
 #include <kaccel.h>
+#include <kaction.h>
+#include <kapplication.h>
+#include <kconfig.h>
 #include <kdebug.h>
+#include <kfilterbase.h>
+#include <kfilterdev.h>
+#include <kglobal.h>
 #include <kinstance.h>
+#include <kio/job.h>
 #include <klocale.h>
 #include <kiconloader.h>
-#include <kio/job.h>
-#include <kaction.h>
-#include <kstdaction.h>
 #include <kmessagebox.h>
-#include <kglobal.h>
+#include <kmimetype.h>
+#include <kprogress.h>
 #include <kstandarddirs.h>
-#include <kconfig.h>
+#include <kstdaction.h>
+#include <ktempfile.h>
+
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -33,6 +40,7 @@
 #include "gotodialog.h"
 #include "kpagetest.h"
 #include "kviewpart.moc"
+#include "pageSize.h"
 #include "pageSizeDialog.h"
 
 extern "C"
@@ -66,7 +74,12 @@ KParts::Part *KViewPartFactory::createPartObject( QWidget *parentWidget, const c
   QString partname = "";
   if (args.count() >= 1)
     partname = args[0];
-  KParts::Part *obj = new KViewPart(partname, parentWidget, widgetName, parent, name);
+  KViewPart *obj = new KViewPart(partname, parentWidget, widgetName, parent, name);
+  if (!obj->isValid())
+  {
+     delete obj;
+     return 0;
+  }
   return obj;
 }
 
@@ -80,9 +93,13 @@ KInstance *KViewPartFactory::instance()
 
 
 KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widgetName, QObject *parent, const char *name)
-  : KViewPart_Iface(parent, name), _partname(partname), _numberOfPages(0),
+  : KViewPart_Iface(parent, name), _partname(partname), multiPage(0), _numberOfPages(0),
   _currentPage(0)
 {
+  tmpUnzipped = 0L;
+
+  documentHasRequestedPaperSize = false;
+
   pageChangeIsConnected = false;
   setInstance(KViewPartFactory::instance());
 
@@ -90,6 +107,8 @@ KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widget
   connect(watch, SIGNAL(dirty(const QString&)), this, SLOT(fileChanged(const QString&)));
   watch->startScan();
 
+  connect(&previewPaintInitiator, SIGNAL(timeout()), this, SLOT(doRepaintScrollBoxBackground()) );
+
   mainWidget = new QWidget(parentWidget, widgetName);
   mainWidget->setFocusPolicy(QWidget::StrongFocus);
   setWidget(mainWidget);
@@ -122,14 +141,18 @@ KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widget
   if (factory) {
     multiPage = (KMultiPage*) factory->create(mainWidget, QString("k%1").arg(_partname).utf8(), "KPart");
   } else {
-    KMessageBox::error(mainWidget, i18n("No module (%1) found").arg(_partname));
-    ::exit(-1);
+    KMessageBox::error(parentWidget, i18n("No module (%1) found").arg(_partname));
   }
+  if (!multiPage)
+     return; // Abort
+  
   hbox->addWidget(multiPage->widget());
 
   // connect to the multi page view
+  connect(this, SIGNAL(scrollbarStatusChanged(bool)), multiPage, SLOT(slotShowScrollbars(bool)));
   connect(multiPage, SIGNAL(numberOfPages(int)), this, SLOT(numberOfPages(int)));
   connect(multiPage, SIGNAL(pageInfo(int, int)), this, SLOT(pageInfo(int, int)) );
+  connect(multiPage, SIGNAL(askingToCheckActions()), this, SLOT(checkActions()) );
   connect(multiPage->scrollView(), SIGNAL(contentsMoving(int,int)), this, SLOT(contentsMoving(int,int)));
   multiPage->scrollView()->installEventFilter(this);
   connect( multiPage, SIGNAL( started( KIO::Job * ) ), this, SIGNAL( started( KIO::Job * ) ) );
@@ -137,19 +160,22 @@ KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widget
   connect( multiPage, SIGNAL( canceled( const QString & ) ), this, SIGNAL( canceled( const QString & ) ) );
   connect( multiPage, SIGNAL( previewChanged(bool)), this, SLOT(updatePreview(bool)));
   connect( multiPage, SIGNAL( setStatusBarText( const QString& ) ), this, SLOT( setStatusBarTextFromMultiPage( const QString& ) ) );
-
+  connect( multiPage, SIGNAL( documentSpecifiedPageSize(const pageSize&) ), this, SLOT( slotPageSizeRequests(const pageSize&) ) );
+  connect( multiPage->scrollView(), SIGNAL( wheelEventReceived(QWheelEvent *)), this,   SLOT(wheelEvent(QWheelEvent *)));
   // settings menu
-  showmarklist = new KToggleAction (i18n("Show &Page List"), 0, this, SLOT(slotShowMarkList()), actionCollection(), "show_page_list");
+  showmarklist = new KToggleAction (i18n("Show &Sidebar"), 0, this, SLOT(slotShowMarkList()), actionCollection(), "show_page_list");
   showPreview = new KToggleAction (i18n("Show P&review"), 0, this, SLOT(slotPreview()), actionCollection(), "show_preview");
   watchAct = new KToggleAction(i18n("&Watch File"), 0, 0, 0, actionCollection(), "watch_file");
+  scrollbarHandling = new KToggleAction (i18n("Show Scrollbars"), 0, 0, 0, actionCollection(), "scrollbarHandling");
+  connect(scrollbarHandling, SIGNAL(toggled(bool)),  multiPage, SLOT(slotShowScrollbars(bool)));
 
   // Orientation menu
   QStringList orientations;
   orientations.append(i18n("Portrait"));
   orientations.append(i18n("Landscape"));
-  orientation = new KSelectAction (i18n("&Orientation"), 0, 0, 0, actionCollection(), "view_orientation");
-  connect (orientation, SIGNAL(activated (int)), this, SLOT(slotOrientation(int)));
+  orientation = new KSelectAction (i18n("Preferred &Orientation"), 0, 0, 0, actionCollection(), "view_orientation");
   orientation->setItems(orientations);
+  connect(orientation, SIGNAL(activated (int)), &userRequestedPaperSize, SLOT(setOrientation(int)));
 
   // Zoom Menu
   zoom_action = new KSelectAction (i18n("&Zoom"), 0, 0, 0, actionCollection(), "view_zoom");
@@ -162,16 +188,19 @@ KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widget
   connect (zoom_action, SIGNAL(activated(const QString &)), this, SLOT(setZoomValue(const QString &)));
 
   // Paper Size Menu
-  media = new KSelectAction (i18n("Paper &Size"), 0, 0, 0, actionCollection(), "view_media");
+  media = new KSelectAction (i18n("Preferred Paper &Size"), 0, 0, 0, actionCollection(), "view_media");
   QStringList items = _paperSize.pageSizeNames();
   items.prepend(i18n("Custom Size..."));
   media->setItems(items);
   connect (media, SIGNAL(activated(int)), this, SLOT(slotMedia(int)));
 
+  useDocumentSpecifiedSize = new KToggleAction(i18n("&Use Document Specified Paper Size"), 0, this, SLOT(slotShowMarkList()), 
+					       actionCollection(), "view_use_document_specified_size");
+  connect(useDocumentSpecifiedSize, SIGNAL(toggled(bool)),  this, SLOT(slotUseDocumentSpecifiedSize(bool)));
 
+  // Zoom Actions
   zoomInAct = KStdAction::zoomIn (this, SLOT(zoomIn()), actionCollection());
   zoomOutAct = KStdAction::zoomOut(this, SLOT(zoomOut()), actionCollection());
-
   fitAct = KStdAction::actualSize(this, SLOT(fitSize()), actionCollection());
   fitPageAct = KStdAction::fitToPage(this, SLOT(fitToPage()), actionCollection());
   fitWidthAct = KStdAction::fitToWidth(this, SLOT(fitToWidth()), actionCollection());
@@ -184,12 +213,13 @@ KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widget
   endAct = KStdAction::lastPage(this, SLOT(lastPage()), actionCollection());
   gotoAct = KStdAction::gotoPage(this, SLOT(goToPage()), actionCollection());
 
-  readDownAct = new KAction(i18n("Read Down Document"), "next",
-			    Key_Space, this, SLOT(readDown() ),
-			    actionCollection(), "go_read_down");
+  readUpAct = new KAction(i18n("Read Up Document"), "previous", SHIFT+Key_Space, this, SLOT(readUp() ), actionCollection(), "go_read_up");
+  readDownAct = new KAction(i18n("Read Down Document"), "next", Key_Space, this, SLOT(readDown() ), actionCollection(), "go_read_down");
 
   printAction = KStdAction::print(this, SLOT(slotPrint()), actionCollection());
-  saveAction = KStdAction::saveAs(multiPage, SLOT(slotSave()), actionCollection());
+  if (multiPage->isReadWrite() )
+    saveAction = KStdAction::save(multiPage, SLOT(slotSave_defaultFilename()), actionCollection());
+  saveAsAction = KStdAction::saveAs(multiPage, SLOT(slotSave()), actionCollection());
 
   // keyboard accelerators
   accel = new KAccel(mainWidget);
@@ -225,29 +255,69 @@ KViewPart::KViewPart(QString partname, QWidget *parentWidget, const char *widget
   // (will be merged automatically)
   insertChildClient( multiPage );
 
+  // Connect _paperSize to the slot that updated the GUI whenever the
+  // paper size changes
+  connect(&_paperSize, SIGNAL(sizeChanged(float, float)), this, SLOT(setPaperSize(float, float)));
+
+  // This will update the menu entries, and modify _paperSize whenever
+  // the user requests a new paper size
+  connect(&userRequestedPaperSize, SIGNAL(sizeChanged(float, float)), this, SLOT(slotUserPreferredSizeChanged(float, float)));
+
   readSettings();
 
   // watch mouse events in the viewport
   QWidget *vp = multiPage->scrollView()->viewport();
   vp->installEventFilter(this);
   // we also have to set an filter on the child
-  // TODO: solve this more elegant
+  // TODO: solve this more elegantly
   if (vp->children()) {
     QWidget *w = (QWidget*)((QObjectList*)(vp->children()))->first();
     w->installEventFilter(this);
   }
 }
 
-
 KViewPart::~KViewPart()
 {
-  writeSettings();
+  if (multiPage)
+    writeSettings();
   delete multiPage;
+  delete tmpUnzipped;
 }
 
 
-void KViewPart::setPaperSize(double w, double h)
+QString KViewPart::pageSizeDescription(void)
 {
+  // Display the current paper size in the status bar. If the format
+  // is one of the known sizes, then it makes sense to say if the
+  // orientation is "portrait" or "landscape". We give this
+  // information also.
+  QString size = " ";
+
+  if (_paperSize.formatNumber() == -1) {
+    if (KGlobal::locale()-> measureSystem() == KLocale::Metric)
+      size += QString("%1x%2 mm").arg(_paperSize.width_in_mm(), 0, 'f', 0).arg(_paperSize.height_in_mm(), 0, 'f', 0);
+    else 
+      size += QString("%1x%2 in").arg(_paperSize.width_in_mm()/25.4, 0, 'g', 2).arg(_paperSize.height_in_mm()/25.4, 0, 'g', 2);
+  } else {
+    size += _paperSize.formatName() + "/";
+    if (_paperSize.getOrientation() == 0)
+      size += i18n("portrait");
+    else
+      size += i18n("landscape");
+  }
+  return size+" ";
+}
+
+
+void KViewPart::setPaperSize(float width_in_mm, float height_in_mm)
+{
+  emit sizeChanged(pageSizeDescription());
+  
+  // And.... well, yes, also set the paper size in the scrollbox and
+  // in the multipage
+  float w = width_in_mm/10.0;
+  float h = height_in_mm/10.0;
+
   // reflect in scrollbox
   scrollBox->setMinimumHeight( int(75.0*h/w+0.5));
 
@@ -260,7 +330,70 @@ void KViewPart::setPaperSize(double w, double h)
 
 QStringList KViewPart::fileFormats()
 {
-  return multiPage->fileFormats();
+  // The multiPage returns a list of endings such as "*.dvi". But the
+  // kviewshell is also able to read compressed files and to
+  // uncompress them on the fly. Thus, we modify the list of supported
+  // file formats which we obtain from the multipage to include
+  // compressed files like "*.dvi.gz". We add "*.dvi.bz2" if support
+  // for bzip2 is compiled into KDE.
+
+  // Check if this version of KDE supports bzip2 
+  bool bzip2Available = (KFilterBase::findFilterByMimeType( "application/x-bzip2" ) != 0L);
+
+  QStringList FF = multiPage->fileFormats();
+  for(QStringList::Iterator it = FF.begin(); it != FF.end(); ++it ) {
+    // Check if we have a mimetype-filter. If so, we don't touch it
+    // and go on.
+    int pos = (*it).find('/');
+    if (pos > 0 && (*it)[pos - 1] != '\\') 
+      break;
+    
+    // Otherwise parse the string and add endings
+    QStringList endings  = QStringList::split( " ", (*it).section( '|', 0, 0 ) ) ;
+    QString descriptions = (*it).section( '|', 1 );
+    QString newFormat;
+    
+    for(QStringList::Iterator it2 = endings.begin(); it2 != endings.end(); ++it2 ) {
+      *it2 = (*it2).stripWhiteSpace();
+      newFormat += *it2 + " ";
+      if ((*it2).find(".gz", -3) == -1) // Paranoia safety check
+	newFormat += (*it2).stripWhiteSpace() + ".gz ";
+      if ((bzip2Available) && ((*it2).find(".bz2", -4) == -1))
+	newFormat += (*it2).stripWhiteSpace() + ".bz2 ";
+    }
+    if (!descriptions.isEmpty())
+      newFormat += "|" + descriptions;
+
+    *it = newFormat;
+  }
+  return FF;
+}
+
+
+void KViewPart::slotSetFullPage(bool fullpage)
+{
+  if (multiPage)
+    multiPage->slotSetFullPage(fullpage);
+  else
+    kdError() << "KViewPart::slotSetFullPage() called without existing multipage" << endl;
+
+  if (fullpage == true) {
+    markList->hide();
+    scrollBox->hide();
+  } else 
+    slotShowMarkList();
+}
+
+
+void KViewPart::slotPageSizeRequests(const pageSize &size) 
+{
+  documentHasRequestedPaperSize = true;
+  documentRequestedPaperSize = size;
+  
+  // Use the document specified paper size, if allowed. The GUI will
+  // be updated automatically by the signal/slot mechanism
+  if (useDocumentSpecifiedSize->isChecked())
+    _paperSize = size;
 }
 
 
@@ -291,7 +424,7 @@ bool KViewPart::openFile()
     // the filename. We decompose the format strings like "*.dvi
     // *.DVI|DVI File" coming from the multipage part.
     QStringList endings;
-    QStringList formats = multiPage->fileFormats();
+    QStringList formats = fileFormats();
     for ( QStringList::Iterator it = formats.begin(); it != formats.end(); ++it ) {
       QString ending = (*it).simplifyWhiteSpace();
       int bar = ending.find('|');
@@ -321,31 +454,124 @@ bool KViewPart::openFile()
     m_url.setPath(QFileInfo(m_file).absFilePath());
   }
 
+  // Set the window caption now, before we do any uncompression and generation of temporary files.
+  tmpFileURL.setPath(m_file);
+  emit setWindowCaption( tmpFileURL.prettyURL() ); // set Window caption WITHOUT the reference part!
+  
+  // Check if the file is compressed
+  KMimeType::Ptr mimetype = KMimeType::findByPath( m_file );
+
+  if (( mimetype->name() == "application/x-gzip" ) || ( mimetype->name() == "application/x-bzip2" ) ||
+      ( mimetype->parentMimeType() == "application/x-gzip" ) || ( mimetype->parentMimeType() == "application/x-bzip2" )) {
+    // The file is compressed. Make a temporary file, and store an uncompressed version there...
+    if (tmpUnzipped != 0L)  // Delete old temporary file
+      delete tmpUnzipped;
+
+    tmpUnzipped = new KTempFile;
+    if (tmpUnzipped == 0L) {
+      KMessageBox::error( mainWidget, i18n( "<qt><strong>File Error!</strong> Could not create temporary file.</qt>") );
+      emit setWindowCaption( QString::null );
+      return false;
+    }
+    tmpUnzipped->setAutoDelete(true);
+    if( tmpUnzipped->status() != 0 ) {
+      KMessageBox::error( mainWidget, i18n( "<qt><strong>File Error!</strong> Could not create temporary file "
+					    "<nobr><strong>%1</strong></nobr>.</qt>").arg( strerror( tmpUnzipped->status() ) ) );
+      emit setWindowCaption( QString::null );
+      return false;
+    }
+
+    QIODevice *filterDev;
+    if (( mimetype->parentMimeType() == "application/x-gzip" ) || ( mimetype->parentMimeType() == "application/x-bzip2" )) 
+      filterDev = KFilterDev::deviceForFile( m_file, mimetype->parentMimeType() );
+    else
+      filterDev = KFilterDev::deviceForFile( m_file );
+    if (filterDev == 0L) {
+      emit setWindowCaption( QString::null );
+      return false;
+    }
+    if( !filterDev->open( IO_ReadOnly ) ) {
+      KMessageBox::detailedError( mainWidget, i18n( "<qt><strong>File Error!</strong> Could not open the file "
+						    "<nobr><strong>%1</strong></nobr> for uncompression. "
+						    "The file will not be loaded.</qt>").arg(m_file),
+				  i18n("<qt>This error typically occurs if you do not have enough permissions to read the file. "
+				       "You can check ownership and permissions if you right-click on the file in the Konqueror "
+				       "file manager and then choose the 'Properties' menu.</qt>"));
+      emit setWindowCaption( QString::null );
+      delete filterDev;
+      return false;
+    }
+    
+    KProgressDialog *prog = new KProgressDialog(0L, "uncompress-progress", 
+						i18n("Uncompressing..."), 
+						i18n("<qt>Uncompressing the file <nobr><strong>%1</strong></nobr>. Please wait.</qt>").arg(m_file));
+    prog->progressBar()->setTotalSteps((int) fi.size()/1024 );
+    prog->progressBar()->setProgress(0);
+    prog->setMinimumDuration(250);
+
+    QByteArray buf( 1024 );
+    int read = 0, wrtn = 0;
+    while( ( read = filterDev->readBlock( buf.data(), buf.size() ) ) > 0 ) {
+      kapp->processEvents();
+      if (prog->wasCancelled()) 
+	break;
+      prog->progressBar()->advance( 1 );
+      
+      wrtn = tmpUnzipped->file()->writeBlock( buf.data(), read );
+      if( read != wrtn )
+	break;
+    }
+    delete filterDev;
+    delete prog;
+    
+    if (prog->wasCancelled())
+      return false;
+    if( (read != 0)||(tmpUnzipped->file()->size() == 0) ) {
+      KMessageBox::detailedError( mainWidget, i18n( "<qt><strong>File Error!</strong> Could not uncompress the file "
+						    "<nobr><strong>%1</strong></nobr>. The file will not be loaded.</qt>").arg( m_file ),
+				  i18n("<qt>This error typically occurs if the file is corrupt. "
+				       "If you want to be sure, try to decompress the file manually using command-line tools.</qt>"));
+      emit setWindowCaption( QString::null );
+      return false;
+    }
+    tmpUnzipped->close();
+    
+    m_file = tmpUnzipped->name();
+  }
+  
+  
   // Now call the openURL-method of the multipage and give it an URL
   // pointing to the downloaded file.
   tmpFileURL.setPath(m_file);
-  emit setWindowCaption( tmpFileURL.prettyURL() ); // set Window caption WITHOUT the reference part!
   tmpFileURL.setRef(m_url.ref()); // Pass the reference part of the URL through to the multipage
-
+  
+  documentHasRequestedPaperSize = false;
   bool r = multiPage->openURL(tmpFileURL);
-
+  
   updateScrollBox();
   markList->select(0);
-
+  
   if (r) {
+    // If the document did not specify a paper size, re-set the paper
+    // size to normal, if necessary
+    if (documentHasRequestedPaperSize == false) 
+      _paperSize = userRequestedPaperSize;
+    
     // start viewing horizontally centered
     QScrollView *sv = multiPage->scrollView();
     if (sv)
       sv->center(sv->contentsWidth()/2, 0);
-
+    
     // Add the file to the watchlist
     watch->addFile( m_file );
   } else {
     m_url = "";
     emit setWindowCaption( "" );
   }
-
+  
   checkActions();
+  scrollBox->setMinimumHeight( int(75.0*_paperSize.height_in_mm()/_paperSize.width_in_mm()+0.5));
+
   return r;
 }
 
@@ -369,73 +595,89 @@ bool KViewPart::closeURL()
     watch->removeFile( m_file );
 
   KParts::ReadOnlyPart::closeURL();
-
+  
   multiPage->closeURL();
-
+  
   m_url = "";
-
+  
   numberOfPages(0);
   checkActions();
-
+  
   emit setWindowCaption("");
-
+  
   return true;
 }
 
 
-void KViewPart::slotOrientation(int)
-{
-  if (orientation->currentItem() == 0)
-    setPaperSize( _paperSize.pageWidth_in_mm()/10.0, _paperSize.pageHeight_in_mm()/10.0);
-  else
-    setPaperSize( _paperSize.pageHeight_in_mm()/10.0, _paperSize.pageWidth_in_mm()/10.0);
-  if (_pageSizeDialog != 0)
-    _pageSizeDialog->setOrientation(orientation->currentItem());
-}
-
-
 void KViewPart::slotMedia(int id)
 {
-  if (id > 0) {
-    _paperSize.setPageSize(media->currentText());
-    slotOrientation(0);
+  // If the user has chosen one of the 'known' paper sizes, set the
+  // user requested paper size to that value. Via signals and slots,
+  // this will update the menus, and also the GUI, if necessary.
+  if (id > 1) {
+    userRequestedPaperSize.setPageSize(media->currentText());
     return;
   }
-
-  // Show the paper size dialog!
+  
+  // If the user has chosen "Custom paper size..", show the paper size
+  // dialog. Construct it, if necessary. The paper size dialog will
+  // know the address of userRequestedPaperSize and change this
+  // member, if the user clicks ok/accept. The signal/slot mechanism
+  // will then make sure that the necessary updates in the GUI are
+  // done.
   if (_pageSizeDialog == 0) {
-    // create the pageSize dialog
-    _pageSizeDialog = new pageSizeDialog(mainWidget);
+    _pageSizeDialog = new pageSizeDialog(mainWidget, &userRequestedPaperSize);
     if (_pageSizeDialog == 0) {
       kdError() << "Could not construct the page size dialog!" << endl;
       return;
     }
-    _pageSizeDialog->setOrientation(orientation->currentItem());
-    connect(_pageSizeDialog, SIGNAL(sizeSelected(const QString&, int)), this, SLOT(slotSizeSelected(const QString&, int)));
   }
-
+  
+  // Reset the "preferred paper size" menu. We don't want to have the
+  // "custom paper size" check if the user aborts the dialog.
+  checkActions();
+  
   // Set or update the paper size dialog to show the currently
   // selected value.
-  media->setCurrentItem(_paperSize.formatNumber()+1); // Give useful setting in case the user aborts the dialog
   _pageSizeDialog->setPageSize(_paperSize.serialize());
   _pageSizeDialog->show();
 }
 
 
-void KViewPart::slotSizeSelected(const QString &sizeString, int ori)
+void KViewPart::slotUserPreferredSizeChanged(float, float)
 {
-  // Check if there is something to do for us. Speeds up the GUI a
-  // lot.
-  if ((ori == orientation->currentItem())&&(sizeString == _paperSize.serialize()))
-    return;
+  checkActions();
+  
+  // Set the paper size that is currently displayed. The GUI is
+  // updated automatically using the signal/slot mechanism.
+  if (!documentHasRequestedPaperSize || !useDocumentSpecifiedSize->isChecked()) 
+    _paperSize = userRequestedPaperSize;
+}
 
-  orientation->setCurrentItem(ori);
-  _paperSize.setPageSize(sizeString);
-  // We never select "Custom paper size" so that the user can select
-  // it several times in a row.
-  if (_paperSize.formatNumber() != -1)
-    media->setCurrentItem(_paperSize.formatNumber()+1);
-  slotOrientation(0); // Calls setPaperSize() and updates the display.
+
+void KViewPart::slotUseDocumentSpecifiedSize(bool doUse)
+{
+#ifdef DEBUG_KVIEWPART
+  kdDebug() << "KViewPart::slotUseDocumentSpecifiedSize() called" << endl;
+#endif
+
+  // Set the paper size that is currently displayed. The GUI is
+  // updated automatically using the signal/slot mechanism.
+  if (doUse && documentHasRequestedPaperSize)
+    _paperSize = documentRequestedPaperSize;
+  else
+    _paperSize = userRequestedPaperSize;
+  checkActions();
+}
+
+
+void KViewPart::doRepaintScrollBoxBackground(void)
+{
+  QPixmap pixmap(scrollBox->width(), scrollBox->height());
+  QPainter p(&pixmap);
+  
+  if (multiPage->preview(&p, scrollBox->width(), scrollBox->height()))
+    scrollBox->setBackgroundPixmap(pixmap);
 }
 
 
@@ -573,15 +815,25 @@ void KViewPart::pageSelected(int nr)
 
 void KViewPart::zoomIn()
 {
-  _zoomVal.setZoomValue(multiPage->setZoom(_zoomVal.zoomIn()));
-  updateScrollBox();
+  float oldVal = _zoomVal.value();
+  float newVal = _zoomVal.zoomIn();
+
+  if (oldVal != newVal) {
+    _zoomVal.setZoomValue(multiPage->setZoom(_zoomVal.zoomIn()));
+    updateScrollBox();
+  }
 }
 
 
 void KViewPart::zoomOut()
 {
-  _zoomVal.setZoomValue(multiPage->setZoom(_zoomVal.zoomOut()));
-  updateScrollBox();
+  float oldVal = _zoomVal.value();
+  float newVal = _zoomVal.zoomOut();
+
+  if (oldVal != newVal) {
+    _zoomVal.setZoomValue(multiPage->setZoom(_zoomVal.zoomOut()));
+    updateScrollBox();
+  }
 }
 
 
@@ -590,8 +842,8 @@ void KViewPart::fitToPage()
   double w, h;
   float _zoom;
 
-  w = multiPage->zoomForWidth(pageSize().width());
-  h = multiPage->zoomForHeight(pageSize().height());
+  w = multiPage->zoomForWidth(sizeOfPage().width());
+  h = multiPage->zoomForHeight(sizeOfPage().height());
 
   if (w < h)
     _zoom = w;
@@ -617,12 +869,8 @@ void KViewPart::fitToHeight()
 {
   // See below, in the documentation of the method "fitToWidth" for an
   // explanation of the complicated calculation we are doing here.
-  int targetHeight  = multiPage->scrollView()->viewportSize(0,0).height()-1;
-  int targetWidth;
-  if (orientation->currentItem() == 0)
-    targetWidth = (int)(targetHeight * _paperSize.pageWidth_in_mm()/_paperSize.pageHeight_in_mm() +0.5);
-  else
-    targetWidth = (int)(targetHeight * _paperSize.pageHeight_in_mm()/_paperSize.pageWidth_in_mm() +0.5);
+  int targetHeight = multiPage->scrollView()->viewportSize(0,0).height()-1;
+  int targetWidth  = (int)(targetHeight * _paperSize.width_in_mm()/_paperSize.height_in_mm() +0.5);
   targetHeight = multiPage->scrollView()->viewportSize(targetWidth, targetHeight).height()-1;
 
   float _zoom = multiPage->zoomForHeight(targetHeight);
@@ -639,22 +887,18 @@ void KViewPart::fitToWidth()
 {
   // There is a slight complication here... if we just take the width
   // of the viewport and scale the contents by a factor x so that it
-  // fits the viewport exactly, then, depending on choosen papersize
+  // fits the viewport exactly, then, depending on chosen papersize
   // (landscape, etc.), the contents may be higher than the viewport
   // and the QScrollview may or may not insert a scrollbar at the
   // right. If the scrollbar appears, then the usable width of the
   // viewport becomes smaller, and scaling by x does not really fit
   // the (now smaller page) anymore.
 
-  // Caluculate the width and heigt of the view, disregarding the
+  // Caluculate the width and height of the view, disregarding the
   // possible complications with scrollbars, e.g. assuming the maximal
   // space is available.
   int targetWidth  = multiPage->scrollView()->viewportSize(0,0).width()-1;
-  int targetHeight;
-  if (orientation->currentItem() == 0)
-    targetHeight = (int)(targetWidth * _paperSize.pageHeight_in_mm()/_paperSize.pageWidth_in_mm() +0.5);
-  else
-    targetHeight = (int)(targetWidth * _paperSize.pageWidth_in_mm()/_paperSize.pageHeight_in_mm() +0.5);
+  int targetHeight = (int)(targetWidth * _paperSize.height_in_mm()/_paperSize.width_in_mm() +0.5);
   // Think again, this time use only the area which is really
   // acessible (which, in case that targetWidth targetHeight don't fit
   // the viewport, is really smaller because of the scrollbars).
@@ -680,7 +924,7 @@ void KViewPart::setZoomValue(const QString &sval)
 }
 
 
-QSize KViewPart::pageSize()
+QSize KViewPart::sizeOfPage()
 {
   QRect r = multiPage->widget()->childrenRect();
   return QSize(r.width(), r.height());
@@ -720,7 +964,23 @@ void KViewPart::checkActions()
   orientation->setEnabled(doc);
 
   printAction->setEnabled(doc);
-  saveAction->setEnabled(doc);
+  if (multiPage->isReadWrite())
+    saveAction->setEnabled(multiPage->isModified());
+  saveAsAction->setEnabled(doc);
+
+  if (documentHasRequestedPaperSize && useDocumentSpecifiedSize->isChecked()) {
+    media->setEnabled(false);
+    orientation->setEnabled(false);
+  } else {
+    if (userRequestedPaperSize.formatNumber() != -1) {
+      orientation->setCurrentItem(userRequestedPaperSize.getOrientation());
+      orientation->setEnabled(true);
+      media->setCurrentItem(userRequestedPaperSize.formatNumber()+1);
+    } else {
+      orientation->setEnabled(false);
+      media->setCurrentItem(userRequestedPaperSize.formatNumber()-1);
+    }
+  }
 }
 
 
@@ -767,21 +1027,81 @@ bool KViewPart::eventFilter(QObject *obj, QEvent *ev)
 }
 
 
+void KViewPart::wheelEvent(QWheelEvent *e)
+{
+  QScrollBar *sb = multiPage->scrollView()->verticalScrollBar();
+  if (sb == 0)
+    return;
+  
+  Q_INT32 pxl = -(e->delta()*sb->lineStep())/60;
+  if (pxl == 0) 
+    if (e->delta() > 0)
+      pxl = -1;
+    else
+      pxl = 1;
+  scroll(pxl);
+}
+
 void KViewPart::scrollUp()
 {
   QScrollBar *sb = multiPage->scrollView()->verticalScrollBar();
-  if (sb)
-    sb->subtractLine();
-  updateScrollBox();
+  if (sb == 0)
+    return;
+
+  scroll(-sb->lineStep());
 }
 
 
 void KViewPart::scrollDown()
 {
   QScrollBar *sb = multiPage->scrollView()->verticalScrollBar();
-  if (sb)
-    sb->addLine();
-  updateScrollBox();
+  if (sb == 0)
+    return;
+
+  scroll(sb->lineStep());
+}
+
+
+void KViewPart::scroll(Q_INT32 deltaInPixel)
+{
+  QScrollBar *sb = multiPage->scrollView()->verticalScrollBar();
+  if (sb == 0) {
+    kdError(4300) << "KViewpart::scroll called without scrollBar" << endl;
+    return;
+  }
+
+  if (deltaInPixel < 0) {
+    if (sb->value() == sb->minValue()) {
+      if ( (page() == 0) || (changePageDelayTimer.isActive()) )
+	return;
+      
+      changePageDelayTimer.stop();
+      prevPage();
+      QScrollView *sv = multiPage->scrollView();
+      scrollTo(sv->contentsX(), sb->maxValue());
+      return;
+    }
+  }
+  
+  if (deltaInPixel > 0) {
+    if (sb->value() == sb->maxValue()) {
+      if ( (page()+1 == pages()) || (changePageDelayTimer.isActive()) )
+	return;
+      
+      changePageDelayTimer.stop();
+      nextPage();
+      QScrollView *sv = multiPage->scrollView();
+      scrollTo(sv->contentsX(), 0);
+      return;
+    }
+  }
+  
+  sb->setValue(sb->value() + deltaInPixel);
+
+  if ( (sb->value() == sb->maxValue()) || (sb->value() == sb->minValue()) )
+    changePageDelayTimer.start(200,true);
+  else
+    changePageDelayTimer.stop();
 }
 
 
@@ -849,20 +1169,21 @@ void KViewPart::scrollTo(int x, int y)
 
 void KViewPart::updatePreview(bool avail)
 {
-  QString size = _paperSize.serialize()+"/";
-  if (orientation->currentItem() == 0)
-    size += i18n("portrait");
-  else
-    size += i18n("landscape");
-  emit sizeChanged(size);
+  if (showPreview->isChecked() && showmarklist->isChecked() && avail)
+    // Rather than calling the method doRepaintScrollBoxBackground()
+    // here (which turns to the multipage to have a pixmap for the
+    // preview drawn), we use a timer here whose timeout() signal is
+    // connected to doRepaintScrollBoxBackground(). Giving the
+    // interval is 0ms will, according to the manual of QT, make the
+    // timer time out only after all the events in the window system's
+    // event queue have been processed.
 
-
-  QPixmap pixmap(scrollBox->width(), scrollBox->height());
-  QPainter p(&pixmap);
-
-  if (showPreview->isChecked() && avail
-      && multiPage->preview(&p, scrollBox->width(), scrollBox->height()))
-    scrollBox->setBackgroundPixmap(pixmap);
+    // The reason: shrinking the pixmap to create the preview often
+    // takes a while. Doing that after all other events are processed
+    // makes sure that the user interface is not blocked. That is
+    // particularly important when the user uses repeat keystrokes to
+    // read through the document.
+    previewPaintInitiator.start(0, true);
   else
     scrollBox->setBackgroundMode(QFrame::PaletteMid);
 }
@@ -897,27 +1218,27 @@ void KViewPart::readSettings()
   slotShowMarkList();
 
   watchAct->setChecked(config->readBoolEntry("WatchFile", true));
-  showPreview->setChecked(config->readBoolEntry("ShowPreview", true));
+  showPreview->setChecked(config->readBoolEntry("ShowPreview", false));
   float _zoom = config->readDoubleNumEntry("Zoom", 1.0);
   if ( (_zoom < ZoomLimits::MinZoom/1000.0) || (_zoom > ZoomLimits::MaxZoom/1000.0))
     _zoom = 1.0;
   _zoomVal.setZoomValue(multiPage->setZoom(_zoom));
 
-  // Read Paper Size and orientation
-  int orient = config->readNumEntry("Orientation", 0 );
-  orientation->setCurrentItem(orient);
-  if (_pageSizeDialog != 0)
-    _pageSizeDialog->setOrientation(orient);
+  // Read Paper Size. and orientation. The GUI is updated
+  // automatically by the signals/slots mechanism whenever
+  // userRequestedPaperSize is changed.
+  userRequestedPaperSize.setPageSize(config->readEntry("PaperFormat", QString::null));
 
-  _paperSize.setPageSize(config->readEntry("PaperFormat", QString::null));
-  // Since "Custom paper size" is the first entry in the media list,
-  // we can just add 1 to the formatNumber. We never select "Custom
-  // paper size" so that the user can select it several times in a row.
-  if (_paperSize.formatNumber() == -1)
-    media->setCurrentItem(-1); // Nothing selected
-  else
-    media->setCurrentItem(_paperSize.formatNumber()+1);
-  slotOrientation(0); // Apply orientation and paperSize settings.
+  // Check if scrollbars should be shown
+  bool sbstatus = config->readBoolEntry("Scrollbars", true);
+  scrollbarHandling->setChecked(sbstatus);
+  emit scrollbarStatusChanged(sbstatus);
+
+  // Check if document specified paper sizes should be shown. We do
+  // not need to take any action here, because this method is called
+  // only in the constructor of the KViewPart when no document is
+  // loaded
+  useDocumentSpecifiedSize->setChecked(config->readBoolEntry("useDocumentSpecifiedSize", true));
 }
 
 
@@ -931,9 +1252,9 @@ void KViewPart::writeSettings()
   config->writeEntry("WatchFile", watchAct->isChecked());
   config->writeEntry("ShowPreview", showPreview->isChecked());
   config->writeEntry("Zoom", _zoomVal.value());
+  config->writeEntry("PaperFormat", userRequestedPaperSize.serialize() );
+  config->writeEntry("Scrollbars", scrollbarHandling->isChecked() );
 
-  config->writeEntry("Orientation", orientation->currentItem() );
-  config->writeEntry("PaperFormat", _paperSize.serialize() );
   config->sync();
 }
 
@@ -943,6 +1264,9 @@ void KViewPart::readDown()
   QScrollBar *sb = multiPage->scrollView()->verticalScrollBar();
   if (sb) {
     if (sb->value() == sb->maxValue()) {
+      if (page()+1 == pages())
+	return;
+
       nextPage();
       QScrollView *sv = multiPage->scrollView();
       scrollTo(sv->contentsX(), 0);
@@ -952,9 +1276,26 @@ void KViewPart::readDown()
 }
 
 
+void KViewPart::readUp()
+{
+  QScrollBar *sb = multiPage->scrollView()->verticalScrollBar();
+  if (sb) {
+    if (sb->value() == sb->minValue()) {
+      if (page() == 0)
+	return;
+
+      prevPage();
+      QScrollView *sv = multiPage->scrollView();
+      scrollTo(sv->contentsX(), sb->maxValue());
+    } else
+      sb->subtractPage();
+  }
+}
+
+
 void KViewPart::connectNotify ( const char *sig )
 {
-  if (QString("2pageChanged(const QString&)") == sig)
+  if (QString(sig).contains("pageChanged"))
     pageChangeIsConnected = true;
 }
 
@@ -971,6 +1312,16 @@ void KViewPart::setStatusBarTextFromMultiPage( const QString &msg )
     emit setStatusBarText(msg);
 }
 
+void KViewPart::partActivateEvent( KParts::PartActivateEvent *ev )
+{
+    QApplication::sendEvent( multiPage, ev );
+}
+
+void KViewPart::guiActivateEvent( KParts::GUIActivateEvent *ev )
+{
+    QApplication::sendEvent( multiPage, ev );
+}
+
 
 KViewPartExtension::KViewPartExtension(KViewPart *parent)
   : KParts::BrowserExtension( parent, "KViewPartExtension")
