@@ -1,5 +1,14 @@
+/*
+ * Parts of this file are
+ * Copyright 2003 Waldo Bastian <bastian@kde.org>
+ *
+ * These parts are free software; you can redistribute and/or modify
+ * them under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2.
+ */
+ 
+
 #include <qtimer.h>
-#include <qdragobject.h>
 #include <qregexp.h>
 
 #include <kiconloader.h>
@@ -10,11 +19,12 @@
 #include <klocale.h>
 #include <kfiledialog.h>
 #include <kmessagebox.h>
+#include <kmenubar.h>
 #include <klibloader.h>
 #include <kstdaction.h>
 #include <khelpmenu.h>
 #include <kedittoolbar.h>
-
+#include <kurldrag.h>
 
 #include <kprogress.h>
 #include <qlabel.h>
@@ -24,6 +34,7 @@
 
 #include "kviewpart_iface.h"
 #include <kstatusbar.h>
+#include <kkeydialog.h>
 #include "kviewshell.moc"
 
 
@@ -57,34 +68,43 @@ KViewShell::KViewShell(QString partname, KAboutData *about)
   closeAction = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
   KStdAction::quit (this, SLOT(slotQuit()), actionCollection());
 
+  // view menu
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,1,90)
+  fullScreenAction = KStdAction::fullScreen(this, SLOT(slotFullScreen()), actionCollection(), this, "fullscreen" );
+
+
   // settings menu
-  showtoolbar = KStdAction::showToolbar(this, SLOT(slotShowToolBar()), actionCollection());
-  showstatusbar = KStdAction::showStatusbar(this, SLOT(slotShowStatusBar()), actionCollection());
+  createStandardStatusBarAction();
+#endif
+  setStandardToolBarMenuEnabled(true);
+
+  KStdAction::keyBindings(this, SLOT(slotConfigureKeys()), actionCollection());
+  KStdAction::configureToolbars(this, SLOT(slotEditToolbar()), actionCollection());
 
   // statusbar connects
   connect( view, SIGNAL( zoomChanged(const QString &) ), this,SLOT( slotChangeZoomText(const QString &) ) );
   connect( view, SIGNAL( pageChanged(const QString &) ), this,SLOT( slotChangePageText(const QString &) ) );
   connect( view, SIGNAL( sizeChanged(const QString &) ), this,SLOT( slotChangeSizeText(const QString &) ) );
 
-  //Creates new object of class statusbar
-  statusbar= new KStatusBar(this,"StatusBar");
-
   m_helpMenu = new KHelpMenu( this, about, true, actionCollection() );
 
   setXMLFile( "kviewshell.rc" );
-
   createGUI(view);
-
   readSettings();
-
   checkActions();
-
   setAcceptDrops(true);
 
+  // If kviewshell is started when another instance of kviewshell runs
+  // in fullscreen mode, the menubar is switched off by default, which
+  // is a nasty surprise for the user: there is no way to access the
+  // menus. To avoid such complications, we switch the menubar on
+  // explicitly.
+  menuBar()->show();
+
   // Add statusbar-widgets for zoom, pagenr and format
-  statusbar->insertItem("", StatusBar_ID_PageNr, 0, true);
-  statusbar->insertFixedItem("XXXX%", StatusBar_ID_Zoom, true);
-  statusbar->insertItem("", StatusBar_ID_PageSize, 0, true);
+  statusBar()->insertItem("", StatusBar_ID_PageNr, 0, true);
+  statusBar()->insertFixedItem("XXXX%", StatusBar_ID_Zoom, true);
+  statusBar()->insertItem(view->pageSizeDescription(), StatusBar_ID_PageSize, 0, true);
 }
 
 
@@ -94,6 +114,7 @@ void KViewShell::checkActions()
 
   closeAction->setEnabled(doc);
   reloadAction->setEnabled(doc);
+  fullScreenAction->setEnabled(doc);
 }
 
 
@@ -106,6 +127,21 @@ KViewShell::~KViewShell()
 
 void KViewShell::slotQuit()
 {
+  // If we are to quit the application while we operate in fullscreen
+  // mode, we need to restore the visibility properties of the
+  // statusbar, toolbar and the menus because these properties are
+  // saved automatically ... and we don't want to start next time
+  // without having menus.
+  if (fullScreenAction->isChecked()) {
+    kdDebug() << "Switching off fullscreen mode before quitting the application" << endl;
+    showNormal();
+    if (isStatusBarShownInNormalMode)
+      statusBar()->show();
+    if (isToolBarShownInNormalMode)
+      toolBar()->show();
+    menuBar()->show();
+    view->slotSetFullPage(false); 
+  }
   kapp->closeAllWindows();
   kapp->quit();
 }
@@ -113,24 +149,18 @@ void KViewShell::slotQuit()
 
 void KViewShell::readSettings()
 {
+  // Hmm using kapp->config() here means all kviewshell-based apps share the same setings... (DF)
+
+  resize(600, 300); // default size if the config file specifies no size
+  setAutoSaveSettings( "General" ); // apply mainwindow settings (size, toolbars, etc.)
+
   KConfig *config = kapp->config();
-
   config->setGroup("General");
 
-  showtoolbar->setChecked(config->readBoolEntry("ShowToolBar", true));
-  slotShowToolBar();
-
-  showstatusbar->setChecked (config->readBoolEntry ("ShowStatusBar", true));
-  slotShowStatusBar();
-
-  int w = config->readNumEntry ("WindowWidth", 600);
-  int h = config->readNumEntry ("WindowHeight", 300);
-  resize(w, h);
-
   recent->loadEntries(config, QString("%1-RecentFiles").arg(_partname));
 
-  // Constant source of anoyment in KDVI < 1.0: the 'recent-files'
-  // menu contains lots of files which don't exist (any long). Thus,
+  // Constant source of annoyance in KDVI < 1.0: the 'recent-files'
+  // menu contains lots of files which don't exist (any longer). Thus,
   // we'll sort out the non-existent files here.
   QStringList items = recent->items();
   for ( QStringList::Iterator it = items.begin(); it != items.end(); ++it ) {
@@ -148,14 +178,7 @@ void KViewShell::readSettings()
 void KViewShell::writeSettings()
 {
   KConfig *config = kapp->config();
-
   config->setGroup( "General" );
-
-  config->writeEntry ("ShowToolBar", showtoolbar->isChecked());
-  config->writeEntry ("ShowStatusBar", showstatusbar->isChecked());
-  config->writeEntry ("WindowWidth", width());
-  config->writeEntry ("WindowHeight", height());
-
   recent->saveEntries(config, QString("%1-RecentFiles").arg(_partname));
 
   config->sync();
@@ -187,6 +210,36 @@ void KViewShell::slotFileOpen()
 }
 
 
+void KViewShell::slotFullScreen()
+{
+  if (fullScreenAction->isChecked()) {
+    // In fullscreen mode, menu- tool- and statusbar are hidden. Save
+    // the visibility flags of these objects here, so that they can
+    // later be properly restored when we switch back to normal mode,
+    // or before we leave the application in slotQuit()
+    isStatusBarShownInNormalMode = statusBar()->isShown();
+    statusBar()->hide();
+    isToolBarShownInNormalMode = toolBar()->isShown();
+    toolBar()->hide();
+    menuBar()->hide();
+    view->slotSetFullPage(true); 
+
+    // Go to fullscreen mode
+    showFullScreen();
+
+    KMessageBox::information(this, i18n("Use the Escape key to leave the fullscreen mode."), i18n("Entering Fullscreen Mode"), "leavingFullScreen");
+  } else {
+    showNormal();
+    if (isStatusBarShownInNormalMode)
+      statusBar()->show();
+    if (isToolBarShownInNormalMode)
+      toolBar()->show();
+    menuBar()->show();
+    view->slotSetFullPage(false); 
+  }
+}
+
+
 void KViewShell::slotFileClose()
 {
   view->closeURL();
@@ -194,50 +247,48 @@ void KViewShell::slotFileClose()
   checkActions();
 }
 
-
-void KViewShell::slotShowStatusBar()
+void KViewShell::slotConfigureKeys()
 {
-  if (showstatusbar->isChecked())
-    statusBar()->show();
-  else
-    statusBar()->hide();
+  KKeyDialog dlg( true, this );
+
+  dlg.insert( actionCollection() );
+  dlg.insert( view->actionCollection() );
+
+  dlg.configure();
 }
 
-
-void KViewShell::slotShowToolBar()
-{
-  if (showtoolbar->isChecked())
-    toolBar()->show();
-  else
-    toolBar()->hide();
-}
-
-
 void KViewShell::slotEditToolbar()
 {
-  KEditToolbar dlg(actionCollection());
-  if (dlg.exec())
-    createGUI(view);
+  saveMainWindowSettings( KGlobal::config(), autoSaveGroup() );
+  KEditToolbar dlg(factory());
+  connect( &dlg, SIGNAL( newToolbarConfig() ), SLOT( slotNewToolbarConfig() ) );
+  dlg.exec();
 }
 
 
+void KViewShell::slotNewToolbarConfig()
+{
+  applyMainWindowSettings( KGlobal::config(), autoSaveGroup() );
+}
+
 void KViewShell::dragEnterEvent(QDragEnterEvent *event)
 {
-  if (QUriDrag::canDecode(event)) {
-    QStringList urls;
-    QUriDrag::decodeToUnicodeUris(event, urls);
+  if (KURLDrag::canDecode(event)) {
+    KURL::List urls;
+    KURLDrag::decode(event, urls);
     if (urls.count() > 0) {
+      const KURL &url = urls.first();
       // look if the extension is right
       QStringList list = view->fileFormats();
       QStringList::Iterator it;
       for (it = list.begin(); it != list.end(); ++it) {
-	QString ext = *it;
+	QString ext = (*it);
 	int pos = ext.find('|');
 	if (pos >= 0)
 	  ext = ext.left(pos);
 
 	QRegExp re(ext, false, true);
-	if (re.search(urls[0]) >= 0) {
+	if (re.search(url.fileName()) >= 0) {
 	  event->accept(true);
 	  return;
 	}
@@ -250,28 +301,38 @@ void KViewShell::dragEnterEvent(QDragEnterEvent *event)
 
 void KViewShell::dropEvent(QDropEvent *event)
 {
-  if (QUriDrag::canDecode(event)) {
-    QStringList urls;
-    QUriDrag::decodeToUnicodeUris(event, urls);
-    if (urls.count() > 0)
-      openURL(KURL(urls[0]));
+  KURL::List urls;
+  if (KURLDrag::decode(event, urls) && !urls.isEmpty())
+    openURL(urls.first());
+}
+
+
+void KViewShell::keyPressEvent(QKeyEvent *event)
+{
+  // The Escape Key is used to return to normal mode from fullscreen
+  // mode
+  if ((event->key() == Qt::Key_Escape) && (fullScreenAction->isChecked())) {
+    showNormal();
+    return;
   }
+  // If we can't use the key event, pass it on
+  event->ignore();
 }
 
 
 void KViewShell::slotChangePageText(const QString &message)
 {
-  statusbar->changeItem(" "+message+" ",StatusBar_ID_PageNr);
+  statusBar()->changeItem(" "+message+" ",StatusBar_ID_PageNr);
 }
 
 
 void KViewShell::slotChangeSizeText(const QString &message)
 {
-  statusbar->changeItem(" "+message+" ",StatusBar_ID_PageSize);
+  statusBar()->changeItem(" "+message+" ",StatusBar_ID_PageSize);
 }
 
 
 void KViewShell::slotChangeZoomText(const QString &message)
 {
-  statusbar->changeItem(" "+message+" ",StatusBar_ID_Zoom);
+  statusBar()->changeItem(" "+message+" ",StatusBar_ID_Zoom);
 }
