@@ -12,7 +12,6 @@
 
 class KRecentFilesAction;
 class ScrollBox;
-class KToggleAction;
 class KViewPart_Iface;
 class KHelpMenu;
 class KAboutData;
@@ -27,7 +26,7 @@ class KViewShell : public KParts::MainWindow
 public:
   KStatusBar *statusbar;
   KStatusBar *action;
-  
+
   KViewShell(QString partname, KAboutData *);
   virtual ~KViewShell();
 
@@ -37,11 +36,12 @@ public slots:
 
 protected slots:
   void slotFileOpen();
-  void slotShowToolBar();
-  void slotShowStatusBar();
+  void slotFullScreen();
   void slotQuit();
+  void slotConfigureKeys();
   void slotEditToolbar();
   void slotFileClose();
+  void slotNewToolbarConfig();
 
   void slotChangeZoomText(const QString &);
   void slotChangePageText(const QString &);
@@ -55,7 +55,7 @@ protected:
 
   void dragEnterEvent(QDragEnterEvent *event);
   void dropEvent(QDropEvent *event);
-
+  void keyPressEvent(QKeyEvent * e);
 
 private:
   QString _partname;
@@ -65,9 +65,15 @@ private:
   QString cwd;
 
   KAction *closeAction, *reloadAction;
-  KToggleAction *showtoolbar, *showstatusbar;
-
+  KToggleAction *fullScreenAction;
   KHelpMenu *m_helpMenu;
+
+  // In the attribute, the status of the statusbar (shown of hidden)
+  // is saved when the kviewshell switches to fullscreen mode. The
+  // statusbar can then be restored when the application returns to normal mode.
+  bool isStatusBarShownInNormalMode;
+  // ditto, for the toolbar
+  bool isToolBarShownInNormalMode;
 };
 
 
