@@ -2,7 +2,7 @@
 //
 // Part of KVIEWSHELL - A framework for multipage text/gfx viewers
 //
-// (C) 2002 Stefan Kebekus
+// (C) 2002-2003 Stefan Kebekus
 // Distributed under the GPL
 
 // Add header files alphabetically
@@ -14,6 +14,7 @@
 #include <qstringlist.h>
 
 #include "pageSize.h"
+#include "units.h"
 
 struct pageSizeItem 
 {
@@ -51,7 +52,7 @@ pageSize::pageSize(void)
 }
 
 
-void pageSize::setPageSize(QString name)
+bool pageSize::setPageSize(QString name)
 {
   // See if we can recognize the string
   QString currentName;
@@ -63,32 +64,76 @@ void pageSize::setPageSize(QString name)
       pageWidth  = staticList[currentSize].width;
       pageHeight = staticList[currentSize].height;
       emit(sizeChanged(pageWidth, pageHeight));
-      return;
+      return true;
     }
   }
 
-  // Check if the string is of type "<number>x<number>"
-  bool wok, hok;
-  pageWidth  = name.section('x',0,0).toFloat(&wok);
-  pageHeight = name.section('x',1,1).toFloat(&hok);
-  if (wok && hok) {
-    rectifySizes();
-    emit(sizeChanged(pageWidth, pageHeight));
-    reconstructCurrentSize();
-    return;
+  // Check if the string contains 'x'. If yes, we assume it is of type
+  // "<number>x<number>". If yes, the first number is interpreted as
+  // the width in mm, the second as the height in mm
+  if (name.find('x') >= 0) {
+    bool wok, hok;
+    float pageWidth_tmp  = name.section('x',0,0).toFloat(&wok);
+    float pageHeight_tmp = name.section('x',1,1).toFloat(&hok);
+    if (wok && hok) {
+      pageWidth  = pageWidth_tmp;
+      pageHeight = pageHeight_tmp;
+      
+      rectifySizes();
+      reconstructCurrentSize();
+      emit(sizeChanged(pageWidth, pageHeight));
+      return true;
+    }
   }
-
+  
+  // Check if the string contains ','. If yes, we assume it is of type
+  // "<number><unit>,<number><uni>". The first number is supposed to
+  // be the width, the second the height.
+  if (name.find(',') >= 0) {
+    bool wok, hok;
+    float pageWidth_tmp  = distance::convertToMM(name.section(',',0,0), &wok);
+    float pageHeight_tmp = distance::convertToMM(name.section(',',1,1), &hok);
+    if (wok && hok) {
+      pageWidth  = pageWidth_tmp;
+      pageHeight = pageHeight_tmp;
+      
+      rectifySizes();
+      reconstructCurrentSize();
+      emit(sizeChanged(pageWidth, pageHeight));
+      return true;
+    } 
+  }  
+  
   // Last resource. Set the default, in case the string is
   // unintelligible to us.
   currentSize = defaultPageSize();
   pageWidth  = staticList[currentSize].width;
   pageHeight = staticList[currentSize].height;
+  kdError() << "pageSize::setPageSize: could not parse '" << name << "'. Using " << staticList[currentSize].name << " as a default." << endl;
   emit(sizeChanged(pageWidth, pageHeight));
+  return false;
+}
+
+
+void pageSize::setPageSize(float width, float height)
+{
+  double oldPageWidth  = pageWidth;
+  double oldPageHeight = pageHeight;
+
+  pageWidth  = width;
+  pageHeight = height;
+  
+  rectifySizes();
+  reconstructCurrentSize();
+  if ( fabs(pageWidth - oldPageWidth) > 2.0 || fabs(pageHeight - oldPageHeight) > 2.0)
+    emit(sizeChanged(pageWidth, pageHeight));
 }
 
 
 void pageSize::setPageSize(QString width, QString widthUnits, QString height, QString heightUnits)
 {
+  double oldPageWidth  = pageWidth;
+  double oldPageHeight = pageHeight;
   pageWidth  = width.toFloat();
   pageHeight = height.toFloat();
 
@@ -111,8 +156,24 @@ void pageSize::setPageSize(QString width, QString widthUnits, QString height, QS
     pageHeight *= 25.4;
 
   rectifySizes();
-  emit(sizeChanged(pageWidth, pageHeight));
   reconstructCurrentSize();
+  if ( fabs(pageWidth - oldPageWidth) > 2.0 || fabs(pageHeight - oldPageHeight) > 2.0)
+    emit(sizeChanged(pageWidth, pageHeight));
+}
+
+
+pageSize &pageSize::operator= (const pageSize &src)
+{
+  double oldPageWidth  = pageWidth;
+  double oldPageHeight = pageHeight;
+
+  currentSize = src.currentSize;
+  pageWidth   = src.pageWidth;
+  pageHeight  = src.pageHeight;
+
+  if ( fabs(pageWidth - oldPageWidth) > 2.0 || fabs(pageHeight - oldPageHeight) > 2.0)
+    emit(sizeChanged(pageWidth, pageHeight));
+  return *this;
 }
 
 
@@ -131,7 +192,8 @@ void pageSize::rectifySizes(void)
   return;
 }
 
-QString pageSize::preferredUnit(void)
+
+QString pageSize::preferredUnit(void) const
 {
   if (currentSize >= 0)
     return staticList[currentSize].preferredUnit;
@@ -144,7 +206,7 @@ QString pageSize::preferredUnit(void)
 }
 
 
-QString pageSize::widthString(QString unit)
+QString pageSize::widthString(QString unit) const
 {
   QString answer = "--";
 
@@ -159,7 +221,7 @@ QString pageSize::widthString(QString unit)
 }
 
 
-QString pageSize::heightString(QString unit)
+QString pageSize::heightString(QString unit) const
 {
   QString answer = "--";
 
@@ -174,7 +236,7 @@ QString pageSize::heightString(QString unit)
 }
 
 
-QStringList pageSize::pageSizeNames(void)
+QStringList pageSize::pageSizeNames(void) 
 {
   QStringList names;
   
@@ -185,7 +247,7 @@ QStringList pageSize::pageSizeNames(void)
 }
 
 
-QString pageSize::formatName(void)
+QString pageSize::formatName(void) const
 {
   if (currentSize >= 0)
     return staticList[currentSize].name;
@@ -194,9 +256,41 @@ QString pageSize::formatName(void)
 }
 
 
-QString pageSize::serialize(void)
+int pageSize::getOrientation(void) const
 {
-  if (currentSize >= 0)
+  if (currentSize == -1) {
+    kdError() << "pageSize::getOrientation: getOrientation called for page format that does not have a name." << endl;
+    return 0;
+  }
+
+  if (pageWidth == staticList[currentSize].width)
+    return 0;
+  else
+    return 1;
+}
+
+
+void pageSize::setOrientation(int orient)
+{
+  if (currentSize == -1) {
+    kdError() << "pageSize::setOrientation: setOrientation called for page format that does not have a name." << endl;
+    return;
+  }
+  
+  if (orient == 1) {
+    pageWidth  = staticList[currentSize].height;
+    pageHeight = staticList[currentSize].width;
+  } else {
+    pageWidth  = staticList[currentSize].width;
+    pageHeight = staticList[currentSize].height;
+  }
+  emit(sizeChanged(pageWidth, pageHeight));
+}
+
+
+QString pageSize::serialize(void) const
+{
+  if ((currentSize >= 0) && (fabs(staticList[currentSize].height-pageHeight) <= 0.5))
     return staticList[currentSize].name;
   else 
     return QString("%1x%2").arg(pageWidth).arg(pageHeight);
@@ -205,19 +299,29 @@ QString pageSize::serialize(void)
 
 void pageSize::reconstructCurrentSize(void)
 {
-  for(int i=0; staticList[i].name != 0; i++) 
+  for(int i=0; staticList[i].name != 0; i++) {
     if ((fabs(staticList[i].width - pageWidth) <= 2) && (fabs(staticList[i].height - pageHeight) <= 2)) {
       currentSize = i;
       pageWidth  = staticList[currentSize].width;
       pageHeight = staticList[currentSize].height;
       return;
     }
+    if ((fabs(staticList[i].height - pageWidth) <= 2) && (fabs(staticList[i].width - pageHeight) <= 2)) {
+      currentSize = i;
+      pageWidth  = staticList[currentSize].height;
+      pageHeight = staticList[currentSize].width;
+      return;
+    }
+  }
   currentSize = -1;
   return;
 }
 
 int pageSize::defaultPageSize(void)
 {
+  // FIXME: static_cast<QPrinter::PageSize>(KGlobal::locale()->pageSize())
+  //        is the proper solution here.  Then you can determine the values
+  //        without using your hardcoded table too!
   if (KGlobal::locale()-> measureSystem() == KLocale::Metric)
     return defaultMetricPaperSize;
   else
