@@ -1,6 +1,6 @@
 /*
  *   pageSizeDialog, for kviewshell
- *   This file: Copyright (C) 2002 Stefan Kebekus, kebekus@kde.org
+ *   This file: Copyright (C) 2002-2003 Stefan Kebekus, kebekus@kde.org
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
  */
 #ifndef _PAGESIZE_DIALOG_H_
@@ -23,21 +23,20 @@
 #include <kdialogbase.h>
 
 class pageSizeWidget;
+class pageSize;
 
 class pageSizeDialog : public KDialogBase
 {
   Q_OBJECT
     
 public:
-  pageSizeDialog( QWidget *parent=0, const char *name=0, bool modal=true);
+  // Constructs the page size Dialog. The pointer userPrefdPageSize
+  // points to a pageSize object which will be set the the chosen
+  // value whenever the user clicks on 'accept' or 'ok'. Programmers
+  // can then connect to the signal sizeChanged() of the object to be
+  // informed about the changes.
+  pageSizeDialog( QWidget *parent=0, pageSize *userPrefdPageSize=0, const char *name=0, bool modal=true);
   void setPageSize(QString);
-  void setOrientation(int);
-
-signals: 
-  // Emitted to indicate that the user has chosen a size. The string
-  // is suitable to be fed into the setPageSize() method of a pageSize
-  // object.
-  void sizeSelected(const QString &size, int orientation);
 
 protected slots:
   virtual void slotOk();
@@ -45,6 +44,7 @@ protected slots:
   
  private:
   pageSizeWidget *pageSizeW;
+  pageSize       *userPreferredPageSize;
 };
 
 
