@@ -25,11 +25,11 @@ public:
   /** Returns a list like "33%", "100%", etc. If you call
       zoomNames(void) more than once, it is guaranteed that the same
       list of strings will be returned. */
-  QStringList zoomNames(void) {return valueNames;};
+  QStringList zoomNames(void) const {return valueNames;};
 
   float       zoomIn(void);
   float       zoomOut(void);
-  float       value(void) {return _zoomValue;};
+  float       value(void) const {return _zoomValue;};
 
 public slots:
   void        setZoomValue(float);
