@@ -16,7 +16,7 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 
-   $Id: devselector.cpp,v 1.20 2002/09/19 10:24:10 freitag Exp $
+   $Id: devselector.cpp,v 1.21 2003/05/18 19:57:26 binner Exp $
 */
 
 #include <stdlib.h>
@@ -61,7 +61,7 @@ DeviceSelector::DeviceSelector( QWidget *parent, QStrList& devList,
    label->resize( 100, 350 );
    topLayout->addWidget( label );
 
-   selectBox = new QButtonGroup( 1, Horizontal, i18n( "Select a Scan Device" ),
+   selectBox = new QButtonGroup( 1, Horizontal, i18n( "Select Scan Device" ),
 				 page, "ButtonBox");
    Q_CHECK_PTR( selectBox );
    selectBox->setExclusive( true );
