@@ -1,5 +1,5 @@
 /* This file is part of the KDE Project
-   Copyright (C) 1999 Klaas Freitag <freitag@suse.de>  
+   Copyright (C) 1999 Klaas Freitag <freitag@suse.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -16,14 +16,14 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 
-   $Id: imgscaledialog.cpp,v 1.7 2002/09/19 10:24:10 freitag Exp $
+   $Id: imgscaledialog.cpp,v 1.9 2003/12/10 17:47:31 mlaurent Exp $
 */
 
 #include <klocale.h>
 #include <kdebug.h>
 #include <qbuttongroup.h>
 #include <qradiobutton.h>
-
+#include <knumvalidator.h>
 #include "imgscaledialog.h"
 
 /* ############################################################################## */
@@ -35,10 +35,10 @@ ImgScaleDialog::ImgScaleDialog( QWidget *parent, int curr_sel,
                  Ok|Cancel, Ok, true )
 {
    // setCaption (i18n ("Image Zoom"));
-   selected = curr_sel;   
+   selected = curr_sel;
    int        one_is_selected = false;
    enableButtonSeparator( false );
-   
+
    // (void) new QLabel( , main, "Page");
    //
    // makeMainWidget();
@@ -46,10 +46,10 @@ ImgScaleDialog::ImgScaleDialog( QWidget *parent, int curr_sel,
    setMainWidget(radios);
    Q_CHECK_PTR(radios);
    radios->setTitle( i18n("Select Image Zoom") );
-   
+
    connect( radios, SIGNAL( clicked( int )),
 	    this, SLOT( setSelValue( int )));
-   
+
    // left gap: smaller Image
    QRadioButton *rb25 = new QRadioButton (i18n ("25 %"), radios);
    if( curr_sel == 25 ){
@@ -68,55 +68,55 @@ ImgScaleDialog::ImgScaleDialog( QWidget *parent, int curr_sel,
       rb75->setChecked( true );
       one_is_selected = true;
    }
-   
+
    QRadioButton *rb100 = new QRadioButton (i18n ("100 %"), radios);
    if( curr_sel == 100 ) {
       rb100->setChecked( true );
       one_is_selected = true;
    }
-   
+
    QRadioButton *rb150 = new QRadioButton (i18n ("150 %"), radios);
    if( curr_sel == 150 ) {
       rb150->setChecked( true );
       one_is_selected = true;
    }
-   
+
    QRadioButton *rb200 = new QRadioButton (i18n ("200 %"), radios );
    if( curr_sel == 200 ) {
       rb200->setChecked( true );
       one_is_selected = true;
    }
-   
+
    QRadioButton *rb300 = new QRadioButton (i18n ("300 %"), radios );
    if( curr_sel == 300 ) {
       rb300->setChecked( true );
       one_is_selected = true;
    }
-   
+
    QRadioButton *rb400 = new QRadioButton (i18n ("400 %"), radios);
    if( curr_sel == 400 ) {
       rb400->setChecked( true );
       one_is_selected = true;
    }
-   
+
    // Custom Scaler at the bottom
    QRadioButton *rbCust = new QRadioButton (i18n ("Custom scale factor:"),
 					    radios);
    if( ! one_is_selected )
       rbCust->setChecked( true );
-      
-   
+
+
    leCust = new QLineEdit( radios );
    QString sn;
    sn.setNum(curr_sel );
-   
+   leCust->setValidator( new KIntValidator( leCust ) );
    leCust->setText(sn );
    connect( leCust, SIGNAL( textChanged( const QString& )),
 	    this, SLOT( customChanged( const QString& )));
    connect( rbCust, SIGNAL( toggled( bool )),
 	    this, SLOT(enableAndFocus(bool)));
    leCust->setEnabled( rbCust->isChecked());
-   
+
 
 }
 
@@ -128,7 +128,7 @@ void ImgScaleDialog::customChanged( const QString& s )
       selected = okval;
       emit( customScaleChange( okval ));
    } else {
-      kdDebug(28000) << "ERR: To large, to smale, or whatever shitty !" << endl;
+      kdDebug(29000) << "ERR: To large, to smale, or whatever shitty !" << endl;
    }
 }
 
@@ -147,11 +147,11 @@ void ImgScaleDialog::setSelValue( int val )
    // Check if value is in Range
    if( val >= 0 && val < (int) translator_size ) {
       selected = translator[val];
-   
+
       // Custom size selected
       if( selected == -1 ) {
 	 QString s = leCust->text();
-      
+
 	 bool ok;
 	 int  okval = s.toInt( &ok );
 	 if( ok ) {
@@ -162,13 +162,13 @@ void ImgScaleDialog::setSelValue( int val )
 	 }
       } // Selection is not custom
    } else {
-      kdDebug(28000) << "ERR: Invalid size selected !" << endl;
+      kdDebug(29000) << "ERR: Invalid size selected !" << endl;
    }
    // debug( "SetValue: Selected Scale is %d", selected );
 }
 
 
-int ImgScaleDialog::getSelected() const 
+int ImgScaleDialog::getSelected() const
 {
    return( selected );
 }
