@@ -16,7 +16,7 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 
-   $Id: kscanoption.cpp,v 1.19 2002/09/23 15:41:44 freitag Exp $
+   $Id: kscanoption.cpp,v 1.20 2003/07/01 16:22:08 mueller Exp $
 */
 
 #include <stdlib.h>
@@ -45,7 +45,7 @@
 // #define MIN_PREVIEW_DPI 20
 // this is nowhere used!? Additionally, it's defined to 75 in kscandevice.cpp
 
-/* switch to show some from time to time usefull alloc-messages */
+/* switch to show some from time to time useful alloc-messages */
 #undef MEM_DEBUG
 
 #undef APPLY_IN_SITU
